# ----------------------------------------------------------------------------
# - warn-arion-connection-point                                              -
# - wild cruncher: connection point                                    arion -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       warn_arion_connection_point
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Arion connection point

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_DIRECTION, \
    WHDB_XPOINT_TYPE
from wcr_core.dbstore.wdbs_meta_type import WdbsRecordMetaType, \
    WDBS_STRING_META
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.system.db.wphdb_connection_point_base import \
    WphdbConnectionPointSimpleBase

WARN_ARION_XPOINT_CODE = 'wcr.system.xpoint-arion'

WARN_ARION_XPOINT_NAME = 'Arion'

WARN_ARION_XPOINT_PREFIX = 'arn'


WARN_XPOINT_ARN_LEID_KEY = 'leid'

WARN_XPOINT_ARN_CONFIG_META = WdbsRecordMetaType(
    (WARN_XPOINT_ARN_LEID_KEY, WDBS_STRING_META, 'Logical equipment ID'))

WARN_XPOINT_ARN_DFLT_ARGS = {
    WHDB_XPOINT_TYPE: WARN_ARION_XPOINT_CODE,
    WHDB_XPOINT_DIRECTION: WDBS_XPNT_BI
}


class WarnArionXPoint(WphdbConnectionPointSimpleBase):
    """
    Arion connection point.
    """

    def __init__(self, xpoint, wreg):
        super().__init__(
            WARN_ARION_XPOINT_NAME, WARN_ARION_XPOINT_PREFIX, xpoint, wreg)

    def getLogicalEquipmentId(self):
        """
        Get the logical equipment ID for this connection point.

        :return: Logical equipment ID
        :rtype: str
        """
        return self._getConfig(WARN_XPOINT_ARN_LEID_KEY)

    def setLogicalEquipmentId(self, leid):
        """
        Set the logical equipment ID for this connection point.

        :param leid: Logical equipment ID
        :type leid: str
        """
        return self._setConfig(leid, WARN_XPOINT_ARN_LEID_KEY)
