# ----------------------------------------------------------------------------
# - scmn-memoization                                                         -
# - silkan python library: memoization utilities                             -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       scmn_memoization
   :platform:     Unix, Windows
   :synopsis:     Silkan Python Library - memoization utilities

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

import functools


class memoize1(object):
    """
    Memoization decorator.
    Memoizes the decorated once.
    """
    def __init__(self, func):
        self.__func = func
        self.__last = None

    def __call__(self, *args):
        if self.__last is not None:
            la, lr = self.__last
            if la == args:
                return lr

        r = self.__func(*args)
        self.__last = (args, r)
        return r

    # noinspection PyUnusedLocal
    def __get__(self, obj, objtype):
        return functools.partial(self.__call__, obj)
