# ----------------------------------------------------------------------------
# - wact-clean-workspace-action                                              -
# - wild cruncher: clean workspace action                     core / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wact_clean_workspace_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Clean workspace action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_process import ScmnProcStreamHandlerP
from wcr_backend.wbke_make_execute import WbkeMakeExecute
from wcr_core.action.wact_action import WactActionBase, \
    WactActionCommandMixin


class _WbkeCleanWorkspaceAction(WactActionBase, WactActionCommandMixin):
    """
    Project editor action to clean the workspace.
    """

    def __init__(self):
        super().__init__(
            'Clean Workspace',
            'Clean project workspace')

    def _doExecute(self, actx, paramValuePairs):
        def run(output):
            try:
                wbke_cleanWorkspace(pctx, output)

                output.handleControlMsg('Finished cleaning workspace',
                                        ScmnProcStreamHandlerP.CTRL_FINISHED)
            except:
                output.handleControlMsg('Failed to clean workspace',
                                        ScmnProcStreamHandlerP.CTRL_FAILED)
                raise

        pctx = actx.getProjectContext()
        return self._runCommand(actx, 'clean', 'Clean', run)


def wbke_cleanWorkspace(prjContext, output):
    """
    Clean the project's workspace.

    :param prjContext: Project context
    :type prjContext: WcorProjectContextP
    :param output: Stream output handler
    :type output: ScmnProcStreamHandlerP
    """
    make = WbkeMakeExecute(prjContext)
    make.doCleanProject(output)


#: Clean workspace action
WBKE_CLEAN_WORKSPACE_ACTION = _WbkeCleanWorkspaceAction()
