# ----------------------------------------------------------------------------
# - wcgn-code-generator                                                      -
# - wild cruncher: code generator api                      backend / codegen -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgn_code_generator
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Code generator API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WcgnCodeGeneratorP(metaclass=ABCMeta):
    """
    Protocol for code generators.
    """

    @abstractmethod
    def canGenerate(self, entity):
        """
        Whether the generator can generate code for the given entity.

        :param entity: Database entity
        :type entity: WhdbBaseP
        """
        raise NotImplementedError()

    @abstractmethod
    def generate(self, entity, iostream):
        """
        Generate code for the given entity.

        :param entity: Database entity
        :type entity: WhdbBaseP
        :param iostream: Output-/error-handler
        :type iostream: ScmnProcStreamHandlerP
        """
        raise NotImplementedError()

    @abstractmethod
    def getEntryPoint(self, inam, sgrp):
        """
        Get the function entry point.

        :param inam: Name
        :type inam: str
        :param sgrp: Scheduling group
        :type sgrp: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getInputTypeList(self, inam, sgrp):
        """
        Get the function input type list in the form of a list of
        name/type tuple.

        :param inam: Function name
        :type inam: str
        :param sgrp: Scheduling group
        :type sgrp: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getOutputTypeList(self, inam, sgrp):
        """
        Get the function output type list in the form of a list of
        name/type tuple.

        :param inam: Function name
        :type inam: str
        :param sgrp: Scheduling group
        :type sgrp: str
        """
        raise NotImplementedError()


class WcgnCodeGeneratorFactoryP(metaclass=ABCMeta):
    """
    Protocol for code generator factories.
    """

    @abstractmethod
    def getName(self):
        """
        Get the descriptive name for the code generators produced.

        :return: Code generator name
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def createCodeGenerator(self, pctx, profile=None):
        """
        Create a new code generator.

        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param profile: Profile name
        :type profile: str | None
        :return: Newly created code generator
        :rtype: WcgnCodeGeneratorP
        """
        raise NotImplementedError()
