# ----------------------------------------------------------------------------
# - wmact-build-action                                                       -
# - wild cruncher: model build action                 core / model / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmact_build_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Model build action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_process import ScmnProcStreamHandlerP
from wcr_backend.actions.wbke_clean_workspace_action import wbke_cleanWorkspace
from wcr_backend.modules.model.actions.wmbke_compile_action import \
    WMBKE_COMP_TYPE_PARAM, wmbke_compile
from wcr_backend.modules.model.actions.wmbke_generate_code_action import \
    wmbke_generateCode
from wcr_core.action.wact_action import WactActionCommandMixin, \
    WACT_ACTION_EXIT_OK, WACT_ACTION_EXIT_CANCELLED
from wcr_core.actions.wact_verify_db_action import \
    WactDatabaseVerificationException
from wcr_core.modules.model.actions.wmact_action_base import WmactActionBase
from wcr_core.modules.model.db.wmhdb_type_propagation import \
    WmhdbElementsException


class _WmbkeBuildAction(WmactActionBase, WactActionCommandMixin):
    """
    Project editor action to clean workspace, and generate and compile the
    code.
    """

    def __init__(self):
        WmactActionBase.__init__(
            self, 'Build',
            'Build the project (clean/generate/compile)',
            WMBKE_COMP_TYPE_PARAM)

    def _isProtocolException(self, e):
        return isinstance(e, WactDatabaseVerificationException) or \
            isinstance(e, WmhdbElementsException)

    def _doExecute(self, actx, paramValuePairs):
        def run(output):
            try:
                wbke_cleanWorkspace(pctx, output)
                wmbke_generateCode(pctx, output)
                wmbke_compile(pctx, compType, output)

                output.handleControlMsg(
                    'Finished building project',
                    ScmnProcStreamHandlerP.CTRL_FINISHED)
            except WactDatabaseVerificationException:
                output.handleControlMsg(
                    'Database verification failed',
                    ScmnProcStreamHandlerP.CTRL_FAILED)
                raise
            except WmhdbElementsException:
                output.handleControlMsg(
                    'Type propagation failed',
                    ScmnProcStreamHandlerP.CTRL_FAILED)
                raise
            except:
                output.handleControlMsg(
                    'Failed to build code',
                    ScmnProcStreamHandlerP.CTRL_FAILED)
                raise

        compType = self._getParamValue(
            WMBKE_COMP_TYPE_PARAM, actx, paramValuePairs)
        """:type: str | None"""

        if compType is None:
            return WACT_ACTION_EXIT_CANCELLED

        pctx = actx.getProjectContext()
        self._runCommand(actx, 'compile', 'Compile', run)
        return WACT_ACTION_EXIT_OK


#: Model build action
WMBKE_BUILD_ACTION = _WmbkeBuildAction()
