# ----------------------------------------------------------------------------
# - wmact-compile-action                                                     -
# - wild cruncher: model compile action               core / model / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmact_compile_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Model compile action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_process import ScmnProcStreamHandlerP
from wcr_backend.wbke_make_execute import WbkeMakeExecute
from wcr_core.action.wact_action import WactActionCommandMixin, \
    WACT_ACTION_EXIT_CANCELLED, WACT_ACTION_EXIT_OK
from wcr_core.action.wact_action_parameter import \
    WactFixedMultiValueActionParameter
from wcr_core.action.wact_action_parameter_types import \
    WactActionStringParameterType
from wcr_core.modules.model.actions.wmact_action_base import WmactActionBase


WMBKE_COMP_TYPE_POSSIBLE_VALUES = [
    ('', 'Optimized', 'Compile to optimized code'),
    ('debug', 'Debug', 'Compile to debug code')]

WMBKE_COMP_TYPE_PARAM = WactFixedMultiValueActionParameter(
    'comptype', 'Compilation type', 'Compilation type (optimized/debug)',
    WactActionStringParameterType(), WMBKE_COMP_TYPE_POSSIBLE_VALUES,
    'Select the compilation type')


class _WmbkeCompileAction(WmactActionBase, WactActionCommandMixin):
    """
    Project editor action to compile the generated code.
    """

    def __init__(self):
        super().__init__(
            'Compile', 'Compile project or instance', WMBKE_COMP_TYPE_PARAM)

    def _doExecute(self, actx, paramValuePairs):
        def run(output):
            try:
                wmbke_compile(pctx, compType, output)

                output.handleControlMsg('Compilation completed',
                                        ScmnProcStreamHandlerP.CTRL_FINISHED)
            except:
                output.handleControlMsg('Compilation failed',
                                        ScmnProcStreamHandlerP.CTRL_FAILED)
                raise

        compType = self._getParamValue(
            WMBKE_COMP_TYPE_PARAM, actx, paramValuePairs)
        """:type: str | None"""

        if compType is None:
            return WACT_ACTION_EXIT_CANCELLED

        pctx = actx.getProjectContext()

        self._runCommand(actx, 'compile', 'Compile', run)
        return WACT_ACTION_EXIT_OK


def wmbke_compile(projectCtx, compType, output):
    """
    Compile the project's code.

    :param projectCtx: Project context
    :type projectCtx: WcorProjectContextP
    :param compType: Compilation type (e.g. debug or optimized)
    :type compType: str
    :param output: Stream output handler
    :type output: ScmnProcStreamHandlerP
    """
    make = WbkeMakeExecute(projectCtx)
    if compType:
        make.addEnvironment('SGRPRJCOMPILE', compType)

    top = projectCtx.getProjectDB().getTopHierarchy()
    """:type: WmhdbTopHierarchy"""
    for sgrp in top.getSchedulingGroupSet():
        if sgrp != 'none':
            make.doAllRule(sgrp, output)


#: Model compile action
WMBKE_COMPILE_ACTION = _WmbkeCompileAction()
