# ----------------------------------------------------------------------------
# - wmact-generate-code-action                                               -
# - wild cruncher: code generation action             core / model / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmact_generate_code_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Code generation action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_process import ScmnProcStreamHandlerP
from wcr_backend.modules.model.aob.wmaob_builder import WmaobBuilder
from wcr_core.action.wact_action import WactActionCommandMixin
from wcr_core.actions.wact_verify_db_action import \
    wact_verifyProjectDatabase, WactDatabaseVerificationException
from wcr_core.modules.model.actions.wmact_action_base import WmactActionBase
from wcr_core.modules.model.db.wmhdb_type_propagation import \
    WmhdbTypePropagator, WmhdbElementsException
from wplg_cold.wcld_variables_parser import WcldColdVarsParser
from wcr_backend.modules.model.aob.wmaob_callback_gene import \
    WmaobCallbacksGenerator
from wplg_cold.wcld_builder import WcldColdBuilder
from wcr_backend.modules.model.aob.wmaob_group_builder import WmaobGroupBuilder
import os


class _WmbkeGenerateCodeAction(WmactActionBase, WactActionCommandMixin):
    """
    Project editor action to generate code for the entire block or the
    selected item(s).
    """

    def __init__(self):
        super().__init__(
            'Generate Code',
            'Generate code for project or selected instance(s)')

    def _isProtocolException(self, e):
        return isinstance(e, WactDatabaseVerificationException) or \
            isinstance(e, WmhdbElementsException)

    def _doExecute(self, actx, paramValuePairs):
        """
        :type actx: WactActionExecutionContextP
        """
        def run(output):
            try:
                wact_verifyProjectDatabase(pctx)
                wmbke_generateCode(pctx, output)

                output.handleControlMsg(
                    'Code generation completed',
                    ScmnProcStreamHandlerP.CTRL_FINISHED)
            except WactDatabaseVerificationException:
                output.handleControlMsg(
                    'Database verification failed',
                    ScmnProcStreamHandlerP.CTRL_FAILED)
                raise
            except WmhdbElementsException:
                output.handleControlMsg(
                    'Type propagation failed',
                    ScmnProcStreamHandlerP.CTRL_FAILED)
                raise
            except:
                output.handleControlMsg(
                    'Code generation failed',
                    ScmnProcStreamHandlerP.CTRL_FAILED)
                raise

        pctx = actx.getProjectContext()
        return self._runCommand(actx, 'generate', 'Generate', run)


def wmbke_generateCode(pctx, output):
    """
    Generate code for the project.

    :param pctx: Project context
    :type pctx: WcorProjectContextP
    :param output: Stream output handler
    :type output: ScmnProcStreamHandlerP
    """
    # the profile to be used for code generation
    prof = 'default'
    # the tools that parses cold outputs : .vars file
    coldparser = WcldColdVarsParser(None)
    # conversion dictionary
    paramconv = {}
    # the object to initialize the project callback functions
    mdlcallbackinit = WmaobCallbacksGenerator(pctx, prof)
    coldacces = WcldColdBuilder(pctx)
    # the top hierarchy bock
    tblk = pctx.getProjectDB().getTopHierarchy()
    tprp = WmhdbTypePropagator()
    fbld = WmaobBuilder(pctx, prof)
    if mdlcallbackinit.canUseCallback:
        # the group builder
        gb = WmaobGroupBuilder(pctx, fbld.getCodeGen(), prof)
        # change the expression paramerts into simple variables
        gb.processParams(tblk, mdlcallbackinit.getSchedulingName(),
                          dic=paramconv, formatname=True)
        if os.path.exists(os.path.join(pctx.getProjectWorkspace().
                                               getProjectPath(),
                                       'default',
                                       mdlcallbackinit.getSchedulingName(),
                                       'CldGlobals.vars')):
            file = os.path.join(pctx.getProjectWorkspace().getProjectPath(),
                                'default', mdlcallbackinit.getSchedulingName(),
                                'CldGlobals.vars')
            coldparser = WcldColdVarsParser(file)
    tprp.propagateTypes(tblk, coldparser, coldacces)
    gb.cleanFiles()
    gb.restoreParams(tblk, paramconv)
    fbld.generate(tblk, output)


#: Model code generation action
WMBKE_CODE_GENERATION_ACTION = _WmbkeGenerateCodeAction()
