# ----------------------------------------------------------------------------
# - wmaob-builder                                                            -
# - wild cruncher:aob builder                                                -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2014 amaury darsch                                    -
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
..  module::      wmaob_builder
   :platform:     Unix, Windows
   :synopsis:     Factory builder environment

.. moduleauthor:: Amaury Darsch <amaury.darsch@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------

from wcr_backend.modules.model.aob.wmaob_top_builder import WmaobTopBuilder
from wcr_backend.modules.model.aob.wmaob_group_builder import WmaobGroupBuilder
from wcr_core.modules.model.db.wmhdb_top_block import WmhdbTopHierarchy
from wcr_core.modules.model.db.wmhdb_scheduling_group import WmhdbGroupBlock


WMAOB_CODE_GENERATORS = []
""":type: list[WcgnCodeGeneratorFactoryP]"""


# ----------------------------------------------------------------------------
# - class section                                                           -
# ----------------------------------------------------------------------------

class WmaobBuilder:
    """
    The WmaobBuilder is the principal class in the wild cruncher build
    environment. The class is designed to build in the operating namespace
    a set of instances by type. The class operates principally within a
    project and a profile.
    """

    def __init__(self, pctx, prof=None):
        """
        Create a builder by context and profile

        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param prof: Profile name
        :type prof: str | None
        """
        # save the profile
        self.prof = prof or 'default'

        #: Code generators
        self.__codegens = [b.createCodeGenerator(pctx, prof)
                           for b in WMAOB_CODE_GENERATORS]
        """:type: list[WcgnCodeGeneratorP]"""
        # create a scheduling group builder
        self.sgrb = WmaobGroupBuilder(pctx, self.__codegens, prof)
        # create a top hierarchy builder
        self.topb = WmaobTopBuilder(pctx, self.__codegens, prof)

    def generate(self, inst, strm=None):
        """
        Generate the workspace code for an instance. This is the entry point
        in the builder. If the instance is not supported, nothing is
        produced.

        :param inst: the instance to process
        :param strm: Process output/error stream handler
        :type strm: ScmnProcStreamHandlerP
        """
        # check for top hierarchy
        if isinstance(inst, WmhdbTopHierarchy):
            self.topb.generate(inst, strm=strm)
        # check for scheduling group
        elif isinstance(inst, WmhdbGroupBlock):
            self.sgrb.generate(inst, strm=strm)
        else:
            for codegen in self.__codegens:
                if codegen.canGenerate(inst):
                    codegen.generate(inst, strm)

    def getCodeGen(self):
        return self.__codegens