# ----------------------------------------------------------------------------
# - wmaob-data-flow                                                          -
# - wild cruncher: aob data flow evaluation                                  -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 amaury darsch                                    -
# - author    (c) 2012-2012 PiV                                              -
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmaob_data_flow
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher data flow evaluation

.. moduleauthor:: PiV <pierre.villalon@silkan.com>
"""
# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------

from wcr_core.modules.model.db.wmhdb_block import WmhdbBlock
from wcr_core.modules.model.db.wmhdb_rti_type import WmhdbRtiType


# ----------------------------------------------------------------------------
# - class section                                                            -
# ----------------------------------------------------------------------------

class WmaobDataFlow:
    """
    Evaluate the data flow for an instance
    """

    def __init__(self, pctx):
        """
        Create a data flow evaluator by context.

        :type  pctx: WcorProjectContextP
        :param pctx: the application context
        """
        self.__pctx = pctx

    def evaluateBlock(self, blok):
        """
        Evaluate the data flow for this model and update the database
        accordingly.

        :param blok: the model block to be evaluated and updated
        :type  blok: WmhdbBlock
        """
        # check for a valid model block
        if not isinstance(blok, WmhdbBlock): return
        # for the time being let's say that everything is scalar / real
        for w in blok.getConnections():
            wire = w
            """:type: WmhdbWire"""
            # get the inferred signal type
            t = wire.getInferredRtiType()
            # reset it which is call it with default values
            if t is None:
                wire.setInferredRtiType(WmhdbRtiType())
        # evaluate the instances
        for i in blok.getBlockAndModelInstances():
            self.evaluateInstance(i)

    def evaluateInstance(self, inst):
        """
        Evaluate the data flow for this instance and update the database
        accordingly.

        :param inst: the instance to be evaluated and updated
        :type  inst: WmhdbModelInstance
        """
        # noinspection PyTypeChecker
        self.evaluateBlock(inst.getModel())
