# ----------------------------------------------------------------------------
# - wmaob-utils                                                              -
# - wild cruncher:aob wmi utility functions                                  -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 amaury darsch                                    -
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
..  module::   wmaob_utils
:platform:     Unix, Windows
:synopsis:     Factory systsem utility

.. moduleauthor:: Amaury Darsch <amaury.darsch@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------

import os
import string
from wcr_core.modules.model.db.wmhdb_scheduling_group import WmhdbGroupBlock
from wcr_core.modules.model.db.wmhdb_instance import WmhdbInstance
from wcr_core.modules.model.db.wmhdb_instance import WmhdbModelInstance,\
    WmhdbBlockInstance
from wcr_core.modules.model.db.wmhdb_rti_type import wmhdb_getCommonType

# ----------------------------------------------------------------------------
# - Constante section                                                        -
# ----------------------------------------------------------------------------

SPE_CHAR_CONV = {'@': '_ab_',
                 '^': '_ac_',
                 '}': '_af_',
                 'à': '_ag_',
                 '{': '_ao_',
                 '\'': '_ap_',
                 '\\': '_as_',
                 'ç': '_cc_',
                 '<': '_ci_',
                 ')': '_cp_',
                 '>': '_cs_',
                 ':': '_dp_',
                 '&': '_ec_',
                 'é': '_ee_',
                 'è': '_eg_',
                 '=': '_eq_',
                 '"': '_gi_',
                 'µ': '_mm_',
                 '*': '_mu_',
                 '(': '_op_',
                 '%': '_pc_',
                 '!': '_pe_',
                 '|': '_pi_',
                 '+': '_pl_',
                 '.': '_pt_',
                 ';': '_pv_',
                 '_': '_sb_',
                 '/': '_sl_',
                 ' ': '_sp_',
                 '-': '_ti_',
                 '~': '_tl_',
                 'ù': '_ug_',
                 ',': '_vg_',
                 }


MDL_CHAR = ('[', ']', '{', '}', '(', ')', '=', '\'', '.', ',', ';', ':', '%',
            '!', '@', '*', '-', '/', '\\', '&', '^', '+')

# ----------------------------------------------------------------------------
# - public section                                                           -
# ----------------------------------------------------------------------------

def wmaob_get_template():
    """
    Get the default system template
    """
    # check for posix
    if os.name == 'posix': return wmaob_get_unx_template()
    # check for wonderland
    if os.name == 'nt': return wmaob_get_win_template()


def wmaob_get_unx_template():
    """
    Get the default unix wmi template
    """
    return 'wcr_unx_wmimb.mak'


def wmaob_get_win_template():
    """
    Get the default windows wmi template
    """
    return 'wcr_mgw_wmimb.mak'

def wmaob_C_format_name(name):
    """
    format name to be compatible with a C use
    """
    lname = name
    lname = lname.replace('_', SPE_CHAR_CONV['_'])
    for c in SPE_CHAR_CONV.keys():
        if c != '_':
            lname = lname.replace(c, SPE_CHAR_CONV[c])
    return lname

def wmaob_format_model_names(model):
    """
    format model blocs names to be with a C use befor a flat
    """
    for li in model.getBlockAndModelInstances():
        if isinstance(li, WmhdbModelInstance):
            li.setName(wmaob_C_format_name(li.getName()))
        if isinstance(li, WmhdbBlockInstance):
            wmaob_format_model_names(li.getBlock())
            li.setName(wmaob_C_format_name(li.getName()))
    for i in model.getOperatorInstances():
        i.setName(wmaob_C_format_name(i.getName()))
    return 0

def get_expre_param(blk):

    expre = list()
    params = blk.getParameters()
    for para in params:
        if para.getValidType() and para.getValidType().getName() != "STR":
            pararti = para.rtiType
            #tow cases for an expression :
            # we can't deduce by our self a# common type and became NOS type
            # or it is a scalar how is a string and have interpreted character.
            if pararti.getDimension() >= 2:
                if pararti.getName() == "NOS":
                    expre.append(para.getName())
            if pararti.getDimension() == 1:
                if pararti.getName() != "STR":
                    if wmhdb_getCommonType([para.rtiType,
                                            para.getValidType()]) \
                            == para.getValidType().getName():
                        continue
                else:
                    if not str_is_number(para.getValue()):
                        if is_m_expression(para.getValue()):
                            expre.append(para.getName())
    return expre

def str_is_number(pstr):
    sep = 1
    for i in pstr:
        if not i.isdigit():
            if (i == '.' or i == ',') and sep > 0:
                sep = 0
            else:
                return False
    return True

def is_m_expression(pstr):
    for c in MDL_CHAR:
        if pstr.find(c) != -1:
            return True
    return False
