# ----------------------------------------------------------------------------
# - wmfog-builder                                                            -
# - wild cruncher:fog builder                                                -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2013 piv                                              -
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
..  module::      wmfog_builder
   :platform:     Unix, Windows
   :synopsis:     builder environment for operators

.. moduleauthor:: piv <pierre.villalon@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------
from silkan_common.scmn_mime_type import scmn_mime_type, SCMN_MIME_ID_C
from wcr_core.modules.model.db.wmhdb_function import WmhdbFunction
from wcr_backend.modules.model.fog.wmfog_function_code_gen import WmfogFuncCode


# ----------------------------------------------------------------------------
# - class section                                                           -
# ----------------------------------------------------------------------------

class WmfogBuilder:
    """
    The WmfogBuilder is the top level class that to generate the code for any
    operators. The class is designed to operate on the workspace to produce
    output files directly from function description as WmhdbFunction objects.
    """

    def __init__(self, pctx, prof="default"):
        """
        Create a builder by context

        :param pctx: the application context
        :type pctx:  WcorProjectContextP
        :param prof: the profile name
        """
        # get the workspace
        self.wspc = pctx.getProjectWorkspace()
        self.prof = prof
        # create a function code object
        self.fcode = WmfogFuncCode(pctx)

    def getDestinationPath(self, inam, sgrp):
        """
        Get the destination file path

        :param inam: the instance name to process
        :param sgrp: the instance scheduling group
        """
        # get the destination name
        dnam = inam + '.' + scmn_mime_type(SCMN_MIME_ID_C).extension
        # map the workspace path
        dpth = self.wspc.getFilePath(dnam, self.prof, sgrp)
        return dpth

    def generate(self, func):
        """
        Generate the code for a given function.

        :param func: the function which needs code generation
        :type  func: WmhdbFunction
        """
        if not isinstance(func, WmhdbFunction): return
        # get the operator name and check ignore
        if func.getOperator().hasFlag('nofog'): return
        # get the func name and scheduling group
        name = func.getName() + '.' + scmn_mime_type(SCMN_MIME_ID_C).extension
        sgrp = func.getSchedulingGroup()
        # get the function code
        txt = self.fcode.getCode(func)
        # produce the code into the workspace
        self.wspc.importText(name, self.prof, sgrp, txt)
