# ----------------------------------------------------------------------------
# - wmfog-parameter                                                          -
# - wild cruncher: implicit function parameter management                    -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 amaury darsch                                    -
# - author    (c) 2012-2013 piv                                              -
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
..  module::      wmfog_parameter
   :platform:     Unix, Windows
   :synopsis:     implicit function parameter management

.. moduleauthor:: piv <pierre.villalon@silkan.com>
"""

from wcr_core.modules.model.db.wmhdb_rti_type import WmhdbRtiType


# ----------------------------------------------------------------------------
# - fog util functions                                                     -
# ----------------------------------------------------------------------------

def wmfog_get_plusx_implicit_params(func):
    """
    """
    values = []
    cons = func.getInputConnectors()
    for con in cons:
        if con.isInvertedPolarity():
            values.append('-1.0')
        else:
            values.append('1.0')
    pstr = '[{}]'.format(';'.join(values))
    return [WmfogParameter(name='coefficient', value=pstr)]


def wmfog_get_multx_implicit_params(func):
    """
    """
    result = ''
    cons = func.getInputConnectors()
    for con in cons:
        if con.isInvertedPolarity():
            result += '/'
        else:
            result += '*'
    return [WmfogParameter(name='operator', value=result)]


class WmfogParameter(object):
    """
    Parameter representation for FOG code generation.
    """

    # noinspection PyUnusedLocal
    def __init__(self, param=None, name=None, value=None, rtyp=None):
        """
        :param param: Source parameter
        :type param: WmhdbParameter
        :param name: Parameter name
        :type name: str
        :param value: Parameter value
        :type value: object
        :param rtyp: Parameter type
        :type rtyp: WmhdbRtiType
        """
        self.__name = name or (param.getName() if param else None)
        self.__value = value or (param.getValue() if param else None)

    def getName(self):
        """
        Get the parameter's name.

        :return: Name of the parameter
        :rtype: str
        """
        return self.__name

    def getValue(self):
        """
        Get the parameter's value.

        :return: Value of the parameter
        :rtype: str
        """
        return self.__value

    @property
    def rtiType(self):
        """
        Get the parameter's type.

        :return: Type of the parameter
        :rtype: WmhdbRtiType
        """
        return WmhdbRtiType.getRtiTypeForStrValue(self.getValue())
