# ----------------------------------------------------------------------------
# - wmbke-plugin-install                                                     -
# - wild cruncher: plugin install context                    backend / model -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmbke_plugin_install
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Plugin install context and manager

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_backend.plugin.wbke_backend_install_context import \
    WbkeBackendModuleInstallContextBase
from wcr_backend.plugin.wbke_backend_install_context_factory import \
    WbkeBackendModuleInstallContextFactoryBase
from wcr_core.modules.model.wmmod_application_module import \
    WMMOD_MODEL_APP_CODE


class WmbkeModelBackendModuleInstallContext(
        WbkeBackendModuleInstallContextBase):
    """
    Backend module install context.
    """

    def __init__(self, plgCode, plgName, plgDesc, cgnfs):
        """
        :param plgCode: Plugin code
        :type plgCode: str
        :param plgName: Plugin name
        :type plgName: str
        :param plgDesc: Plugin description
        :type plgDesc: str
        :param cgnfs: Code-generator factory list
        :type cgnfs: list[WcgnCodeGeneratorFactoryP]
        """
        super().__init__(plgCode, plgName, plgDesc)

        self.__codeGenFactoryList = cgnfs

        self.__codegenFys = []
        """:type: list[WcgnCodeGeneratorFactoryP]"""

    def commit(self):
        self.__codeGenFactoryList.extend(self.__codegenFys)

    def installCodeGeneratorFactory(self, factory):
        self.__codegenFys.append(factory)


class WmbkeModelBackendModuleInstallContextFactory(
        WbkeBackendModuleInstallContextFactoryBase):
    """
    Backend module install context manager.
    """

    def __init__(self, cgnfs):
        """
        :param cgnfs: Code-generator factory list
        :type cgnfs: list[WcgnCodeGeneratorFactoryP]
        """
        self.__codeGenFactoryList = cgnfs

    def getModuleCode(self):
        return WMMOD_MODEL_APP_CODE

    def createInstallContext(self, plgCode, plgName, plgDesc, *args, **kwargs):
        return WmbkeModelBackendModuleInstallContext(
            plgCode, plgName, plgDesc, self.__codeGenFactoryList)
