# ----------------------------------------------------------------------------
# - wsbke-generate-drs-action                                                -
# - wild cruncher: sim. generate drs action   backend / simulation / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wsbke_generate_drs_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Simulation DRS generation action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_backend.modules.simulation.aob.wsaob_builder import WsaobBuilder
from wcr_core.modules.simulation.actions.wsact_action_base import \
    WsactActionBase
from wcr_core.modules.simulation.db.wshdb_block import wshdb_pruneBindings, \
    WshdbTopBlock
from wcr_core.modules.simulation.db.wshdb_block_flattener import wshdb_flatten
from wcr_core.modules.simulation.wsmod_project_context import \
    WsmodSimulationProjectContextP


class _WsbkeGenerateDrsAction(WsactActionBase):
    """
    Project editor action to generate DRS.
    """

    def __init__(self):
        super().__init__('Build', 'Build simulator')

    def _doExecute(self, actx, paramValuePairs):
        """
        :type actx: WactActionExecutionContextP
        """
        pctx = actx.getProjectContext()
        assert(isinstance(pctx, WsmodSimulationProjectContextP))
        topBlk = pctx.getProjectDB().getTopHierarchy()
        assert(isinstance(topBlk, WshdbTopBlock))

        wshdb_pruneBindings(topBlk)
        wsbke_generateDrs(pctx)


def wsbke_generateDrs(projectCtx):
    """
    Build project in the workspace.

    :param projectCtx: Project context
    :type projectCtx: WsmodSimulationProjectContextP
    """
    fbld = WsaobBuilder(projectCtx)
    block = projectCtx.getProjectDB().getTopHierarchy()
    # noinspection PyTypeChecker
    flat = wshdb_flatten(block)

    # fpdb = BasPdbProjectDatabase()
    # fpdb.setModule(prjCtx.getProjectDB().getModule())
    # fpdb.tops.setTopHierarchy(flat)
    # with open('/tmp/out.xml', 'w') as f:
    #     BasPdbXml().serialize(fpdb, f)
    # prjCtx.getGlobalContext().handleFile('/tmp/out.xml')
    # print('===================================\n'
    #       'IN:\n'
    #       '-----------------------------------')
    # bas_hds_printBlock(block)
    # print('===================================\n'
    #       'FLAT:\n'
    #       '-----------------------------------')
    # bas_hds_printBlock(flat)
    fbld.generate(flat)


WSBKE_GENERATE_DRS_ACTION = _WsbkeGenerateDrsAction()
