# ----------------------------------------------------------------------------
# - wsaob-builder                                                            -
# - wild cruncher:sfs builder                                                -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2013 amaury darsch                                    -
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
..  module::      wsaob_builder
   :platform:     Unix, Windows
   :synopsis:     Factory builder environment

.. moduleauthor:: Amaury Darsch <amaury.darsch@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------
from wcr_core.modules.simulation.db.wshdb_block import WshdbBlock
from wcr_backend.modules.simulation.aob.wsaob_drs_builder import \
    WsaobDrsBuilder


# ----------------------------------------------------------------------------
# - class section                                                           -
# ----------------------------------------------------------------------------

class WsaobBuilder:
    """
    The WsaobBuilder is the principal class in the wild cruncher simulator
    biuld environment. The class is designed to build in the operating
    namespace all the files needed to configure a proper simulator. The class
    operates principally within a project and a profile.
    """

    def __init__(self, pctx, prof="default"):
        """
        Create a builder by context and profile

        :param pctx: the context
        :type pctx: WsmodSimulationProjectContextP
        :param prof: the profile name
        """
        # save the profile
        self.__prof = prof
        # create a drs builder
        self.__drsb = WsaobDrsBuilder(pctx, prof)

    # noinspection PyUnusedLocal
    def generate(self, block, strm=None):
        """
        Generate the workspace code for a block. This is the entry point
        in the builder. If the instance is not supported, nothing is
        produced.

        :param block: the block to process
        :type block: WshdbBlock
        :param strm: Process output/error stream handler
        :type  strm: ScmnProcStreamHandlerP
        """
        # check for top hierarchy
        if isinstance(block, WshdbBlock):
            self.__drsb.generate(block, strm)
