# ----------------------------------------------------------------------------
# - wpbke-generate-config-action                                             -
# - wild cruncher: sys. generate config action    backend / system / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 samuel longchamps                                -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpbke_generate_config_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - System configuration file generation action

.. moduleauthor:: Samuel Longchamps <samuel.longchamps@silkan.com>
"""
from wcr_backend.modules.system.aob.wpaob_tutor_config_generator import \
    WpaobTutorConfigGenerator
from wcr_core.action.wact_action import WACT_ACTION_EXIT_OK
from wcr_core.action.wact_action_api import WACT_ACTION_EXECUTION_FAILED_EVENT
from wcr_core.action.wact_action_execution_context import \
    WactActionExecutionContextP
from wcr_core.actions.wact_verify_db_action import wact_verifyProjectDatabase, \
    WactDatabaseVerificationException, WACT_VERIFY_DATABASE_ACTION
from wcr_core.modules.system.actions.wpact_action_base import WpactActionBase
from wcr_core.modules.system.wpmod_project_context_api import \
    WpmodSystemProjectContextP


class _WpbkeGenerateConfigAction(WpactActionBase):
    """
    Project editor action to generate system configuration file.
    """

    def __init__(self):
        super().__init__('Generate',
                         'Generate configuration file for use in Tutor')

    def _doExecute(self, actx, paramValuePairs):
        """
        :param actx: Action execution context
        :type actx: WactActionExecutionContextP
        """
        pctx = actx.getProjectContext()
        assert(isinstance(pctx, WpmodSystemProjectContextP))
        try:
            actx.statusMessage('Verifying database...')
            wact_verifyProjectDatabase(pctx)
            actx.statusMessage('Database verification successful, generation '
                               'in process...')
            wpbke_generateConfigFile(pctx, verifyDB=False)
            actx.statusMessage('Generation completed without any error')
            return WACT_ACTION_EXIT_OK
        except Exception as e:
            if isinstance(e, WactDatabaseVerificationException):
                actx.statusMessage(WpaobTutorConfigGenerator.ERROR_MESSAGE +
                                   "Database verification failed ({} errors "
                                   "reported)".format(
                                   len(e.getResult().allErrors)))
                # Show database verifier's detail error window by notifying
                # of a failed verification through its action
                WACT_VERIFY_DATABASE_ACTION._notifyObservers(
                    actx, WACT_ACTION_EXECUTION_FAILED_EVENT, e)
            else:
                actx.statusMessage(str(e))


def wpbke_generateConfigFile(prjCtx, prof=None,
                             block=None, strm=None, header=True,
                             verifyDB=True):
    """
    Generate system configuration of project in the workspace.

    :param prjCtx: Project context
    :type prjCtx: WpmodSystemProjectContextP
    :param prof: Profile name
    :type prof: str | None
    :param block: the block to generate from, None uses top block by default
    :type block: WphdbBlockP | None
    :param strm: Process output/error stream handler
    :type  strm: ScmnProcStreamHandlerP | None
    :param header: True to write the XML header, False to skip it.
    :type header: bool
    :param verifyDB: True calls database verification first, if it fails the
        generation is aborted, False skips it.
    :type verifyDB: bool
    """
    if verifyDB:
        wact_verifyProjectDatabase(prjCtx)
    gen = WpaobTutorConfigGenerator(prjCtx, prof)
    assert gen
    gen.generate(block, strm, header)


WPBKE_GENERATE_CONFIG_ACTION = _WpbkeGenerateConfigAction()
