# ----------------------------------------------------------------------------
# - wbke-backend-install-context                                             -
# - wild cruncher: backend install context                  backend / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbke_backend_install_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Backend install context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from abc import abstractmethod

from wcr_core.plugin.wplg_install_context import \
    WplgPlatformInstallContextBase, WplgModuleInstallContextBase
from wcr_core.plugin.wplg_install_context_api import WplgInstallContextP, \
    WplgModuleInstallContextP, WplgPlatformInstallContextP


WBKE_BACKEND_INSTALL_CODE = 'backend'


# noinspection PyAbstractClass
class WbkeBackendInstallContextMixin(WplgInstallContextP):
    """
    Backend install context mixin.
    """

    def getComponentCode(self):
        return WBKE_BACKEND_INSTALL_CODE


# noinspection PyAbstractClass
class WbkeBackendPlatformInstallContextP(WplgPlatformInstallContextP):
    """
    Backend platform install context protocol.
    """
    pass


# noinspection PyAbstractClass
class WbkeBackendPlatformInstallContextBase(
        WplgPlatformInstallContextBase, WbkeBackendInstallContextMixin,
        WbkeBackendPlatformInstallContextP):
    def __init__(self, plgCode, plgName, plgDesc):
        """
        :param plgCode: Plugin code
        :type plgCode: str
        :param plgName: Plugin name
        :type plgName: str
        :param plgDesc: Plugin description
        :type plgDesc: str
        """
        super().__init__(plgCode, plgName, plgDesc)


# noinspection PyAbstractClass
class WbkeBackendModuleInstallContextP(WplgModuleInstallContextP):
    """
    Backend module install context protocol.
    """

    @abstractmethod
    def installCodeGeneratorFactory(self, factory):
        """
        Install a code gneration factory.

        :param factory: Code generator factory to install.
        :type factory: WcgnCodeGeneratorFactoryP
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WbkeBackendModuleInstallContextBase(WplgModuleInstallContextBase,
                                          WbkeBackendInstallContextMixin,
                                          WbkeBackendModuleInstallContextP):
    def __init__(self, plgCode, plgName, plgDesc):
        """
        :param plgCode: Plugin code
        :type plgCode: str
        :param plgName: Plugin name
        :type plgName: str
        :param plgDesc: Plugin description
        :type plgDesc: str
        """
        super().__init__(plgCode, plgName, plgDesc)
