# ----------------------------------------------------------------------------
# - wact-action-execution-context                                            -
# - wild cruncher: action execution context                    core / action -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wact_action_execution_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Action execution context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod

from silkan_common.scmn_log import ScmnMessageTargetP
from wcr_core.action.wact_parameter_value_source_api import \
    WactParameterValueSourceP


# noinspection PyAbstractClass
class WactActionExecutionContextP(WactParameterValueSourceP,
                                  ScmnMessageTargetP):
    @abstractmethod
    def getProjectContext(self):
        """
        Project context.

        :return: Project context
        :rtype: WcorProjectContextP
        """
        raise NotImplementedError()

    @abstractmethod
    def statusMessage(self, msg):
        """
        Add a status message.

        :param msg: Message to display
        :type msg: str
        """
        raise NotImplementedError()

    @abstractmethod
    def executeCommand(self, cmndCode, cmndTitle, func, successF=None,
                       errorF=None):
        """
        Execute an external command with an text output widget.

        :param func: Function to execute
        :type func: (ScmnProcStreamHandlerP) -> _
        :param cmndTitle: Title for the output widget
        :type cmndTitle: str
        :param cmndCode: Code for the output widget, for re-use
        :type cmndCode: str
        :param successF: Function to execute after successful execution
        :type successF: () -> _
        :param errorF: Function to execute after failed execution
        :type errorF: (Exception, str) -> _
        """
        raise NotImplementedError()
