# ----------------------------------------------------------------------------
# - wact-action-parameter                                                    -
# - wild cruncher: action parameters                           core / action -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wact_action_parameter
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Action parameters

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.action.wact_action_parameter_api import WactActionParameterP, \
    WactFixedSelectionActionParameterP


# noinspection PyAbstractClass
class _WactActionParameterBase(WactActionParameterP):
    """
    Generic action parameter implementation.
    """

    def __init__(self, code, name, description, paramType, optional=False):
        """
        :param code: Parameter code
        :type code: str
        :param name: Parameter name
        :type name: str
        :param description: Parameter description
        :type description: str
        :param paramType: Parameter type
        :type paramType: WactStringableActionParameterTypeP
        :param optional: Flag indicating that the parameter is optional
        :type optional: bool
        """
        #: Parameter code
        self.__code = code
        #: Parameter name
        self.__name = name
        #: Parameter description
        self.__description = description
        #: Parameter type
        self.__paramType = paramType
        #: Whether the parameter is optional
        self.__optional = optional

    def getCode(self):
        return self.__code

    def getName(self):
        return self.__name

    def getDescription(self):
        return self.__description

    def getParamType(self):
        return self.__paramType

    def isOptional(self):
        return self.__optional


class WactActionParameter(_WactActionParameterBase):
    """
    Generic action parameter implementation.
    """

    def __init__(self, code, name, description, paramType, optional=False):
        """
        :param code: Parameter code
        :type code: str
        :param name: Parameter name
        :type name: str
        :param description: Parameter description
        :type description: str
        :param paramType: Parameter type
        :type paramType: WactStringableActionParameterTypeP
        :param optional: Flag indicating that the parameter is optional
        :type optional: bool
        """
        super().__init__(code, name, description, paramType, optional)


class WactFixedMultiValueActionParameter(_WactActionParameterBase,
                                         WactFixedSelectionActionParameterP):
    """
    Generic action parameter implementation.
    """

    def __init__(self, code, name, description, paramType,
                 possibleValues, choiceMessage=None, optional=False):
        """
        :param code: Parameter code
        :type code: str
        :param name: Parameter name
        :type name: str
        :param description: Parameter description
        :type description: str
        :param paramType: Parameter type
        :type paramType: WactStringableActionParameterTypeP
        :param possibleValues: Possible values for the parameter
        :type possibleValues: list[(object, str, str)]
        :param choiceMessage: Question to ask the user to be able to choose
        :type choiceMessage: str | None
        :param optional: Flag indicating that the parameter is optional
        :type optional: bool
        """
        super().__init__(code, name, description, paramType, optional)

        self.__choiceMessage = choiceMessage
        self.__possibleValues = possibleValues

    def getChoiceMessage(self):
        """
        :return: Message allowing the user to make a decision
        :rtype: str | None
        """
        return self.__choiceMessage

    def getPossibleValues(self):
        return iter(self.__possibleValues)
