# ----------------------------------------------------------------------------
# - wact-action-parameter-api                                                -
# - wild cruncher: action parameter api                         core /action -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wact_action_parameter_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Action parameter API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod, ABCMeta


class WactActionParameterP(metaclass=ABCMeta):
    """
    Protocol for action parameters.
    """

    @abstractmethod
    def getParamType(self):
        """
        Get the type of the action parameter.

        :return: Type of the action parameter
        :rtype: WactActionParameterTypeP
        """
        raise NotImplementedError()

    @abstractmethod
    def getCode(self):
        """
        Get the code for the action parameter.
        The parameter code is used to identify parameters unambiguously.

        :return: Parameter code
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getName(self):
        """
        Get the name of the action parameter.

        :return: Name of the action parameter
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getDescription(self):
        """
        Get the description for the action parameter.

        :return: Description for the action parameter
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def isOptional(self):
        """
        Whether the parameter is optional.

        Execution calls with non-optional parameters missing will result in
        exceptions being thrown.

        :return: True iff the parameter is optional.
        :rtype: bool
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WactFixedSelectionActionParameterP(WactActionParameterP):
    @abstractmethod
    def getPossibleValues(self):
        """
        Get the possible values for the parameter.

        :return: Possible values for the parameter
        :rtype: Iterable[(object, str, str)]
        """
        raise NotImplementedError()
