# ----------------------------------------------------------------------------
# - wcmn_graph_layouter_grid                                                 -
# - wild cruncher: layouter                                    core / common -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 samuel longchamps                                -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcmn_graph_layouter_grid
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher graph layouter

.. moduleauthor:: Samuel Longchamps <samuel.longchamps@silkan.com>
"""
import math
from wcr_core.common.wcmn_graph_layouter import WcmnGraphLayouterP, \
    WCMN_GRAPH_LAYOUTER_DEFAULT_SPACING


class WcmnGraphLayouterGrid(WcmnGraphLayouterP):
    """
    Grid Layouter class
    """

    def __init__(self, spacing=None):
        """
        :param spacing: Positive spacing value
        :type spacing: float | None
        """
        if spacing is not None and spacing < 0:
            raise ValueError('The spacing must be positive ({spc} < 0)'.format(
                spc=spacing))

        self.__spacing = spacing or WCMN_GRAPH_LAYOUTER_DEFAULT_SPACING

    def getSpacing(self):
        """
        return: Spacing value
        rtype: float
        """
        return self.__spacing

    def applyLayout(self, graph):
        """
        Organize the passed graph object as a grid

        :param graph: Object representing a set of nodes and edges between them
        :type graph: _WcmnLayouterGraph
        """

        if not graph:
            raise ValueError('Cannot apply grid layout: graph is None')

        def mkps(icnt):
            offs = (icnt - 1) * self.__spacing / 2
            return [i * self.__spacing - offs for i in range(icnt)]

        stts = graph.getNodes()
        scnt = len(stts)

        if scnt == 1:
            stts[0].setPosition(0, 0)
            return
        elif scnt == 0:
            raise ValueError('Cannot apply layout: No nodes in graph')

        ccnt = math.ceil(math.sqrt(scnt))
        rcnt = math.ceil(scnt / ccnt)
        xlst = mkps(ccnt)
        ylst = mkps(rcnt)

        rows = [stts[i:i + ccnt] for i in range(0, scnt, ccnt)]

        for ypos, row in zip(ylst, rows):
            _xls = xlst if len(row) == len(xlst) else mkps(len(row))
            for xpos, stat in zip(_xls, row):
                stat.setPosition(xpos, ypos)
