# ----------------------------------------------------------------------------
# - wctl-controller-value-provider-api                                       -
# - wild cruncher: controller value provider api           core / controller -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wctl_controller_value_provider_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Controller value provider API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WctlControllerValueProviderBaseP(metaclass=ABCMeta):
    """
    Base protocol for controller value providers.
    """
    pass


class WctlConnectionControllerValueProviderP(WctlControllerValueProviderBaseP):
    """
    Protocol for connection controller value providers.
    """
    pass


class WctlInstanceControllerValueProviderP(WctlControllerValueProviderBaseP):
    """
    Protocol for instance controller value providers.
    """

    @abstractmethod
    def provideNewConnectionPointDirection(self, instance, **kwargs):
        """
        Provide the direction for a connection point to be created.

        :param instance: Instance the connection point is to be created for
        :type instance: WhdbInstanceP
        :param kwargs: Additional keyword arguments for the connection point
                       creation
        :type kwargs: dict[str, object]
        :return: Direction for the new connection point
        :rtype: int | None
        """
        raise NotImplementedError()

    @abstractmethod
    def provideNewConnectionPointType(self, instance, **kwargs):
        """
        Provide the type for a connection point to be created.

        :param instance: Instance the connection point is to be created for
        :type instance: WhdbInstanceP
        :param kwargs: Additional keyword arguments for the connection point
                       creation
        :type kwargs: dict[str, object]
        :return: Type for the new connection point
        :rtype: str | None
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WctlControllerValueProviderP(
        WctlConnectionControllerValueProviderP,
        WctlInstanceControllerValueProviderP):
    """
    Protocol for instance controller value providers.
    """
    pass
