# ----------------------------------------------------------------------------
# - wctl-default-controllers                                                 -
# - wild cruncher: default controllers                    core / controllers -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wctl_default_controllers
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Default controllers

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from wcr_core.controller.wctl_controller_mixins import *
from wcr_core.db_api.whdb_connection_api import WhdbConnectionP
from wcr_core.db_api.whdb_instance_api import WhdbInstanceP
from wcr_core.db_api.whdb_shape_api import WhdbShapeP


class WctlDefaultItemBase(WctlControllerP):
    def __init__(self, fcty):
        """
        :param fcty: Controller factory
        :type fcty: WctlControllerFactoryP
        """
        self.__factory = fcty

    @property
    def factory(self):
        """
        :rtyp: WctlControllerFactoryP
        """
        return self.__factory


# noinspection PyAbstractClass
class WctlDefaultItemControllerBase(WctlDefaultItemBase,
                                    WctlItemControllerP):
    def __init__(self, fcty):
        """
        :param fcty: Controller factory
        :type fcty: WctlControllerFactoryP
        """
        super().__init__(fcty)


class WctlDefaultInstanceController(WctlDefaultItemControllerBase,
                                    WctlInstanceControllerDefaultXPointMixin,
                                    WctlInstanceControllerDefaultDeleteMixin,
                                    WctlInstanceControllerP):
    def __init__(self, fcty):
        """
        :param fcty: Controller factory
        :type fcty: WctlControllerFactoryP
        """
        super().__init__(fcty)


class WctlDefaultConnectionController(
        WctlDefaultItemControllerBase,
        WctlConnectionControllerDefaultCreateMixin,
        WctlConnectionControllerDefaultDeleteMixin,
        WctlConnectionControllerP):
    def __init__(self, fcty):
        """
        :param fcty: Controller factory
        :type fcty: WctlControllerFactoryP
        """
        super().__init__(fcty)


class WctlDefaultShapeController(
        WctlDefaultItemControllerBase,
        WctlShapeControllerDefaultCreateMixin,
        WctlShapeControllerDefaultDeleteMixin,
        WctlShapeControllerP):
    def __init__(self, fcty):
        """
        :param fcty: Controller factory
        :type fcty: WctlControllerFactoryP
        """
        super().__init__(fcty)


class WctlSimpleControllerFactoryBase(WctlControllerFactoryP):
    def __init__(self, instCtlr=None, connCtlr=None, shapeCtlr=None):
        self.__instCtlr = instCtlr or WctlDefaultInstanceController(self)
        self.__connCtlr = connCtlr or WctlDefaultConnectionController(self)
        self.__shapeCtlr = shapeCtlr or WctlDefaultShapeController(self)

    def getItemController(self, item):
        if isinstance(item, WhdbInstanceP):
            return self.__instCtlr
        elif isinstance(item, WhdbConnectionP):
            return self.__connCtlr
        elif isinstance(item, WhdbShapeP):
            return self.__shapeCtlr
        else:
            raise ValueError(
                'No controller available for {} ({})'.format(item, type(item)))

    def getInstanceController(self):
        return self.__instCtlr

    def getConnectionController(self):
        return self.__connCtlr

    def getShapeController(self):
        return self.__shapeCtlr


class WctlDefaultControllerFactory(WctlSimpleControllerFactoryBase):
    def __init__(self):
        super().__init__()


#: Generic controller factory
WCTL_DEFAULT_CONTROLLER_FACTORY = WctlDefaultControllerFactory()
