# ----------------------------------------------------------------------------
# - whdb-button                                                              -
# - wild cruncher: interactive elements                            core / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_button
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher general hierarchy interactive elements

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_geometry import ScmnRectangle
from wcr_core.db_api.whdb_button_api import WhdbButtonP
from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import WDBS_POINT_META, WDBS_FLOAT_META, \
    WDBS_META_RECTANGLE
from .whdb_protocols import WhdbBase, WHDB_BBOX_KEY, WHDB_Z_VALUE_KEY, \
    WhdbHasLocation, WhdbHasZValue, WHDB_POSITION_KEY


class WhdbButton(WhdbBase, WhdbHasLocation, WhdbHasZValue, WhdbButtonP):
    """
    Button class.
    """

    META = WdbsMetaDescription(
        WdbsMetaField(WHDB_POSITION_KEY, WDBS_POINT_META),
        WdbsMetaField(WHDB_Z_VALUE_KEY, WDBS_FLOAT_META),
        WdbsMetaField(WHDB_BBOX_KEY, WDBS_META_RECTANGLE,
                      temporary=True))

    def __init__(self, button, wreg):
        """
        :param button: Raw button
        :type button: WdbsButtonP
        :param wreg: Raw element wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        WhdbBase.__init__(self, button, wreg)

        self.__rawButton = button

    def getTypeName(self):
        return 'button'

    @property
    def title(self):
        return self.__rawButton.getTitle()

    @title.setter
    def title(self, title):
        self.__rawButton.setTitle(title)

    @property
    def command(self):
        return self.__rawButton.getCommand()

    @command.setter
    def command(self, command):
        self.__rawButton.setCommand(command)

    @property
    def parameters(self):
        return {p.getName(): p.getValue() for p
                in self.__rawButton.getParameters()}

    @parameters.setter
    def parameters(self, parameters):
        for n, v in parameters.items():
            self.__rawButton.getOrCreateParameter(n).setValue(v)

    @property
    def boundingBox(self):
        return self.__rawButton.getMeta(WHDB_BBOX_KEY)

    # noinspection PyShadowingNames
    def setBoundingBox(self, x, y, width, height):
        self.__rawButton.setMeta(ScmnRectangle(x, y, width, height),
                                 WHDB_BBOX_KEY)
