# ----------------------------------------------------------------------------
# - whdb-connection-path-builder                                             -
# - wild cruncher: path builder interface                          core / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_connection_path_builder
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher path builder interface

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from abc import ABCMeta, abstractmethod

#: Default minimum path segment length
WHDB_DEFAULT_MIN_LINE_LENGTH = 14


class WhdbConnectionPathBuilder(metaclass=ABCMeta):
    """
    Path builder interface.

    A path builder generates a path between two connection points.
    """

    @abstractmethod
    def createPath(self, sobj, scon, eobj, econ):
        """
        Calculate the points along the Manhattan path between two connection
        points.

        :param sobj: Instance associated with the first connection point.
        :type sobj: WhdbInstanceP
        :param scon: First connection point
        :type scon: WhdbConnectionPointP
        :param eobj: Instance associated with the second connection point.
        :type eobj: WhdbInstanceP
        :param econ: Second connection point
        :type econ: WhdbConnectionPointP
        :return: List of points along the path
        :rtype: list[(float, float)]
        """
        raise NotImplementedError()

    @abstractmethod
    def simplifyPath(self, path, remf, pntf=None):
        """
        Simplify a given path, if applicable/possible.

        :param path: Path to simplify
        :type path: Iterable[(float, float)]
        :param remf: Function to remove a point
        :type remf: (object) -> None
        :param pntf: Function to retrieve coordinates of a point
        :type pntf: (object) -> (float, float)
        :return: Simplified path
        :rtype: Iterable[(float, float)]
        """
        raise NotImplementedError()
