# ----------------------------------------------------------------------------
# - whdb-connection-utils                                                    -
# - wild cruncher: connection utility functions                    core / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_connection_utils
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher connection utility functions

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db_api.whdb_connection_point_api import WhdbConnectionPointP
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_OUT, \
    WDBS_XPNT_IN

X = 0
Y = 1
N = (0, -1)
E = (1, 0)
S = (0, 1)
W = (-1, 0)


def whdb_xOrY(drct):
    """
    Whether the given direction is affects the X or Y coordinate.

    :param drct: dir
    :type drct: (int, int)
    :return: X or Y
    :rtype: int
    """
    if drct[X] != 0: return X
    if drct[Y] != 0: return Y


def whdb_dirFromGravity(gvty):
    """
    Convert a connection point gravity to a direction.

    :param gvty: Connection point gravity
    :return: Direction
    :rtype: (int, int)
    """
    if gvty == WhdbConnectionPointP.North: return N
    if gvty == WhdbConnectionPointP.East: return E
    if gvty == WhdbConnectionPointP.South: return S
    if gvty == WhdbConnectionPointP.West: return W


def whdb_flipConnType(ctyp):
    """
    Get the opposite connection point type.

    :param ctyp: Connection point type to return the opposite for
    :type ctyp: int
    :return: Opposite connection point type of the given one
    :rtype: int
    """
    if ctyp == WDBS_XPNT_IN:
        return WDBS_XPNT_OUT
    elif ctyp == WDBS_XPNT_OUT:
        return WDBS_XPNT_IN
    else:
        return ctyp


def whdb_getConnectionDescription(conn):
    """
    Get the textual description for the given connection.

    :param conn: Connection to get name for
    :type conn: WhdbConnectionP
    :return: Description for the given connection
    :rtype: str
    """
    name = conn.getName()
    so = conn.getStartObject()
    soName = (so.getName() or so.getTypeName()) if so is not None else 'None'
    sc = conn.getStartConnectionPoint()
    scName = sc.getDescription() if sc is not None else 'None'
    do = conn.getEndObject()
    doName = (do.getName() or do.getTypeName()) if do is not None else 'None'
    dc = conn.getEndConnectionPoint()
    dcName = dc.getDescription() if dc is not None else 'None'

    n = '{so}.{sc} - {eo}.{ec}'.format(
        so=soName, sc=scName, eo=doName, ec=dcName)
    if name:
        n = '{0} ({1})'.format(name, n)

    return n
