# ----------------------------------------------------------------------------
# - whdb-model                                                               -
# - wild cruncher: models                                          core / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_model
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher general hierarchy models

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_mime_type import *
from wcr_core.common.wcmn_mime_type import *
from wcr_core.db_api.whdb_model_api import WhdbModelP
from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import *

from .whdb_protocols import WhdbHasMedia, WhdbBase, WhdbNamedEntity
from .whdb_utils import whdb_getPortNamesForModel

WHDB_MODEL_PORTS_KEY = 'ports'

WHDB_MODEL_PAYLOAD_TYPES = \
    WCMN_PAYLOAD_MODEL | \
    WCMN_PAYLOAD_ENTRY_POINTS | \
    WCMN_PAYLOAD_MODEL_PORTS

WHDB_PORTNAMES_META = WdbsDictionaryMetaType(
    WDBS_INTEGER_META,
    WdbsListMetaType(
        WdbsRecordMetaType(('name', WDBS_STRING_META),
                           ('label', WDBS_STRING_META))))


class WhdbModel(WhdbBase, WhdbNamedEntity, WhdbHasMedia, WhdbModelP):
    """
    External models.
    """
    META = WdbsMetaDescription(
        WhdbHasMedia.META,
        WdbsMetaField(WHDB_MODEL_PORTS_KEY, WHDB_PORTNAMES_META))

    def __init__(self, model, wreg):
        """
        :param model: Raw block
        :type model: WdbsModelP
        :param wreg: Raw element wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        WhdbBase.__init__(self, model, wreg)
        WhdbNamedEntity.__init__(self, model)

        self.__rawModel = model

    def getTypeName(self):
        return 'model'

    def setPath(self, path):
        self.__rawModel.setPath(path)

    def getPath(self):
        return self.__rawModel.getPath()

    def setEntryPoint(self, epnt):
        self.__rawModel.setEntryPoint(epnt)

    def getEntryPoint(self):
        return self.__rawModel.getEntryPoint()

    def getPortNames(self):
        names = self.__rawModel.getMeta(WHDB_MODEL_PORTS_KEY)
        if names:
            # noinspection PyUnresolvedReferences
            return {cdir: [(p['name'], p['label']) for p in prts]
                    for cdir, prts in names.items()}
        else:
            return {}

    def resetPortNames(self):
        names = whdb_getPortNamesForModel(self)
        if names is not None:
            self.__rawModel.setMeta(names, WHDB_MODEL_PORTS_KEY)


def whdb_getMimeForModelFile(path):
    """
    Get the MIME-type for the file at the given path.

    :param path: Path
    :type path: str
    :return: MIME-type
    :rtype: str
    """
    mtps = scmn_mime_types(path)
    for mime in mtps:
        if mime.payloadTypes & WHDB_MODEL_PAYLOAD_TYPES:
            return mime.mimeType
    return SCMN_MIME_ID_UNKNOWN
