# ----------------------------------------------------------------------------
# - whdb-top-level-factory                                                   -
# - wild cruncher: top level factory                               core / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_top_level_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher top level factory

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
import os

from wcr_core.dbstore.immutable.wdbi_context import WdbiContext
from wcr_core.dbstore.mutable.wdbm_context import WdbmContext

from .whdb_block import WHDB_TOPLEVEL_BLOCK_CODE


#: Whether to use the immutable DB
_WHDB_USE_IMMUTABLE_DB = False


def whdb_setImmutableDb(immdb=True):
    """
    Activate use of immutable DB.
    """
    global _WHDB_USE_IMMUTABLE_DB
    _WHDB_USE_IMMUTABLE_DB = immdb


def whdb_isImmutableDb():
    """
    Whether to use the immutable DB

    :return: True iff the immutable DB is to be used
    :rtype: bool
    """
    return _WHDB_USE_IMMUTABLE_DB or 'WCR_IMMDB' in os.environ


def whdb_createTopLevelRawBlock(metaPrv, name=None, info=None, uuid=None):
    """
    Create a top-level block, depending on the immutable DB setting.

    :param metaPrv: Database context
    :type metaPrv: WdbsMetaProviderP
    :param name: Name of the new block
    :type name: str
    :param info: Info string for the new block
    :type info: str
    :param uuid: UUID for the new block (will be created if `None`)
    :type uuid: UUID | str | None
    :return: Top-level raw block
    :rtype: WdbsBlockP
    """
    if whdb_isImmutableDb():
        from wcr_core.dbstore.immutable.wdbi_block import \
            WdbiBlockForNode, wdbi_createBlock

        return WdbiBlockForNode(
            WdbiContext(metaPrv),
            wdbi_createBlock(
                metaPrv, WHDB_TOPLEVEL_BLOCK_CODE, name, info, uuid))
    else:
        from wcr_core.dbstore.mutable.wdbm_block import WdbmBlock

        return WdbmBlock(
            WdbmContext(metaPrv), WHDB_TOPLEVEL_BLOCK_CODE, name, info, uuid)
