# ----------------------------------------------------------------------------
# - whdb-button-api                                                          -
# - wild cruncher: button api                                  core / db-api -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_button_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Button API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod
from wcr_core.db_api.whdb_protocols_api import WhdbHasZValueP, \
    WhdbHasLocationP, WhdbBaseP


# noinspection PyAbstractClass
class WhdbButtonP(WhdbHasLocationP, WhdbHasZValueP, WhdbBaseP):
    """
    Protocol for buttons.
    """

    @property
    @abstractmethod
    def title(self):
        """
        Get the button title.

        :return: Title of the button
        :rtype: str
        """
        raise NotImplementedError()

    @title.setter
    @abstractmethod
    def title(self, title):
        """
        Set the button title.

        :param title: New button title
        :type title: str
        """
        raise NotImplementedError()

    @property
    @abstractmethod
    def command(self):
        """
        Get the button command.

        :return: Command associated with the button
        :rtype: str
        """
        raise NotImplementedError()

    @command.setter
    @abstractmethod
    def command(self, command):
        """
        Set the button command.

        :param command: New command for the button
        :type command: str
        """
        raise NotImplementedError()

    @property
    @abstractmethod
    def parameters(self):
        """
        Get the command parameter values.

        :return: Parameter values
        :rtype: dict[str, str]
        """
        raise NotImplementedError()

    @parameters.setter
    @abstractmethod
    def parameters(self, parameters):
        """
        Set the command parameter values.

        :param parameters: New parametetr values
        :type parameters: dict[str, str]
        """
        raise NotImplementedError()

    @property
    @abstractmethod
    def boundingBox(self):
        """
        Get the bounding box.

        :return: Bounding box
        :rtype: ScmnRectangle
        """
        raise NotImplementedError()

    # noinspection PyShadowingNames
    @abstractmethod
    def setBoundingBox(self, x, y, width, height):
        """
        Set the bounding box.

        :param x: X-position
        :type x: float
        :param y: Y-position
        :type y: float
        :param width: Width
        :type width: float
        :param height: Height
        :type height: float
        """
        raise NotImplementedError()
