# ----------------------------------------------------------------------------
# - whdb-connection-api                                                      -
# - wild cruncher: connection api                              core / db-api -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_connection_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Connection API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod
from wcr_core.db_api.whdb_protocols_api import WhdbHasZValueP, \
    WhdbNamedEntityP, WhdbBaseP


# noinspection PyAbstractClass
class WhdbConnectionP(WhdbNamedEntityP, WhdbHasZValueP, WhdbBaseP):
    """
    Protocol for connections.
    """

    @abstractmethod
    def getConnectionTypeDescription(self):
        raise NotImplementedError()

    @abstractmethod
    def getStartObject(self):
        """
        Get the connection's start object.

        :return: Start object of the connection
        :rtype: WhdbInstanceP
        """
        raise NotImplementedError()

    @abstractmethod
    def getStartConnectionPoint(self):
        """
        Get the connection's start connection point.

        :return: Start connection point of the connection
        :rtype: WhdbConnectionPointP
        """
        raise NotImplementedError()

    @abstractmethod
    def getEndObject(self):
        """
        Get the connection's end object.

        :return: Ent object of the connection
        :rtype: WhdbInstanceP
        """
        raise NotImplementedError()

    @abstractmethod
    def getEndConnectionPoint(self):
        """
        Get the connection's end connection point.

        :return: End connection point of the connection
        :rtype: WhdbConnectionPointP
        """
        raise NotImplementedError()

    @abstractmethod
    def isConnected(self, ix, cx=None):
        """
        Check if an object is connected to the connection
        :param ix: the instance object to check
        :type ix: WhdbInstanceP
        :param cx: the optional connection point object
        :type cx: WhdbConnectionPointP
        """
        raise NotImplementedError()

    @abstractmethod
    def isValid(self):
        raise NotImplementedError()

    @abstractmethod
    def setKeepPath(self, kpth=True):
        """
        Indicate that the path has been manually altered.
        After calling the function, the connection will attempt to maintain as
        much of the alterations as possible.
        Calling getPath().resetPath() will allow fully automated path routing
        again.

        :param kpth: Whether to keep the path
        :type kpth: bool
        """
        raise NotImplementedError()

    @abstractmethod
    def isKeepPath(self):
        """
        Whether the path has been manually altered, i.e. whether the connection
        attempt to preserve the path when its endpoints move.

        :return: Whether the connection is trying to maintain the path.
        :rtype: bool
        """
        raise NotImplementedError()

    @abstractmethod
    def getPath(self):
        """
        The path of the connection.

        :return: Path
        :rtype: WhdbConnectionPathP
        """
        raise NotImplementedError()
