# ----------------------------------------------------------------------------
# - whdb-model-api                                                           -
# - wild cruncher: model api                                   core / db-api -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_model_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Model API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod
from wcr_core.db_api.whdb_protocols_api import WhdbNamedEntityP, \
    WhdbHasMediaP, WhdbBaseP


WHDB_DEFAULT_MODEL_CODE = 'wcr.core.model-default'


# noinspection PyAbstractClass
class WhdbModelP(WhdbNamedEntityP, WhdbHasMediaP, WhdbBaseP):
    """
    External models.
    """

    @abstractmethod
    def setPath(self, path):
        """
        Set the file path of the associated text model.

        :param path: Path of the associated text model
        :type path: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getPath(self):
        """
        Get the file path of the associated text model.

        :return: Path of the associated text model
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def setEntryPoint(self, epnt):
        """
        Set the name of the associated entry function.

        :param epnt: Name of the associated entry function
        :type epnt: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getEntryPoint(self):
        """
        Get the name of the associated entry function.

        :return: Name of the associated entry function
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getPortNames(self):
        """
        Get all port-names for the model, grouped by port direction.

        :return: Port names as {direction:[(name, label)]}
        :rtype: dict[int, list[(str, str)]]
        """
        raise NotImplementedError()

    @abstractmethod
    def resetPortNames(self):
        """
        Reset the stored port-names, i.e. re-read them from the associated
        model.
        """
        raise NotImplementedError()
