# ----------------------------------------------------------------------------
# - whdb-observer_api                                                        -
# - wild cruncher: observer protocol                           core / db-api -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_observer_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Observer protocol

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore_api.wdbs_observer_api import *


WHDB_EVENT_COUNT = WDBS_EVENT_COUNT + 1


WHDB_NO_EVENT = WDBS_NO_EVENT
WHDB_CREATE_EVENT = WDBS_CREATE_EVENT
WHDB_DELETE_EVENT = WDBS_DELETE_EVENT
WHDB_UPDATE_CORE_EVENT = WDBS_UPDATE_CORE_EVENT
WHDB_UPDATE_META_EVENT = WDBS_UPDATE_META_EVENT
WHDB_UPDATE_PARAM_EVENT = WDBS_UPDATE_PARAM_EVENT
WHDB_CHILD_ADDED_EVENT = WDBS_CHILD_ADDED_EVENT
WHDB_CHILD_REMOVED_EVENT = WDBS_CHILD_REMOVED_EVENT

WHDB_ENTITY_RESET_EVENT = (2 ** i for i in range(
    WDBS_EVENT_COUNT, WHDB_EVENT_COUNT))

WHDB_UPDATE_EVENT = WHDB_UPDATE_CORE_EVENT | WHDB_UPDATE_META_EVENT | \
    WHDB_UPDATE_PARAM_EVENT | WHDB_CHILD_ADDED_EVENT | WHDB_CHILD_REMOVED_EVENT


class WhdbObserverP(metaclass=ABCMeta):
    """
    Protocol for observers of HDB entities.
    """

    @abstractmethod
    def dbNotify(self, obj, event, *args, **kwargs):
        """
        Callback for notifications from an observed HDB entity.

        :param obj: Source entity of the notification
        :type obj: WhdbBaseP
        :param event: Event code
        :type event: int
        :param args: Event-specific arguments
        :param kwargs: Event-specific keyword-arguments
        """
        raise NotImplementedError()


class WhdbObservableP(metaclass=ABCMeta):
    """
    Protocol for observable HDB entities.
    """

    @abstractmethod
    def registerObserver(self, observer, event=None):
        """
        Register an observer with the entity.

        :param observer: Observer to register
        :type observer: WhdbObserverP
        :param event: Events to notify the observer for (`None` for all events)
        :type event: int
        """
        raise NotImplementedError()

    @abstractmethod
    def unregisterObserver(self, observer):
        """
        Unregister a previously registered observer from the entity.

        :param observer: Observer to unregister
        :type observer: WhdbObserverP
        """
        raise NotImplementedError()
