# ----------------------------------------------------------------------------
# - whdb-parameter-api                                                       -
# - wild cruncher: parameter api                               core / db-api -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whdb_parameter_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Parameter API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WhdbParameterP(metaclass=ABCMeta):
    """
    The WhdbParameterP class is the generic silkan hierarchy parameter
    object. A parameter is imply defined by a name, information string
    and value. One specificity of the object is to provide a copy mechanism.
    """

    @abstractmethod
    def getDescription(self):
        """
        Get the parameter's description.

        :return: Description of the parameter
        :rtype: WcmnParameterDescriptionP
        """
        raise NotImplementedError()

    @abstractmethod
    def getName(self):
        """
        Get the parameter name.

        :return: Name of the parameter
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getInfo(self):
        """
        Get the parameter info.

        :return: Info for the parameter
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getValue(self):
        """
        Get the parameter value.

        :return: Value of the parameter
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def setValue(self, val):
        """
        Set the parameter value.

        :param val: The new parameter value
        :type val: str
        """
        raise NotImplementedError()


class WhdbHasParametersP(metaclass=ABCMeta):
    @abstractmethod
    def getParameters(self):
        """
        Get the parameters.

        :return: Parameters
        :rtype: Iterable[WhdbParameterP]
        """
        raise NotImplementedError()

    @abstractmethod
    def getParameter(self, name):
        """
        Get the parameter with the given name.

        :param name: Name to look for
        :type name: str
        :return: Parameter with the given name
        :rtype: WhdbParameterP
        """
        raise NotImplementedError()

    @abstractmethod
    def getParameterValue(self, name):
        """
        Get the value for the parameter with the given name, or `None` if no
        such parameter exists.

        :param name: Name to look for
        :type name: str
        :return: Value for the parameter with the given name, or `None` if no
                 such parameter exists
        :rtype: object
        """
        raise NotImplementedError()
