# ----------------------------------------------------------------------------
# - wdbi-connection-point                                                    -
# - wild cruncher:ish connector objects           core / dbstore / immutable -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbi_connection_point
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher immutable connection point objects

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.immutable.wdbi_tree import WdbiFieldNode
from wcr_core.dbstore.immutable.wdbi_tree_db import WDBI_CONNECTOR_NODE, \
    WdbiDbArrayNode
from wcr_core.dbstore.immutable.wdbi_entity import WdbiNodeEntityBase, \
    WdbiTreepathEntityBase, WdbiEntityMixinP
from wcr_core.dbstore_api.wdbs_connection_point_api import WdbsConnectionPointP


_WDBI_XPOINT_TYPE = 0
_WDBI_XPOINT_DIRECTION = 1


def wdbi_createConnectionPoint(metaPrv, ctyp, uuid=None, meta=None, **kwargs):
    """
    Create a new connection point node.

    :param metaPrv: Meta-description provider
    :type metaPrv: WdbsMetaProviderP
    :param ctyp: Connection point type
    :type ctyp: str
    :param uuid: UUID
    :type uuid: UUID | str | None
    :param meta: Meta description
    :type meta: WdbsMetaDescriptionP | None
    :param kwargs: Additional meta-data
    :return: Newly created connection point node
    :rtype: WdbiDbArrayNode
    """
    assert bool(ctyp)
    assert 'cdir' in kwargs

    if meta is None:
        meta = metaPrv.getMetaDescription(WdbsConnectionPointP, ctyp)

    return WdbiDbArrayNode(
        uuid, WDBI_CONNECTOR_NODE, meta,
        [WdbiFieldNode(None, 0, 'type', ctyp),
         WdbiFieldNode(None, 0, 'cdir', kwargs.get('cdir'))],
        **kwargs)


# noinspection PyAbstractClass,PyUnresolvedReferences
class WdbiConnectionPointEntityMixin(WdbiEntityMixinP, WdbsConnectionPointP):
    """
    Wrapper for connection point nodes.
    """

    def getEntityType(self):
        return self._current().get(_WDBI_XPOINT_TYPE).getValue()

    def getDirection(self):
        return self._current().get(_WDBI_XPOINT_DIRECTION).getValue()


class WdbiConnectionPointForNode(WdbiNodeEntityBase,
                                 WdbiConnectionPointEntityMixin):
    """
    Wrapper for connection point nodes, handling current node internally.
    """

    def __init__(self, dctx, node):
        """
        :param dctx: Database context
        :type dctx: WdbiContext
        :param node: Node to be wrapped
        :type node: WdbiDbArrayNode
        """
        WdbiNodeEntityBase.__init__(self, dctx, node)


class WdbiConnectionPointForTreepath(WdbiTreepathEntityBase,
                                     WdbiConnectionPointEntityMixin):
    """
    Wrapper for externally managed connection point nodes.
    """

    def __init__(self, dctx, currentF, updateF, ident):
        """
        :param dctx: Database context
        :type dctx: WdbiContext
        :param currentF: Function providing the current connector node
        :type currentF: (int) -> WdbiDbArrayNode
        :param updateF: Function to update the current connector node
        :type updateF: ((int, WdbiDbArrayNode) -> v) -> _
        :param ident: Ident of the wrapped node
        :type ident: int
        """
        WdbiTreepathEntityBase.__init__(self, dctx, currentF, updateF, ident)
