# ----------------------------------------------------------------------------
# - wdbi-context                                                             -
# - wild cruncher: database context               core / dbstore / immutable -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbi_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - DBI database context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from weakref import ref

from wcr_core.dbstore.wdbs_context import WdbsContext


class WdbiContext(WdbsContext):
    def __init__(self, metaPrv):
        """
        :param metaPrv: Meta-description provider
        :type metaPrv: WdbsMetaProviderP
        """
        super().__init__(metaPrv)

        self.__registry = {}

    def wrap(self, wpid, func, *args, **kwargs):
        """
        Get the wrapper for the node with the given id, or create a new one.

        :param wpid: Node id
        :type wpid: UUID
        :param func: Wrapper creation function
        :type func: (*_) -> WdbiWrapperP
        :param args: Arguments for the wrapper creation function
        :param kwargs: Keyword-arguments for the wrapper creation function
        :return: Wrapper for the node with the given id
        :rtype: WdbiEntityP
        """
        wref = self.__registry.get(wpid)
        if wref is not None:
            wrapped = wref()
            if wrapped is not None:
                return wrapped

        wrapped = func(self, *args, **kwargs)
        self.__registry[wpid] = ref(wrapped)
        return wrapped
