# ----------------------------------------------------------------------------
# - wdbi-db-utils                                                            -
# - wild cruncher:ish utility functions           core / dbstore / immutable -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbi_db_utils
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher immutable database utility functions

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.immutable.wdbi_tree_db import *

_WDBI_NODE_NAMES = {
    0: 'GENERIC',
    1: 'CONNECTOR',
    2: 'OPERATOR_INSTANCE',
    3: 'BLOCK_INSTANCE',
    4: 'MODEL_INSTANCE',
    5: 'BLOCK',
    6: 'MODEL',
    7: 'PARAMETER',
    8: 'PATH',
    9: 'POINT',
    10: 'SHAPE',
    11: 'WIRE',
    12: 'BUTTON',
    101: 'CONNECTOR_LIST',
    103: 'INSTANCE_LIST',
    104: 'WIRE_LIST',
    105: 'SHAPE_LIST',
    106: 'PARAM_DICT',
    107: 'BUTTON_LIST',
    999: 'META'
}


def wdbi_printDbNode(node, indent='', first=None, file=None):
    """
    Recursively print the structure of the given node.

    :param node: Node to be printed
    :type node: WdbiDbNodeP
    :param indent: Current indendation prefix
    :type indent: str
    :param first: Indendation prefix for first line
    :type first: str
    """
    nodeType = _WDBI_NODE_NAMES.get(node.nodeType, '?')

    # noinspection PyShadowingNames
    def formatMultiNodes(ntyp, fnds):
        flds = ', '.join(
            '{}: {}'.format(f.getFieldName(), f.getValue()) for f in fnds)
        # noinspection PyUnresolvedReferences
        return '{i}{k}#{t} {fs}@{id} {m}'.format(
            k=nodeType, t=ntyp,
            id=node.getUUID(), i=first or indent,
            fs='<{}> '.format(flds) if flds else '', m=getMeta())

    def getMeta():
        return str(node.getAllMeta()) if isinstance(node, WdbiDbNodeP) else ''

    if isinstance(node, WdbiDictNode):
        fnds = [f for _, f in node if isinstance(f, WdbiFieldNode)]
        print(formatMultiNodes('D', fnds), file=file)

        for k, v in node:
            if isinstance(v, WdbiFieldNode):
                continue
            print('{i} +-{k}:'.format(i=indent, k=k), file=file)
            wdbi_printDbNode(v, indent + ' |   ', file=file)
    elif isinstance(node, WdbiListNode):
        fnds = [f for f in node if isinstance(f, WdbiFieldNode)]
        print(formatMultiNodes('L', fnds), file=file)

        for v in node:
            if isinstance(v, WdbiFieldNode):
                continue
            wdbi_printDbNode(
                v, indent + '   ', first=indent + ' * ', file=file)
    elif isinstance(node, WdbiArrayNode):
        fnds = [f for f in node if isinstance(f, WdbiFieldNode)]
        print(formatMultiNodes('A', fnds), file=file)

        for v in node:
            if isinstance(v, WdbiFieldNode):
                continue
            wdbi_printDbNode(
                v, indent + '   ', first=indent + ' * ', file=file)
    elif isinstance(node, WdbiLeafNode):
        print(
            '{i}{t}#L @{id} {m}'.format(
                t=nodeType, id=node.getUUID(), i=first or indent,
                m=getMeta()), file=file)
    elif isinstance(node, WdbiDbFieldNode):
        print(
            '{i}{t}#F {n}: {v} @{id} {m}'.format(
                t=nodeType, id=node.getUUID(), n=node.getFieldName(),
                v=node.getValue(), i=first or indent, m=getMeta()),
            file=file)
    else:
        print(
            '{i}{t}#{n} {s}'.format(
                t=nodeType, n=node.__class__.__name__, s=node,
                i=first or indent),
            file=file)
