# ----------------------------------------------------------------------------
# - wdbi-shape                                                               -
# - wild cruncher:ish shape objects               core / dbstore / immutable -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbi_shape
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher immutable shape objects

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.immutable.wdbi_tree import WdbiFieldNode
from wcr_core.dbstore.immutable.wdbi_tree_db import WDBI_SHAPE_NODE, \
    WdbiDbArrayNode
from wcr_core.dbstore.immutable.wdbi_entity import WdbiNodeEntityBase, \
    WdbiTreepathEntityBase, WdbiEntityMixinP
from wcr_core.dbstore_api.wdbs_shape_api import WdbsShapeP


_WDBI_SHAPE_TYPE = 0


def wdbi_createShape(metaPrv, shapeType, uuid=None, meta=None, **kwargs):
    """
    Create a new shape node.

    :param metaPrv: Meta-description provider
    :type metaPrv: WdbsMetaProviderP
    :param shapeType: Shape type for new node
    :type shapeType: str
    :param uuid: UUID
    :type uuid: UUID | str | None
    :param meta: Meta description
    :type meta: WdbsMetaDescriptionP | None
    :param kwargs: Additional meta-data
    :return: Newly created shape node
    :rtype: WdbiDbArrayNode
    """
    if meta is None:
        meta = metaPrv.getMetaDescription(WdbsShapeP, shapeType)

    return WdbiDbArrayNode(
        uuid, WDBI_SHAPE_NODE, meta,
        [WdbiFieldNode(None, 0, 'type', shapeType)],
        **kwargs)


# noinspection PyUnresolvedReferences,PyAbstractClass
class WdbiShapeEntityMixin(WdbiEntityMixinP, WdbsShapeP):
    """
    Wrapper for shape nodes.
    """

    def getEntityType(self):
        return self._current().get(_WDBI_SHAPE_TYPE).getValue()


class WdbiShapeForNode(WdbiNodeEntityBase, WdbiShapeEntityMixin):
    """
    Wrapper for shape nodes, handling current node internally.
    """

    def __init__(self, dctx, node):
        """
        :param dctx: Database context
        :type dctx: WdbiContext
        :param node: Node to be wrapped
        :type node: WdbiDbArrayNode
        """
        WdbiNodeEntityBase.__init__(self, dctx, node)


class WdbiShapeForTreepath(WdbiTreepathEntityBase, WdbiShapeEntityMixin):
    """
    Wrapper for externally managed shape nodes.
    """

    def __init__(self, dctx, currentF, updateF, ident):
        """
        :param dctx: Database context
        :type dctx: WdbiContext
        :param currentF: Function providing the current shape node
        :type currentF: (int) -> WdbiDbArrayNode
        :param updateF: Function to update the current shape node
        :type updateF: ((int, WdbiDbArrayNode) -> WdbiDbArrayNode) -> None
        :param ident: Ident of the wrapped node
        :type ident: int
        """
        WdbiTreepathEntityBase.__init__(self, dctx, currentF, updateF, ident)
