# ----------------------------------------------------------------------------
# - wdbi-utils                                                               -
# - wild cruncher:ish utility functions           core / dbstore / immutable -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbi_utils
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher immutable database utility functions

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.immutable.wdbi_tree import *


def wdbi_printNode(node, indent='', first=None):
    """
    Recursively print the structure of the given node.

    :param node: Node to be printed
    :type node: WdbiNodeP
    :param indent: Current indendation prefix
    :type indent: str
    :param first: Indendation prefix for first line
    :type first: str
    """
    if isinstance(node, WdbiDictNode):
        print('{i}DICT @{id} @@{pid}'.format(
            id=node.getUUID(), pid=id(node), i=first or indent))

        for k, v in node:
            print('{i} +-{k}:'.format(i=indent, k=k))
            wdbi_printNode(v, indent + ' |   ')
    elif isinstance(node, WdbiListNode):
        print('{i}LIST @{id} @@{pid}'.format(
            id=node.getUUID(), pid=id(node), i=first or indent))

        for v in node:
            wdbi_printNode(v, indent + '   ', first=indent + ' * ')
    elif isinstance(node, WdbiArrayNode):
        print('{i}ARRAY @{id} @@{pid}'.format(
            id=node.getUUID(), pid=id(node), i=first or indent))

        for v in node:
            wdbi_printNode(v, indent + '   ', first=indent + ' * ')
    elif isinstance(node, WdbiLeafNode):
        print('{i}NODE @{id} @@{pid}'.format(
            id=node.getUUID(), pid=id(node), i=first or indent))
    elif isinstance(node, WdbiFieldNode):
        print('{i}FIELD @{id} @@{pid}'.format(
            id=node.getUUID(), pid=id(node), i=first or indent))
    else:
        print('{i}UNKNOWN {n} {s}'.format(
            n=type(node), s=node, i=first or indent))


def wdbi_path2str(path):
    """
    Get the string representation of a path.

    :param path: Path to obtain string representation for
    :type path: Iterable
    :return: String representation of given path
    :rtype: str
    """

    def pstr(p):
        try:
            return p.getUUID()
        except AttributeError:
            return p

    return str(list(map(pstr, path)))
