# ----------------------------------------------------------------------------
# - wdbm-button                                                              -
# - wild cruncher:msh interactive elements          core / dbstore / mutable -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbm_button
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher mutable interactive elements

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.mutable.wdbm_parameter import WdbmHasParametersMixin
from wcr_core.dbstore.mutable.wdbm_entity import WdbmEntityBase
from wcr_core.dbstore_api.wdbs_buttons_api import WdbsButtonP
from wcr_core.dbstore_api.wdbs_observer_api import WDBS_UPDATE_CORE_EVENT


class WdbmButton(WdbmEntityBase, WdbmHasParametersMixin, WdbsButtonP):
    def __init__(self, dctx, uuid=None, title=None, command=None, params=None,
                 meta=None, **kwargs):
        """
        :param dctx: Database context
        :type dctx: WdbsContextP
        :param title: Button title
        :type title: str
        :param command: Command associated with button
        :type command: str
        :param bpos: Position
        :type bpos: (float, float)
        :param params: Command parameter values
        :type params: list[(str, object)]
        :param uuid: UUID
        :type uuid: UUID | str | None
        :param meta: Meta-data description
        :type meta: WdbsMetaDescriptionP | None
        :param kwargs: Additional meta-data
        """
        WdbmEntityBase.__init__(self, dctx, uuid, meta, **kwargs)
        WdbmHasParametersMixin.__init__(self, dctx, params)

        #: Button title
        self.__title = title if title is not None else ''
        #: Command
        self.__command = command if command is not None else ''

    def doDelete(self, recurse=True):
        WdbmHasParametersMixin.doDelete(self)
        super().doDelete()

    def getCommand(self):
        return self.__command

    def setCommand(self, command):
        self.__command = command
        self._notifyObservers(WDBS_UPDATE_CORE_EVENT)

    def getTitle(self):
        return self.__title

    def setTitle(self, title):
        self.__title = title
        self._notifyObservers(WDBS_UPDATE_CORE_EVENT)
