# ----------------------------------------------------------------------------
# - wdbm-connection                                                          -
# - wild cruncher:msh model connection object       core / dbstore / mutable -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbm_connection
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher mutable silkan hierarchy model connection

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.mutable.wdbm_geometry import WdbmPath
from wcr_core.dbstore.mutable.wdbm_entity import WdbmNamedEntityMixin, \
    WdbmEntityBase
from wcr_core.dbstore_api.wdbs_connection_api import WdbsConnectionP
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_IN
from wcr_core.dbstore_api.wdbs_observer_api import WDBS_UPDATE_CORE_EVENT


class WdbmConnection(WdbmEntityBase, WdbmNamedEntityMixin, WdbsConnectionP):
    """
    The WdbmConnection is the silkan hierarchical connection object.
    A connection is defined by name and connect two connection points.
    """

    def __init__(self, dctx, so, sc, eo, ec, name=None, info=None, path=None,
                 uuid=None, meta=None, **kwargs):
        """
        :param dctx: Database context
        :type dctx: WdbsContextP
        :param so: the starting object
        :type so: WdbsInstanceP
        :param sc: the starting connector
        :type sc: WdbsConnectionPointP
        :param eo: the ending object
        :type eo: WdbsInstanceP
        :param ec: the ending connector
        :type ec: WdbsConnectionPointP
        :param name: the wire name
        :type name: str
        :param info: the wire information
        :type info: str
        :param path: Connection path
        :type path: [(float, float)]
        :param uuid: UUID
        :type uuid: UUID | str | None
        :param meta: Meta-data description
        :type meta: WdbsMetaDescriptionP | None
        :param kwargs: Additional meta-data
        """
        if so is None: raise ValueError('Start-object cannot be None')
        if sc is None: raise ValueError('Start-connector cannot be None')
        if eo is None: raise ValueError('End-object cannot be None')
        if ec is None: raise ValueError('End-connector cannot be None')

        WdbmEntityBase.__init__(self, dctx, uuid, meta, **kwargs)
        WdbmNamedEntityMixin.__init__(self, name, info)

        cord = sc.getDirection() != WDBS_XPNT_IN
        #: Start object
        self.__srcObject = so if cord else eo
        #: Start connector
        self.__srcXPoint = sc if cord else ec
        #: End object
        self.__destObject = eo if cord else so
        #: End connector
        self.__destXPoint = ec if cord else sc

        self.__path = WdbmPath(self.context, path) if path else None

        #sc.addConnection(self)
        #ec.addConnection(self)

    def doDelete(self, recurse=True):
        #self.__srcXPoint.removeConnection(self)
        #self.__destXPoint.removeConnection(self)
        path = self.__path
        if path is not None:
            path.doDelete()
        super().doDelete()

    def getStartObject(self):
        return self.__srcObject

    def getStartXPoint(self):
        return self.__srcXPoint

    def getEndObject(self):
        return self.__destObject

    def getEndXPoint(self):
        return self.__destXPoint

    def getPath(self):
        return self.__path

    def setPath(self, path):
        self.__path = WdbmPath(self.context, path)
        self._notifyObservers(WDBS_UPDATE_CORE_EVENT)
