# ----------------------------------------------------------------------------
# - wdbm-geometry                                                            -
# - wild cruncher:msh geometry object               core / dbstore / mutable -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbm_geometry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher mutable silkan hierarchy geometric objects

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.mutable.wdbm_entity import WdbmEntityBase
from wcr_core.dbstore_api.wdbs_geometry_api import WdbsPathP
from silkan_common.scmn_geometry import ScmnPointMP
from wcr_core.dbstore_api.wdbs_observer_api import WDBS_CHILD_REMOVED_EVENT, \
    WDBS_CHILD_ADDED_EVENT


class WdbmPoint(ScmnPointMP):
    """
    The WdbmPoint class is a two-dimensional geometric point.
    """

    def __init__(self, x, y):
        """
        :param x: the x coordinate
        :param y: the y coordinate
        """
        self.__x = x
        self.__y = y

    def getX(self):
        return self.__x

    def getY(self):
        return self.__y

    def setXY(self, x, y):
        self.__x = x
        self.__y = y

    def setX(self, x):
        self.__x = x

    def setY(self, y):
        self.__y = y

    def __iter__(self):
        return iter((self.__x, self.__y))

    def __str__(self):
        return '({}, {})'.format(self.__x, self.__y)

    def __repr__(self):
        return str(self)


# ----------------------------------------------------------------------------
# - path class                                                               -
# ----------------------------------------------------------------------------

class WdbmPath(WdbmEntityBase, WdbsPathP):
    """
    The WdbmPath class is a geometric path defined as a collection of points.
    The first point is the start of the path and the last point is the end.
    """

    def __init__(self, dctx, plst=None, uuid=None, meta=None, **kwargs):
        """
        :param dctx: Database context
        :type dctx: WdbsContextP
        :param plst: Point list
        :type plst: Iterable[(float, float)] | None
        :param uuid: UUID
        :type uuid: UUID | str | None
        :param meta: Meta-data description
        :type meta: WdbsMetaDescriptionP | None
        """
        WdbmEntityBase.__init__(
            self, dctx=dctx, uuid=uuid, meta=meta, **kwargs)

        self.__plst = [WdbmPoint(*p) for p in plst] if plst else []

    def getPoints(self):
        return self.__plst

    def addPoint(self, x, y, pos=None):
        point = WdbmPoint(x, y)
        if pos is None:
            self.__plst.append(point)
        else:
            self.__plst[pos:pos] = [point]
        self._notifyObservers(WDBS_CHILD_ADDED_EVENT)
        return point

    def removePoint(self, point):
        self.__plst.remove(point)
        self._notifyObservers(WDBS_CHILD_REMOVED_EVENT)

    def __iter__(self):
        """
        :rtype: Iterator[WdbmPoint] | Iterable[WdbmPoint]
        """
        return iter(self.__plst)
