# ----------------------------------------------------------------------------
# - wdbm-model                                                               -
# - wild cruncher:msh model object                  core / dbstore / mutable -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2013 amaury darsch                                    -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbm_model
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher mutable base model object

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from wcr_core.dbstore.mutable.wdbm_entity import WdbmTypedEntityBase, \
    WdbmNamedEntityMixin
from wcr_core.dbstore_api.wdbs_model_api import WdbsModelP
from wcr_core.dbstore_api.wdbs_observer_api import WDBS_UPDATE_CORE_EVENT


class WdbmModel(WdbmTypedEntityBase, WdbmNamedEntityMixin, WdbsModelP):
    """
    The WdbmModel is a base class for a model abstraction of the silkan
    hierarchy.
    """

    def __init__(self, dctx, mtyp, path=None, entryPoint=None,
                 name=None, info=None, uuid=None, meta=None, **kwargs):
        """
        :param dctx: Database context
        :type dctx: WdbsContextP
        :param mtyp: Structural model type
        :type mtyp: str
        :param path: Path to model file
        :type path: str
        :param entryPoint: Entry point (e.g. function name)
        :type entryPoint: str
        :param name: Model name
        :type name: str
        :param info: Model info string
        :type info: str
        :param uuid: UUID
        :type uuid: UUID | str | None
        :param meta: Meta-data description
        :type meta: WdbsMetaDescriptionP | None
        :param kwargs: Additional meta-data
        """
        WdbmTypedEntityBase.__init__(
            self, mtyp, dctx, uuid, meta, **kwargs)
        WdbmNamedEntityMixin.__init__(self, name, info)

        #: Model file path
        self.__path = path if path is not None else ''
        #: Model file entry point
        self.__entryPoint = entryPoint if entryPoint is not None else ''

    def getPath(self):
        return self.__path

    def getEntryPoint(self):
        return self.__entryPoint

    def setPath(self, path):
        self.__path = path
        self._notifyObservers(WDBS_UPDATE_CORE_EVENT)

    def setEntryPoint(self, entryPoint):
        self.__entryPoint = entryPoint
        self._notifyObservers(WDBS_UPDATE_CORE_EVENT)
