# ----------------------------------------------------------------------------
# - wdbs_meta_proxy                                                          -
# - wild cruncher: meta-data proxy types                      core / dbstore -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbs-meta-proxy
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher meta-data proxy types

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore_api.wdbs_meta_type_api import WdbsListMetaTypeP


class WdbsListMetaTypeProxy(WdbsListMetaTypeP):
    """
    Proxy-class for list meta-types.
    """

    def __init__(self, proxied=None):
        """
        :param proxied: Initial proxied element
        :type proxied: WdbsListMetaTypeP | None
        """
        self.__proxied = proxied

    def setProxiedElement(self, proxied):
        """
        Set the proxied list element

        :param proxied: Proxied element
        :type proxied: WdbsListMetaTypeP | None
        """
        self.__proxied = proxied

    def getProxiedElement(self):
        """
        Get the proxied element.

        :return: Proxied element, or `None`
        :rtype: WdbsListMetaTypeP | None
        """
        return self.__proxied

    def __str__(self):
        return '&[{}]'.format(
            self.__proxied.__class__.__name__ if self.__proxied else 'UNKNOWN')

    def __repr__(self):
        return str(self)

    def asValue(self, meta):
        proxied = self.__proxied
        return proxied.asValue(meta) if proxied else None

    def getEmptyValue(self):
        proxied = self.__proxied
        return proxied.getEmptyValue() if proxied else None

    def getValueType(self):
        proxied = self.__proxied
        return proxied.getValueType() if proxied else None

    def setFromValue(self, setF, *args, **kwargs):
        proxied = self.__proxied
        if proxied:
            proxied.setFromValue(setF, *args, **kwargs)

    def getStructName(self):
        proxied = self.__proxied
        return proxied.getStructName() if proxied else None

    def getName(self):
        proxied = self.__proxied
        return proxied.getName() if proxied else None

    def getKey(self, item, data):
        proxied = self.__proxied
        return proxied.getKey(item, data) if proxied else None
