# ----------------------------------------------------------------------------
# - wdbs-connection                                                          -
# - wild cruncher:rsh connection object                   core / dbstore-api -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2013 clemens oertel                                   -
# - copyright (c) 2012-2013 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbs_connection
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher raw hierarchy connection objects

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod

from wcr_core.dbstore_api.wdbs_entity_api import WdbsNamedEntityP


# noinspection PyAbstractClass
class WdbsConnectionP(WdbsNamedEntityP):
    """
    Raw connection object.
    """

    @abstractmethod
    def getStartObject(self):
        """
        Get the connection's start object.

        :return: Start object
        :rtype: WdbsInstanceP
        """
        raise NotImplementedError()

    @abstractmethod
    def getStartXPoint(self):
        """
        Get the connection's start connection point.

        :return: Start connection point
        :rtype: WdbsConnectionPointP
        """
        raise NotImplementedError()

    @abstractmethod
    def getEndObject(self):
        """
        Get the connection's end object.

        :return: End object
        :rtype: WdbsInstanceP
        """
        raise NotImplementedError()

    @abstractmethod
    def getEndXPoint(self):
        """
        Get the connection's end connection point.

        :return: End connection point
        :rtype: WdbsConnectionPointP
        """
        raise NotImplementedError()

    @abstractmethod
    def getPath(self):
        """
        Get the connection's path.

        :return: Path
        :rtype: WdbsPathP
        """
        raise NotImplementedError()

    @abstractmethod
    def setPath(self, path):
        """
        Set the connection's path.

        :param path: New path [(x, y)]
        :type path: Iterable[(float, float)]
        """
        raise NotImplementedError()
