# ----------------------------------------------------------------------------
# - wdbs-geometry                                                            -
# - wild cruncher: geometrical elements                   core / dbstore-api -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbs_geometry
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher scene items geometrical elements

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod

from wcr_core.dbstore_api.wdbs_entity_api import WdbsEntityP


# noinspection PyAbstractClass
class WdbsPathP(WdbsEntityP):
    @abstractmethod
    def getPoints(self):
        """
        Get the collection of points as a list.

        :return: List of paints making up the path
        :rtype: list[ScmnPointMP]
        """
        raise NotImplementedError()

    @abstractmethod
    def addPoint(self, x, y, pos=None):
        """
        Add a point by coordinate

        :param x: the x coordinate
        :type x: float
        :param y: the y coordinate
        :type y: float
        """
        raise NotImplementedError()

    @abstractmethod
    def removePoint(self, point):
        """
        Remove a point from the path.

        :param point: Point to be removed
        :type point: ScmnPointP
        """
        raise NotImplementedError()

    @abstractmethod
    def __iter__(self):
        """
        :rtype: Iterator[ScmnPointMP]
        """
        raise NotImplementedError()
