# ----------------------------------------------------------------------------
# - wdbs-meta-description                                                    -
# - wild cruncher:rsh meta descriptions                   core / dbstore-api -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbs_meta_description
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher raw hierarchy meta descriptions

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WdbsMetaFieldP(metaclass=ABCMeta):
    """
    Meta-data field.
    """
    __slots__ = ()

    @property
    @abstractmethod
    def name(self):
        """
        Get the field's name.

        :return: Field name
        :rtype: str
        """
        raise NotImplementedError()

    @property
    @abstractmethod
    def title(self):
        """
        Get the field's title.

        :return: Field title
        :rtype: str
        """
        raise NotImplementedError()

    @property
    @abstractmethod
    def is_temporary(self):
        """
        Whether the field holds temporary/non-serializable data.

        :return: Temporary data
        :rtype: bool
        """
        raise NotImplementedError()

    @property
    @abstractmethod
    def metaType(self):
        """
        Extract a value for this field from a given string.

        :return: Meta-data type
        :rtype: WdbsMetaTypeP
        """
        raise NotImplementedError()

    @abstractmethod
    def addRWFuncs(self, code, version, readF, writeF=None):
        """
        Add adapter read/write functions for this meta-data field for the given
        serialization type/version.

        :param code: Code of the serialization type
        :type code: str
        :param version: Version of the serialization
        :type version: str
        :param readF: Read-function, from serialization to data
        :type readF: (object) -> object
        :param writeF: Write-function, from data to serialization
        :type writeF: (object) -> object | None
        """
        raise NotImplementedError()

    @abstractmethod
    def getRWFuncs(self, code, version):
        """
        Get the adapter read/write functions for the given serialization
        type/version, if any.

        :param code: Code of the serialization type
        :type code: str
        :param version: Version of the serialization
        :type version: str
        :return: Read- and write-functions
        :rtype: ((object) -> object, (object) -> object)
        """
        raise NotImplementedError()


class WdbsMetaDescriptionP(metaclass=ABCMeta):
    """
    Meta-data description for an entity class.
    """

    __slots__ = ()

    @property
    @abstractmethod
    def fields(self):
        """
        Get the meta-data fields.

        :return: Meta data fields
        :rtype: Iterable[WdbsMetaFieldP]
        """
        raise NotImplementedError()

    @abstractmethod
    def __getitem__(self, item):
        raise NotImplementedError()
