# ----------------------------------------------------------------------------
# - wdbs-observer-api                                                        -
# - wild cruncher: api for dbstore observers              core / dbstore-api -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbs_observer_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - API for dbstore observers

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


#: Total number of event types
WDBS_EVENT_COUNT = 8


# DBSTORE notification events
WDBS_NO_EVENT, \
    WDBS_CREATE_EVENT, \
    WDBS_DELETE_EVENT, \
    WDBS_UPDATE_CORE_EVENT, \
    WDBS_UPDATE_META_EVENT, \
    WDBS_UPDATE_PARAM_EVENT, \
    WDBS_CHILD_ADDED_EVENT, \
    WDBS_CHILD_REMOVED_EVENT, \
    WDBS_ANY_EVENT = (2 ** i for i in range(WDBS_EVENT_COUNT + 1))
WDBS_ANY_EVENT -= 1

WDBS_CHILD_EVENT = WDBS_CHILD_ADDED_EVENT | WDBS_CHILD_REMOVED_EVENT

WDBS_UPDATE_EVENT = WDBS_UPDATE_CORE_EVENT | WDBS_UPDATE_META_EVENT | \
    WDBS_UPDATE_PARAM_EVENT | WDBS_CHILD_ADDED_EVENT | WDBS_CHILD_REMOVED_EVENT


class WdbsObserverP(metaclass=ABCMeta):
    """
    Protocol for observers of dbstore entities.
    """
    @abstractmethod
    def dbNotify(self, obj, event, *args, **kwargs):
        """
        Callback for notification about changes in dbstore entities.

        :param obj: Changed dbstore entity
        :type obj: WdbsEntityP
        :param event: Event
        :type event: int
        :param args: Event/entity-specific arguments
        :param kwargs: Event/entity-specific keyword arguments
        """
        raise NotImplementedError()


class WdbsObservableP(metaclass=ABCMeta):
    """
    Protocol for observable dbstore entities.
    """
    __slots__ = ()

    @abstractmethod
    def registerObserver(self, observer, event=None):
        """
        Register an observer for notification about changes.

        :param observer: Observer
        :type observer: WdbsObserverP
        :param event: Mask defining events of interest (currently ignored)
        :type event: int
        """
        raise NotImplementedError()

    @abstractmethod
    def unregisterObserver(self, observer):
        """
        Unregister an observer.

        :param observer: Observer
        :type observer: WdbsObserverP
        """
        raise NotImplementedError()
