# ----------------------------------------------------------------------------
# - wdbs-parameter                                                           -
# - wild cruncher:rsh parameter object                    core / dbstore-api -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wdbs_parameter
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher raw hierarchy parameter objects

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod

from wcr_core.dbstore_api.wdbs_entity_api import WdbsEntityP


# noinspection PyAbstractClass
class WdbsParameterP(WdbsEntityP):
    """
    Parameter object.
    """

    @abstractmethod
    def getName(self):
        """
        Get the parameter's name.

        :return: Parameter name
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getValue(self):
        """
        Get the parameter's value.

        :return: Parameter value
        :rtype: str | None
        """
        raise NotImplementedError()

    @abstractmethod
    def setValue(self, value):
        """
        Set the parameter's value.

        :param value: New parameter value
        :type value: str | None
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WdbsEntityParametersP(WdbsEntityP):
    """
    Protocol for raw hierarchy entities holding parameters.
    """
    __slots__ = ()

    @abstractmethod
    def getParameterCount(self):
        """
        Get the number of parameters.

        :return: Number of parameters
        :rtype: int
        """
        raise NotImplementedError()

    @abstractmethod
    def getParameters(self):
        """
        Get the parameters.

        :return: Parameter iterable
        :rtype: Iterable[WdbsParameterP]
        """
        raise NotImplementedError()

    @abstractmethod
    def getParameter(self, name):
        """
        Get the parameter with the given name.

        :param name: Name of parameter to retrieve
        :type name: str
        :return: Parameter with the given name
        :rtype: WdbsParameterP
        """
        raise NotImplementedError()

    @abstractmethod
    def getOrCreateParameter(self, name):
        """
        Get the parameter with the given name, and create it if it does not yet
        exist.

        :param name: Name of parameter to retrieve
        :type name: str
        :return: Parameter with the given name
        :rtype: WdbsParameterP
        """
        raise NotImplementedError()

    @abstractmethod
    def removeParameter(self, name):
        """
        Remove the parameter with the given name.

        :param name: Name of parameter to be removed
        :type name: str
        """
        raise NotImplementedError()

    @abstractmethod
    def clearParameters(self):
        """
        Remove all parameters.
        """
        raise NotImplementedError()
