# ----------------------------------------------------------------------------
# - wbhdb-connection-points                                                  -
# - wild cruncher: connection points                         core / bus / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbhdb_connection_points
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Bus connection points

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_TYPE, \
    WHDB_XPOINT_DIRECTION
from wcr_core.dbstore.wdbs_meta_type import WdbsRecordMetaType, \
    WDBS_STRING_META
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.bus.db.wbhdb_connection_point_base import \
    WbhdbConnectionPointBase


WBHDB_XPOINT_UDP_CODE = 'wcr.bus.xpoint-udp'
WBHDB_XPOINT_TCP_CODE = 'wcr.bus.xpoint-tcp'

WBHDB_XPOINT_UDP_KEY_INTERFACE = 'udp-interface'
WBHDB_XPOINT_TCP_KEY_INTERFACE = 'tcp-interface'

WBHDB_XPOINT_UDP_CONFIG_META = WdbsRecordMetaType(
    (WBHDB_XPOINT_UDP_KEY_INTERFACE, WDBS_STRING_META,
     'UDP interface'),)
WBHDB_XPOINT_TCP_CONFIG_META = WdbsRecordMetaType(
    (WBHDB_XPOINT_TCP_KEY_INTERFACE, WDBS_STRING_META,
     'TCP interface'),)

WBHDB_XPOINT_UDP_DFLT_ARGS = {
    WHDB_XPOINT_TYPE: WBHDB_XPOINT_UDP_CODE,
    WHDB_XPOINT_DIRECTION: WDBS_XPNT_BI
}
WBHDB_XPOINT_TCP_DFLT_ARGS = {
    WHDB_XPOINT_TYPE: WBHDB_XPOINT_TCP_CODE,
    WHDB_XPOINT_DIRECTION: WDBS_XPNT_BI
}


class WbhdbUDPXPoint(WbhdbConnectionPointBase):
    """
    Ethernet connection point.
    """

    def getInterface(self):
        return self._getConfig(WBHDB_XPOINT_UDP_KEY_INTERFACE)

    def getTypeName(self):
        return 'UDP connection point'

    def getNamePrefix(self):
        return 'udp'

    def getDescription(self, **kwargs):
        interface = self.getInterface()

        return '{} (UDP{})'.format(
            self.getName(),
            ', {}: {}'.format(WBHDB_XPOINT_UDP_KEY_INTERFACE, interface)
            if interface else '')

    def getCode(self):
        return WBHDB_XPOINT_UDP_KEY_INTERFACE



class WbhdbTCPXPoint(WbhdbConnectionPointBase):
    """
    Ethernet connection point.
    """

    def getInterface(self):
        return self._getConfig(WBHDB_XPOINT_TCP_KEY_INTERFACE)

    def getTypeName(self):
        return 'TCP connection point'

    def getNamePrefix(self):
        return 'tcp'

    def getDescription(self, **kwargs):
        interface = self.getInterface()

        return '{} (TCP{})'.format(
            self.getName(),
            ', {}: {}'.format(WBHDB_XPOINT_TCP_KEY_INTERFACE, interface)
            if interface else '')

    def getCode(self):
        return WBHDB_XPOINT_TCP_CODE
