# ----------------------------------------------------------------------------
# - wbhdb-external-module-instance                                           -
# - wild cruncher: external module instances                 core / bus / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbhdb_external_module_instance
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher - External module instance

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from uuid import UUID
import re

from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_DIRECTION, \
    WHDB_XPOINT_NAME
from wcr_core.dbstore.wdbs_util import wdbs_makeUUID
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.bus.db.wbhdb_standard_operators import WBHDB_EXT_MODULE
from wcr_core.db.whdb_instance import WhdbOperatorInstance

from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.bus.db.wbhdb_messages_format_meta import \
    WBHDB_MESSAGE_BASE, WBHDB_FILE_KEY, WBHDB_NAME_KEY

_WPHDB_XPOINT_NAME_PATTERN = re.compile(r"^(\w+)-(\d+)$")
WBHDB_META_EXT_MODULE_KEY = "BBmessages_hosted"


class WbhdbExtModuleInstance(WhdbOperatorInstance):
    """
    Operator-instance for host instances.
    """

    META = WdbsMetaDescription(
        WhdbOperatorInstance.META,
        WdbsMetaField(
            WBHDB_META_EXT_MODULE_KEY,
            WdbsDictionaryMetaType(WDBS_STRING_META,
                                   WBHDB_MESSAGE_BASE)))
    """
    META : {WBHDB_META_EXT_MODULE_KEY : {file+name:(file,name)}}
    """

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        WhdbOperatorInstance.__init__(self, WBHDB_EXT_MODULE, inst, wreg)

    def _createRawConnectionPoint(self, ctyp, **kwargs):
        if WHDB_XPOINT_DIRECTION not in kwargs:
            kwargs[WHDB_XPOINT_DIRECTION] = WDBS_XPNT_BI
        return super()._createRawConnectionPoint(ctyp, **kwargs)

    def createConnectionPoint(self, ctyp=None, **kwargs):
        nameGiven = bool(kwargs.get(WHDB_XPOINT_NAME))
        xpnt = super().createConnectionPoint(ctyp, **kwargs)
        """:type: WphdbConnectionPointBase"""
        if not nameGiven:
            prfx = xpnt.getNamePrefix()
            mxnr = 0
            for xpnt in self.getConnectionPoints(ctyp=ctyp):
                m = _WPHDB_XPOINT_NAME_PATTERN.match(xpnt.getName())
                if m and m.group(1) == prfx:
                    mxnr = max(int(m.group(2)), mxnr)
            xpnt.setName('{p}-{n}'.format(p=prfx, n=mxnr + 1))
        return xpnt

    def getMessages(self):
        lmk = sorted(self.rawElement.getMeta(WBHDB_META_EXT_MODULE_KEY).keys())
        lm=[]
        for k in lmk:
            mt=(self.rawElement.getMeta(WBHDB_META_EXT_MODULE_KEY,
                                        k,WBHDB_FILE_KEY),
                self.rawElement.getMeta(WBHDB_META_EXT_MODULE_KEY,
                                        k,WBHDB_NAME_KEY))
            lm.append(mt)
        return lm

    def addMessage(self,message):
        """
        :param message: structur to save message
                        (file, messagename)
        :type message: (str, str)
        """
        newkey=self.formatMessageKey((message[0],message[1]))
        self.rawElement.setMeta({WBHDB_FILE_KEY:message[0],
                                 WBHDB_NAME_KEY:message[1]},
                                WBHDB_META_EXT_MODULE_KEY, newkey)

    def removeAllMessages(self):
        lm=self.rawElement.getMeta(WBHDB_META_EXT_MODULE_KEY).keys()
        for messagekey in lm:
            self.rawElement.removeMeta(WBHDB_META_EXT_MODULE_KEY, messagekey)

    def formatMessageKey(self, datamessage):
        """
        :param datamessage: dhe data used to format key
        :type (str,str)
        """
        return datamessage[0]+' - '+datamessage[1]


def _wphdb_getUuid(item):
    return item if isinstance(item, UUID) else item.uuid
