# ----------------------------------------------------------------------------
# - wbhdb-meta-provider                                                      -
# - wild cruncher: meta-data provider                        core / bus / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbhdb_meta_provider
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy meta-data provider

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_meta_provider import WhdbMetaProvider
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.modules.model.db.wmhdb_block import WmhdbBlock
from wcr_core.modules.bus.db.wbhdb_logibus_instance \
    import WbhdbLogiBusInstance
from wcr_core.modules.bus.db.wbhdb_external_module_instance \
    import WbhdbExtModuleInstance
from wcr_core.modules.bus.db.wbhdb_standard_operators \
    import WBHDB_BUS_CODE, WBHDB_EXT_MODULE_CODE
from wcr_core.modules.bus.db.wbhdb_connection_point_base import \
    WbhdbConnectionPointBase, WBHDB_XPOINT_CONFIG_KEY
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaDescription, \
    WdbsMetaField

WBHDB_STD_OPERINST_META = {WBHDB_BUS_CODE: WbhdbLogiBusInstance.META,
                           WBHDB_EXT_MODULE_CODE: WbhdbExtModuleInstance.META}

class WbhdbMetaProvider(WhdbMetaProvider):
    def __init__(self, blockMetas, xpntCfgMetas):
        super().__init__()

        self.__blockMetas = blockMetas
        self.__xpntCfgMeta = xpntCfgMetas

        self.__xpntMeta = {}

    def getBlockMeta(self, btyp=None):
        if btyp:
            meta = self.__blockMetas.get(btyp)
            if meta:
                return meta
        return WmhdbBlock.META

    def getOperatorInstanceMeta(self, oper=None):
        opMeta = WBHDB_STD_OPERINST_META.get(oper)
        return opMeta if opMeta is not None else WhdbOperatorInstance.META

    def getConnectionPointMeta(self, xtyp=None):
        if not xtyp:
            return WbhdbConnectionPointBase.META

        meta = self.__xpntMeta.get(xtyp)
        if meta:
            return meta

        cfgMeta = self.__xpntCfgMeta.get(xtyp)
        if not cfgMeta:
            return WbhdbConnectionPointBase.META

        meta = WdbsMetaDescription(
            WbhdbConnectionPointBase.META,
            WdbsMetaField(WBHDB_XPOINT_CONFIG_KEY, cfgMeta))
        self.__xpntMeta[xtyp] = meta
        return meta