# ----------------------------------------------------------------------------
# - wbhdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry                 core / bus / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbhdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy raw data wrapper registry

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.db.whdb_block import WHDB_TOPLEVEL_BLOCK_CODE
from wcr_core.modules.bus.db.wbhdb_top_block import WbhdbTopHierarchy
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.db.whdb_block import WhdbBlock
from wcr_core.modules.bus.db.wbhdb_external_module_instance import \
    WbhdbExtModuleInstance
from wcr_core.modules.bus.db.wbhdb_logibus_instance import WbhdbLogiBusInstance
from wcr_core.modules.bus.db.wbhdb_standard_operators import \
WBHDB_BUS_CODE, WBHDB_EXT_MODULE_CODE
from wcr_core.db_api.whdb_protocols_api import WHDB_HALIGN_LEFT, \
    WHDB_HALIGN_RIGHT
from wcr_core.modules.bus.db.wbhdb_operator import WBHDB_OPER, WbhdbOperator
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP
from wcr_core.dbstore_api.wdbs_connection_point_api import WdbsConnectionPointP
from wcr_core.modules.bus.db.wbhdb_connection_point_manager import \
    WbhdbConnectionPointManager
from wcr_core.modules.bus.db.wbhdb_meta_provider import WBHDB_STD_OPERINST_META
from wcr_frontend.modules.bus import _WBGUI_ICONS
from wcr_frontend.modules.bus.gui.wbgui_props_dialog_factory import \
    WBHDB_STD_OPERINST_PROPDLG

from wcr_frontend.modules.bus.gui.wbgui_external_module_instance_props_dialog import \
    WbguiExtModulePropertiesDialog

WBHDB_STD_OPERINST_CLS = {WBHDB_BUS_CODE: WbhdbLogiBusInstance,
                          WBHDB_EXT_MODULE_CODE: WbhdbExtModuleInstance,}


class WbhdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """
    def __init__(self, blockClzzs, xpntClzzs):
        """

        """
        super().__init__()

        self.__blockClasses = blockClzzs
        #: Connection point classes, by code
        self.__xpntClzzs = xpntClzzs

    def createWrapper(self, obj, *args):
        if isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WHDB_TOPLEVEL_BLOCK_CODE:
                return WbhdbTopHierarchy(obj, self)
            else:
                return WhdbBlock(obj, self)
        elif isinstance(obj, WdbsOperatorInstanceP):
            return self._createOperatorInstance(obj)
        elif isinstance(obj, WdbsConnectionPointP):
            clazz = self.__xpntClzzs.get(obj.getEntityType())
            assert clazz, str(self.__xpntClzzs)
            return clazz(obj, self)
        return super().createWrapper(obj, *args)


    def _createOperatorInstance(self, opInst):
        """
        Create an operator instance object based on the type of operator

        :param opInst: Raw operator instance
        :type opInst: WdbsOperatorInstanceP
        :return: Operator instance created
        :rtype: WhdbOperatorInstanceP
        """
        operator = opInst.getOperator()
        cls = WBHDB_STD_OPERINST_CLS.get(operator)

        return cls(opInst, self) if cls is not None else \
            WhdbOperatorInstance(WBHDB_OPER[operator], opInst, self)


def wbhdb_create_and_instanciate_module(opInst):
    module_code = opInst.getOperator().getCode()
    mod_name = module_code.split('.')[-1]
    WBHDB_STD_OPERINST_CLS[WBHDB_EXT_MODULE_CODE] = WbhdbExtModuleInstance
    WBHDB_STD_OPERINST_META[WBHDB_EXT_MODULE_CODE] = WbhdbExtModuleInstance.META
    _WBGUI_ICONS[WBHDB_EXT_MODULE_CODE] = ':/img/wpgui_clstr.png'
    oper = WbhdbOperator(code=WBHDB_EXT_MODULE_CODE,
                         xmngr=WbhdbConnectionPointManager(xpntTypes=None),
                         name=opInst.getName(), info=mod_name, symbol=mod_name,
                         labelAlign=WHDB_HALIGN_LEFT)
    WBHDB_STD_OPERINST_PROPDLG[oper] = WbguiExtModulePropertiesDialog
    return oper
