# ----------------------------------------------------------------------------
# - wbmod_application_module                                                 -
# - wild cruncher: application module                             core / bus -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbmod_application_module
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher bus designer application module

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_top_level_factory import whdb_createTopLevelRawBlock
from wcr_core.modules.bus.db.wbhdb_meta_provider import \
    WbhdbMetaProvider
from wcr_core.modules.bus.db.wbhdb_wrapper_registry import \
    WbhdbWrapperRegistry
from wcr_core.project.wprj_project_context import WprjProjectContext
from wcr_core.modules.bus.db.wbhdb_verifier import \
    WbhdbHierarchyVerifier
# from wcr_core.modules.bus.controller.wbctl_controllers import \
#     WBCTL_CONTROLLER_FACTORY
from wcr_core.modules.bus.db.wbhdb_operator import WBHDB_OPER, WbhdbOperator
from wcr_core.modules.bus.db.wbhdb_standard_operators import \
    WBHDB_DEFAULT_OPS
from wcr_core.modules.wmod_application_module import \
    WMOD_DEFAULT_BLOCK_TYPES, WmodApplicationModuleBase, \
    WmodApplicationModuleP, WmodApplicationModuleMP
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_BUS
from wcr_core.db_api.whdb_block_api import WHDB_TOPLEVEL_BLOCK_CODE, \
    WHDB_DEFAULT_BLOCK_CODE
from wcr_core.modules.bus.db.wbhdb_top_block import WbhdbTopHierarchy
from wcr_core.modules.wmod_block_type import WmodBlockType, \
    WMOD_SYSTEM_BLOCK_FLAG
from wcr_core.modules.bus.wbmod_project_context import WbmodBusProjectContext

WBMOD_BUS_APP_CODE = 'bus'

WBMOD_BLOCK_TYPES = [
    WmodBlockType(WHDB_TOPLEVEL_BLOCK_CODE, 'Top-level',
                  WbhdbTopHierarchy.META, WbhdbTopHierarchy,
                  WMOD_SYSTEM_BLOCK_FLAG)]

# noinspection PyAbstractClass
class WbmodBusApplicationModuleP(WmodApplicationModuleP):
    pass


# noinspection PyAbstractClass
class WbmodBusApplicationModuleMP(WmodApplicationModuleMP,
                                  WbmodBusApplicationModuleP):
    pass


class _WbmodBusApplicationModule(WmodApplicationModuleBase,
                                 WbmodBusApplicationModuleMP):
    def __init__(self, operRegistry):
        super().__init__(
            code=WBMOD_BUS_APP_CODE,
            name='Bus View',
            blockTypes=WBMOD_BLOCK_TYPES,
            defaultOps=WBHDB_DEFAULT_OPS,
            operRegistry=operRegistry)
        #: Distribution service types
        self.__distribSvcTypes = {}
        """:type: dict[str, WpmodDistributionServiceTypeP]"""

        #: Connection point types
        self.__xpointTypes = {}
        """:type: dict[str, WpmodConnectionPointTypeP]"""



    def createHierarchyVerifier(self, pctx):
        return WbhdbHierarchyVerifier(self._getDbVerifierModules())

    def getPayloadType(self):
        return WCMN_PAYLOAD_BLOCK_BUS

    def createTopLevelBlock(self, name=None, info=None):
        blockMetas = {b.getCode(): b.getMeta() for b in self.getBlockTypes()}
        blockClzzs = {b.getCode(): b.getClass() for b in self.getBlockTypes()}
        xpntClzzs = {x.getCode(): x.getClass() for x
                     in self.getConnectionPointTypes()}
        """:type: dict[str, () -> WphdbConnectionPointBase]"""
        xpntCfgMetas = {x.getCode(): x.getConfigMeta() for x
                        in self.getConnectionPointTypes()}
        """:type: dict[str, WdbsMetaTypeP]"""
        return wbmod_create_top_block(blockMetas, blockClzzs,
                                      xpntCfgMetas, xpntClzzs,
                                      name=name, info=info)

    def createProjectContext(self, gContext, projectDB, logger=None):
        return WbmodBusProjectContext(gContext, self, projectDB, logger)#,
                                      # WBCTL_CONTROLLER_FACTORY)

    def addDistributionServiceType(self, dsvc):
        self.__distribSvcTypes[dsvc.getCode()] = dsvc

    def getDistributionServiceType(self, code):
        return self.__distribSvcTypes.get(code)

    def getDistributionServiceTypes(self):
        return self.__distribSvcTypes.values()

    def addConnectionPointType(self, xpntType):
        self.__xpointTypes[xpntType.getCode()] = xpntType

    def getConnectionPointType(self, code):
        return self.__xpointTypes.get(code)

    def getConnectionPointTypes(self):
        return self.__xpointTypes.values()

    def getFileExtention(self):
        return '.bus'

def wbmod_create_top_block(blockMetas, blockClzzs,
                           xpntCfgMetas, xpntClzzs,
                           name=None, info=None):

    tblk = whdb_createTopLevelRawBlock(WbhdbMetaProvider(blockMetas,
                                                         xpntCfgMetas),
                                       name, info)
    rgty = WbhdbWrapperRegistry(blockClzzs, xpntClzzs)
    return rgty.createWrapper(tblk)

#: HDS Application module singleton
WBMOD_BUS_APP_MODULE = _WbmodBusApplicationModule(WBHDB_OPER)
