# ----------------------------------------------------------------------------
# - whctl-controllers                                                        -
# - wild cruncher: hardware controllers         core / hardware / controller -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whctl_controllers
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Model controllers

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""

from wcr_core.controller.wctl_controller_mixins import *
from wcr_core.controller.wctl_default_controllers import \
    WctlSimpleControllerFactoryBase
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WhhdbHardwareXPoint


class WhctlControllerBase(WctlControllerP):
    def __init__(self, fcty):
        """
        :param fcty: Controller factory
        :type fcty: WbctlControllerFactory
        """
        self.__factory = fcty

    @property
    def factory(self):
        return self.__factory


class WhctlConnectionController(WhctlControllerBase,
                                WctlConnectionControllerDefaultCreateMixin,
                                WctlConnectionControllerDefaultDeleteMixin,
                                WctlConnectionControllerP):


    def __init__(self, fcty):
        """
    :param fcty: Controller factory
    :type fcty: WpctlControllerFactory
    """
        super().__init__(fcty)

    def canCreateConnection(self, block, sinst, einst, sxpnt=None, expnt=None,
                            **kwargs):
        """"
        :type block: WhdbBlock
        :type sinst: WhdbInstanceP
        :type einst: WhdbInstanceP
        :type sxpnt: WhdbConnectionPointP | None
        :type expnt: WhdbConnectionPointP | None
        :rtype: bool
        """
        if not sxpnt or not expnt:
            return False
        if not isinstance(sxpnt,WhhdbHardwareXPoint):
            return False
        if not isinstance(expnt, WhhdbHardwareXPoint):
            return False

        if sxpnt == expnt:
            return False

        return super().canCreateConnection(block, sinst, einst,
                                           sxpnt=sxpnt, expnt=expnt,
                                           **kwargs)

class WhctlControllerFactory(WctlSimpleControllerFactoryBase):
    def __init__(self):
        super().__init__(connCtlr=WhctlConnectionController(self))

#: Model controller factory
WHCTL_CONTROLLER_FACTORY = WhctlControllerFactory()
