# ----------------------------------------------------------------------------
# - whhdb-connection-point-base-meta                                         -
# - wild cruncher: hardware connection point meta-data      core/hardware/db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_connection_point_base_meta
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - hardware connection-point-base meta-data

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_core.db.whdb_parameter import *
from wcr_core.dbstore.wdbs_meta_type import *

WHHDB_PORT_SPEED_KEY = 'speed'
WHHDB_TYPE_KEY = 'type'
WHHDB_PORT_NUM_KEY = 'PNum'
WHHDB_PORT_BBONE_KEY = 'BackBone'
WHHDB_PORT_DIRECTION_KEY = 'Direction'
WHHDB_PORT_STATUS_MASK_KEY = 'StatusMask'
WHHDB_PORT_STATUS_MONITOR_KEY = 'StatusMonitor'
WHHDB_PORT_CARD_KEY = 'cardName'
WHHDB_PORT_ACTIVE_KEY = 'active'
WHHDB_PORT_PTP_ACTIVE_KEY = 'ptpActive'

CONF_KEYS = [WHHDB_PORT_SPEED_KEY, WHHDB_TYPE_KEY, WHHDB_PORT_NUM_KEY,
             WHHDB_PORT_BBONE_KEY, WHHDB_PORT_DIRECTION_KEY,
             WHHDB_PORT_STATUS_MONITOR_KEY, WHHDB_PORT_CARD_KEY,
             WHHDB_PORT_ACTIVE_KEY, WHHDB_PORT_STATUS_MASK_KEY,
             WHHDB_PORT_PTP_ACTIVE_KEY]

WHHDB_PORT_SPEED_TAG = 'Bandwidth (/s)'
WHHDB_PORT_SPEED_PARAMETER = (WHHDB_PORT_SPEED_KEY, WDBS_STRING_META,
                              WHHDB_PORT_SPEED_TAG)

WHHDB_PORT_STATUS_MONITOR_TAG = 'Status Monitor'
WHHDB_STATUS_MONITOR_PARAMETER = (WHHDB_PORT_STATUS_MONITOR_KEY, WDBS_STRING_META,
                                  WHHDB_PORT_STATUS_MONITOR_TAG)

WHHDB_PORT_PTP_ACTIVE_TAG = 'Ptp Active'
WHHDB_PTP_ACTIVE_PARAMETER = (WHHDB_PORT_PTP_ACTIVE_KEY, WDBS_STRING_META,
                              WHHDB_PORT_PTP_ACTIVE_TAG)

WHHDB_PORT_TYPE_TAG = 'Port Type'
WHHDB_TYPE_PARAMETER = (WHHDB_TYPE_KEY, WDBS_STRING_META, WHHDB_PORT_TYPE_TAG)

WHHDB_PORT_NUMB_TAG = 'Port Number'
WHHDB_PORT_NUMBER_PARAMETER = (WHHDB_PORT_NUM_KEY, WDBS_INTEGER_NONE_META,
                               WHHDB_PORT_NUMB_TAG)

WHHDB_PORT_BBONE_TAG = 'Is Back Bone'
WHHDB_PORT_BBONE_PARAMETER = (WHHDB_PORT_BBONE_KEY, WDBS_STRING_META,
                           WHHDB_PORT_BBONE_TAG)

WHHDB_PORT_CARD_PARAMETER = (WHHDB_PORT_CARD_KEY, WDBS_STRING_META,
                             WHHDB_PORT_CARD_KEY)

WHHDB_PORT_ACTIVE_PARAMETER = (WHHDB_PORT_ACTIVE_KEY, WDBS_STRING_META,
                             WHHDB_PORT_ACTIVE_KEY)

WHHDB_PORT_STATUS_MASK_TAG = 'StatusMask'
WHHDB_PORT_STATUS_MASK_PARAMETER = (WHHDB_PORT_STATUS_MASK_KEY, WDBS_STRING_META,
                                    WHHDB_PORT_STATUS_MASK_TAG)

# Conf port data
WHHDB_PORT_CONF_TAG = 'Port Conf'
WHHDB_PORT_CONF_KEY = 'conf'
WHHDB_PORT_CONF_PARAMETER = (WHHDB_PORT_CONF_KEY, WDBS_STRING_META,
                             WHHDB_PORT_CONF_TAG)

WHHDB_PORT_CETRAC_CONF_TAG = 'CetraC Conf'
WHHDB_PORT_CETRAC_CONF_KEY = 'CetraCConf'
WHHDB_PORT_CETRAC_CONF_PARAMETER = (WHHDB_PORT_CETRAC_CONF_KEY, WDBS_STRING_META,
                                    WHHDB_PORT_CETRAC_CONF_TAG)
# AFDX DATA
WHHDB_PORT_AFDX_DELAY_TAG = 'max Delay'
WHHDB_PORT_AFDX_DELAY_KEY = 'maxDelay'
WHHDB_PORT_AFDX_DELAY_PARAMETER = (WHHDB_PORT_AFDX_DELAY_KEY, WDBS_STRING_META,
                                   WHHDB_PORT_AFDX_DELAY_TAG)

WHHDB_PORT_AFDX_HPBS_TAG = 'high Priority Buffer Size'
WHHDB_PORT_AFDX_HPBS_KEY = 'highPriorityBufferSize'
WHHDB_PORT_AFDX_HPBS_PARAMETER = (WHHDB_PORT_AFDX_HPBS_KEY, WDBS_STRING_META,
                                  WHHDB_PORT_AFDX_HPBS_TAG)

WHHDB_PORT_AFDX_LPBS_TAG = 'low Priority Buffer Size'
WHHDB_PORT_AFDX_LPBS_KEY = 'lowPriorityBufferSize'
WHHDB_PORT_AFDX_LPBS_PARAMETER = (WHHDB_PORT_AFDX_LPBS_KEY, WDBS_STRING_META,
                                  WHHDB_PORT_AFDX_LPBS_TAG)

WHHDB_PORT_AFDX_NET_ID_TAG = 'network Id'
WHHDB_PORT_AFDX_NET_ID_KEY = 'networkId'
WHHDB_PORT_AFDX_NET_ID_PARAMETER = (WHHDB_PORT_AFDX_NET_ID_KEY, WDBS_STRING_META,
                                    WHHDB_PORT_AFDX_NET_ID_TAG)

WHHDB_PORT_AFDX_INTERFACE_ID_TAG = 'interface Id'
WHHDB_PORT_AFDX_INTERFACE_ID_KEY = 'interFaceId'
WHHDB_PORT_AFDX_INTERFACE_ID_PARAMETER = (WHHDB_PORT_AFDX_INTERFACE_ID_KEY, WDBS_STRING_META,
                                          WHHDB_PORT_AFDX_INTERFACE_ID_TAG)

WHHDB_PORT_AFDX_SIDE_ID_TAG = 'side Id'
WHHDB_PORT_AFDX_SIDE_ID_KEY = 'sideId'
WHHDB_PORT_AFDX_SIDE_ID_PARAMETER = (WHHDB_PORT_AFDX_SIDE_ID_KEY, WDBS_STRING_META,
                                     WHHDB_PORT_AFDX_SIDE_ID_TAG)

WHHDB_PORT_AFDX_LOCATION_ID_TAG = 'location Id'
WHHDB_PORT_AFDX_LOCATION_ID_KEY = 'locationId'
WHHDB_PORT_AFDX_LOCATION_ID_PARAMETER = (WHHDB_PORT_AFDX_LOCATION_ID_KEY, WDBS_STRING_META,
                                         WHHDB_PORT_AFDX_LOCATION_ID_TAG)
# DISCRETE DATA
WHHDB_PORT_DIS_EDMI_TAG = 'Event data mask in'
WHHDB_PORT_DIS_EDMI_KEY = 'eventDataMaskIn'
WHHDB_PORT_DIS_EDMI_PARAMETER = (WHHDB_PORT_DIS_EDMI_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_DIS_EDMI_TAG)

WHHDB_PORT_DIS_EDMO_TAG = 'Event data mask out'
WHHDB_PORT_DIS_EDMO_KEY = 'eventDataMaskOut'
WHHDB_PORT_DIS_EDMO_PARAMETER = (WHHDB_PORT_DIS_EDMO_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_DIS_EDMO_TAG)

WHHDB_PORT_DIS_CDMI_TAG = 'Cyclic data mask in'
WHHDB_PORT_DIS_CDMI_KEY = 'cyclicDataMaskIn'
WHHDB_PORT_DIS_CDMI_PARAMETER = (WHHDB_PORT_DIS_CDMI_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_DIS_CDMI_TAG)

WHHDB_PORT_DIS_CDMO_TAG = 'Cyclic data mask out'
WHHDB_PORT_DIS_CDMO_KEY = 'cyclicDataMaskOut'
WHHDB_PORT_DIS_CDMO_PARAMETER = (WHHDB_PORT_DIS_CDMO_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_DIS_CDMO_TAG)

WHHDB_PORT_DIS_BUFFER_TAG = 'Nb Buffer'
WHHDB_PORT_DIS_BUFFER_KEY = 'nbBuffer'
WHHDB_PORT_DIS_BUFFER_PARAMETER = (WHHDB_PORT_DIS_BUFFER_KEY, WDBS_STRING_META,
                                   WHHDB_PORT_DIS_BUFFER_TAG)

WHHDB_PORT_DIS_ID_TAG = 'Producer id'
WHHDB_PORT_DIS_ID_KEY = 'producerId'
WHHDB_PORT_DIS_ID_PARAMETER = (WHHDB_PORT_DIS_ID_KEY, WDBS_STRING_META,
                               WHHDB_PORT_DIS_ID_TAG)

WHHDB_PORT_DIC_CYC_LEN_TAG = 'Cycle length'
WHHDB_PORT_DIS_CYC_LEN_KEY = 'cycleLength'
WHHDB_PORT_DIS_CYC_LEN_PARAMETER = (WHHDB_PORT_DIS_CYC_LEN_KEY, WDBS_STRING_META,
                                    WHHDB_PORT_DIC_CYC_LEN_TAG)

# FIBER CHANNEL DATA
WHHDB_PORT_FB_MFL_TAG = 'Max frame length'
WHHDB_PORT_FB_MFL_KEY = 'maxFrameLength'
WHHDB_PORT_FB_MFL_PARAMETER = (WHHDB_PORT_FB_MFL_KEY, WDBS_STRING_META,
                               WHHDB_PORT_FB_MFL_TAG)

WHHDB_PORT_FB_IPCONF_TAG = 'Ip config'
WHHDB_PORT_FB_IPCONF_KEY = 'ipConfiguration'
WHHDB_PORT_FB_IPCONF_PARAMETER = (WHHDB_PORT_FB_IPCONF_KEY, WDBS_STRING_META,
                                  WHHDB_PORT_FB_IPCONF_TAG)

WHHDB_PORT_FB_FRFT_TAG = 'Fifo receive full treshold'
WHHDB_PORT_FB_FRFT_KEY = 'fifoReceiveFullTreshold'
WHHDB_PORT_FB_FRFT_PARAMETER = (WHHDB_PORT_FB_FRFT_KEY, WDBS_STRING_META,
                                WHHDB_PORT_FB_FRFT_TAG)

WHHDB_PORT_FB_FTFT_TAG = 'Fifo transmit full treshold'
WHHDB_PORT_FB_FTFT_KEY = 'fifoTransmitFullTreshold'
WHHDB_PORT_FB_FTFT_PARAMETER = (WHHDB_PORT_FB_FTFT_KEY, WDBS_STRING_META,
                                WHHDB_PORT_FB_FTFT_TAG)

WHHDB_PORT_FB_FTET_TAG = 'Fifo transmit empty treshold'
WHHDB_PORT_FB_FTET_KEY = 'fifoTransmitEmptyTreshold'
WHHDB_PORT_FB_FTET_PARAMETER = (WHHDB_PORT_FB_FTET_KEY, WDBS_STRING_META,
                                WHHDB_PORT_FB_FTET_TAG)

WHHDB_PORT_FB_FRAET_TAG = 'Fifo receive almost empty treshold'
WHHDB_PORT_FB_FRAET_KEY = 'fifoReceiveAlmostEmptyTreshold'
WHHDB_PORT_FB_FRAET_PARAMETER = (WHHDB_PORT_FB_FRAET_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_FB_FRAET_TAG)

WHHDB_PORT_FB_FRAFT_TAG = 'Fifo receive almost full treshold'
WHHDB_PORT_FB_FRAFT_KEY = 'fifoReceiveAlmostFullTreshold'
WHHDB_PORT_FB_FRAFT_PARAMETER = (WHHDB_PORT_FB_FRAFT_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_FB_FRAFT_TAG)

WHHDB_PORT_FB_FTAET_TAG = 'Fifo transmit almost empty treshold'
WHHDB_PORT_FB_FTAET_KEY = 'fifoTransmitAlmostEmptyTreshold'
WHHDB_PORT_FB_FTAET_PARAMETER = (WHHDB_PORT_FB_FTAET_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_FB_FTAET_TAG)

WHHDB_PORT_FB_FTAFT_TAG = 'Fifo transmit almost full treshold'
WHHDB_PORT_FB_FTAFT_KEY = 'fifoTransmitAlmostFullTreshold'
WHHDB_PORT_FB_FTAFT_PARAMETER = (WHHDB_PORT_FB_FTAFT_KEY, WDBS_STRING_META,
                                 WHHDB_PORT_FB_FTAFT_TAG)

WHHDB_PORT_FB_ITV_TAG = 'Initialization timer value'
WHHDB_PORT_FB_ITV_KEY = 'initializationTimerValue'
WHHDB_PORT_FB_ITV_PARAMETER = (WHHDB_PORT_FB_ITV_KEY, WDBS_STRING_META,
                               WHHDB_PORT_FB_ITV_TAG)

WHHDB_PORT_FB_ETV_TAG = 'Event timeout value'
WHHDB_PORT_FB_ETV_KEY = 'eventTimeoutValue'
WHHDB_PORT_FB_ETV_PARAMETER = (WHHDB_PORT_FB_ETV_KEY, WDBS_STRING_META,
                               WHHDB_PORT_FB_ETV_TAG)

# 1553 DATA
WHHDB_PORT_1553_ID_TAG = 'Producer id'
WHHDB_PORT_1553_ID_KEY = 'prodId'
WHHDB_PORT_1553_ID_PARAMETER = (WHHDB_PORT_1553_ID_KEY, WDBS_STRING_META,
                                WHHDB_PORT_1553_ID_TAG)

WHHDB_PORT_1553_SEQUENCE_TAG = 'Sequence number'
WHHDB_PORT_1553_SEQUENCE_KEY = 'sequenceNumber'
WHHDB_PORT_1553_SEQUENCE_PARAMETER = (WHHDB_PORT_1553_SEQUENCE_KEY, WDBS_STRING_META,
                                      WHHDB_PORT_1553_SEQUENCE_TAG)

WHHDB_PORT_1553_CYCLE_TAG = 'Clock cycle'
WHHDB_PORT_1553_CYCLE_KEY = 'clockCycle'
WHHDB_PORT_1553_CYCLE_PARAMETER = (WHHDB_PORT_1553_CYCLE_KEY, WDBS_STRING_META,
                                   WHHDB_PORT_1553_CYCLE_TAG)

# ARINC825 DATA
WHHDB_PORT_ARINC825_TYPE_TAG = 'Can Type '
WHHDB_PORT_ARINC825_TYPE_KEY = 'canType'
WHHDB_PORT_ARINC825_TYPE_PARAMETER = (WHHDB_PORT_ARINC825_TYPE_KEY, WDBS_STRING_META,
                                      WHHDB_PORT_ARINC825_TYPE_TAG)

WHHDB_PORT_ARINC825_SPEED_TAG = 'Baud Rate (k/s)'
WHHDB_PORT_ARINC825_SPEED_KEY = 'speed '
WHHDB_PORT_ARINC825_SPEED_PARAMETER = (WHHDB_PORT_ARINC825_SPEED_KEY, WDBS_STRING_META,
                                       WHHDB_PORT_ARINC825_SPEED_TAG)

WHHDB_PORT_ARINC825_USER_SPEED_TAG = 'User Baud Rate'
WHHDB_PORT_ARINC825_USER_SPEED_KEY = 'userSpeed'
WHHDB_PORT_ARINC825_USER_SPEED_PARAMETER = (WHHDB_PORT_ARINC825_USER_SPEED_KEY, WDBS_STRING_META,
                                            WHHDB_PORT_ARINC825_USER_SPEED_TAG)

WHHDB_PORT_ARINC825_ARBITRATION_SPEED_TAG = 'Arbitration Baud Rate (FD) '
WHHDB_PORT_ARINC825_ARBITRATION_SPEED_KEY = 'arbitBaudRate'
WHHDB_PORT_ARINC825_ARBITRATION_SPEED_PARAMETER = (WHHDB_PORT_ARINC825_ARBITRATION_SPEED_KEY, WDBS_STRING_META,
                                                   WHHDB_PORT_ARINC825_ARBITRATION_SPEED_TAG)

WHHDB_PORT_ARINC825_DATA_SPEED_TAG = 'Data Baud Rate (FD) '
WHHDB_PORT_ARINC825_DATA_SPEED_KEY = 'dataBaudRate'
WHHDB_PORT_ARINC825_DATA_SPEED_PARAMETER = (WHHDB_PORT_ARINC825_DATA_SPEED_KEY, WDBS_STRING_META,
                                            WHHDB_PORT_ARINC825_DATA_SPEED_TAG)


WHHDB_PORT_ARINC825_CYCLIC_TAG = 'Cyclic Mode'
WHHDB_PORT_ARINC825_CYCLIC_KEY = 'cyclicMode'
WHHDB_PORT_ARINC825_CYCLIC_PARAMETER = (WHHDB_PORT_ARINC825_CYCLIC_KEY, WDBS_STRING_META,
                                        WHHDB_PORT_ARINC825_CYCLIC_TAG)

WHHDB_PORT_ARINC825_LISTEN_MODE_TAG = 'Listen Only Mode'
WHHDB_PORT_ARINC825_LISTEN_MODE_KEY = 'listenOnlyMode'
WHHDB_PORT_ARINC825_LISTEN_MODE_PARAMETER = (WHHDB_PORT_ARINC825_LISTEN_MODE_KEY, WDBS_STRING_META,
                                             WHHDB_PORT_ARINC825_LISTEN_MODE_TAG)

WHHDB_PORT_ARINC825_MAX_FRAME_TAG = 'Max nb Frame'
WHHDB_PORT_ARINC825_MAX_FRAME_KEY = 'maxNbFrame'
WHHDB_PORT_ARINC825_MAX_FRAME_PARAMETER = (WHHDB_PORT_ARINC825_MAX_FRAME_KEY, WDBS_STRING_META,
                                           WHHDB_PORT_ARINC825_MAX_FRAME_TAG)

WHHDB_PORT_ARINC825_TIMEOUT_TAG = 'Timeout'
WHHDB_PORT_ARINC825_TIMEOUT_KEY = 'timeout'
WHHDB_PORT_ARINC825_TIMEOUT_PARAMETER = (WHHDB_PORT_ARINC825_TIMEOUT_KEY, WDBS_STRING_META,
                                         WHHDB_PORT_ARINC825_TIMEOUT_TAG)

WHHDB_AFDX_KEYS = [WHHDB_PORT_AFDX_DELAY_KEY, WHHDB_PORT_AFDX_HPBS_KEY,
                   WHHDB_PORT_AFDX_LPBS_KEY, WHHDB_PORT_AFDX_LPBS_KEY,
                   WHHDB_PORT_AFDX_NET_ID_KEY,
                   WHHDB_PORT_AFDX_INTERFACE_ID_KEY, WHHDB_PORT_AFDX_SIDE_ID_KEY,
                   WHHDB_PORT_AFDX_LOCATION_ID_KEY]

WHHDB_DISCRET_KEYS = [WHHDB_PORT_DIS_EDMI_KEY, WHHDB_PORT_DIS_CDMI_KEY,
                      WHHDB_PORT_DIS_EDMO_KEY, WHHDB_PORT_DIS_CDMO_KEY,
                      WHHDB_PORT_DIS_BUFFER_KEY, WHHDB_PORT_DIS_ID_KEY,
                      WHHDB_PORT_DIS_CYC_LEN_KEY]

WHHDB_FIBER_KEYS = []

WHHDB_1553_KEYS = [WHHDB_PORT_1553_ID_KEY, WHHDB_PORT_1553_SEQUENCE_KEY,
                   WHHDB_PORT_1553_CYCLE_KEY]

WHHDB_ARINC825_KEYS = [WHHDB_PORT_ARINC825_TYPE_KEY, WHHDB_PORT_ARINC825_SPEED_KEY,
                       WHHDB_PORT_ARINC825_USER_SPEED_KEY, WHHDB_PORT_ARINC825_LISTEN_MODE_KEY,
                       WHHDB_PORT_CETRAC_CONF_KEY, WHHDB_PORT_ARINC825_CYCLIC_KEY,
                       WHHDB_PORT_ARINC825_MAX_FRAME_KEY, WHHDB_PORT_ARINC825_TIMEOUT_KEY,
                       WHHDB_PORT_ARINC825_ARBITRATION_SPEED_KEY, WHHDB_PORT_ARINC825_DATA_SPEED_KEY]
