# ----------------------------------------------------------------------------
# - wbhdb-connection-points                                                  -
# - wild cruncher: connection points                         core / bus / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_connection_points
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Hardware connection points

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_TYPE, \
    WHDB_XPOINT_DIRECTION
from wcr_core.dbstore.wdbs_meta_type import WdbsRecordMetaType, \
    WDBS_STRING_META
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.hardware.db.whhdb_connection_point_base import \
    WhhdbConnectionPointBase, WHHDB_XPOINT_DISCRET_KEY, WHHDB_XPOINT_FIBER_CHANNEL_KEY,\
    WHHDB_XPOINT_1553_KEY, WHHDB_XPOINT_ARINC825_KEY
from wcr_core.modules.hardware.db.whhdb_connection_point_base_meta import *


WHHDB_XPOINT_AFDX_CODE = 'AFDX'
WHHDB_XPOINT_ETHERNET_CODE = 'Ethernet'
WHHDB_XPOINT_CETRAC_CODE = 'CetraC'
WHHDB_XPOINT_DISCRET_CODE = 'Discrete'
WHHDB_XPOINT_FIBER_CHANNEL_CODE = 'FiberChannel'
WHHDB_XPOINT_1553_CODE = '1553'
WHHDB_XPOINT_ARINC825_CODE = 'ARINC825'
WHHDB_XPOINT_DEFAULT_CODE = 'Undefined'

WHHDB_XPOINT_NAMES = [WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_ETHERNET_CODE,
                      WHHDB_XPOINT_CETRAC_CODE]

WHHDB_MODULES_IO_NAMES = [WHHDB_XPOINT_DISCRET_CODE, WHHDB_XPOINT_FIBER_CHANNEL_CODE,
                          WHHDB_XPOINT_1553_CODE, WHHDB_XPOINT_ARINC825_CODE]

WHHDB_XPOINT_CONFIG_META = WdbsRecordMetaType(
    WHHDB_PORT_NUMBER_PARAMETER,
    WHHDB_STATUS_MONITOR_PARAMETER,
    WHHDB_TYPE_PARAMETER,
    WHHDB_PORT_SPEED_PARAMETER,
    WHHDB_PORT_BBONE_PARAMETER,
    WHHDB_PORT_CARD_PARAMETER,
    WHHDB_PORT_ACTIVE_PARAMETER,
    WHHDB_PORT_STATUS_MASK_PARAMETER,
    WHHDB_PTP_ACTIVE_PARAMETER,
    )

WHHDB_XPOINT_CONFIG_DIS_META = WdbsRecordMetaType(
    WHHDB_PORT_DIS_EDMI_PARAMETER,
    WHHDB_PORT_DIS_EDMO_PARAMETER,
    WHHDB_PORT_DIS_CDMI_PARAMETER,
    WHHDB_PORT_DIS_CDMO_PARAMETER,
    WHHDB_PORT_DIS_BUFFER_PARAMETER,
    WHHDB_PORT_DIS_ID_PARAMETER,
    WHHDB_PORT_DIS_CYC_LEN_PARAMETER
    )

WHHDB_XPOINT_CONFIG_FB_META = WdbsRecordMetaType()

WHHDB_XPOINT_CONFIG_AFDX_META = WdbsRecordMetaType(
    WHHDB_PORT_AFDX_DELAY_PARAMETER,
    WHHDB_PORT_AFDX_HPBS_PARAMETER,
    WHHDB_PORT_AFDX_LPBS_PARAMETER,
    WHHDB_PORT_AFDX_NET_ID_PARAMETER,
    WHHDB_PORT_AFDX_INTERFACE_ID_PARAMETER,
    WHHDB_PORT_AFDX_SIDE_ID_PARAMETER,
    WHHDB_PORT_AFDX_LOCATION_ID_PARAMETER)

WHHDB_XPOINT_CONFIG_1553_META = WdbsRecordMetaType(
    WHHDB_PORT_1553_ID_PARAMETER,
    WHHDB_PORT_1553_SEQUENCE_PARAMETER,
    WHHDB_PORT_1553_CYCLE_PARAMETER,)

WHHDB_XPOINT_CONFIG_ARINC825_META = WdbsRecordMetaType(
    WHHDB_PORT_ARINC825_TYPE_PARAMETER,
    WHHDB_PORT_ARINC825_SPEED_PARAMETER,
    WHHDB_PORT_ARINC825_USER_SPEED_PARAMETER,
    WHHDB_PORT_ARINC825_ARBITRATION_SPEED_PARAMETER,
    WHHDB_PORT_ARINC825_DATA_SPEED_PARAMETER,
    WHHDB_PORT_ARINC825_LISTEN_MODE_PARAMETER,
    WHHDB_PORT_CETRAC_CONF_PARAMETER,
    WHHDB_PORT_ARINC825_CYCLIC_PARAMETER,
    WHHDB_PORT_ARINC825_MAX_FRAME_PARAMETER,
    WHHDB_PORT_ARINC825_TIMEOUT_PARAMETER)

WHHDB_XPOINT_DFLT_ARGS = {
    WHDB_XPOINT_DIRECTION: WDBS_XPNT_BI,
    WHHDB_XPOINT_DISCRET_KEY: WHHDB_XPOINT_CONFIG_DIS_META,
    WHHDB_XPOINT_FIBER_CHANNEL_KEY: WHHDB_XPOINT_CONFIG_FB_META,
    WHHDB_XPOINT_1553_KEY: WHHDB_XPOINT_CONFIG_1553_META,
    WHHDB_XPOINT_ARINC825_KEY: WHHDB_XPOINT_CONFIG_ARINC825_META,
}


class WhhdbHardwareXPoint(WhhdbConnectionPointBase):
    """
    Standart connection point.
    """
    def __init__(self, xpoint, wreg, code=None):
        super().__init__(xpoint, wreg)
        if code:
            self.setCode(code)
        if self.getCode() in (WHHDB_XPOINT_NAMES + WHHDB_MODULES_IO_NAMES):
            tnam = self.getCode()
        else:
            raise Exception('Connector type is unknown !')
        #: Connection point type name
        self.__xpntTypeName = tnam
        #: Connection point name prefix
        self.__prefix = tnam[:4]

    def getInterface(self):
        return self._getConfig(WHHDB_TYPE_KEY)

    def getNamePrefix(self):
        return self.__prefix

    def getTypeName(self):
        return '{} connection point'.format(self.__xpntTypeName)

    def getDescription(self, **kwargs):
        return '{n} ({t})'.format(n=self.getName(), t=self.getCode())

    def getSpeed(self):
        return self._getConfig(WHHDB_PORT_SPEED_KEY)

    def setSpeed(self, debit):
        self._setConfig(debit, WHHDB_PORT_SPEED_KEY)

    def getCard(self):
        card = self._getConfig(WHHDB_PORT_CARD_KEY)
        if '.' in card:
            card, _ = card.split('.')
            return card
        else:
            return card

    def getIoCard(self):
        card = self._getConfig(WHHDB_PORT_CARD_KEY)
        if '.' in card:
            _, moduleIo = card.split('.')
            return moduleIo
        else:
            raise Exception('No IoCard linked to this port.')

    def setCard(self, debit):
        self._setConfig(debit, WHHDB_PORT_CARD_KEY)

    def getActive(self):
        return self._getConfig(WHHDB_PORT_ACTIVE_KEY)

    def setActive(self, state):
        self._setConfig(state, WHHDB_PORT_ACTIVE_KEY)

    def getCode(self):
        return self._getConfig(WHHDB_TYPE_KEY)

    def setCode(self, code):
        self._setConfig(code, WHHDB_TYPE_KEY)

    def getPortNum(self):
        return self._getConfig(WHHDB_PORT_NUM_KEY)

    def setPortNum(self, num):
        self._setConfig(num, WHHDB_PORT_NUM_KEY)

    def getStatusMask(self):
        return self._getConfig(WHHDB_PORT_STATUS_MASK_KEY)

    def setStatusMask(self, stMask):
        self._setConfig(stMask, WHHDB_PORT_STATUS_MASK_KEY)

    def getStatusMonitor(self):
        return self._getConfig(WHHDB_PORT_STATUS_MONITOR_KEY)

    def setStatusMonitor(self, stMonitor):
        self._setConfig(stMonitor, WHHDB_PORT_STATUS_MONITOR_KEY)

    def getPtpActive(self):
        return self._getConfig(WHHDB_PORT_PTP_ACTIVE_KEY)

    def setPtpActve(self, ptpActive):
        self._setConfig(ptpActive, WHHDB_PORT_PTP_ACTIVE_KEY)

    def isBackBone(self):
        return self._getConfig(WHHDB_PORT_BBONE_KEY)

    def setBackBone(self, bb):
        self._setConfig(bb, WHHDB_PORT_BBONE_KEY)