# ----------------------------------------------------------------------------
# - whhdb-instance                                                           -
# - wild cruncher:wsh hardware instance object          core / hardware / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_instance
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher hardware instance

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------
from abc import abstractmethod

from wcr_core.modules.hardware.db.whhdb_instance_base import WhhdbInstanceBaseP


# ----------------------------------------------------------------------------
# - instance class                                                           -
# ----------------------------------------------------------------------------


# noinspection PyAbstractClass
class WhhdbInstance(WhhdbInstanceBaseP):
    """
    The WmhdbInstance class is the Silkan hierarchy model instance. An
    instance is a binding between a name and a model. The instance name is
    unique and can be optionally associated with an information string. The
    instance connectors are associated with the model connectors by port id.
    """

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsInstanceP
        :param wreg: Raw element wrapper registry
        :type wreg: WmhdbWrapperRegistry
        """
        super().__init__(inst, wreg)

        self.__rawInstance = inst
        self.__wrapperRegistry = wreg

    def clearConnectors(self):
        """
        Remove all connectors.
        """
        # noinspection PyUnresolvedReferences
        self.clearConnectionPoints()

    def setConnectorOrder(self, orderF):
        """
        Reorder the list of connectors based on the given ordering function.

        :param orderF: Ordering function
        :type orderF: (WmhdbConnector) -> object
        """
        # noinspection PyTypeChecker
        self.setConnectionPointOrder(orderF)

    @abstractmethod
    def getPortName(self, port):
        raise NotImplementedError()
