# ----------------------------------------------------------------------------
# - whhdb-meta-provider                                                      -
# - wild cruncher: meta-data provider                   core / hardware / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_meta_provider
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy meta-data provider

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_meta_provider import WhdbMetaProvider
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaDescription, \
    WdbsMetaField
from wcr_core.modules.hardware.db.whhdb_generic_Host_instance import \
    WHHDB_CETRAC_RACK_IO_CODE
from wcr_core.modules.hardware.db.whhdb_Host_instance import WhhdbHostInstance
from wcr_core.modules.hardware.db.whhdb_iocard_instance import \
    WhhdbIoCardInstance
from wcr_core.modules.hardware.db.whhdb_rack_instance import WhhdbRackInstance
from wcr_core.modules.hardware.db.whhdb_Cetrac_switch_instance import \
    WhhdbCetracSwitchInstance
from wcr_core.modules.hardware.db.whhdb_kazymir_switch_instance import \
    WhhdbKazymirSwitchInstance
from wcr_core.modules.hardware.db.whhdb_standard_operators import \
    WHHDB_HOST_CODE, WHHDB_CETRAC_SWITCH_CODE, WHHDB_RACK_IO_CODE, \
    IO_CARDS_CODE, WHHDB_KAZYMIR_FIBER_CODE, WHHDB_KAZYMIR_CETRAC_CODE,\
    WHHDB_KAZYMIR_CODE
from wcr_core.modules.hardware.db.whhdb_connection_point_base import \
    WhhdbConnectionPointBase, WHHDB_XPOINT_CONFIG_KEY, \
    WHHDB_XPOINT_DISCRET_KEY, WHHDB_XPOINT_FIBER_CHANNEL_KEY, \
    WHHDB_XPOINT_AFDX_KEY, WHHDB_XPOINT_1553_KEY, WHHDB_XPOINT_ARINC825_KEY
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_CONFIG_FB_META, WHHDB_XPOINT_CONFIG_DIS_META, \
    WHHDB_XPOINT_CONFIG_AFDX_META, WHHDB_XPOINT_CONFIG_1553_META, \
    WHHDB_XPOINT_CONFIG_ARINC825_META

WHHDB_STD_OPERINST_META = {WHHDB_HOST_CODE: WhhdbHostInstance.META,
                           WHHDB_CETRAC_RACK_IO_CODE: WhhdbRackInstance.META,
                           WHHDB_RACK_IO_CODE: WhhdbRackInstance.META,
                           WHHDB_CETRAC_SWITCH_CODE:
                               WhhdbCetracSwitchInstance.META,
                           WHHDB_KAZYMIR_FIBER_CODE:
                               WhhdbKazymirSwitchInstance.META,
                           WHHDB_KAZYMIR_CETRAC_CODE:
                               WhhdbKazymirSwitchInstance.META,
                           WHHDB_KAZYMIR_CODE:
                               WhhdbKazymirSwitchInstance.META,
                           }


class WhhdbMetaProvider(WhdbMetaProvider):
    def __init__(self, xpntCfgMeta):
        super().__init__()


        #: Connection point configuration meta-data
        self.__xpntCfgMeta = xpntCfgMeta
        #: Connection point meta-data cache (incl. base and config)
        self.__xpntMeta = {}

    def getOperatorInstanceMeta(self, oper=None):
        if oper in IO_CARDS_CODE:
            opMeta = WhhdbIoCardInstance.META
        else:
            opMeta = WHHDB_STD_OPERINST_META.get(oper)
        return opMeta if opMeta is not None else WhdbOperatorInstance.META

    def getConnectionPointMeta(self, xtyp=None):
        if not xtyp:
            return WhhdbConnectionPointBase.META

        meta = self.__xpntMeta.get(xtyp)
        if meta:
            return meta

        cfgMeta = self.__xpntCfgMeta.get(xtyp)
        if not cfgMeta:
            return WhhdbConnectionPointBase.META

        meta = WdbsMetaDescription(
            WhhdbConnectionPointBase.META,
            WdbsMetaField(WHHDB_XPOINT_CONFIG_KEY, cfgMeta),
            WdbsMetaField(WHHDB_XPOINT_DISCRET_KEY,
                          WHHDB_XPOINT_CONFIG_DIS_META),
            WdbsMetaField(WHHDB_XPOINT_FIBER_CHANNEL_KEY,
                          WHHDB_XPOINT_CONFIG_FB_META),
            WdbsMetaField(WHHDB_XPOINT_AFDX_KEY,
                          WHHDB_XPOINT_CONFIG_AFDX_META),
            WdbsMetaField(WHHDB_XPOINT_1553_KEY,
                          WHHDB_XPOINT_CONFIG_1553_META),
            WdbsMetaField(WHHDB_XPOINT_ARINC825_KEY,
                          WHHDB_XPOINT_CONFIG_ARINC825_META),
        )

        self.__xpntMeta[xtyp] = meta
        return meta