# ----------------------------------------------------------------------------
# - whhdb-operator                                                           -
# - wild cruncher:wsh hardware operator object          core / hardware / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_operator
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher model function object

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------
from wcr_core.db.whdb_operator import WhdbOperatorRegistry, WhdbOperator


# ----------------------------------------------------------------------------
# - base operator class                                                      -
# ----------------------------------------------------------------------------


class WhhdbOperator(WhdbOperator):
    """
    The WcrOperator class is the silkan hierarchical operator object. An
    operator is defined by name and info. It is the operator name which
    defines its functionality. The operator contains an icon representation
    as well as a collection of connectors.
    """
    # TODO: Do not extend HasMediaP

    def __init__(self, code, xmngr, name=None, info=None, meta=None,
                 parameters=None, flags=None, icons=None, shape=None,
                 symbol=None, labelAlign=None):
        """
        Create a new operator by name and info

        :param code: Operator code
        :type code: str
        :param xmngr: Connector manager
        :type xmngr: WhdbConnectionPointManagerP
        :param name: the operator name
        :type name: str
        :param info: the operator info
        :type info: str
        :param meta: Operator meta data
        :type meta: dict[str, object]
        :param parameters: Operator parameters
                           (name, label, info, value, type, flags)
        :type parameters: list[(str, str, str, str|None, str|None, int|None)]
        :param flags: Flags
        :type flags: set[str]
        :param icons: Operator icon paths
        :type icons: dict[str, str]
        :param shape: Operator shape
        :type shape: list[(float, float)]
        :param symbol: Text symbol
        :type symbol: str
        :param labelAlign: Alignment for labels
        :type labelAlign: int
        """
        super().__init__(code, xmngr, name, info, meta, parameters,
                         flags, icons, shape, symbol, labelAlign)

#: Global HDS function operator registry
WHHDB_OPER = WhdbOperatorRegistry()
