# ----------------------------------------------------------------------------
# - whhdb-standard-operators                                                 -
# - wild cruncher:standard operators                    core / hardware / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_standard_operators
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher standard operators

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""


from wcr_core.db_api.whdb_protocols_api import WHDB_HALIGN_CENTER
from wcr_core.modules.hardware.db.whhdb_connection_point_base import \
    WHHDB_XPOINT_CONFIG_KEY, WHHDB_XPOINT_DISCRET_KEY, \
    WHHDB_XPOINT_FIBER_CHANNEL_KEY, WHHDB_XPOINT_AFDX_KEY, \
    WHHDB_XPOINT_1553_KEY, WHHDB_XPOINT_ARINC825_KEY

from wcr_core.modules.hardware.db.whhdb_connection_point_base_meta import \
    WHHDB_PORT_ARINC825_SPEED_KEY, WHHDB_PORT_ARINC825_LISTEN_MODE_KEY,\
    WHHDB_PORT_ARINC825_CYCLIC_KEY
from wcr_core.modules.hardware.db.whhdb_connection_point_manager import \
    WhhdbConnectionPointManager
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_CETRAC_CODE, WHHDB_XPOINT_DISCRET_CODE, \
    WHHDB_XPOINT_FIBER_CHANNEL_CODE, WHHDB_XPOINT_1553_CODE, WHHDB_XPOINT_ARINC825_CODE
from wcr_core.modules.hardware.db.whhdb_operator import WHHDB_OPER, \
    WhhdbOperator
from wcr_core.modules.hardware.db.whhdb_rack_instance_meta import IO_TYPES
from wcr_core.modules.hardware.plugin.whplg_library_reader import \
    _whplg_create_xpoint_manager

WHHDB_KAZYMIR_CODE = 'wcr.cets.kaz'
WHHDB_KAZYMIR_SWITCH = WhhdbOperator(
    code=WHHDB_KAZYMIR_CODE,
    xmngr=WhhdbConnectionPointManager(xpntTypes=None),
    name='Kazymir',
    info='Kazymir',
    symbol='',
    labelAlign=WHDB_HALIGN_CENTER)

WHHDB_KAZYMIR_FIBER_CODE = 'wcr.cets.kazf'
WHHDB_KAZYMIR_FIBER_SWITCH = WhhdbOperator(
    code=WHHDB_KAZYMIR_FIBER_CODE,
    xmngr=WhhdbConnectionPointManager(xpntTypes=None),
    name='Kazymir',
    info='Kazymir',
    symbol='',
    labelAlign=WHDB_HALIGN_CENTER)

WHHDB_KAZYMIR_CETRAC_CODE = 'wcr.cets.kazc'
WHHDB_KAZYMIR_CETRAC_SWITCH = WhhdbOperator(
    code=WHHDB_KAZYMIR_CETRAC_CODE,
    xmngr=WhhdbConnectionPointManager(xpntTypes=None),
    name='Kazymir',
    info='Kazymir',
    symbol='',
    labelAlign=WHDB_HALIGN_CENTER)


WHHDB_CETRAC_SWITCH_CODE = 'wcr.hardware.std.switch'

WHHDB_CETRAC_SWITCH = WhhdbOperator(
    code=WHHDB_CETRAC_SWITCH_CODE,
    xmngr=WhhdbConnectionPointManager(xpntTypes=None),
    name='Switch',
    info='switch',
    symbol='',
    labelAlign=WHDB_HALIGN_CENTER)


WHHDB_HOST_CODE = 'wcr.hardware.std.HOST'


WHHDB_HOST = WhhdbOperator(
    code=WHHDB_HOST_CODE,
    xmngr=WhhdbConnectionPointManager(xpntTypes=None),
    name='Host',
    info='Host',
    symbol='',
    labelAlign=WHDB_HALIGN_CENTER)


WHHDB_RACK_IO_CODE = 'wcr.hardware.std.RACK'

RACK_POINT = [('single', {'cdir': 3,
                          WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                     'speed': '1G',
                                     'type': WHHDB_XPOINT_CETRAC_CODE,
                                     'Direction': 'East',
                                     'PNum': 1},
                          WHHDB_XPOINT_DISCRET_KEY: {},
                          WHHDB_XPOINT_FIBER_CHANNEL_KEY: {},
                          WHHDB_XPOINT_AFDX_KEY: {},
                          'name': 'port_cetrac1'})]
PARAMETER_RACK = [['id', 'Rack id', 'Rack id', 0, '', '']]

WHHDB_RACK = WhhdbOperator(
    code=WHHDB_RACK_IO_CODE,
    xmngr=_whplg_create_xpoint_manager(RACK_POINT),
    name='RACKIO',
    info='RACKIO',
    symbol='RackIO',
    parameters=PARAMETER_RACK,
    labelAlign=WHDB_HALIGN_CENTER)


WHHDB_CETRAC_CARD_CODE = 'wcr.hardware.std.CETRACCard'

CETRAC_POINT = [('single', {'cdir': 3,
                            WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                       'speed': '1G',
                                       'type': WHHDB_XPOINT_CETRAC_CODE,
                                       'Direction': 'East',
                                       'PNum': 6},
                            WHHDB_XPOINT_DISCRET_KEY: {},
                            WHHDB_XPOINT_FIBER_CHANNEL_KEY: {},
                            WHHDB_XPOINT_AFDX_KEY: {},
                            'name': 'port_cetrac1'})]

WHHDB_CETRAC_CARD = WhhdbOperator(
    code=WHHDB_CETRAC_CARD_CODE,
    xmngr=_whplg_create_xpoint_manager(CETRAC_POINT),
    name='CetraCCard',
    info='CetracCard',
    symbol='',
    meta={'type': 'External'},
    labelAlign=WHDB_HALIGN_CENTER)

AFDX_POINT = [('single', {'cdir': 3,
                          WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                     'speed': '1G',
                                     'type': 'AFDX',
                                     'Direction': 'East',
                                     'PNum': 1},
                          WHHDB_XPOINT_DISCRET_KEY: {},
                          WHHDB_XPOINT_FIBER_CHANNEL_KEY: {},
                          WHHDB_XPOINT_AFDX_KEY: {},
                          'name': 'port_afdx1'})]

WHHDB_AFDX_CARD_CODE = 'wcr.hardware.std.AFDXCard'

WHHDB_AFDX_CARD = WhhdbOperator(
    code=WHHDB_AFDX_CARD_CODE,
    xmngr=_whplg_create_xpoint_manager(AFDX_POINT),
    name='AFDXCard',
    info='AFDXCard',
    symbol='',
    meta={'type': 'External'},
    labelAlign=WHDB_HALIGN_CENTER)

ETHERNET_POINT = [('single', {'cdir': 3,
                              WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                         'speed': '1G',
                                         'type': 'Ethernet',
                                         'Direction': 'East',
                                         'PNum': 1},
                              WHHDB_XPOINT_DISCRET_KEY: {},
                              WHHDB_XPOINT_FIBER_CHANNEL_KEY: {},
                              WHHDB_XPOINT_AFDX_KEY: {},
                              'name': 'port_eth1'})]

WHHDB_ETHERNET_CARD_CODE = 'wcr.hardware.std.EthernetCard'

WHHDB_ETHERNET_CARD = WhhdbOperator(
    code=WHHDB_ETHERNET_CARD_CODE,
    xmngr=_whplg_create_xpoint_manager(ETHERNET_POINT),
    name='EthernetCard',
    info='EthernetCard',
    symbol='',
    meta={'type': 'External'},
    labelAlign=WHDB_HALIGN_CENTER)

DISCRET_POINT = [('single', {'cdir': 3,
                          WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                     'speed': '1G',
                                     'type': 'Discrete',
                                     'Direction': 'East',
                                     'active': 'On',
                                     'PNum': 1},
                          WHHDB_XPOINT_DISCRET_KEY: {},
                          WHHDB_XPOINT_FIBER_CHANNEL_KEY: {},
                          WHHDB_XPOINT_AFDX_KEY: {},
                          'name': 'p1'})]

WHHDB_DISCRET_CARD_CODE = 'wcr.hardware.std.DiscretCard'

WHHDB_DISCRET_CARD = WhhdbOperator(
    code=WHHDB_DISCRET_CARD_CODE,
    xmngr=_whplg_create_xpoint_manager(DISCRET_POINT),
    name='DiscreteModule',
    info='DiscreteModule',
    symbol='',
    meta={'type': WHHDB_XPOINT_DISCRET_CODE},
    labelAlign=WHDB_HALIGN_CENTER)

FIBER_POINT = [('single', {'cdir': 3,
                          WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                     'speed': '1G',
                                     'type': 'FiberChannel',
                                     'Direction': 'East',
                                     'active': 'On',
                                     'PNum': 1},
                          WHHDB_XPOINT_DISCRET_KEY: {},
                          WHHDB_XPOINT_FIBER_CHANNEL_KEY: {},
                          WHHDB_XPOINT_AFDX_KEY: {},
                          'name': 'p1'})]

WHHDB_FIBER_CARD_CODE = 'wcr.hardware.std.FiberCard'

WHHDB_FIBER_CARD = WhhdbOperator(
    code=WHHDB_FIBER_CARD_CODE,
    xmngr=_whplg_create_xpoint_manager(FIBER_POINT),
    name='FiberModule',
    info='FiberModule',
    symbol='',
    meta={'type': WHHDB_XPOINT_FIBER_CHANNEL_CODE},
    labelAlign=WHDB_HALIGN_CENTER)

MIL1553_POINT = [('single', {'cdir': 3,
                  WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                            'speed': '1G',
                                            'type': WHHDB_XPOINT_1553_CODE,
                                            'Direction': 'East',
                                            'active': 'On',
                                            'PNum': 1},
                  WHHDB_XPOINT_1553_KEY: {},
                  'name': 'p1'})]

WHHDB_1553_CARD_CODE = 'wcr.hardware.std.1553Card'

WHHDB_1553_CARD = WhhdbOperator(
    code=WHHDB_1553_CARD_CODE,
    xmngr=_whplg_create_xpoint_manager(MIL1553_POINT),
    name='1553Module',
    info='1553Module',
    symbol='',
    meta={'type': WHHDB_XPOINT_1553_CODE},
    labelAlign=WHDB_HALIGN_CENTER)

ARINC825_POINT = [('single', {'cdir': 3,
                  WHHDB_XPOINT_CONFIG_KEY: {'BackBone': 'No',
                                            'speed': '',
                                            'type': WHHDB_XPOINT_ARINC825_CODE,
                                            'Direction': 'East',
                                            'active': 'On',
                                            'PNum': 1},
                  WHHDB_XPOINT_ARINC825_KEY: {WHHDB_PORT_ARINC825_SPEED_KEY: '500K',
                                              WHHDB_PORT_ARINC825_CYCLIC_KEY: 'No',
                                              WHHDB_PORT_ARINC825_LISTEN_MODE_KEY: 'No'},
                  'name': 'p1'})]

WHHDB_ARINC825_CARD_CODE = 'wcr.hardware.std.Arinc825Card'

WHHDB_ARINC825_CARD = WhhdbOperator(
    code=WHHDB_ARINC825_CARD_CODE,
    xmngr=_whplg_create_xpoint_manager(ARINC825_POINT),
    name='ARINC825Module',
    info='ARINC825Module',
    symbol='',
    meta={'type': WHHDB_XPOINT_ARINC825_CODE},
    labelAlign=WHDB_HALIGN_CENTER)


WHHDB_CETRAC_RACK_IO_CODE = 'wcr.cetr.rio'
IO_CARDS_CODE = ['wcr.ioc.{}'.format(code.lower()) for code in IO_TYPES]
DEFAULT_CARDS = [WHHDB_CETRAC_CARD, WHHDB_AFDX_CARD, WHHDB_ETHERNET_CARD,
                 WHHDB_FIBER_CARD, WHHDB_DISCRET_CARD, WHHDB_1553_CARD,
                 WHHDB_ARINC825_CARD]
DEFAULT_CARDS_CODES = [card.getCode() for card in DEFAULT_CARDS]
WHHDB_DEFAULT_OPS = [WHHDB_RACK, WHHDB_HOST, WHHDB_CETRAC_SWITCH,
                     WHHDB_CETRAC_CARD, WHHDB_AFDX_CARD, WHHDB_ETHERNET_CARD,
                     WHHDB_FIBER_CARD, WHHDB_DISCRET_CARD, WHHDB_1553_CARD,
                     WHHDB_ARINC825_CARD]
WHHDB_OPER.addOperators(WHHDB_DEFAULT_OPS)
