# ----------------------------------------------------------------------------
# - whhdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry            core / hardware / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whhdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy raw data wrapper registry

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP, \
    WdbsBlockInstanceP, WdbsModelInstanceP
from wcr_core.modules.hardware.db.whhdb_connection_points import WhhdbHardwareXPoint
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.modules.hardware.db.whhdb_top_block import WhhdbTopHierarchy
from wcr_core.modules.hardware.db.whhdb_block import WhhdbBlock
from wcr_core.db.whdb_block import WHDB_TOPLEVEL_BLOCK_CODE
from wcr_core.modules.hardware.db.whhdb_standard_operators import \
    WHHDB_HOST_CODE, WHHDB_CETRAC_SWITCH_CODE, WHHDB_RACK_IO_CODE, \
    IO_CARDS_CODE, WHHDB_KAZYMIR_CETRAC_CODE, WHHDB_KAZYMIR_FIBER_CODE,\
    WHHDB_KAZYMIR_CODE
from wcr_core.modules.hardware.db.whhdb_Host_instance import\
    WhhdbHostInstance
from wcr_core.modules.hardware.db.whhdb_generic_Host_instance import\
    WHHDB_CETRAC_RACK_IO_CODE
from wcr_core.modules.hardware.db.whhdb_rack_instance import \
    WhhdbRackInstance
from wcr_core.modules.hardware.db.whhdb_iocard_instance import \
    WhhdbIoCardInstance
from wcr_core.modules.hardware.db.whhdb_Cetrac_switch_instance import \
    WhhdbCetracSwitchInstance
from wcr_core.modules.hardware.db.whhdb_kazymir_switch_instance import \
    WhhdbKazymirSwitchInstance
from wcr_core.modules.hardware.db.whhdb_operator_instance import \
    WhhdbOperatorInstance
from wcr_core.dbstore_api.wdbs_connection_point_api import WdbsConnectionPointP


WHHDB_STD_OPERINST_CLS = {WHHDB_HOST_CODE: WhhdbHostInstance,
                          WHHDB_RACK_IO_CODE: WhhdbRackInstance,
                          WHHDB_CETRAC_RACK_IO_CODE: WhhdbRackInstance,
                          WHHDB_CETRAC_SWITCH_CODE: WhhdbCetracSwitchInstance,
                          WHHDB_KAZYMIR_CETRAC_CODE: WhhdbKazymirSwitchInstance,
                          WHHDB_KAZYMIR_FIBER_CODE: WhhdbKazymirSwitchInstance,
                          WHHDB_KAZYMIR_CODE: WhhdbKazymirSwitchInstance,}


class WhhdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """
    def __init__(self, xpntClzzs):
        """
        :param xpntClzzs: Connection point classes/constructors, by code
        :type xpntClzzs: dict[str, () -> WphdbConnectionPointBase]
        """
        super().__init__()

        self.__xpntClzzs = xpntClzzs

    def createWrapper(self, obj, *args):

        if isinstance(obj, WdbsOperatorInstanceP):
            return self._createOperatorInstance(obj)
        elif isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WHDB_TOPLEVEL_BLOCK_CODE:
                return WhhdbTopHierarchy(obj, self)
            else:
                return WhhdbBlock(obj, self)
        elif isinstance(obj, WdbsConnectionPointP):
            clazz = self.__xpntClzzs.get(obj.getEntityType())

            assert clazz, str(self.__xpntClzzs)
            if clazz == WhhdbHardwareXPoint:
                return clazz(obj, self, obj.getEntityType())
            else:
                return clazz(obj, self)
        return super().createWrapper(obj, *args)

    def _createOperatorInstance(self, opInst):
        """
        Create an operator instance object based on the type of operator

        :param opInst: Raw operator instance
        :type opInst: WdbsOperatorInstanceP
        :return: Operator instance created
        :rtype: WhdbOperatorInstanceP
        """
        operator = opInst.getOperator()
        if operator.lower() in IO_CARDS_CODE:
            return WhhdbIoCardInstance(opInst, self)
        cls = WHHDB_STD_OPERINST_CLS.get(operator)
        return cls(opInst, self) if cls is not None else \
            WhhdbOperatorInstance(opInst, self)