# ----------------------------------------------------------------------------
# - whmod_application_module                                                 -
# - wild cruncher: application module                        core / hardware -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whmod_application_module
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher hardware designer application module

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_top_level_factory import whdb_createTopLevelRawBlock
from wcr_core.modules.hardware.db.whhdb_meta_provider import \
    WhhdbMetaProvider
from wcr_core.modules.hardware.db.whhdb_wrapper_registry import \
    WhhdbWrapperRegistry
from wcr_core.project.wprj_project_context import WprjProjectContext
from wcr_core.modules.hardware.db.whhdb_verifier import \
    WhhdbHierarchyVerifier
from wcr_core.modules.hardware.controller.whctl_controllers import \
    WHCTL_CONTROLLER_FACTORY
from wcr_core.modules.hardware.db.whhdb_operator import WHHDB_OPER
from wcr_core.modules.hardware.db.whhdb_standard_operators import \
    WHHDB_DEFAULT_OPS
from wcr_core.modules.wmod_application_module import \
    WMOD_DEFAULT_BLOCK_TYPES, WmodApplicationModuleBase, \
    WmodApplicationModuleP, WmodApplicationModuleMP
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_HARDWARE




from wcr_core.db_api.whdb_block_api import WHDB_TOPLEVEL_BLOCK_CODE, \
    WHDB_DEFAULT_BLOCK_CODE
from wcr_core.modules.hardware.db.whhdb_top_block import WhhdbTopHierarchy
from wcr_core.modules.wmod_block_type import WmodBlockType, \
    WMOD_SYSTEM_BLOCK_FLAG


WHMOD_HARDWARE_APP_CODE = 'hardware'

WHMOD_BLOCK_TYPES = [
    WmodBlockType(WHDB_TOPLEVEL_BLOCK_CODE, 'Top-level',
                  WhhdbTopHierarchy.META, WhhdbTopHierarchy,
                  WMOD_SYSTEM_BLOCK_FLAG),
]

# noinspection PyAbstractClass
class WhmodHardwareApplicationModuleP(WmodApplicationModuleP):
    pass


# noinspection PyAbstractClass
class WhmodHardwareApplicationModuleMP(WmodApplicationModuleMP,
                                       WhmodHardwareApplicationModuleP):
    pass


class _WhmodHardwareApplicationModule(WmodApplicationModuleBase,
                                         WhmodHardwareApplicationModuleMP):
    def __init__(self, operRegistry):
        super().__init__(
            code=WHMOD_HARDWARE_APP_CODE,
            name='Hardware View',
            blockTypes=WHMOD_BLOCK_TYPES,
            defaultOps=WHHDB_DEFAULT_OPS,
            operRegistry=operRegistry)

        #: Connection point types
        self.__xpointTypes = {}
        self.__operRegistry = operRegistry
        """:type: dict[str, WbmodConnectionPointTypeP]"""

    @property
    def operRegistry(self):
        return self.__operRegistry

    def createHierarchyVerifier(self, pctx):
        return WhhdbHierarchyVerifier(self._getDbVerifierModules())

    def getPayloadType(self):
        return WCMN_PAYLOAD_BLOCK_HARDWARE

    def createTopLevelBlock(self, name=None, info=None):
        return whmod_create_top_block(self, name, info)

    def createProjectContext(self, gContext, projectDB, logger=None):
        return WprjProjectContext(
            gContext, self, projectDB, logger, WHCTL_CONTROLLER_FACTORY)

    def addConnectionPointType(self, xpntType):
        self.__xpointTypes[xpntType.getCode()] = xpntType

    def getConnectionPointType(self, code):
        return self.__xpointTypes.get(code)

    def getConnectionPointTypes(self):
        return self.__xpointTypes.values()

    def getFileExtention(self):
        return '.hw'


def whmod_create_top_block(module, name=None, info=None):
    xpntClzzs = {x.getCode(): x.getClass() for x
                 in module.getConnectionPointTypes()}
    """:type: dict[str, () -> WphdbConnectionPointBase]"""
    xpntCfgMetas = {x.getCode(): x.getConfigMeta() for x
                    in module.getConnectionPointTypes()}
    """:type: dict[str, WdbsMetaTypeP]"""
    rgty = WhhdbWrapperRegistry(xpntClzzs)
    tblk = whdb_createTopLevelRawBlock(WhhdbMetaProvider(xpntCfgMetas), name, info)
    return rgty.createWrapper(tblk)


#: HDS Application module singleton
WHMOD_HARDWARE_APP_MODULE = _WhmodHardwareApplicationModule(WHHDB_OPER)
