# ----------------------------------------------------------------------------
# - wihdb-block                                                              -
# - wild cruncher:wsh block object                           core / icd / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wihdb_block
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy blocks

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_core.db.whdb_block import WhdbBlock
from wcr_core.modules.icd.db.wihdb_operator import WIHDB_OPER
from wcr_core.db.whdb_connection_point import WHDB_DEFAULT_XPOINT_CODE
from wcr_core.db.whdb_protocols import WHDB_ROTATION_KEY, WHDB_HFLIP_KEY, \
    WHDB_VFLIP_KEY, WHDB_POSITION_KEY, WHDB_Z_VALUE_KEY, \
    WHDB_DIMENSION_KEY


# noinspection PyAbstractClass
class WihdbBlock(WhdbBlock):
    """
    The WihdbModelBlock class is the silkan hierarchy model block. It is used
    to construct one hierarchical level by grouping instances, functions and
    wire into several list. A block is a model and therefore is defined by name
    with an optional information string.
    """

    def __init__(self, block, wreg):
        """
        :param block: Raw block
        :type block: WdbsBlockP
        :param wreg: Raw element wrapper registry
        :type wreg: WmhdbWrapperRegistry
        """
        WhdbBlock.__init__(self, block, wreg)

    def _createRawOperatorInstance(self, oper, *args, **kwargs):
        if isinstance(oper, str):
            oper = WIHDB_OPER[oper]
        return wihdb_createOperatorInstance(
            self.__rawBlock, oper, *args, **kwargs)


def wihdb_createOperatorInstance(block, oper, name=None, info=None,
                                 pos=None, zpos=None, width=None,
                                 height=None, rotation=None, vflip=None,
                                 hflip=None, prms=None):
    """
    :param block: Block to create function in
    :type block: WdbsBlockP
    :param oper: Operator code
    :type oper: WmhdbOperator
    :param name: Name of the instance
    :type name: str
    :param info: Info string for the instance
    :type info: str
    :param pos: Position
    :type pos: (float, float)
    :param zpos: Z-value
    :type zpos: int
    :param width: Width
    :type width: float
    :param height: Height
    :type height: float
    :param rotation: Rotation
    :type rotation: float
    :param vflip: Vertically flipped
    :type vflip: bool
    :param hflip: Horizontally flipped
    :type hflip: bool
    :param prms: Parameter values
    :type prms: Iterable[(str, object)]
    :return: The newly created operator instance
    :rtype: WdbsOperatorInstanceP
    """
    if width is not None and height is not None:
        dim = (width, height)
    else:
        dim = None
    fclst = [
        (WHDB_DEFAULT_XPOINT_CODE, None, xdsc.kwargs)
        for xdsc in oper.createInitialConnectionPoints()]

    return block.createOperatorInstance(
        oper.getCode(), name, info, fclst, prms, **{
            WHDB_POSITION_KEY: pos or (0, 0),
            WHDB_Z_VALUE_KEY: zpos,
            WHDB_DIMENSION_KEY: dim,
            WHDB_ROTATION_KEY: rotation or 0,
            WHDB_HFLIP_KEY: hflip or False,
            WHDB_VFLIP_KEY: vflip or False})