# ----------------------------------------------------------------------------
# - wihdb-module-instance                                                    -
# - wild cruncher: icd module instances                      core / icd / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wihdb_module_instance
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher - icdmodule instance

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.modules.icd.db.wihdb_operator import WIHDB_OPER
from wcr_core.modules.icd.db.wihdb_standard_operators import \
    WIHDB_MODULE
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaField, \
    WdbsMetaDescription
from wcr_core.modules.icd.db.wihdb_connection_point_manager import \
    _wihdb_resetXPointPositions
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_DIRECTION,\
    WHDB_XPOINT_FACING, WHDB_XPOINT_NAME
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI, \
    WDBS_XPNT_IN, WDBS_XPNT_OUT

WIHDB_DATAS_KEY = "message"
WIHDB_CONSPROD_KEY = 'consprod'
WIHDB_STORAGE_KEY = 'storage'
WIHDB_MODULE_TYPE_KEY = 'moduleType'


class WihdbModuleInstance(WhdbOperatorInstance):
    """
    Operator-instance for External Module instances.
    """

    META = WdbsMetaDescription(
        WhdbOperatorInstance.META,
        WdbsMetaField(WIHDB_DATAS_KEY,
                      WdbsDictionaryMetaType(WDBS_STRING_META,
                                WdbsTupleMetaType(WDBS_STRING_META,
                                                  WDBS_STRING_META,
                                                  WDBS_STRING_META,
                                                  WDBS_STRING_META
                                                    ))),
        WdbsMetaField(WIHDB_MODULE_TYPE_KEY,
                      WDBS_STRING_META)
    )

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        operName = inst.getOperator()
        if operName:
            oper = WIHDB_OPER[operName]
        else:
            oper = WIHDB_MODULE
        WhdbOperatorInstance.__init__(self, oper, inst, wreg)
        if oper.getModuleType():
            self._setModuleType(oper.getModuleType())
        if not tuple(self.getConnectionPoints()):
            entries = oper.getEntries()
            if entries:
                for msgName, (_, direction, _, _) in entries.items():
                    self._createRawConnectionPoint('message', msgName, direction)
                _wihdb_resetXPointPositions(self.getConnectionPoints())

    def _createRawConnectionPoint(self, ctyp, globmsg, direction):
        kwargs = {WHDB_XPOINT_FACING: 2, WHDB_XPOINT_NAME: globmsg}
        kwargs[WHDB_XPOINT_DIRECTION] = wihdb_get_metadirection(direction)
        return super()._createRawConnectionPoint(ctyp, **kwargs)

    def updateDirection(self, newdir, globmsg):
        for conn in self.getConnectionPoints():
            if conn.getName() == globmsg:
                conn.setDirection(newdir)

    def getModuleType(self):
        """
        return the module type
        """
        return self.rawElement.getMeta(WIHDB_MODULE_TYPE_KEY)

    def _setModuleType(self, moduleType):
        """
        set the module type
        """
        self.rawElement.setMeta(moduleType, WIHDB_MODULE_TYPE_KEY)

    def getData(self):
        """
        return the messages of the module
        """
        return self.rawElement.getMeta(WIHDB_DATAS_KEY)

    def setData(self, moduledat, top=None):
        """
        set the messages for the module
        """
        datas = {}
        if top:
            for msg, (_,  consprod, storage, period) in moduledat.items():
                type_ = getTypeFromModuleMsg(top, msg, self.getName())
                datas[msg] = (type_, consprod, storage, period)
        else:
            for msg, (type_,  consprod, storage, period) in moduledat.items():
                datas[msg] = (type_, consprod, storage, period)
        self.rawElement.setMeta(datas, WIHDB_DATAS_KEY)


def wihdb_get_metadirection(direction):
    """
        transform from C or P (consumer/producer) into the meta equivalent
        :param direction: str
        :return: metaDir: in
    """
    metaDir = WDBS_XPNT_BI
    if direction == 'C':
        metaDir = WDBS_XPNT_IN
    elif direction == 'P':
        metaDir = WDBS_XPNT_OUT
    return metaDir


def wihdb_get_realdirection(metadirection):
    """
        transform the meta direction into C or P (consumer/producer)
        :param direction: str
        :return: metaDir: in
        """
    direction = ''
    if metadirection == WDBS_XPNT_IN:
        direction = 'C'
    elif metadirection == WDBS_XPNT_OUT:
        direction = 'P'
    return direction


def getTypeFromModuleMsg(top, msg, modName):
    allBindedMsg = top.getAllMessageBindings()
    allMessages = top.getAllMessages()
    for glMsg, msgs in allBindedMsg.items():
        compMsg = '{}.{}'.format(modName, msg)
        for message in msgs:
            if compMsg in message:
                return allMessages[glMsg][0]
