# ----------------------------------------------------------------------------
# - wihdb-module--instance                                                   -
# - Dezygner wight box messages instances                core / message / db -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 philippe lambert                                 -
# - copyright (c) 2018-2018 silkan                                           -
# ----------------------------------------------------------------------------

"""
.. module::       wihdb_wbwmessage_instance
   :platform:     Unix, Windows
   :synopsis:     Dezygner - Host instance

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""

from wcr_core.modules.icd.db.wihdb_standard_operators import \
    WIHDB_WBMESSAGE
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaField, \
    WdbsMetaDescription
from wcr_core.dbstore.wdbs_meta_type import *

WIHDB_WGT_BOX_SIZE_KEY = "whiteboxsize"
WIHDB_WGT_BOX_DATA_KEY = "wightboxdata"


class WihdbWightBoxMessageInstance(WhdbOperatorInstance):
    """
    Operator-instance for External Module instances.
    """

    META = WdbsMetaDescription(
        WhdbOperatorInstance.META,
        WdbsMetaField(WIHDB_WGT_BOX_SIZE_KEY,
                      WdbsIntegerMetaType(default=64)),
        WdbsMetaField(WIHDB_WGT_BOX_DATA_KEY,
                      WdbsListMetaType(
                          WdbsTupleMetaType(WDBS_STRING_META,
                                                WDBS_INTEGER_META,
                                                WDBS_INTEGER_META))))

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        WhdbOperatorInstance.__init__(self, WIHDB_WBMESSAGE, inst, wreg)

    def getSize(self):
        """
        return the size of the message
        :param msgName: The name of the wanted message
        :type msgName: str
        :return:
        """
        return self.rawElement.getMeta(WIHDB_WGT_BOX_SIZE_KEY)

    def setSize(self, nsize):
        """
        Set a new value to Size
        :param nsize: the new size to set
        :type nsize: int
        """
        self.rawElement.setMeta(int(nsize), WIHDB_WGT_BOX_SIZE_KEY)

    def getDatas(self):
        """
        return the data ans positions of the message
        """
        return self.rawElement.getMeta(WIHDB_WGT_BOX_DATA_KEY)

    def setDatas(self, datas):
        """
        set the Datas for the message
        """
        self.rawElement.setMeta(datas, WIHDB_WGT_BOX_DATA_KEY)

    def addData(self, dataname, start, end):
        """
        add a data to the message
        """
        dat = self.getDatas()
        if self.canInsert(start, end):
            dat.append((dataname, start, end))
        else:
            raise Exception("No room to insert data.")
        self.setDatas(dat)

    def canEdgeAt(self, startpos):
        for data in self.getDatas():
            if data[1] <= startpos <= data[2]:
                return False
        return 0 <= startpos < self.getSize()

    def canInsert(self, start, end):
        if end <= start or \
                not self.canEdgeAt(start) or \
                not self.canEdgeAt(end):
            return False
        for data in self.getDatas():
            if start <= data[1] <= end or start <= data[2] <= end:
                return False
        return True

    def removeData(self, data):
        datas = self.getDatas()
        newdatas=[]
        for dat in datas:
            if dat[0] != data:
                newdatas.append(dat)
        self.setDatas(newdatas)

