# ----------------------------------------------------------------------------
# - wimod_application_module                                                 -
# - wild cruncher: application module                             core / icd -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wimod_application_module
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher ICD designer application module

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_top_level_factory import whdb_createTopLevelRawBlock
from wcr_core.modules.icd.db.wihdb_meta_provider import WihdbMetaProvider
from wcr_core.project.wprj_project_context import WprjProjectContext
from wcr_core.modules.icd.db.wihdb_verifier import WihdbHierarchyVerifier
from wcr_core.modules.icd.controller.wictl_controllers import \
    WICTL_CONTROLLER_FACTORY
from wcr_core.modules.icd.db.wihdb_operator import WIHDB_OPER, WihdbOperator
from wcr_core.modules.wmod_application_module import \
    WMOD_DEFAULT_BLOCK_TYPES, WmodApplicationModuleBase, \
    WmodApplicationModuleP, WmodApplicationModuleMP
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_ICD

from wcr_core.db_api.whdb_block_api import WHDB_TOPLEVEL_BLOCK_CODE, \
    WHDB_DEFAULT_BLOCK_CODE
from wcr_core.modules.icd.db.wihdb_top_block import WihdbTopHierarchy
from wcr_core.modules.wmod_block_type import WmodBlockType, \
    WMOD_SYSTEM_BLOCK_FLAG


WIMOD_ICD_APP_CODE = 'icd'
WIMOD_ICD_APP_NAME = 'Logical View'

WIMOD_BLOCK_TYPES = [
    WmodBlockType(WHDB_TOPLEVEL_BLOCK_CODE, 'Top-level',
                  WihdbTopHierarchy.META, WihdbTopHierarchy,
                  WMOD_SYSTEM_BLOCK_FLAG)]

# noinspection PyAbstractClass
class WimodICDApplicationModuleP(WmodApplicationModuleP):
    pass


# noinspection PyAbstractClass
class WimodICDApplicationModuleMP(WmodApplicationModuleMP,
                                       WimodICDApplicationModuleP):
    pass


class _WimodICDApplicationModule(WmodApplicationModuleBase,
                                         WimodICDApplicationModuleMP):
    def __init__(self, operRegistry):
        self.operatorRegistry = operRegistry
        super().__init__(
            code=WIMOD_ICD_APP_CODE,
            name=WIMOD_ICD_APP_NAME,
            blockTypes=WIMOD_BLOCK_TYPES,
            # defaultOps=WIHDB_DEFAULT_OPS,
            operRegistry=operRegistry)

    def createHierarchyVerifier(self, pctx):
        return WihdbHierarchyVerifier(self._getDbVerifierModules())

    def getPayloadType(self):
        return WCMN_PAYLOAD_BLOCK_ICD

    def createTopLevelBlock(self, name=None, info=None):
        blockMetas = {b.getCode():b.getMeta() for b in self.getBlockTypes()}
        blockClzzs = {b.getCode():b.getClass() for b in self.getBlockTypes()}
        return wimod_create_top_block(blockMetas, blockClzzs, name, info)

    def createProjectContext(self, gContext, projectDB, logger=None):
        return WprjProjectContext(
            gContext, self, projectDB, logger, WICTL_CONTROLLER_FACTORY)

    def getFileExtention(self):
        return '.icd'


def wimod_create_top_block(blockMetas, blockClzzs, name=None, info=None):
    from wcr_core.modules.icd.db.wihdb_wrapper_registry import WihdbWrapperRegistry
    tblk = whdb_createTopLevelRawBlock(
        WihdbMetaProvider(blockMetas), name, info)
    rgty = WihdbWrapperRegistry(blockClzzs)
    return rgty.createWrapper(tblk)


#: HDS Application module singleton
WIMOD_ICD_APP_MODULE = _WimodICDApplicationModule(WIHDB_OPER)
