# ----------------------------------------------------------------------------
# - wmehdb-top-block                                                         -
# - wild cruncher:wsh top hierarchy object               core / message / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 lambert philippe                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmehdb_top_block
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher top hierarchy object

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.modules.message.db.wmehdb_block import WmehdbBlock
from wcr_core.db.whdb_block import WhdbBlock
from silkan_common.scmn_file_path import scmn_path_rel2file, scmn_path_mkabs, \
    scmn_isfile, scmn_path_dir, scmn_relpath
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.bus.db.wbhdb_top_block_meta import *

WIHDB_META_ICD_KEY = "ICD_sourcefile"

class WmehdbTopHierarchy(WhdbBlock):
    """
    The WmhdbTopHierarchy is the top hierarchy model class. A top hierarchy
    is at the same time a model and an instance. The class provides several
    methods for creating the hierarchy.
    """
    META = WdbsMetaDescription(
        WmehdbBlock.META,
        WdbsMetaField(WIHDB_META_ICD_KEY,WdbsTupleMetaType(WDBS_STRING_META,
                                                           WDBS_STRING_META)))


    def __init__(self, block, wreg):
        """
        :param block: Raw block
        :type block: WdbsBlockP
        :param wreg: Wrapper registry
        :type wreg: WmhdbWrapperRegistry
        """
        super().__init__(block, wreg)
        self._srdfilepath = None
        self.Module=[]

    def setSourcePath(self, path):
        if path is not None:
            self._srdfilepath = path

    def getModules(self):
        return self.Module

    def setModules(self, modulel):
        self.Module = modulel

    def setICDFile(self, path, name):
        self.rawElement.setMeta((name, str(path)), WIHDB_META_ICD_KEY)

    def getICDFile(self):
        name, path = self.rawElement.getMeta(WIHDB_META_ICD_KEY)
        abspathicd = scmn_path_mkabs(path,
                                      bdir=scmn_path_dir(self._srdfilepath))
        if scmn_isfile(abspathicd):
            return [abspathicd]
        return []

    def getICDNameFile(self, path):
        name, metapath = self.rawElement.getMeta(WIHDB_META_ICD_KEY)
        abspath = scmn_path_mkabs(metapath,
                                  bdir=scmn_path_dir(self._srdfilepath))
        if self._srdfilepath:
            relpath = scmn_relpath(path, self._srdfilepath)
        else:
            relpath=path+"inval"
        if path == abspath or path == relpath:
            return name
        return None

    def formatToSave(self, npath):
        ndir = scmn_path_dir(npath)
        msgpathl = self.getICDFile()
        for path in msgpathl:
            if scmn_isfile(path):
                name = self.getICDNameFile(path)
                msgrela = scmn_relpath(path, ndir)
                if name is None:
                    raise Exception("ICD path given not in meta")
                self.setICDFile(msgrela, name)
        self.setSourcePath(npath)
