# ----------------------------------------------------------------------------
# - wmehdb-wrapper-registry                                                  -
# - wild cruncher: raw data wrapper registry             core / message / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmehdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy raw data wrapper registry

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.modules.message.db.wmehdb_wgtbxmessage_instance import \
    WmehdbWightBoxMessageInstance
from wcr_core.modules.message.db.wmehdb_operator import WMEHDB_OPER
from wcr_core.modules.message.db.wmehdb_standard_operators import \
    WMEHDB_WBMESSAGE_CODE
from wcr_core.db.whdb_block import WhdbBlock
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.modules.message.db.wmehdb_top_block import WmehdbTopHierarchy
from wcr_core.db_api.whdb_block_api import WHDB_TOPLEVEL_BLOCK_CODE

WMEHDB_STD_OPERINST_CLS = {WMEHDB_WBMESSAGE_CODE:
                           WmehdbWightBoxMessageInstance}


class WmehdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """
    def __init__(self, blockClzzs):
        """

        """
        super().__init__()
        self.__blockClasses = blockClzzs

    def createWrapper(self, obj, *args):
        if isinstance(obj, WdbsOperatorInstanceP):
            return self._createOperatorInstance(obj)
        elif isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WHDB_TOPLEVEL_BLOCK_CODE:
                return WmehdbTopHierarchy(obj, self)
            else:
                return WhdbBlock(obj, self)

        return super().createWrapper(obj, *args)

    def _createOperatorInstance(self, opInst):
        """
        Create an operator instance object based on the type of operator

        :param opInst: Raw operator instance
        :type opInst: WdbsOperatorInstanceP
        :return: Operator instance created
        :rtype: WhdbOperatorInstanceP
        """
        operator = opInst.getOperator()
        cls = WMEHDB_STD_OPERINST_CLS.get(operator)

        return cls(opInst, self) if cls is not None else \
            WhdbOperatorInstance(WMEHDB_OPER[operator], opInst, self)
