# ----------------------------------------------------------------------------
# - wmemod_application_module                                                -
# - wild cruncher: application module                         core / message -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmemod_application_module
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher message designer application module

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_top_level_factory import whdb_createTopLevelRawBlock
from wcr_core.modules.message.db.wmehdb_meta_provider import \
    WmehdbMetaProvider
from wcr_core.modules.message.db.wmehdb_wrapper_registry import \
    WmehdbWrapperRegistry
from wcr_core.project.wprj_project_context import WprjProjectContext
from wcr_core.modules.message.db.wmehdb_verifier import \
    WmehdbHierarchyVerifier
from wcr_core.modules.message.controller.wmectl_controllers import \
    WMECTL_CONTROLLER_FACTORY
from wcr_core.modules.message.db.wmehdb_operator import WMEHDB_OPER
from wcr_core.modules.message.db.wmehdb_standard_operators import \
    WMEHDB_DEFAULT_OPS
from wcr_core.modules.message.db.wmehdb_operator import WmehdbOperator
from wcr_core.modules.wmod_application_module import \
    WMOD_DEFAULT_BLOCK_TYPES, WmodApplicationModuleBase, \
    WmodApplicationModuleP, WmodApplicationModuleMP
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_MESSAGE
from wcr_core.modules.message.wmemod_project_context import \
    WmemodMessageProjectContext




from wcr_core.db_api.whdb_block_api import WHDB_TOPLEVEL_BLOCK_CODE, \
    WHDB_DEFAULT_BLOCK_CODE
from wcr_core.modules.message.db.wmehdb_top_block import WmehdbTopHierarchy
from wcr_core.modules.wmod_block_type import WmodBlockType, \
    WMOD_SYSTEM_BLOCK_FLAG
from wcr_core.modules.message.db.wmehdb_block import WmehdbBlock


WMEMOD_MESSAGE_APP_CODE = 'message'

WMEMOD_BLOCK_TYPES = [
    WmodBlockType(WHDB_TOPLEVEL_BLOCK_CODE, 'Top-level',
                  WmehdbTopHierarchy.META, WmehdbTopHierarchy,
                  WMOD_SYSTEM_BLOCK_FLAG)]

# noinspection PyAbstractClass
class WmemodMessageApplicationModuleP(WmodApplicationModuleP):
    pass


# noinspection PyAbstractClass
class WmemodMessageApplicationModuleMP(WmodApplicationModuleMP,
                                       WmemodMessageApplicationModuleP):
    pass


class _WmemodMessageApplicationModule(WmodApplicationModuleBase,
                                         WmemodMessageApplicationModuleMP):
    def __init__(self, operRegistry):
        super().__init__(
            code=WMEMOD_MESSAGE_APP_CODE,
            name='Message View',
            blockTypes=WMEMOD_BLOCK_TYPES,
            defaultOps=WMEHDB_DEFAULT_OPS,
            operRegistry=operRegistry)

    def createHierarchyVerifier(self, pctx):
        return WmehdbHierarchyVerifier(self._getDbVerifierModules())

    def getPayloadType(self):
        return WCMN_PAYLOAD_BLOCK_MESSAGE

    def createTopLevelBlock(self, name=None, info=None):
        blockMetas = {b.getCode(): b.getMeta() for b in self.getBlockTypes()}
        blockClzzs = {b.getCode(): b.getClass() for b in self.getBlockTypes()}

        return wmemod_create_top_block(blockMetas, blockClzzs,
                                        name=name, info=info)

    def createProjectContext(self, gContext, projectDB, logger=None):
        return WmemodMessageProjectContext(
            gContext, self, projectDB, logger)

    def getFileExtention(self):
        return '.msg'


def wmemod_create_top_block(blockMetas, blockClzzs, name=None, info=None):
    rgty = WmehdbWrapperRegistry(blockClzzs)
    tblk = whdb_createTopLevelRawBlock(WmehdbMetaProvider(blockMetas), name, info)
    return rgty.createWrapper(tblk)


#: HDS Application module singleton
WMEMOD_MESSAGE_APP_MODULE = _WmemodMessageApplicationModule(WMEHDB_OPER)
