# ----------------------------------------------------------------------------
# - wmact-propagate-types-action                                             -
# - wild cruncher: model type propagation action      core / model / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmact_propagate_types_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Model type propagation action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.modules.model.actions.wmact_action_base import WmactActionBase
from wcr_core.modules.model.db.wmhdb_type_propagation import \
    WmhdbTypePropagator, WmhdbElementsException
from wplg_cold.wcld_variables_parser import WcldColdVarsParser
from wcr_backend.modules.model.aob.wmaob_callback_gene import \
    WmaobCallbacksGenerator
from wplg_cold.wcld_builder import WcldColdBuilder
from wcr_backend.modules.model.aob.wmaob_group_builder import WmaobGroupBuilder
from wcr_backend.modules.model.aob.wmaob_builder import WmaobBuilder
import os

class _WmactPropagateTypesAction(WmactActionBase):
    """
    Project editor action to trigger type propagation.
    """

    def __init__(self):
        super().__init__(
            'Propagate Types',
            'Propagate Types throughout the Project')

    def _isProtocolException(self, e):
        return isinstance(e, WmhdbElementsException)

    def _doExecute(self, actx, paramValuePairs):
        """
        Execute the action associated with propagate type buton
        """
        # the tools that parses cold outputs : .vars file
        coldparser = WcldColdVarsParser(None)
        # the profile to be used for code generation
        prof = 'default'
        paramconv={}
        pctx = actx.getProjectContext()
        coldacces = WcldColdBuilder(pctx)
        mdlcallbackinit = WmaobCallbacksGenerator(pctx, prof)
        tblk = pctx.getProjectDB().getTopHierarchy()
        """:type: WmhdbTopHierarchy"""
        tprp = WmhdbTypePropagator()
        fbld = WmaobBuilder(pctx, prof)
        if mdlcallbackinit.canUseCallback:
            gb = WmaobGroupBuilder(pctx, fbld.getCodeGen(), prof)
            gb.processParams(tblk, mdlcallbackinit.getSchedulingName(),
                              dic=paramconv, formatname=True)
            if os.path.exists(os.path.join(pctx.getProjectWorkspace().
                                                   getProjectPath(),
                                           prof,
                                           mdlcallbackinit.getSchedulingName(),
                                           'CldGlobals.vars')):
                file = os.path.join(pctx.getProjectWorkspace().getProjectPath(),
                                    prof, mdlcallbackinit.getSchedulingName(),
                                    'CldGlobals.vars')
                coldparser = WcldColdVarsParser(file)
        tprp.propagateTypes(tblk, coldparser, coldacces)
        gb.cleanFiles()
        gb.restoreParams(tblk, paramconv)


#: Model type propagation actino
WMACT_PROPAGATE_TYPES_ACTION = _WmactPropagateTypesAction()
