# ----------------------------------------------------------------------------
# - wmhdb-block-base                                                         -
# - wild cruncher:model block base class                   core / model / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2013 clemens oertel                                   -
# - copyright (c) 2013-2013 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmhdb_block_base
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher model block base class

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from abc import abstractmethod

from wcr_core.db_api.whdb_block_api import WhdbEntityTargetP, WhdbBlockP


# noinspection PyAbstractClass
class WmhdbBlockP(WhdbBlockP):
    """
    Abstract base class for model blocks.
    """

    @abstractmethod
    def getBlockAndModelInstances(self):
        raise NotImplementedError()

    @abstractmethod
    def getInstanceInputConnectorsUserTypes(self, instance):
        """
        Get the user types for the instance's input connectors.

        :param instance: Instance to return types for
        :type instance: WmhdbHasConnectorsP
        :return: User types of the given instance's input connectors
        :rtype: [WmhdbRtiType]
        """
        raise NotImplementedError()

    @abstractmethod
    def getInstanceInputConnectorsInferredTypes(self, instance):
        """
        Get the inferred types for the given instance's input connectors.

        :param instance: Instance to return types for
        :type instance: WmhdbHasConnectorsP
        :return: Inferred types of the given instance's input connectors
        :rtype: [WmhdbRtiType]
        """
        raise NotImplementedError()

    @abstractmethod
    def getInstanceOutputConnectorsUserTypes(self, instance):
        """
        Get the user types for the given instance's output connectors.

        :param instance: Instance to return types for
        :type instance: WmhdbHasConnectorsP
        :return: User types of the given instance's output connectors
        :rtype: [WmhdbRtiType]
        """
        raise NotImplementedError()

    @abstractmethod
    def getInstanceOutputConnectorsInferredTypes(self, instance):
        """
        Get the inferred types for the given instance's output connectors.

        :param instance: Instance to return types for
        :type instance: WmhdbHasConnectorsP
        :return: Inferred types of the given instance's output connectors
        :rtype: [WmhdbRtiType]
        """
        raise NotImplementedError()

    def getInputWires(self, instance):
        """
        Get the list of input wires connected to the given instance.

        :param instance: Instance to return input wires for
        :type instance: WmhdbHasConnectorsP
        :return: Input wires for the given instance
        :rtype: [WmhdbWire]
        """
        raise NotImplementedError()

    def getOutputWires(self, instance):
        """
        Get the list of output wires connected to the given instance.

        :param instance: Instance to return output wires for
        :type instance: WmhdbHasConnectorsP
        :return: Output wires for the given instance
        :rtype: [WmhdbWire]
        """
        raise NotImplementedError()

    @abstractmethod
    def resetWireList(self):
        """
        Reset the wire list, including its associated path.
        """
        raise NotImplementedError()
