# ----------------------------------------------------------------------------
# - wmhdb-block-flattener                                                    -
# - wild cruncher:wsh flat block model object              core / model / db -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2013 PiV                                              -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmhdb_block_flattener
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher flat block model object

.. moduleauthor:: PiV <pierre.villalon@silkan.com>
"""
from wcr_core.db.whdb_block_flattener import WhdbBlockFlattener
from wcr_core.db.whdb_operator import WHDB_PORT_OPERATOR_FLAG
from wcr_core.db.whdb_connection_point import WHDB_XPOINT_PORT
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP
from wcr_core.modules.model.db.wmhdb_block import WmhdbBlock
from wcr_core.modules.model.db.wmhdb_operator import WMHDB_OPER
from wcr_core.modules.model.db.wmhdb_wire import WMHDB_WIRE_UTYPE_KEY, \
    WMHDB_WIRE_ITYPE_KEY
from wcr_core.modules.model.db.wmhdb_standard_operators import \
    WMHDB_PORT_NAME_PARAMETER


def wmhdb_merge_types(l):
    """
    :return: merge all the type in the list
    :rtype : WmhdbRtiType
    :param l: the list of type to be merged
    :type  l: Iterable[WmhdbRtiType]
    """
    lst = list(l)
    result = None
    for t in lst:
        if t is not None:
            if result is not None:
                if (t.getType() != result.getType() or
                            t.getXdim() != result.getXdim() or
                            t.getYdim() != result.getYdim()):
                    raise ValueError(
                        'can only merge exact same types,'
                        'got {} with {} vs. {}'.format(lst, result, t))
            else:
                result = t
    return result


class WmhdbBlockFlattener(WhdbBlockFlattener):
    def isPort(self, obj):
        if not isinstance(obj, WdbsOperatorInstanceP):
            return False
        return WMHDB_OPER.get(obj.getOperator()).hasFlag(
            WHDB_PORT_OPERATOR_FLAG)

    def getPortForXPoint(self, xpnt, block):
        if not isinstance(block, WdbsBlockP):
            return None
        pid = xpnt.getMeta(WHDB_XPOINT_PORT)
        if pid is None:
            return None
        for f in (i for i in block.getInstances()
                  if isinstance(i, WdbsOperatorInstanceP)):
            # noinspection PyUnresolvedReferences
            p = f.getParameter(WMHDB_PORT_NAME_PARAMETER)
            if p is not None and p.getValue() == pid:
                return f
        return None

    def combineConnectionMetaData(self, cnctn, *cnctns):
        utyp = wmhdb_merge_types(map(
            lambda i: i.getMeta(WMHDB_WIRE_UTYPE_KEY), filter(None, cnctns)))
        cnctn.setMeta(utyp, WMHDB_WIRE_UTYPE_KEY)

        ityp = wmhdb_merge_types(map(
            lambda i: i.getMeta(WMHDB_WIRE_ITYPE_KEY), filter(None, cnctns)))
        cnctn.setMeta(ityp, WMHDB_WIRE_ITYPE_KEY)


def wmhdb_flatten(block, **meta):
    """
    Flatten a HDS block.

    :param block: Block to flatten
    :type block: WmhdbBlock
    :param meta: Additional meta-data for elements in flattened block
    :type meta: {str:object}
    :return: Flattened block
    :rtype: WmhdbBlock
    """
    flat = WmhdbBlockFlattener().flatten(block.rawElement, **meta)
    # noinspection PyTypeChecker
    return WmhdbBlock(flat, block.wrapperRegistry)
