# ----------------------------------------------------------------------------
# - wmhdb-instance-base                                                      -
# - wild cruncher:wsh instance base object                 core / model / db -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2013 amaury darsch                                    -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmhdb_instance_base
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance base object

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------

from wcr_core.db.whdb_instance import WhdbInstance
from wcr_core.db_api.whdb_instance_api import WhdbInstanceP
from wcr_core.db_api.whdb_protocols_api import WhdbHasLocationP, WhdbHasFlagsP
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaDescription, \
    WdbsMetaField
from wcr_core.dbstore.wdbs_meta_type import WDBS_STRING_META
from wcr_core.modules.model.db.wmhdb_protocols import *
from wcr_core.modules.model.db.wmhdb_connection_point import WmhdbConnector


# ----------------------------------------------------------------------------
# - instance class                                                           -
# ----------------------------------------------------------------------------

# noinspection PyAbstractClass
class WmhdbInstanceBaseP(WhdbInstanceP, WmhdbHasConnectorsP,
                         WmhdbHasSchedulingGroupMP,
                         WmhdbHasSchedulingGroupP, WhdbHasFlagsP,
                         WhdbHasLocationP, WhdbHasDimensionP):
    META = WdbsMetaDescription(
        WhdbInstance.META,
        WdbsMetaField(WMHDB_SGROUP_KEY, WDBS_STRING_META))

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsInstanceP
        :param wreg: Raw element wrapper registry
        :type wreg: WmhdbWrapperRegistry
        """
        self.__rawInstance = inst
        self.__wrapperRegistry = wreg

    def getSchedulingGroup(self):
        return self.__rawInstance.getMeta(WMHDB_SGROUP_KEY)

    def setSchedulingGroup(self, schedGroup):
        self.__rawInstance.setMeta(schedGroup, WMHDB_SGROUP_KEY)

    def getInputConnectors(self):
        # noinspection PyUnresolvedReferences
        return self.getConnectionPoints(cdir=WmhdbConnector.Input)

    def getOutputConnectors(self):
        # noinspection PyUnresolvedReferences
        return self.getConnectionPoints(cdir=WmhdbConnector.Output)

    def getConnectionPointDescription(self, conn, **kwargs):
        return conn.getDescription(**kwargs)

    def getConnectorByName(self, name):
        # noinspection PyUnresolvedReferences
        return next((c for c in self.getConnectionPoints()
                     if c.getName() == name), None)

    def getConnectorByPortRef(self, ref):
        # noinspection PyUnresolvedReferences
        return next((c for c in self.getConnectionPoints()
                     if c.getPortReference() == ref), None)
