# ----------------------------------------------------------------------------
# - wmhdb-parameter                                                          -
# - wild cruncher:wsh model parameter object               core / model / db -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2012 amaury darsch                                    -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmhdb_parameter
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher model parameter object

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_parameter import WhdbParameter
from wcr_core.modules.model.db.wmhdb_rti_type import WmhdbRtiType


class WmhdbParameter(WhdbParameter):
    """
    The WmhdbParameter class is the generic silkan hierarchy parameter
    object. A parameter is imply defined by a name, information string
    and value. One specificity of the object is to provide a copy mechanism.
    """

    def __init__(self, param, desc):
        """
        :param param: Raw parameter
        :type param: WdbsParameterP
        :param desc: Parameter description
        :type desc: WcmnParameterDescriptionP
        """
        super().__init__(param, desc)

    def getValidType(self):
        """
        Get the parameter valid types.

        :return: Value of the parameter
        :rtype: WmhdbRtiType
        """
        # noinspection PyUnresolvedReferences
        return self.getDescription().validType

    @property
    def rtiType(self):
        """
        Get the type for the parameter's value, if it can be
        determined.

        :return: The type for the parameter, or `None`
        :rtype: WmhdbRtiType
        """

        return WmhdbRtiType.getRtiTypeForStrValue(self.getValue()) or \
               self.getValidType()
