# ----------------------------------------------------------------------------
# - wmhdb-scheduling-group                                                   -
# - wild cruncher:wsh scheduling group model object        core / model / db -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2013 amaury darsch                                    -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmhdb_scheduling_group
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher flat block model object

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.modules.model.db.wmhdb_block import WmhdbBlock
from wcr_core.modules.model.db.wmhdb_block_flattener import wmhdb_flatten
from wcr_core.modules.model.db.wmhdb_protocols import \
    WmhdbHasSchedulingGroupP, WMHDB_SGROUP_KEY
from wcr_core.modules.model.db.wmhdb_standard_operators import \
    WMHDB_BUS_FLAG, WMHDB_BUS_NAME_PARAMETER


WMHDB_SGROUP_BLOCK_CODE = 'wcr.model.block-sgroup'


class WmhdbGroupBlock(WmhdbBlock, WmhdbHasSchedulingGroupP):
    """
    The WmhdbModelGroup is the scheduling group block model. The scheduling
    group model is identified by the special scheduling group media type. A
    scheduling group is created as a model block by name, info and scheduling
    group name. The scheduling group object provides a flatten method
    which builds a flatten object for that scheduling group.
    """
    _PORT_FLAG = WMHDB_BUS_FLAG

    _PORT_NAME_PARAM = WMHDB_BUS_NAME_PARAMETER

    def __init__(self, sgrp, block, wreg):
        """
        :param sgrp: Scheduling group name
        :type sgrp: str
        :param block: Raw block
        :type block: WdbsBlockP
        :param wreg: Raw element wrapper registry
        :type wreg: WmhdbWrapperRegistryP
        """
        super().__init__(block, wreg)

        self.__sgrp = sgrp

    def getSchedulingGroup(self):
        return self.__sgrp

    def flatten(self):
        """
        Flatten a scheduling group by creating a flatten block

        :return: Flattened block for scheduling group
        :rtype: WmhdbBlock
        """
        return wmhdb_flatten(
            self, **{WMHDB_SGROUP_KEY: self.getSchedulingGroup()})
