# ----------------------------------------------------------------------------
# - wmhdb-text-model                                                         -
# - wild cruncher:wsh model text object                    core / model / db -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2013 amaury darsch                                    -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------
"""
.. module::       wmhdb_text_model
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wsh models

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import WDBS_TEXT_META
from wcr_core.modules.model.db.wmhdb_model_base import WmhdbModel
from silkan_common.scmn_file_path import scmn_expshenv


WMHDB_MODEL_TEXT_KEY = 'text'


class WmhdbModelText(WmhdbModel):
    META = WdbsMetaDescription(
        WmhdbModel.META,
        WdbsMetaField(WMHDB_MODEL_TEXT_KEY, WDBS_TEXT_META))

    def __init__(self, model, wreg):
        """
        :param model: Raw text model
        :type model: WdbsModelP
        :param wreg: Raw element wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        WmhdbModel.__init__(self, model, wreg)

        self.__rawModel = model

    def getTypeName(self):
        return 'text-model'

    def getText(self):
        """
        Get the stored contents of the text model, if any.

        :return: Stored contents of the text model, or `None`
        :rtype: str
        """
        text = self.__rawModel.getMeta(WMHDB_MODEL_TEXT_KEY)
        if not text:
            text = self._loadText()
            self.__rawModel.setMeta(text, WMHDB_MODEL_TEXT_KEY)
        return text

    def _loadText(self):
        path = self.getPath()
        # check for path
        if path is not None:
            # expand the path
            path = scmn_expshenv(path)
            with open(path, 'r') as f:
                return f.read()
