# ----------------------------------------------------------------------------
# - wfact_layout_action                                                      -
# - wild cruncher: layout action               core / statemachine / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 samuel longchamps                                -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfact_layout_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - State machine layout action

.. moduleauthor:: Samuel Longchamps <samuel.longchamps@silkan.com>
"""
from wcr_core.action.wact_action import WACT_ACTION_EXIT_CANCELLED, \
    WACT_ACTION_EXIT_OK
from wcr_core.action.wact_action_parameter import \
    WactFixedMultiValueActionParameter
from wcr_core.action.wact_action_parameter_types import \
    WactActionStringParameterType
from wcr_core.common.wcmn_graph_layouter_circular import \
    WcmnGraphLayouterCircular
from wcr_core.common.wcmn_graph_layouter_grid import WcmnGraphLayouterGrid
from wcr_core.common.wcmn_graph_layouter_spring import WcmnGraphLayouterSpring

from wcr_core.modules.statemachine.actions.wfact_action_base \
    import WfactActionBase
from wcr_core.modules.statemachine.wfmod_layouter import WfmodLayouter


WFACT_LAYOUT_TYPE_POSSIBLE_VALUES = [
    ('spring', 'Optimal', 'Place nodes to minimize crossing and overlapping'),
    ('grid', 'Grid', 'Place nodes on a grid'),
    ('circular', 'Circle', 'Place nodes on a circle')]

WFACT_LAYOUT_TYPE_PARAM = WactFixedMultiValueActionParameter(
    'layouttype', 'Layout type', 'Layout type to apply',
    WactActionStringParameterType(), WFACT_LAYOUT_TYPE_POSSIBLE_VALUES,
    'Select a layout to apply')


class _WfactLayoutAction(WfactActionBase):
    def __init__(self):
        super().__init__('Lay out Schematic',
                         'Apply a layout to the state machine schematic',
                         WFACT_LAYOUT_TYPE_PARAM)

    def _doExecute(self, actx, paramValuePairs):
        """
        :type actx: WactActionExecutionContextP
        """

        layoutType = self._getParamValue(
            WFACT_LAYOUT_TYPE_PARAM, actx, paramValuePairs)
        """:type: str | None"""

        if layoutType == (WFACT_LAYOUT_TYPE_POSSIBLE_VALUES[0])[0]:
            layout = WcmnGraphLayouterSpring(spacing=100)
        elif layoutType == (WFACT_LAYOUT_TYPE_POSSIBLE_VALUES[1])[0]:
            layout = WcmnGraphLayouterGrid()
        elif layoutType == (WFACT_LAYOUT_TYPE_POSSIBLE_VALUES[2])[0]:
            layout = WcmnGraphLayouterCircular()
        elif layoutType is None:
            return WACT_ACTION_EXIT_CANCELLED
        else:
            raise ValueError('Invalid layout type')

        actx.statusMessage('Laying out the schematic...')
        wfact_applyLayout(actx.getProjectContext(), layout)
        actx.statusMessage('Layout applied successfully to schematic')

        return WACT_ACTION_EXIT_OK


def wfact_applyLayout(pctx, layout):
    """
    Apply the passed layout to the whole project

    :param pctx: Project context
    :type pctx: WcorProjectContextP
    :param layout: Layout to be applied
    :type layout: WcmnGraphLayouterP
    """
    topBlk = pctx.getProjectDB().getTopHierarchy()
    """:type: WfhdbTopBlock"""
    if not topBlk:
        raise ValueError("No top block for the current project")

    WfmodLayouter(layout).applyLayout(topBlk)

# : State machine layout action
WFACT_LAYOUT_ACTION = _WfactLayoutAction()
