# ----------------------------------------------------------------------------
# - wfhdb-protocols                                                          -
# - wild cruncher:fsm protocols                     core / statemachine / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfhdb_protocols
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan state machine protocols

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaField, \
    WdbsMetaDescription
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.dbstore_api.wdbs_meta_description_api import *
from wcr_core.dbstore.wdbs_meta_proxy import WdbsListMetaTypeProxy

WFHDB_STATE_ID_KEY = 'state-id'

WFHDB_SOURCE_KEY = 'import-source'

WFHDB_INITIAL_STATE_KEY = 'initial'

WFHDB_FINAL_STATE_KEY = 'final'

WFHDB_HISTORY_STATE_KEY = 'history'

WFHDB_HISTORY_TYPE_KEY = 'type'

WFHDB_HISTORY_TYPE_NONE = None

WFHDB_HISTORY_TYPE_DEEP = 'deep'

WFHDB_HISTORY_TYPE_SHALLOW = 'shallow'

WFHDB_ONENTRY_KEY = 'onentry'

WFHDB_ONEXIT_KEY = 'onexit'

WFHDB_CONTENT_EXPR_KEY = 'expr'

WFHDB_CONTENT_TEXT_KEY = 'content'

WFHDB_PARAM_NAME_KEY = 'name'

WFHDB_PARAM_EXPR_KEY = 'expr'

WFHDB_PARAM_LOCATION_KEY = 'location'

WFHDB_EXEC_ASSIGN_KEY = 'assign'

WFHDB_EXEC_ASSIGN_LOC_KEY = 'loc'

WFHDB_EXEC_ASSIGN_EXPR_KEY = 'expr'

WFHDB_EXEC_SCRIPT_KEY = 'script'

WFHDB_EXEC_LOG_KEY = 'log'

WFHDB_EXEC_LOG_LABEL_KEY = 'label'

WFHDB_EXEC_LOG_EXPR_KEY = 'expr'

WFHDB_EXEC_RAISE_KEY = 'raise'

WFHDB_EXEC_SEND_KEY = 'send'

WFHDB_EXEC_SEND_EVENT_KEY = 'event'

WFHDB_EXEC_SEND_TARGET_KEY = 'target'

WFHDB_EXEC_SEND_TYPE_KEY = 'type'

WFHDB_EXEC_SEND_ID_KEY = 'id'

WFHDB_EXEC_SEND_DELAY_KEY = 'delay'

WFHDB_EXEC_SEND_PARAMS_KEY = 'params'

WFHDB_EXEC_SEND_CONTENT_KEY = 'content'

WFHDB_EXEC_IF_KEY = 'if'

WFHDB_EXEC_IF_COND_KEY = 'cond'

WFHDB_EXEC_IF_EXECS_KEY = 'execs'

WFHDB_EXEC_INVOKE_KEY = 'invoke'

WFHDB_EXEC_INVOKE_TYPE_KEY = 'type'

WFHDB_EXEC_INVOKE_SRC_KEY = 'source'

WFHDB_EXEC_INVOKE_ID_KEY = 'id'

WFHDB_EXEC_INVOKE_AUTOFWD_KEY = 'autofwd'

WFHDB_EXEC_INVOKE_PARAMS_KEY = 'params'

WFHDB_EXEC_INVOKE_FINALIZE_KEY = 'finalize'

WFHDB_EXEC_INVOKE_CONTENT_KEY = 'content'

WFHDB_DATA_KEY = 'data'

WFHDB_DATA_ID_KEY = 'id'

WFHDB_DATA_SRC_KEY = 'source'

WFHDB_DATA_EXPRE_KEY = 'expression'

_WFHDB_EXECUTABLE_META_PROXY = WdbsListMetaTypeProxy()

_WFHDB_EXEC_PARAMS_META = WdbsListMetaType(
    WdbsRecordMetaType(
        (WFHDB_PARAM_NAME_KEY, WDBS_STRING_META),
        (WFHDB_PARAM_EXPR_KEY, WDBS_STRING_META),
        (WFHDB_PARAM_LOCATION_KEY, WDBS_STRING_META)))

_WFHDB_EXEC_CONTENT_META = WdbsRecordMetaType(
    (WFHDB_CONTENT_EXPR_KEY, WDBS_STRING_META),
    (WFHDB_CONTENT_TEXT_KEY, WDBS_TEXT_META))

WFHDB_EXECUTABLE_ASSIGN_META = WdbsRecordMetaType(
    (WFHDB_EXEC_ASSIGN_LOC_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_ASSIGN_EXPR_KEY, WDBS_STRING_META))

WFHDB_EXECUTABLE_IF_BRANCH_META = WdbsRecordMetaType(
    (WFHDB_EXEC_IF_COND_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_IF_EXECS_KEY, _WFHDB_EXECUTABLE_META_PROXY))

WFHDB_EXECUTABLE_IF_META = WdbsListMetaType(
    WFHDB_EXECUTABLE_IF_BRANCH_META)

WFHDB_EXECUTABLE_INVOKE_META = WdbsRecordMetaType(
    (WFHDB_EXEC_INVOKE_TYPE_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_INVOKE_SRC_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_INVOKE_ID_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_INVOKE_AUTOFWD_KEY, WDBS_BOOL_META),
    (WFHDB_EXEC_INVOKE_CONTENT_KEY, _WFHDB_EXEC_CONTENT_META),
    (WFHDB_EXEC_INVOKE_PARAMS_KEY, _WFHDB_EXEC_PARAMS_META),
    (WFHDB_EXEC_INVOKE_FINALIZE_KEY, _WFHDB_EXECUTABLE_META_PROXY))

WFHDB_DATA_META = WdbsRecordMetaType(
    (WFHDB_DATA_ID_KEY, WDBS_STRING_META),
    (WFHDB_DATA_SRC_KEY, WDBS_TEXT_META),
    (WFHDB_DATA_EXPRE_KEY, WDBS_TEXT_META))

WFHDB_EXECUTABLE_LOG_META = WdbsRecordMetaType(
    (WFHDB_EXEC_LOG_LABEL_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_LOG_EXPR_KEY, WDBS_STRING_META))

WFHDB_EXECUTABLE_RAISE_META = WDBS_STRING_META

WFHDB_EXECUTABLE_SCRIPT_META = WDBS_TEXT_META

WFHDB_EXECUTABLE_SEND_META = WdbsRecordMetaType(
    (WFHDB_EXEC_SEND_EVENT_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_SEND_TARGET_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_SEND_TYPE_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_SEND_ID_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_SEND_DELAY_KEY, WDBS_TIME_META),
    (WFHDB_EXEC_SEND_CONTENT_KEY, _WFHDB_EXEC_CONTENT_META),
    (WFHDB_EXEC_SEND_PARAMS_KEY, _WFHDB_EXEC_PARAMS_META))

WFHDB_EXECUTABLE_META_UNION = WdbsUnionMetaType(
    (WFHDB_EXEC_ASSIGN_KEY, WFHDB_EXECUTABLE_ASSIGN_META),
    (WFHDB_EXEC_IF_KEY, WFHDB_EXECUTABLE_IF_META),
    (WFHDB_EXEC_INVOKE_KEY, WFHDB_EXECUTABLE_INVOKE_META),
    (WFHDB_EXEC_LOG_KEY, WFHDB_EXECUTABLE_LOG_META),
    (WFHDB_EXEC_RAISE_KEY, WFHDB_EXECUTABLE_RAISE_META),
    (WFHDB_EXEC_SCRIPT_KEY, WDBS_STRING_META),
    (WFHDB_EXEC_SEND_KEY, WFHDB_EXECUTABLE_SEND_META))

WFHDB_DATA_META_UNION = WdbsUnionMetaType(
    (WFHDB_DATA_KEY, WFHDB_DATA_META))

WFHDB_EXECUTABLE_META = WdbsListMetaType(WFHDB_EXECUTABLE_META_UNION)

_WFHDB_EXECUTABLE_META_PROXY.setProxiedElement(WFHDB_EXECUTABLE_META)


class WfhdbStateP(metaclass=ABCMeta):
    """
    Protocol for state entities.
    """

    @abstractmethod
    def getStateId(self):
        """
        Get the entity's state id.

        :return: The state id of the entity
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def setStateId(self, stateId):
        """
        Set the entity's state id.

        :param stateId: The new state id of the entity
        :type stateId: str
        """
        raise NotImplementedError()


class WfhdbStateMixin(WfhdbStateP):
    """
    Protocol for state entities.
    """

    META = WdbsMetaDescription(
        WdbsMetaField(WFHDB_STATE_ID_KEY, WDBS_STRING_META))

    @property
    @abstractmethod
    def rawElement(self):
        """
        :rtype: WdbsEntityP
        """
        raise NotImplementedError()

    def getStateId(self):
        """
        Get the entity's state id.

        :return: The state id of the entity
        :rtype: str
        """
        return self.rawElement.getMeta(WFHDB_STATE_ID_KEY)

    def setStateId(self, stateId):
        """
        Set the entity's state id.

        :param stateId: The new state id of the entity
        :type stateId: str
        """
        self.rawElement.setMeta(stateId, WFHDB_STATE_ID_KEY)


class WfhdbHasSourceP(metaclass=ABCMeta):
    """
    Protocol for state entities.
    """

    @abstractmethod
    def getSource(self):
        """
        Return the source path the block was imported from, if applicable.

        :return: Source path, if any
        :rtype: str | None
        """
        raise NotImplementedError()

    @abstractmethod
    def setSource(self, source):
        """
        Set the source path the block was imported from, if applicable.

        :param source: Source path, if any
        :type source: str | None
        """
        raise NotImplementedError()


class WfhdbHasSourceMixin(WfhdbHasSourceP):
    """
    Protocol for state entities.
    """

    META = WdbsMetaDescription(
        WdbsMetaField(WFHDB_SOURCE_KEY, WDBS_STRING_META))

    @property
    @abstractmethod
    def rawElement(self):
        """
        :rtype: WdbsEntityP
        """
        raise NotImplementedError()

    def getSource(self):
        """
        Return the source path the block was imported from, if applicable.

        :return: Source path, if any
        :rtype: str | None
        """
        return self.rawElement.getMeta(WFHDB_SOURCE_KEY)

    def setSource(self, source):
        """
        Set the source path the block was imported from, if applicable.

        :param source: Source path, if any
        :type source: str | None
        """
        self.rawElement.setMeta(source, WFHDB_SOURCE_KEY)


class WfhdbMayBeInitialP(metaclass=ABCMeta):
    """
    Protocol for potentially initial state entities.
    """

    @abstractmethod
    def isInitialState(self):
        """
        Whether the state is the initial state for its level.

        :return: True iff this state is the initial state
        :rtype: bool
        """
        raise NotImplementedError()

    @abstractmethod
    def setInitialState(self, initial):
        """
        Set whether the state is the initial state for its level.

        :param initial: If true, the state will be the initial state
        :type initial: bool
        """
        raise NotImplementedError()


class WfhdbMayBeInitialMixin(WfhdbMayBeInitialP):
    """
    Protocol for potentially initial state entities.
    """

    META = WdbsMetaDescription(
        WdbsMetaField(WFHDB_INITIAL_STATE_KEY, WDBS_BOOL_META))

    @property
    @abstractmethod
    def rawElement(self):
        """
        :rtype: WdbsEntityP
        """
        raise NotImplementedError()

    def isInitialState(self):
        """
        Whether the state is the initial state for its level.

        :return: True iff this state is the initial state
        :rtype: bool
        """
        initial = self.rawElement.getMeta(WFHDB_INITIAL_STATE_KEY)
        return initial or False

    def setInitialState(self, initial):
        """
        Set whether the state is the initial state for its level.

        :param initial: If true, the state will be the initial state
        :type initial: bool
        """
        self.rawElement.setMeta(initial, WFHDB_INITIAL_STATE_KEY)


class WfhdbMayBeFinalP(metaclass=ABCMeta):
    """
    Protocol for potentially final state entities.
    """

    @abstractmethod
    def isFinalState(self):
        """
        Whether the state is the final state for its level.

        :return: True iff this state is the final state
        :rtype: bool
        """
        raise NotImplementedError()

    @abstractmethod
    def setFinalState(self, final):
        """
        Set whether the state is the final state for its level.

        :param final: If true, the state will be the final state
        :type final: bool
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WfhdbMayBeFinalMixin(WfhdbMayBeFinalP):
    """
    Protocol for potentially final state entities.
    """

    META = WdbsMetaDescription(
        WdbsMetaField(WFHDB_FINAL_STATE_KEY, WDBS_BOOL_META))

    @property
    @abstractmethod
    def rawElement(self):
        """
        :rtype: WdbsEntityP
        """
        raise NotImplementedError()

    def isFinalState(self):
        """
        Whether the state is the final state for its level.

        :return: True iff this state is the final state
        :rtype: bool
        """
        final = self.rawElement.getMeta(WFHDB_FINAL_STATE_KEY)
        return final or False

    def setFinalState(self, final):
        """
        Set whether the state is the final state for its level.

        :param final: If true, the state will be the final state
        :type final: bool
        """
        self.rawElement.setMeta(final, WFHDB_FINAL_STATE_KEY)


class WfhdbExecutableP(metaclass=ABCMeta):
    """
    Protocol for state entities.
    """

    @abstractmethod
    def getOnEntryExec(self):
        """
        Get the state's on-entry executable text.

        :return: On-entry executables
        :rtype: list[(str, object)] | None
        """
        raise NotImplementedError()

    @abstractmethod
    def setOnEntryExec(self, exctb):
        """
        Set the state's on-entry executables.

        :param exctb: On-entry executables
        :type exctb: list[(str, object)] | None
        """
        raise NotImplementedError()

    @abstractmethod
    def getOnExitExec(self):
        """
        Get the state's on-exit executable text.

        :return: On-exit executables
        :rtype: list[(str, object)] | None
        """
        raise NotImplementedError()

    @abstractmethod
    def setOnExitExec(self, exctb):
        """
        Set the state's on-exit executables.

        :param exctb: On-exit executables
        :type exctb: list[(str, object)] | None
        """
        raise NotImplementedError()


class WfhdbExecutableMixin(WfhdbExecutableP):
    """
    Protocol for state entities.
    """

    META = WdbsMetaDescription(
        WdbsMetaField(WFHDB_ONENTRY_KEY, WFHDB_EXECUTABLE_META),
        WdbsMetaField(WFHDB_ONEXIT_KEY, WFHDB_EXECUTABLE_META))

    @property
    @abstractmethod
    def rawElement(self):
        """
        :rtype: WdbsEntityP
        """
        raise NotImplementedError()

    def getOnEntryExec(self):
        """
        Get the state's on-entry executable text.

        :return: On-entry executables
        :rtype: list[(str, object)] | None
        """
        return self.rawElement.getMeta(WFHDB_ONENTRY_KEY)

    def setOnEntryExec(self, exctb):
        """
        Set the state's on-entry executables.

        :param exctb: On-entry executables
        :type exctb: list[(str, object)] | None
        """
        self.rawElement.setMeta(exctb, WFHDB_ONENTRY_KEY)

    def getOnExitExec(self):
        """
        Get the state's on-exit executable text.

        :return: On-exit executables
        :rtype: list[(str, object)] | None
        """
        return self.rawElement.getMeta(WFHDB_ONEXIT_KEY)

    def setOnExitExec(self, exctb):
        """
        Set the state's on-exit executables.

        :param exctb: On-exit executables
        :type exctb: list[(str, object)] | None
        """
        self.rawElement.setMeta(exctb, WFHDB_ONEXIT_KEY)

class WfhdbMayBeHistoryP(metaclass=ABCMeta):
    """
    Protocol for potentially history state entities.
    """

    @abstractmethod
    def isHistoryState(self):
        """
        Whether the state is the history state for its level.

        :return: True iff this state is the history state
        :rtype: bool
        """
        raise NotImplementedError()

    @abstractmethod
    def setHistoryState(self, history):
        """
        Set whether the state is the history state for its level.

        :param history: If true, the state will be the history state
        :type history: bool
        """
        raise NotImplementedError()

    def getHistoryType(self):
        """
        Get the type of the history for its level..

        :param type: the type of history node
        :type type: str
        """
        raise NotImplementedError()

    def setHistoryType(self, type):
        """
        Set the type of history for its level.

        :param type: If true, the state will be the history state
        :type type: str
        """
        raise NotImplementedError()

# noinspection PyAbstractClass
class WfhdbMayBeHistoryMixin(WfhdbMayBeHistoryP):
    """
    Protocol for potentially history state entities.
    """

    META = WdbsMetaDescription(
        WdbsMetaField(WFHDB_HISTORY_STATE_KEY, WDBS_BOOL_META),
        WdbsMetaField(WFHDB_HISTORY_TYPE_KEY, WDBS_STRING_META))

    @property
    @abstractmethod
    def rawElement(self):
        """
        :rtype: WdbsEntityP
        """
        raise NotImplementedError()

    def isHistoryState(self):
        """
        Whether the state is the history state for its level.

        :return: True iff this state is the history state
        :rtype: bool
        """
        history = self.rawElement.getMeta(WFHDB_HISTORY_STATE_KEY)
        return history or False

    def setHistoryState(self, history):
        """
        Set whether the state is the history state for its level.

        :param history: If true, the state will be the history state
        :type history: bool
        """
        self.rawElement.setMeta(history, WFHDB_HISTORY_STATE_KEY)


    def getHistoryType(self):
        """
        Get the type of the history for its level..

        :param type: the type of history node
        :type type: str
        """
        return self.rawElement.getMeta(WFHDB_HISTORY_TYPE_KEY)

    def setHistoryType(self, type):
        """
        Set the type of history for its level.

        :param type: If true, the state will be the history state
        :type type: str
        """
        self.rawElement.setMeta(type, WFHDB_HISTORY_TYPE_KEY)

