# ----------------------------------------------------------------------------
# - wfhdb-standard-operators                                                 -
# - wild cruncher:standard operators                core / statemachine / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfhdb_standard_operators
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher state machine hierarchy standard operators

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_operator import WhdbOperator
from wcr_core.db_api.whdb_connection_point_api import WhdbConnectionPointP
from wcr_core.db_api.whdb_protocols_api import WHDB_HALIGN_CENTER
from wcr_core.db.whdb_connection_point import WHDB_DEFAULT_XPOINT_CODE
from wcr_core.db.whdb_connection_point_manager import \
    WhdbFixedConnectionPointManager, WhdbXPointDesc
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_IN, \
    WDBS_XPNT_OUT
from wcr_core.modules.statemachine.db.wfhdb_operator import WFHDB_OPER, WfhdbOperator
from wcr_core.modules.statemachine.db.wfhdb_connection_point_manager import \
    WfhdbConnectionPointManager


WFHDB_STATE_CODE = 'wcr.statemachine.std.state'

WFHDB_XBT_TARGET_PROXY_CODE = 'wcr.statemachine.std.xbt-target'

WFHDB_XBT_SOURCE_PROXY_CODE = 'wcr.statemachine.std.xbt-source'

#: State operator
WFHDB_STATE = WfhdbOperator(
    code=WFHDB_STATE_CODE,
    xmng=WfhdbConnectionPointManager(),
    name='State',
    info='State',
    symbol='',
    labelAlign=WHDB_HALIGN_CENTER)

WFHDB_XBT_TARGET_PROXY = WfhdbOperator(
    code=WFHDB_XBT_TARGET_PROXY_CODE,
    xmng=WhdbFixedConnectionPointManager([
        WhdbXPointDesc(
            ctyp=WHDB_DEFAULT_XPOINT_CODE, cdir=WDBS_XPNT_IN,
            name='IN', cpos=(-1.0, 0.0), fcng=WhdbConnectionPointP.West)]),
    name='Out-Transition',
    info='',
    symbol='OUT',
    labelAlign=WHDB_HALIGN_CENTER)

WFHDB_XBT_SOURCE_PROXY = WfhdbOperator(
    code=WFHDB_XBT_SOURCE_PROXY_CODE,
    xmng=WhdbFixedConnectionPointManager([
        WhdbXPointDesc(
            ctyp=WHDB_DEFAULT_XPOINT_CODE, cdir=WDBS_XPNT_OUT,
            name='OUT', cpos=(1.0, 0.0), fcng=WhdbConnectionPointP.East)]),
    name='In-Transition',
    info='',
    symbol='IN',
    labelAlign=WHDB_HALIGN_CENTER)

WFHDB_XBT_PROXY_OPS = (WFHDB_XBT_TARGET_PROXY, WFHDB_XBT_SOURCE_PROXY)

WFHDB_XBT_PROXY_OP_CODES = \
    (WFHDB_XBT_TARGET_PROXY_CODE, WFHDB_XBT_SOURCE_PROXY_CODE)

#: State machine default operators
WFHDB_DEFAULT_OPS = [WFHDB_STATE]

WFHDB_OPER.addOperators(WFHDB_DEFAULT_OPS)
WFHDB_OPER.addOperator(WFHDB_XBT_TARGET_PROXY)
WFHDB_OPER.addOperator(WFHDB_XBT_SOURCE_PROXY)
