# ----------------------------------------------------------------------------
# - wfhdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry        core / statemachine / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfhdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher state machine hierarchy raw data wrapper
                  registry

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.dbstore_api.wdbs_connection_api import WdbsConnectionP
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP, \
    WdbsBlockInstanceP
from wcr_core.modules.statemachine.db.wfhdb_transition import WfhdbTransition
from wcr_core.modules.statemachine.db.wfhdb_instance import *
from wcr_core.modules.statemachine.db.wfhdb_block import *
from wcr_core.modules.statemachine.db.wfhdb_standard_operators import *


class WfhdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """

    def createWrapper(self, obj, *args):
        if isinstance(obj, WdbsOperatorInstanceP):
            oper = obj.getOperator()
            if obj.getOperator() == WFHDB_STATE_CODE:
                return WfhdbStateInstance(obj, self)
            elif obj.getOperator() == WFHDB_XBT_SOURCE_PROXY_CODE:
                return WfhdbXbtSourceProxyInstance(obj, self)
            elif obj.getOperator() == WFHDB_XBT_TARGET_PROXY_CODE:
                return WfhdbXbtTargetProxyInstance(obj, self)
            else:
                return WhdbOperatorInstance(WFHDB_OPER[oper], obj, self)
        elif isinstance(obj, WdbsBlockInstanceP):
            return WfhdbBlockInstance(obj, self)
        elif isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WFHDB_PARALLEL_BLOCK_CODE:
                return WfhdbParallelBlock(obj, self)
            elif obj.getEntityType() == WFHDB_NESTED_BLOCK_CODE:
                return WfhdbNestedBlock(obj, self)
            else:
                return WfhdbTopBlock(obj, self)
        elif isinstance(obj, WdbsConnectionP):
            return WfhdbTransition(obj, self)
        else:
            return super().createWrapper(obj, *args)
