# ----------------------------------------------------------------------------
# - wfhdb-xbt-utils                                                          -
# - wild cruncher: xbt database utility functions   core / statemachine / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfhdb_xbt_utils
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - XBT database utility functions

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.modules.statemachine.db.wfhdb_instance import \
    WfhdbXbtTargetProxyInstance, WfhdbXbtSourceProxyInstance
from wcr_core.modules.statemachine.db.wfhdb_transition import WfhdbTransition


class WfhdbXbtException(Exception):
    def __init__(self, label, description):
        """
        :type label: str
        :type description: str
        """
        super().__init__(label)

        self.__label = label
        self.__description = description

    def getLabel(self):
        """
        :rtype: str
        """
        return self.__label

    def getDescription(self):
        """
        :rtype: str
        """
        return self.__description

    def __str__(self):
        return '{} ({})'.format(self.__label, self.__description)


def wfhdb_is_xbt_state(state):
    """
    Whether the given state is a XBT-state.

    :param state: State to test for XBT status
    :type state: WfhdbStateP | WhdbInstanceP
    :return: True iff the given state is a XBT-state
    :rtype: bool
    """
    return isinstance(state, WfhdbXbtTargetProxyInstance) or \
        isinstance(state, WfhdbXbtSourceProxyInstance)


def wfhdb_is_xbt_transition(transition):
    """
    Whether the given state is a XBT-transition.

    :param transition: Transition to test for XBT status
    :type transition: WfhdbTransition
    :return: True iff the given transition is a XBT-transition
    :rtype: bool
    """
    return wfhdb_is_xbt_state(transition.getStartObject()) or \
        wfhdb_is_xbt_state(transition.getEndObject())


def wfhdb_is_xbt_entity(entity):
    """
    Whether the given entity is a XBT-entity.

    :param entity: Entity to test for XBT status
    :type entity: WhdbBaseP
    :return: True iff the given entity is a XBT-entity
    :rtype: bool
    """
    if isinstance(entity, WfhdbTransition):
        return wfhdb_is_xbt_transition(entity)
    return isinstance(entity, WfhdbXbtTargetProxyInstance) or \
        isinstance(entity, WfhdbXbtSourceProxyInstance)
