# ----------------------------------------------------------------------------
# - wfhdb-xmlio                                                              -
# - wild cruncher:wsh xml serialization/deser.      core / statemachine / db -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfhdb_xmlio
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher model DB xml serialization/deserialization

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
import traceback
from xml.etree.ElementTree import ElementTree

from wcr_core.common.wcmn_file_handler import WcmnFileReaderBase
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_MODEL_PORTS, \
    WCMN_PAYLOAD_ENTRY_POINTS
from wcr_core.xmlio.wxio_xmlio import wxio_deserializeXml
from wcr_core.project.wprj_xmlio import WPRJ_SILKAN_XML_MIME_TYPE, \
    WPRJ_SILKAN_XML_MIME_NAME


class WfhdbSilkanXmlReader(WcmnFileReaderBase):
    def __init__(self):
        super().__init__(
            [WPRJ_SILKAN_XML_MIME_TYPE],
            WCMN_PAYLOAD_MODEL_PORTS | WCMN_PAYLOAD_ENTRY_POINTS)

    @property
    def name(self):
        return 'Silkan StateMachine-XML'

    def readText(self, text, payloadHandler, xctx, types=None):
        return super().readText(text, payloadHandler, xctx, types)

    def readFile(self, file, payloadHandler, xctx, types=None):
        from ..wfmod_application_module import WFMOD_STATEMACHINE_APP_CODE, \
            WFMOD_STATEMACHINE_APP_MODULE

        typesRead = 0
        if not types:
            types = payloadHandler.handledPayloadTypes

        tree = ElementTree(file=file.path)
        hnod = tree.getroot().find('hierarchy')
        if hnod is not None and \
                hnod.get('module') != WFMOD_STATEMACHINE_APP_CODE:
            return typesRead

        try:
            model = WFMOD_STATEMACHINE_APP_MODULE.createTopLevelBlock()
            wxio_deserializeXml(
                tree, 'tophm', model.rawElement)

            if types & WCMN_PAYLOAD_ENTRY_POINTS:
                typesRead |= WCMN_PAYLOAD_ENTRY_POINTS
                payloadHandler.handlePayload(
                    WCMN_PAYLOAD_ENTRY_POINTS, WPRJ_SILKAN_XML_MIME_NAME,
                    [model.getName() or 'StateMachine'])
            if types & WCMN_PAYLOAD_MODEL_PORTS:
                typesRead |= WCMN_PAYLOAD_MODEL_PORTS
                payloadHandler.handlePayload(
                    WCMN_PAYLOAD_MODEL_PORTS, WPRJ_SILKAN_XML_MIME_NAME,
                    lambda x: model.getPortNames())
        except:
            traceback.print_exc()
        return typesRead


WPRJ_SILKAN_XML_MIME_TYPE.addReader(WfhdbSilkanXmlReader())
