# ----------------------------------------------------------------------------
# - wfmod-application-module                                                 -
# - wild cruncher: application module                    core / statemachine -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfmod_application_module
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher state machine application module

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
import wcr_core.modules.statemachine.db.wfhdb_xmlio
from wcr_core.action.wact_action_api import WACT_WCR_ACTION_CODE,\
    WACT_ACTION_W_DB_MODIFY
from wcr_core.db.whdb_top_level_factory import whdb_createTopLevelRawBlock
from wcr_core.modules.statemachine.db.wfhdb_meta_provider import \
    WfhdbMetaProvider
from wcr_core.modules.statemachine.db.wfhdb_wrapper_registry import \
    WfhdbWrapperRegistry
from wcr_core.modules.statemachine.db.wfhdb_verifier import \
    WfhdbHierarchyVerifier
from wcr_core.modules.wmod_block_type import WmodBlockType
from wcr_core.modules.wmod_application_module import \
    WmodApplicationModuleBase, WmodApplicationModuleP, WmodApplicationModuleMP
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_STATEMACHINE
from wcr_core.modules.statemachine.wfmod_project_context import \
    WfmodStatemachineProjectContext
from wcr_core.modules.statemachine.wfmod_event_manager import WfmodEventManager
from wcr_core.modules.statemachine.db.wfhdb_block import \
    WFHDB_NESTED_BLOCK_CODE, WFHDB_PARALLEL_BLOCK_CODE, WfhdbNestedBlock, \
    WfhdbParallelBlock
from wcr_core.modules.statemachine.db.wfhdb_operator import WFHDB_OPER
from wcr_core.modules.statemachine.db.wfhdb_standard_operators import \
    WFHDB_DEFAULT_OPS
from wcr_core.modules.statemachine.actions.wfact_layout_action import \
    WFACT_LAYOUT_ACTION


WFMOD_STATEMACHINE_APP_CODE = 'statemachine'


# noinspection PyAbstractClass
class WfmodStatemachineApplicationModuleP(WmodApplicationModuleP):
    pass


# noinspection PyAbstractClass
class WfmodStatemachineApplicationModuleMP(
        WmodApplicationModuleMP, WfmodStatemachineApplicationModuleP):
    pass


class _WfmodStatemachineApplicationModule(
        WmodApplicationModuleBase, WfmodStatemachineApplicationModuleMP):
    _BLOCK_TYPES = (
        WmodBlockType(
            WFHDB_NESTED_BLOCK_CODE, 'Nested block',
            WfhdbNestedBlock.META, WfhdbNestedBlock),
        WmodBlockType(
            WFHDB_PARALLEL_BLOCK_CODE, 'Parallel block',
            WfhdbParallelBlock.META, WfhdbParallelBlock))

    def __init__(self, operRegistry):
        super().__init__(
            code=WFMOD_STATEMACHINE_APP_CODE,
            name='State Machine',
            blockTypes=self._BLOCK_TYPES,
            defaultOps=WFHDB_DEFAULT_OPS,
            operRegistry=operRegistry)

        self._createDefaultActions()

    def createHierarchyVerifier(self, pctx):
        return WfhdbHierarchyVerifier(pctx, self._getDbVerifierModules())

    def getPayloadType(self):
        return WCMN_PAYLOAD_BLOCK_STATEMACHINE

    def createTopLevelBlock(self, name=None, info=None):
        return wfmod_create_top_block(name, info)

    def createProjectContext(self, gContext, projectDB, logger=None):
        tblk = projectDB.getTopHierarchy()
        """:type: WfhdbTopBlock"""
        eventMngr = WfmodEventManager(tblk)
        return WfmodStatemachineProjectContext(
            gContext, self, eventMngr, projectDB, logger)

    def _createDefaultActions(self):
        actnSection = '{}.core.{}'.format(
            WACT_WCR_ACTION_CODE, WFMOD_STATEMACHINE_APP_CODE)
        self.addAction(
            WFACT_LAYOUT_ACTION, 'layout', actnSection,
            WACT_ACTION_W_DB_MODIFY)


def wfmod_create_top_block(name=None, info=None):
    rgty = WfhdbWrapperRegistry()
    tblk = whdb_createTopLevelRawBlock(WfhdbMetaProvider(), name, info)
    return rgty.createWrapper(tblk)


#: Application module singleton
WFMOD_STATEMACHINE_APP_MODULE = _WfmodStatemachineApplicationModule(WFHDB_OPER)
