# ----------------------------------------------------------------------------
# - wfmod-project-context                                                    -
# - wild cruncher - fsm project context                  core / statemachine -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfmod_project_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher state machine project context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod

from wcr_core.modules.statemachine.controller.wfctl_controllers import \
    WfctlControllerFactory
from wcr_core.modules.statemachine.db.wfhdb_xbt_manager import WfhdbXbtManager
from wcr_core.wcor_context import WcorProjectContextP
from wcr_core.project.wprj_project_context import WprjProjectContext


# noinspection PyAbstractClass
class WfmodStatemachineProjectContextP(WcorProjectContextP):
    @abstractmethod
    def getEventManager(self):
        """
        Get the event manager.

        :return: Event manager
        :rtype: WfmodEventManager
        """
        raise NotImplementedError()

    @abstractmethod
    def getXbtManager(self):
        """
        Get the project's path manager / UUID resolver.

        :return: Path manager / UUID resolver for the project
        :rtype: WfhdbXbtManager
        """
        raise NotImplementedError()


class WfmodStatemachineProjectContext(WprjProjectContext,
                                      WfmodStatemachineProjectContextP):
    """
    Default project context implementation.
    """

    def __init__(self, gContext, module, evntMngr, projectDB, logger=None):
        """
        :param gContext: Global context
        :type gContext: WcorGlobalContextP
        :param module: Application module
        :type module: WmodApplicationModuleP
        :param evntMngr: Event manager
        :type evntMngr: WfmodEventManager
        :param projectDB: Project database
        :type projectDB: WprjProjectDatabase
        :param logger: Logger
        :type logger: ScmnLoggerP
        """
        super().__init__(gContext, module, projectDB, logger)

        self.__eventMngr = evntMngr

        tblk = projectDB.getTopHierarchy()
        """:type: WfhdbTopBlock"""
        self.__xbtManager = WfhdbXbtManager(tblk)
        self.__ctrlFcty = WfctlControllerFactory(self)

    def getEventManager(self):
        return self.__eventMngr

    def getXbtManager(self):
        return self.__xbtManager

    def getControllerFactory(self):
        return self.__ctrlFcty
