# ----------------------------------------------------------------------------
# - wcact-import_hardware-action                                             -
# - wild cruncher: system mapper hardware import action                      -
#                                                   core / sys_map / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcact_import_hardware_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - System mapper import hardware action

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""
from wcr_core.action.wact_action_parameter import WactActionParameter
from wcr_core.action.wact_action_parameter_types import \
    WactActionOpenFileByPayloadParameterType
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_HARDWARE
from wcr_core.modules.sys_map.actions.wcact_action_base import WcactActionBase
from wcr_core.modules.sys_map.actions.wcact_reload_hardware_action \
    import wcact_reload_Hardware


WCACT_HARDWARE_FILE_PARAMETER = WactActionParameter(
    'source', 'Hardware Design', 'Hardware Design file to import',
    WactActionOpenFileByPayloadParameterType(
        WCMN_PAYLOAD_BLOCK_HARDWARE))


class _WcactImportHardwareDesignAction(WcactActionBase):
    """
    Project editor action to set & load the hardware source.
    """

    def __init__(self):
        super().__init__(
            'Set Hardware...',
            'Set & load the Hardware for this system project',
            WCACT_HARDWARE_FILE_PARAMETER)

    def _doExecute(self, actx, paramValuePairs):
        file = self._getParamValue(
            WCACT_HARDWARE_FILE_PARAMETER, actx, paramValuePairs)
        """:type: WcmnFile | None"""
        if not file:
            return
        pctx = actx.getProjectContext()
        """:type: WpmodSystemProjectContextMP"""
        tblk = pctx.getProjectDB().getTopHierarchy()
        """:type: WphdbTopBlock"""
        tblk.setHardwareFile(file)
        wcact_reload_Hardware(pctx)


#: Logical simulation load action
WCACT_IMPORT_HARDWARE_ACTION = _WcactImportHardwareDesignAction()

