# ----------------------------------------------------------------------------
# - wcact-import_logical-action                                              -
# - wild cruncher: system mapper logical import action                       -
#                                                   core / sys_map / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2019-2019 achraf mahti                                     -
# - copyright (c) 2014-2019 silkan / CETRAC                                  -
# ----------------------------------------------------------------------------

"""
.. module::       wcact_import_hardware_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - System mapper import hardware action

.. moduleauthor:: achraf mahti <achraf@cetrac.io>
"""
from wcr_core.action.wact_action_parameter import WactActionParameter
from wcr_core.action.wact_action_parameter_types import \
    WactActionOpenFileByPayloadParameterType
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_ICD
from wcr_core.modules.sys_map.actions.wcact_action_base import WcactActionBase
from wcr_core.modules.sys_map.actions.wcact_reload_logic_action \
    import wcact_reload_logical
from wcr_frontend.gui.wgui_utils import wgui_getOpenFile
from wcr_core.project.wprj_xmlio import WPRJ_ICD_SILKAN_XML_MIME_TYPE


WCACT_LOGICAL_FILE_PARAMETER = WactActionParameter(
    'source', 'Logic Design', 'Logical Design file to import',
    WactActionOpenFileByPayloadParameterType(
        WCMN_PAYLOAD_BLOCK_ICD))


class _WcactImportLogicalDesignAction(WcactActionBase):
    """
    Project editor action to set & load the hardware source.
    """

    def __init__(self):
        super().__init__(
            'Set Logical...',
            'Set & load the Logical for this system project',
            WCACT_LOGICAL_FILE_PARAMETER)

    def _doExecute(self, actx, paramValuePairs):
        file = wgui_getOpenFile("Import Hardware", actx.getEditor(),
                                WPRJ_ICD_SILKAN_XML_MIME_TYPE)

        """:type: WcmnFile | None"""
        if not file:
            return
        pctx = actx.getProjectContext()
        """:type: WpmodSystemProjectContextMP"""
        tblk = pctx.getProjectDB().getTopHierarchy()
        """:type: WphdbTopBlock"""
        tblk.setLogicFile(file)
        wcact_reload_logical(pctx)


#: Logical simulation load action
WCACT_IMPORT_LOGICAL_ACTION = _WcactImportLogicalDesignAction()

