# ----------------------------------------------------------------------------
# - wcact-map-checker-action                                                 -
# - wild cruncher: reload hardware source action    core / sys_map / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 Philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcact_map_checker_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Binding checker action

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_core.modules.sys_map.actions.wcact_action_base import WcactActionBase
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import \
    WVHDB_VLINK_TYPE_AFDX, WVHDB_VLINK_TYPE_CETRAC, WVHDB_VLINK_TYPE_ETNERNET
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_CETRAC_CODE, \
    WHHDB_XPOINT_ETHERNET_CODE

PHY_PORT_LOG_PORT_SUPPORT=[]

WCACT_ACTION_BIND_VERIFY='bindverif'

class WchdbBindsException(Exception):
    def __init__(self, elements, message):
        super().__init__(message)

        self.__elements = elements

    @property
    def elements(self):
        return iter(self.__elements)

class _WcactCheckMappingAction(WcactActionBase):
    """
    Project editor action to check the mapping
    """

    def __init__(self):
        super().__init__(
            'Check mapping',
            'Check if the user mapping is correct')

    def _isProtocolException(self, e):
        return isinstance(e, WchdbBindsException)

    def _doExecute(self, actx, paramValuePairs):
        pctx = actx.getProjectContext()
        """:type: WpmodSystemProjectContextMP"""
        wrongs=wcact_Check_Mapping_Hardware(pctx)
        if len(wrongs) > 0:
            mess = "Folowing ports binding are incorrect :\n"
            for ppn,lpn in wrongs:
                 mess += " "+ppn+" - "+lpn+"\n"
            raise WchdbBindsException(wrongs,mess[:-1])


def wcact_Check_Mapping_Hardware(pctx):
    """
    Check the VL mapping

    :param pctx: Project context
    :type pctx: WpmodSystemProjectContextMP
    """
    tblk = pctx.getProjectDB().getTopHierarchy()
    """:type: WphdbTopBlock"""
    mapper = pctx.getMappingManager()
    errorbind=[]
    usrpbind=tblk.getProdBinds() + tblk.getConsBinds()
#    for ppuuid, vluuid in usrpbind:
#        if (mapper.getPhyPortByUUID(ppuuid).getCode(),
#            mapper.getVlinkByUUID(vluuid).getVLType()) not in \
#            PHY_PORT_LOG_PORT_SUPPORT:
#            errorbind.append((mapper.getPhyPortByUUID(ppuuid).getName(),
#                              mapper.formatName(mapper.getVlinkByUUID(vluuid)))
#                             )
    return errorbind



#: Logical simulation check action
WCACT_CHECK_MAPPING_ACTION = _WcactCheckMappingAction()
