# ----------------------------------------------------------------------------
# - wcact-reload-hardware-action                                             -
# - wild cruncher: reload hardware source action    core / sys_map / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 Philippe lambert                                 -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcact_reload_hardware_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Hardware source reload action

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
import os
from wcr_core.modules.hardware.db.whhdb_top_block import WhhdbTopHierarchy
from wcr_core.modules.hardware.whmod_application_module import \
    WHMOD_HARDWARE_APP_CODE
from wcr_core.modules.sys_map.actions.wcact_action_base import WcactActionBase
from wcr_core.project.wprj_project import wprj_load_project


class _WcactReloadHardwareAction(WcactActionBase):
    """
    Project editor action to reload the hardware source.
    """

    def __init__(self):
        super().__init__(
            'Reload Hardware...',
            'Reload the hardware from disk')

    def _doExecute(self, actx, paramValuePairs):
        pctx = actx.getProjectContext()
        """:type: WpmodSystemProjectContextMP"""
        wcact_reload_Hardware(pctx)


def wcact_reload_Hardware(pctx):
    """
    Reload the hardware source.

    :param pctx: Project context
    :type pctx: WpmodSystemProjectContextMP
    """
    tblk = pctx.getProjectDB().getTopHierarchy()
    """:type: WphdbTopBlock"""
    file = tblk.getHardwareFile()
    if os.path.isfile(str(file)):
        pdb = wprj_load_project(file, pctx)
        if pdb is not None and pdb.getModule() == WHMOD_HARDWARE_APP_CODE:
            simulation = pdb.getTopHierarchy() if pdb else None
            assert isinstance(simulation, WhhdbTopHierarchy)
            pctx.setHardwareFile(simulation)
            return

    pctx.setHardwareFile(None)


#: Logical simulation reload action
WCACT_RELOAD_HARDWARE_ACTION = _WcactReloadHardwareAction()
