# ----------------------------------------------------------------------------
# - wchdb-host-decoy-instance                                                -
# - wild cruncher: Host decoy instances                  core / sys_map / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wchdb_switch_lure_instance
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Host decoy instance

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.sys_map.db.wchdb_standard_operators import \
    WCHDB_SWITCH_LURE, WCHDB_SWITCH_LURE_CODE
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_DIRECTION, \
    WHDB_XPOINT_NAME
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI

WCHDB_SOURCE_UUID_KEY = "uuid_src"

class WchdbSwitchLureInstance(WhdbOperatorInstance):
    """
    Operator-instance for bus instances.
    """

    META = WdbsMetaDescription(
        WhdbOperatorInstance.META,
        WdbsMetaField(WCHDB_SOURCE_UUID_KEY, WDBS_UUID_META),)

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        assert inst.getOperator() == WCHDB_SWITCH_LURE_CODE
        WhdbOperatorInstance.__init__(self, WCHDB_SWITCH_LURE, inst, wreg)
        self.__ioCards = []
        self.__ioBinding = {}

    def _createRawConnectionPoint(self, ctyp, **kwargs):
        if WHDB_XPOINT_DIRECTION not in kwargs:
            kwargs[WHDB_XPOINT_DIRECTION] = WDBS_XPNT_BI
        return super()._createRawConnectionPoint(ctyp, **kwargs)

    def setIoCards(self, ioCards):
        self.__ioCards = ioCards

    def getIoCards(self):
        return self.__ioCards

    def setIoCardBind(self, ioBinding, card):
        self.__ioBinding[ioBinding] = card

    def getIoCardBind(self):
        return self.__ioBinding

    def setUUIDsource(self, uuid):
        self.rawElement.setMeta(uuid, WCHDB_SOURCE_UUID_KEY)

    def getUUIDsource(self):
        return self.rawElement.getMeta(WCHDB_SOURCE_UUID_KEY)
