# ----------------------------------------------------------------------------
# - wchdb-connection-points                                                  -
# - wild cruncher: connection points                     core / sys_map / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wchdb_connection_points
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Sys_map connection points

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_TYPE, \
    WHDB_XPOINT_DIRECTION
from wcr_core.dbstore.wdbs_meta_type import WdbsRecordMetaType, \
    WDBS_STRING_META
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.sys_map.db.wchdb_connection_point_base import \
    WchdbConnectionPointBase
from wcr_core.modules.sys_map.db.wchdb_connection_point_base_meta import \
    WCHDB_SRC_UUID_PARAMETER, WCHDB_SRC_UUID_KEY, WCHDB_PORT_SPEED_KEY, WCHDB_PORT_CARD_KEY,\
    WCHDB_PORT_SPEED_PARAMETER, WCHDB_PORT_CODE_PARAMETER, WCHDB_PORT_CODE_KEY,\
    WCHDB_PORT_ACTIVE_KEY, WCHDB_PORT_ACTIVE_PARAMETER, WCHDB_PORT_CARD_PARAMETER

WCHDB_XPOINT_STD_CODE = 'wcr.sys_map.std-xpoint'

WCHDB_XPOINT_STD_KEY_INTERFACE = 'standard-interface'

WCHDB_XPOINT_STD_CONFIG_META = WdbsRecordMetaType(
    (WCHDB_XPOINT_STD_KEY_INTERFACE, WDBS_STRING_META, 'Standard interface'),
    WCHDB_SRC_UUID_PARAMETER,
    WCHDB_PORT_SPEED_PARAMETER,
    WCHDB_PORT_CODE_PARAMETER,
    WCHDB_PORT_CARD_PARAMETER,
    WCHDB_PORT_ACTIVE_PARAMETER,
)
WCHDB_XPOINT_STD_DFLT_ARGS = {
    WHDB_XPOINT_TYPE: WCHDB_XPOINT_STD_CODE,
    WHDB_XPOINT_DIRECTION: WDBS_XPNT_BI
}


class WchdbStdXPoint(WchdbConnectionPointBase):
    """
    Standart connection point. will correspond to anny hardware connection point
    """

    __description = 'No description set.'

    def getInterface(self):
        return self._getConfig(WCHDB_XPOINT_STD_KEY_INTERFACE)

    def getTypeName(self):
        return 'Standart connection point'

    def getNamePrefix(self):
        return 'Standart'

    def getDescription(self, **kwargs):
        return self.__description

    def getUUIDSrc(self):
        return self._getConfig(WCHDB_SRC_UUID_KEY)

    def setUUIDSrc(self, valuuid):
        self._setConfig(valuuid, WCHDB_SRC_UUID_KEY)

    def setDescription(self, desc):
        if desc is not None:
            self.__description = desc
        else:
            self.__description = 'No description set.'

    def getSpeed(self):
        return self._getConfig(WCHDB_PORT_SPEED_KEY)

    def setSpeed(self, debit):
        self._setConfig(debit, WCHDB_PORT_SPEED_KEY)

    def getCode(self):
        return self._getConfig(WCHDB_PORT_CODE_KEY)

    def setCode(self, debit):
        self._setConfig(debit, WCHDB_PORT_CODE_KEY)

    def getCard(self):
        return self._getConfig(WCHDB_PORT_CARD_KEY)

    def setCard(self, debit):
        self._setConfig(debit, WCHDB_PORT_CARD_KEY)

    def getActive(self):
        return self._getConfig(WCHDB_PORT_ACTIVE_KEY)

    def setActive(self, active):
        self._setConfig(active, WCHDB_PORT_ACTIVE_KEY)