# ----------------------------------------------------------------------------
# - wchdb-meta-provider                                                      -
# - wild cruncher: meta-data provider                    core / sys_map / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wchdb_meta_provider
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy meta-data provider

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_meta_provider import WhdbMetaProvider
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaDescription, \
    WdbsMetaField
from wcr_core.modules.sys_map.db.wchdb_connection_point_base import \
    WchdbConnectionPointBase, WCHDB_XPOINT_CONFIG_KEY
from wcr_core.modules.sys_map.db.wchdb_standard_operators import \
    WCHDB_HOST_LURE_CODE, WCHDB_SWITCH_LURE_CODE, WCHDB_RACK_LURE_CODE
from wcr_core.modules.sys_map.db.wchdb_Host_lure_instance import \
    WchdbHostLureInstance
from wcr_core.modules.sys_map.db.wchdb_rack_lure_instance import \
    WchdbRackLureInstance
from wcr_core.modules.sys_map.db.wchdb_Switch_lure_instance import \
    WchdbSwitchLureInstance
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import \
    WvhdbVLinkInstance
from wcr_core.modules.vlink.db.wvhdb_standard_operators import \
    WVHDB_VLINK_CODE

WCHDB_STD_OPERINST_META={WCHDB_HOST_LURE_CODE:WchdbHostLureInstance.META,
                        WCHDB_RACK_LURE_CODE:WchdbRackLureInstance.META,
                        WCHDB_SWITCH_LURE_CODE:WchdbSwitchLureInstance.META,
                        WVHDB_VLINK_CODE: WvhdbVLinkInstance.META}


class WchdbMetaProvider(WhdbMetaProvider):
    def __init__(self, blockMetas, xpntCfgMeta):
        super().__init__()

        self.__blockMetas= blockMetas
        #: Connection point configuration meta-data
        self.__xpntCfgMeta = xpntCfgMeta
        #: Connection point meta-data cache (incl. base and config)
        self.__xpntMeta = {}

    def getBlockMeta(self, btyp=None):
        if btyp:
            meta = self.__blockMetas.get(btyp)
            if meta:
                return meta
        return WchdbBlock.META

    def getOperatorInstanceMeta(self, oper=None):
        opMeta = WCHDB_STD_OPERINST_META.get(oper)
        return opMeta if opMeta is not None else WhdbOperatorInstance.META

    def getConnectionPointMeta(self, xtyp=None):
        if not xtyp:
            return WchdbConnectionPointBase.META

        meta = self.__xpntMeta.get(xtyp)
        if meta:
            return meta

        cfgMeta = self.__xpntCfgMeta.get(xtyp)
        if not cfgMeta:
            return WchdbConnectionPointBase.META

        meta = WdbsMetaDescription(
            WchdbConnectionPointBase.META,
            WdbsMetaField(WCHDB_XPOINT_CONFIG_KEY, cfgMeta))
        self.__xpntMeta[xtyp] = meta
        return meta