# ----------------------------------------------------------------------------
# - wchdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry             core / sys_map / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wchdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy raw data wrapper registry

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP, \
    WdbsBlockInstanceP, WdbsModelInstanceP

from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.modules.sys_map.db.wchdb_operator import WCHDB_OPER
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.modules.sys_map.db.wchdb_top_block import WchdbTopHierarchy
from wcr_core.modules.sys_map.db.wchdb_block import WchdbBlock
from wcr_core.db.whdb_block import WHDB_TOPLEVEL_BLOCK_CODE
from wcr_core.dbstore_api.wdbs_connection_point_api import WdbsConnectionPointP
from wcr_core.modules.sys_map.db.wchdb_standard_operators import \
    WCHDB_HOST_LURE_CODE, WCHDB_SWITCH_LURE_CODE, WCHDB_RACK_LURE,\
    WCHDB_RACK_LURE_CODE
from wcr_core.modules.sys_map.db.wchdb_Host_lure_instance import \
    WchdbHostLureInstance
from wcr_core.modules.sys_map.db.wchdb_rack_lure_instance import \
    WchdbRackLureInstance
from wcr_core.modules.sys_map.db.wchdb_Switch_lure_instance import \
    WchdbSwitchLureInstance

WCHDB_STD_OPERINST_CLS= {WCHDB_HOST_LURE_CODE: WchdbHostLureInstance,
                         WCHDB_RACK_LURE_CODE: WchdbRackLureInstance,
                         WCHDB_SWITCH_LURE_CODE: WchdbSwitchLureInstance}

class WchdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """
    def __init__(self,blockClzzs, xpntClzzs):
        """
        :param xpntClzzs: Connection point classes/constructors, by code
        :type xpntClzzs: dict[str, () -> WphdbConnectionPointBase]
        """
        super().__init__()

        self.__blockClzzs = blockClzzs
        self.__xpntClzzs = xpntClzzs

    def createWrapper(self, obj, **args):

        if isinstance(obj, WdbsOperatorInstanceP):
            return self._createOperatorInstance(obj)
        elif isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WHDB_TOPLEVEL_BLOCK_CODE:
                return WchdbTopHierarchy(obj, self)
            else:
                return WchdbBlock(obj, self)
        elif isinstance(obj, WdbsConnectionPointP):
            clazz = self.__xpntClzzs.get(obj.getEntityType())
            assert clazz, str(self.__xpntClzzs)
            return clazz(obj, self)
        return super().createWrapper(obj, **args)

    def _createOperatorInstance(self, opInst):
        """
        Create an operator instance object based on the type of operator

        :param opInst: Raw operator instance
        :type opInst: WdbsOperatorInstanceP
        :return: Operator instance created
        :rtype: WhdbOperatorInstanceP
        """
        operator = opInst.getOperator()
        cls = WCHDB_STD_OPERINST_CLS.get(operator)
        return cls(opInst, self) if cls is not None else \
            WhdbOperatorInstance(WCHDB_OPER[operator], opInst, self)
