# ----------------------------------------------------------------------------
# - wcplg-file-config-generator                                              -
# - wild cruncher: file config generator             core / sys_map / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcplg_file_config_generator
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Cetrac file config generator

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""

from silkan_common.scmn_utils import isHexa
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_CETRAC_CODE, WHHDB_XPOINT_ETHERNET_CODE, \
    WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_FIBER_CHANNEL_CODE, \
    WHHDB_XPOINT_DISCRET_CODE, WHHDB_XPOINT_1553_CODE, WHHDB_XPOINT_ARINC825_CODE
from wcr_core.modules.hardware.db.whhdb_Host_instance import WhhdbHostInstance
from wcr_core.modules.hardware.db.whhdb_rack_instance import WhhdbRackInstance
from wcr_core.modules.sys_map.plugin.wcplg_config_generator_params import *
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import \
    WVHDB_VLINK_TYPE_AFDX, WVHDB_VLINK_TYPE_CETRAC, WVHDB_VLINK_TYPE_ETNERNET, \
    WVHDB_VLINK_TYPE_AFDX_PNUMBER, WVHDB_VLINK_TYPE_ARINC825
from wcr_frontend import showErrorWindow

WHHDB_HOST_EXTERNAL_KEY = WhhdbHostInstance.WHHDB_HOST_EXTERNAL_KEY
WHHDB_HOST_CETRAC_KEY = WhhdbHostInstance.WHHDB_HOST_CETRAC_KEY
DFT_MAC_ETH = "00:00:00:00:00:00"
DFT_IP = "0.0.0.0"
DFT_PROT = "UDP"
DFT_STORAGE = "Queue"
DFT_BUFFER = "0x15"
DFT_AFDX_INTERFACE = "0x1"
DFT_FRAMESIZE = "0x512"

APP_PERIODS = {'0':   '0x00',
               '1':   '0x01',
               '2':   '0x02',
               '4':   '0x04',
               '8':   '0x08',
               '16':  '0x10',
               '32':  '0x20',
               '64':  '0x40',
               '128': '0x80',
               '256': '0x100'}

WHHDB_MAPPING_IO_CONNECTORS = []
WHHDB_MAPPING_IO_LINKS = []

FIBER_COMSET = {'maxFrameLength': '0x858',
                'ipConfiguration': '0x29E',
                'fifoReceiveFullTreshold': '0x020',
                'fifoTransmitFullTreshold': '0x20',
                'fifoTransmitEmptyTreshold': '0x20',
                'fifoReceiveAlmostEmptyTreshold': '0x008',
                'fifoReceiveAlmostFullTreshold': '0x008',
                'fifoTransmitAlmostEmptyTreshold': '0x008',
                'fifoTransmitAlmostFullTreshold': '0x008',
                'initializationTimerValue': '0x40D99',
                'eventTimeoutValue': '0x510FF4'}

DISCRETE_COMSET = {'eventDataMaskIn': '',
                   'eventDataMaskOut': '',
                   'cyclicDataMaskIn': '',
                   'cyclicDataMaskOut': '',
                   'nbBuffer': '',
                   'producerId': '',
                   'cycleLength': ''}

MILBUS_COMSET = {'prodId': '',
                 'sequenceNumber': '',
                 'clockCycle': '0x186A0'}

ARINC825_COMSET = {'producerID': '0x1',
                   'acceptanceFilterMode':  '0x0',
                   'clockActivated': 'No',
                   'clockDivider': '0x0',
                   'selfTestEnable': 'No',}

STORAGE_METHOD_CONVERSION = {'Sample': '0x0',
                             'Queue': '0x1'}

MODULE_TYPE_CONVERSION = {'CetraC': 'mitracApp',
                          'Extern': 'CBApp',
                          }

VAR_TYPE_CONVERSION = {'char':   'char',
                       'int':    'int',
                       'int64':  'int', # <---- need to be discussed
                       'float':  'int',
                       'short':  'short'}


def getLsbMsb(mask):
    """
    retrun the lsb and msb in a format of 8 string of hexa
    :param mask: string of 16 char
    :return:
    """
    if len(mask) > 0 and len(mask) != 16:
        showErrorWindow('Configuration Error',
                        'Status report mask mandatory size is 16,'
                        ' has a bad size of {}.',
                        len(mask))
    return mask[:8], mask[8:]


def createConfigurationFileText(smpctx, mport=None):
    global WHHDB_MAPPING_IO_CONNECTORS
    global WHHDB_MAPPING_IO_LINKS
    WHHDB_MAPPING_IO_CONNECTORS = []
    WHHDB_MAPPING_IO_LINKS = []
    smaper = smpctx.getMappingManager()
    top = smpctx.getProjectDB().getTopHierarchy()
    # geting sourc ports for configuration having to be forced as
    # cetrac connections
    if not mport is None:
        hsp, ssp = getcodeurports(mport, smaper.getConections())
    else:
        hsp, ssp = None, None
    # physical architecture description.
    phconfstr = createHWConf(smaper, hsp, ssp)
    # port link list
    phconfstr = createPortLink(phconfstr, smaper)

    # logical architecture description
    logconfstr = '<{}>\n'.format(LOG_GL_DESC_TAG)
    # application list
    logconfstr += createApplicationList(top, smaper)
    # bus list
    logconfstr += createTrafficRouteList(smpctx, smaper)
    logconfstr += '</{}>\n'.format(LOG_GL_DESC_TAG)

    full= \
        '<CETRAC_CONFIGURATION ' \
        'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ' \
        'xsi:noNamespaceSchemaLocation=' \
        '"cetracConfiguratorSchema.xsd">\n' + phconfstr + logconfstr + \
        '</CETRAC_CONFIGURATION>'

    return full


def createHWConf(smaper, hsp, ssp):
    rackId = None
    num = 1
    phconfstr = '<{}>\n'.format(PHY_GL_DESC_TAG)
    # equipement description
    phconfstr += '<{}>\n'.format(PHY_OP_L_TAG)
    # for now the rack id is defined from his ParameterVariable id.
    for r in smaper.getRacks():
        rackId = int(r.getParameterValue('id'))
        phconfstr += createRackPhysicalDescription(r, rackId)
    for h in smaper.getHosts():
        if num == rackId:
            num += 1
        phconfstr += createHostPhysicalDescription(h, num, hsp)
        num += 1
    for s in smaper.getSwitchs():
        if num == rackId:
            num += 1
        phconfstr += createSwitchPhysicalDescription(s, num, ssp)
        num += 1
    phconfstr += createFakeHostDescription(num)
    phconfstr += '</{}>\n'.format(PHY_OP_L_TAG)

    return phconfstr


def createPortLink(phconfstr, smaper):
    phconfstr += '<{}>\n'.format(PHY_PORT_L_TAG)
    for con in smaper.getConections():
        cp1 = con.getStartConnectionPoint()
        stPtName = cp1.getName()
        stCardName = cp1.getCard()
        stEqpName = con.getStartObject().getName()
        cp2 = con.getEndConnectionPoint()
        endPtName = cp2.getName()
        endCardName = cp2.getCard()
        endEqpName = con.getEndObject().getName()
        switchs = smaper.getSwitchs()
        # swith don't have card so we create its name
        if con.getStartObject() in switchs:
            stCardName = '{}_card'.format(stEqpName)
        if con.getEndObject() in switchs:
            endCardName = '{}_card'.format(endEqpName)

        fullName1 = '{}.{}.{}'.format(stEqpName, stCardName, stPtName)
        fullName2 = '{}.{}.{}'.format(endEqpName, endCardName, endPtName)
        if not cp1.getSpeed() == cp2.getSpeed():
            raise Exception("Port of a connection must match speed "
                            "wrong ports: {} : {}".format(fullName1,
                                                          fullName2))

        if not (cp1.getCode() == cp2.getCode()) and (
                WHHDB_XPOINT_CETRAC_CODE not in (cp1.getCode(), cp2.getCode())):
            raise Exception("Port of a connection must match Type "
                            "wrong ports: {} Type {} / {} Type {}".
                            format(fullName1, cp1.getCode(),
                                   fullName2, cp2.getCode()))
        # TODO: adapt when the RackIo is fully functional
        if isinstance(con.getEndObject(), WhhdbRackInstance):
            fullName2 = '{}.{}.{}'.format(stEqpName, stCardName, '1_PORT_ETH')
        elif isinstance(con.getStartObject(), WhhdbRackInstance):
            fullName1 = '{}.{}.{}'.format(stEqpName, stCardName, '1_PORT_ETH')

        phconfstr += '<{} {}="{}" {}="{}"/>\n'.format(PHY_PORT_TAG,
                                                      PHY_PORT_PNAME1_TAG,
                                                      fullName1,
                                                      PHY_PORT_PNAME2_TAG,
                                                      fullName2)

    phconfstr += createFakeLinks()

    phconfstr += '</{}>\n'.format(PHY_PORT_L_TAG)
    phconfstr += '</{}>\n'.format(PHY_GL_DESC_TAG)

    return phconfstr


def getcodeurports(mport, cons):
    for con in cons:
        if mport == con.getStartConnectionPoint():
            return con.getEndConnectionPoint(), mport
        elif mport == con.getEndConnectionPoint():
            return con.getStartConnectionPoint(), mport
    #: selected port is connected to nothing: don't care
    return None, mport
    #raise Exception("Selected port is connected to Nothing")


def hexify(val):
    if not val:
        return hex(0)
    if not isinstance(val, int):
        if isHexa(val):
            return val
        else:
            try:
                val = int(val)
            except:
                return val
    return hex(val)


def createPort(equipstr, c):
    portState = None
    ptpActive = c.getPtpActive()
    if c.getPortNum() is None or c.getPortNum() <= 0:
        raise Exception("Hardware port ({}) without "
                        "Port Number".format(c.getName()))
    equipstr += '<{} {}="{}" {}="{}" {}="{}"'. \
        format(PHY_OP_CARD_PORT_TAG,
               PHY_OP_CARD_PORT_NAME_TAG, c.getName(),
               PHY_OP_CARD_PORT_TYPE_TAG, HARD_NETWORK_TYPE[c.getCode()],
               PHY_OP_CARD_PORT_NUM_TAG, hexify(c.getPortNum())
               )
    if portState:
        equipstr += ' {}="{}" '.format(PHY_OP_CARD_PORT_STATE_TAG, portState)
    if ptpActive:
        equipstr += ' {}="{}" '.format(PHY_OP_CARD_PORT_PTP_TAG, ptpActive)
    equipstr += '>\n'

    return equipstr


def createPortParameters(equipstr, c, cardType):
    fullMask = c.getStatusMask()
    if fullMask:
        portLSBMask, portMSBMask = getLsbMsb(fullMask)
    else:
        portLSBMask, portMSBMask = None, None

    if c.getCode() == WHHDB_XPOINT_ARINC825_CODE:
        ARINC825Meta = c._getConfigARINC825()
        portFreq = ARINC825Meta[PHY_OP_ARINC825_SPEED_TAG]
    else:
        portFreq = c.getSpeed()
    isBackBone = c.isBackBone()

    isCetracEqp = cardType in (WHHDB_HOST_CETRAC_KEY, CET_SW_CARD_TYPE)

    statusMonitor = c.getStatusMonitor() if isCetracEqp else None
    equipstr += '<{} {}="{}" {}="{}"'. \
        format(PHY_OP_CARD_PORT_PARAM_TAG,
               PHY_OP_CARD_PORT_PARAM_BACKBONE_TAG, isBackBone,
               PHY_OP_CARD_PORT_PARAM_FREQ_TAG, portFreq)

    if portLSBMask:
        equipstr += ' {}="0x{}" '.format(PHY_OP_CARD_PORT_PARAM_LSB_TAG,
                                         portLSBMask)
    if portMSBMask:
        equipstr += ' {}="0x{}" '.format(PHY_OP_CARD_PORT_PARAM_MSB_TAG,
                                         portMSBMask)
    if statusMonitor == 'On':
        equipstr += ' {}="{}" '.format(PHY_OP_CARD_PORT_PARAM_STATUS_MONITOR_TAG,
                                       statusMonitor)
    equipstr += '/>\n'

    return equipstr


def createPortDiscretParameters(equipstr, c):
    dicretMeta = c._getConfigDiscret()
    eventDataMaskIn = dicretMeta[PHY_OP_CARD_PORT_DISCRET_EVENT_IN_TAG]
    eventDataMaskOut = dicretMeta[PHY_OP_CARD_PORT_DISCRET_EVENT_OUT_TAG]
    cycliDataMaskIn = dicretMeta[PHY_OP_CARD_PORT_DISCRET_CYCLIC_IN_TAG]
    cycliDataMaskOut = dicretMeta[PHY_OP_CARD_PORT_DISCRET_CYCLIC_OUT_TAG]
    nbBuffer = dicretMeta[PHY_OP_CARD_PORT_DISCRET_BUFFER_TAG]
    producerId = dicretMeta[PHY_OP_CARD_PORT_DISCRET_ID_TAG]
    cycleLength = dicretMeta[PHY_OP_CARD_PORT_DISCRET_CYCLEN_TAG]

    equipstr += '<{} {}="{}" {}="{}" {}="{}" {}="{}" {}="{}" {}="{}" {}="{}"/>\n'. \
        format(PHY_OP_CARD_PORT_DISCRET_TAG,
               PHY_OP_CARD_PORT_DISCRET_EVENT_IN_TAG, hexify(eventDataMaskIn),
               PHY_OP_CARD_PORT_DISCRET_EVENT_OUT_TAG, hexify(eventDataMaskOut),
               PHY_OP_CARD_PORT_DISCRET_CYCLIC_IN_TAG, hexify(cycliDataMaskIn),
               PHY_OP_CARD_PORT_DISCRET_CYCLIC_OUT_TAG, hexify(cycliDataMaskOut),
               PHY_OP_CARD_PORT_DISCRET_BUFFER_TAG, hexify(nbBuffer),
               PHY_OP_CARD_PORT_DISCRET_ID_TAG, hexify(producerId),
               PHY_OP_CARD_PORT_DISCRET_CYCLEN_TAG, hexify(cycleLength))

    return equipstr


def createPort1553Parameters(equipstr, c):
    mil1553Meta = c._getConfig1553()
    producerId = mil1553Meta[PHY_OP_CARD_PORT_1553_ID_TAG]
    sequenceNumber = mil1553Meta[PHY_OP_CARD_PORT_1553_SEQUENCE_TAG]
    clockCycle = mil1553Meta[PHY_OP_CARD_PORT_1553_CYCLE_TAG]

    equipstr += '<{} {}="{}" {}="{}" {}="{}"/>\n'. \
        format(PHY_OP_CARD_PORT_1553_TAG,
               PHY_OP_CARD_PORT_1553_ID_TAG, hexify(producerId),
               PHY_OP_CARD_PORT_1553_SEQUENCE_TAG, hexify(sequenceNumber),
               PHY_OP_CARD_PORT_1553_CYCLE_TAG, hexify(clockCycle))

    return equipstr


def createPortAfdxConf(equipstr, c):
    cetracMeta = c._getConfigAfdx()
    maxDelay = cetracMeta[PHY_OP_CARD_PORT_AFDX_DELAY_TAG]
    highPrioBufferSize = cetracMeta[PHY_OP_CARD_PORT_AFDX_HIGHTBUFFER_TAG]
    lowPrioBufferSize = cetracMeta[PHY_OP_CARD_PORT_AFDX_LOWBUFFER_TAG]
    networkId = cetracMeta[PHY_OP_CARD_PORT_AFDX_NETWORK_TAG]
    interfaceId = cetracMeta[PHY_OP_CARD_PORT_AFDX_INTERFACE_TAG]
    if not interfaceId :
        interfaceId = DFT_AFDX_INTERFACE
    sideId = cetracMeta[PHY_OP_CARD_PORT_AFDX_SIDE_TAG]
    locationId = cetracMeta[PHY_OP_CARD_PORT_AFDX_LOCATION_TAG]


    equipstr += '<{} {}="{}" {}="{}" {}="{}" {}="{}" {}="{}" '\
                '{}="{}" {}="{}"/>\n'. \
        format(PHY_OP_CARD_PORT_AFDX_TAG,
               PHY_OP_CARD_PORT_AFDX_DELAY_TAG, hexify(maxDelay),
               PHY_OP_CARD_PORT_AFDX_HIGHTBUFFER_TAG, hexify(highPrioBufferSize),
               PHY_OP_CARD_PORT_AFDX_LOWBUFFER_TAG, hexify(lowPrioBufferSize),
               PHY_OP_CARD_PORT_AFDX_NETWORK_TAG, hexify(networkId),
               PHY_OP_CARD_PORT_AFDX_INTERFACE_TAG, hexify(interfaceId),
               PHY_OP_CARD_PORT_AFDX_SIDE_TAG, hexify(sideId),
               PHY_OP_CARD_PORT_AFDX_LOCATION_TAG, hexify(locationId)
               )

    return equipstr


def createPortFiberParameters(equipstr, c):
    fiberMeta = c._getConfigFiberChannel()
    maxFrameLength = FIBER_COMSET[PHY_OP_CARD_PORT_FB_MFL_TAG]
    fifoIpConfig = FIBER_COMSET[PHY_OP_CARD_PORT_FB_IPCONF_TAG]
    fifoReceiveFullTreshold = FIBER_COMSET[PHY_OP_CARD_PORT_FB_FRFT_TAG]
    fifoTransmitFullTreshold = FIBER_COMSET[PHY_OP_CARD_PORT_FB_FTFT_TAG]
    fifoTransmitEmptyTreshold = FIBER_COMSET[PHY_OP_CARD_PORT_FB_FTET_TAG]
    fifoReceiveAlmostEmptyTreshold = FIBER_COMSET[PHY_OP_CARD_PORT_FB_FRAET_TAG]
    fifoReceiveAlmostFullTreshold = FIBER_COMSET[PHY_OP_CARD_PORT_FB_FRAFT_TAG]
    fifoTransmitAlmostEmptyTreshold = FIBER_COMSET[PHY_OP_CARD_PORT_FB_FTAET_TAG]
    fifoTransmitAlmostFullTreshold = FIBER_COMSET[PHY_OP_CARD_PORT_FB_FTAFT_TAG]
    initializationTimerValue = FIBER_COMSET[PHY_OP_CARD_PORT_FB_ITV_TAG]
    eventTimeoutValue = FIBER_COMSET[PHY_OP_CARD_PORT_FB_ETV_TAG]

    equipstr += '<{} {}="{}" {}="{}" {}="{}" {}="{}" {}="{}"' \
                ' {}="{}" {}="{}" {}="{}" {}="{}" {}="{}" {}="{}"/>\n'. \
        format(PHY_OP_CARD_PORT_FB_TAG,
               PHY_OP_CARD_PORT_FB_MFL_TAG, hexify(maxFrameLength),
               PHY_OP_CARD_PORT_FB_IPCONF_TAG, hexify(fifoIpConfig),
               PHY_OP_CARD_PORT_FB_FRFT_TAG, hexify(fifoReceiveFullTreshold),
               PHY_OP_CARD_PORT_FB_FTFT_TAG, hexify(fifoTransmitFullTreshold),
               PHY_OP_CARD_PORT_FB_FTET_TAG, hexify(fifoTransmitEmptyTreshold),
               PHY_OP_CARD_PORT_FB_FRAET_TAG, hexify(fifoReceiveAlmostEmptyTreshold),
               PHY_OP_CARD_PORT_FB_FRAFT_TAG, hexify(fifoReceiveAlmostFullTreshold),
               PHY_OP_CARD_PORT_FB_FTAET_TAG, hexify(fifoTransmitAlmostEmptyTreshold),
               PHY_OP_CARD_PORT_FB_FTAFT_TAG, hexify(fifoTransmitAlmostFullTreshold),
               PHY_OP_CARD_PORT_FB_ITV_TAG, hexify(initializationTimerValue),
               PHY_OP_CARD_PORT_FB_ETV_TAG, hexify(eventTimeoutValue)
               )

    return equipstr


def createPortARINC825Parameters(equipstr, c):
    ARINC825Meta = c._getConfigARINC825()
    listenMode = ARINC825Meta[PHY_OP_ARINC825_LISTEN_TAG]
    maxFrame = ARINC825Meta[PHY_OP_ARINC825_MAX_FRAME_TAG]
    arbiterSpeed = ARINC825Meta[PHY_OP_ARINC825_ARBITRATION_SPEED_TAG]
    dataSpeed = ARINC825Meta[PHY_OP_ARINC825_DATA_SPEED_TAG]
    timeout = ARINC825Meta[PHY_OP_ARINC825_TIMEOUT_TAG]
    cyclicMode = ARINC825Meta[PHY_OP_ARINC825_CYCLIC_MODE_TAG]
    prodId = ARINC825_COMSET[PHY_OP_ARINC825_ID_TAG]
    filterMode = ARINC825_COMSET[PHY_OP_ARINC825_FILTER_TAG]
    clockActive = ARINC825_COMSET[PHY_OP_ARINC825_CLOCK_ACTIVATE_TAG]
    clockDivide = ARINC825_COMSET[PHY_OP_ARINC825_CLOCK_DIVIDER_TAG]
    selfTest = ARINC825_COMSET[PHY_OP_ARINC825_SELF_TEST_TAG]

    equipstr += '<{} {}="{}" {}="{}" {}="{}" {}="{}" {}="{}"' \
                ' {}="{}" {}="{}" {}="{}" {}="{}"'. \
        format(PHY_OP_CARD_PORT_ARINC825_TAG,
               PHY_OP_ARINC825_ID_TAG, hexify(prodId),
               PHY_OP_ARINC825_FILTER_TAG, hexify(filterMode),
               PHY_OP_ARINC825_CLOCK_ACTIVATE_TAG, clockActive,
               PHY_OP_ARINC825_CLOCK_DIVIDER_TAG, hexify(clockDivide),
               PHY_OP_ARINC825_LISTEN_TAG, hexify(listenMode),
               PHY_OP_ARINC825_MAX_FRAME_TAG, hexify(maxFrame),
               PHY_OP_ARINC825_SELF_TEST_TAG, selfTest,
               PHY_OP_ARINC825_TIMEOUT_TAG, hexify(timeout),
               PHY_OP_ARINC825_CYCLIC_MODE_TAG, cyclicMode,
               )

    if arbiterSpeed and dataSpeed:
        equipstr += ' {}="{}" {}="{}" '.format(
                PHY_OP_ARINC825_ARBITRATION_SPEED_TAG, arbiterSpeed,
                PHY_OP_ARINC825_DATA_SPEED_TAG, dataSpeed
                )

    equipstr += '/>\n'
    return equipstr


def createFakeHostDescription(num):
    """
    :param host: the extern hosts we write for the io connectors
    :type host: WhhdbHostInstance
    """
    # for each io ports we create an extern host that will have an extern card
    # connected to an extern connector
    equipstr = ''
    for i, c in enumerate(WHHDB_MAPPING_IO_CONNECTORS, 1):
        eqpName = "EQP_IO_{}".format(i)
        WHHDB_MAPPING_IO_LINKS.append(('{}.CARD_1.PORT_1'.format(eqpName), c))
        # Equipement head for Fake host
        equipstr += '<EQUIPMENT eqpName="{}" eqpType="host" eqpPhyNum="{}">\n' \
                    '<CARDS_LIST>\n' \
                    '<CARD cardName="CARD_1" cardType="Extern" cardPhyNum="0x1">\n' \
                    '<PORTS_LIST>\n' \
                    '<PORT portName="PORT_1" portType="EXTERN" portPhyNum="0x1"/>\n' \
                    '</PORTS_LIST>\n' \
                    '</CARD>\n' \
                    '</CARDS_LIST>\n' \
                    '</EQUIPMENT>\n'.format(eqpName, hex(num+i))

    return equipstr


def createFakeLinks():
    """
    generate the links for the Io
    """
    equipstr = ''
    for c1, c2 in WHHDB_MAPPING_IO_LINKS:
        equipstr += '<{} {}="{}" {}="{}"/>\n'.format(PHY_PORT_TAG,
                                                     PHY_PORT_PNAME1_TAG,
                                                     c1,
                                                     PHY_PORT_PNAME2_TAG,
                                                     c2)
    return equipstr


def createHostPhysicalDescription(host, num, configport):
    """
    :param host: the host we right for
    :type host: WhhdbHostInstance
    """
    def getcardtypefor(ct):
        if ct == WHHDB_HOST_EXTERNAL_KEY:
            return EXT_CARD_TYPE
        elif ct == WHHDB_HOST_CETRAC_KEY:
            return CET_CARD_TYPE
        else:
            raise Exception("No card code corresponding "
                            "to card type :{}".format(ct))

    # Equipement head for host
    hostName = host.getName()
    equipstr='<{} {}="{}" {}="{}" {}="{}">\n'.format(PHY_OP_TAG,
                                                     PHY_OP_NAME_TAG,
                                                     hostName,
                                                     PHY_OP_TYPE_TAG,
                                                     HOST_EQ_TYPE,
                                                     PHY_OP_NUM_TAG,
                                                     hexify(num))
    cards = host.getPCICardsDescription()
    # Cards description
    if len(cards) > 0:
        equipstr += '<{}>\n'.format(PHY_OP_CARD_L_TAG)
        ncard = 1
        for card in cards:
            # card tag begin
            cardName = card[0]
            cardType = card[1]
            portBind = card[2]
            ptpGrandMaster = card[3]
            fullMask = card[4]
            maxSkew = card[5]
            rId = host.getId()
            # if card type is cetracpci and a mask is given
            if cardType == WHHDB_HOST_CETRAC_KEY and fullMask:
                cardLSBMask, cardMSBMask = getLsbMsb(fullMask)
            else:
                cardLSBMask, cardMSBMask = None, None
            equipstr += '<{} {}="{}" {}="{}" {}="{}"'. \
                format(PHY_OP_CARD_TAG,
                       PHY_OP_CARD_NAME_TAG, cardName,
                       PHY_OP_CARD_TYPE_TAG,
                       getcardtypefor(cardType),
                       PHY_OP_CARD_NUM_TAG, hexify(ncard))
            if ptpGrandMaster == 'Yes':
                equipstr += ' {}="{}" '.format(PHY_OP_CARD_PTP_GM_TAG,
                                                   ptpGrandMaster)
            if cardLSBMask:
                equipstr += ' {}="0x{}" '.format(PHY_OP_CARD_LSB_TAG,
                                                cardLSBMask)
            if cardMSBMask:
                equipstr += ' {}="0x{}" '.format(PHY_OP_CARD_MSB_TAG,
                                                 cardMSBMask)
            if maxSkew:
                equipstr += ' {}="0x{}" '.format(PHY_OP_CARD_MAX_SKEW_TAG,
                                                 cardMSBMask)
            if rId:
                equipstr += ' {}="{}"'.format(PHY_OP_CARD_ID_TAG, hex(rId))

            equipstr += '>\n'
            equipstr += '<{}>\n'.format(PHY_OP_CARD_PORT_L_TAG)
            # card tag end
            Nport = 1
            # Connection points from cards description
            redondportnum = False
            # get ModuleIo of the card
            ioModules = host.getModuleIoFromCard(cardName)
            modulesPortBind = []
            for module in ioModules:
                modulesPortBind.extend(host.getBindedIoPorts(module))
            modulesPortBind = set(modulesPortBind)
            for c in host.getConnectionPoints():
                # check if port in card or module
                if c.getUUID() in portBind or c.getUUID() in modulesPortBind:
                    if c.getActive() == "Off":
                        continue
                    isDiscret = c.getCode() == WHHDB_XPOINT_DISCRET_CODE
                    isFiberChannel = c.getCode() == WHHDB_XPOINT_FIBER_CHANNEL_CODE
                    isAfdxConf = c.getCode() == WHHDB_XPOINT_AFDX_CODE
                    is1553 = c.getCode() == WHHDB_XPOINT_1553_CODE
                    isARINC825 = c.getCode() == WHHDB_XPOINT_ARINC825_CODE
                    linkName = '{}.{}.{}'.format(hostName, cardName, c.getName())
                    # PORT tag
                    equipstr = createPort(equipstr, c)
                    # PORT_PARAMETERS tag
                    equipstr = createPortParameters(equipstr, c, cardType)
                    if isDiscret:
                        WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
                        # DISCRET_PARAMETERS tag
                        equipstr = createPortDiscretParameters(equipstr, c)
                    elif isFiberChannel:
                        WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
                        # FIBER_CHANNEL_PARAMETERS tag
                        equipstr = createPortFiberParameters(equipstr, c)
                    elif is1553:
                        WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
                        # FIBER_CHANNEL_PARAMETERS tag
                        equipstr = createPort1553Parameters(equipstr, c)
                    elif isAfdxConf:
                        # AFDX_CONFIGURATION tag
                        equipstr = createPortAfdxConf(equipstr, c)
                    elif isARINC825:
                        WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
                        # ARINC825_PARAMETERS tag
                        equipstr = createPortARINC825Parameters(equipstr, c)
                    equipstr += '</{}>\n'.format(PHY_OP_CARD_PORT_TAG)
                    if c.getPortNum() == 5 and \
                                    getcardtypefor(card[1]) == CET_CARD_TYPE:
                        redondportnum = True
                    Nport += 1
            if not redondportnum and getcardtypefor(card[1]) == CET_CARD_TYPE:
                equipstr += '<{} {}="{}" {}="{}" {}="{}">\n'. \
                    format(PHY_OP_CARD_PORT_TAG,
                           PHY_OP_CARD_PORT_NAME_TAG, "Port_PCI",
                           PHY_OP_CARD_PORT_TYPE_TAG, "PCI",
                           PHY_OP_CARD_PORT_NUM_TAG, "0x05")
                equipstr += '<{} {}="{}" {}="{}"/>\n'.\
                    format(PHY_OP_CARD_PORT_PARAM_TAG,
                           PHY_OP_CARD_PORT_PARAM_BACKBONE_TAG, "No",
                           PHY_OP_CARD_PORT_PARAM_FREQ_TAG, "1G",
                           )
                equipstr += '</{}>\n'.format(PHY_OP_CARD_PORT_TAG)
            elif redondportnum:
                showErrorWindow('Hardware Configuration Error',
                                'The CetracPCI card "{}" has a port 5 defined '
                                'which is reserved for PCI port configuration.', card[0])
                raise Exception("A cetracPCI card was incorrectly defined.")
            ncard += 1
            equipstr += '</{}>\n'.format(PHY_OP_CARD_PORT_L_TAG)
            equipstr += '</{}>\n'.format(PHY_OP_CARD_TAG)
        equipstr += '</{}>\n'.format(PHY_OP_CARD_L_TAG)
    equipstr += '</{}>\n'.format(PHY_OP_TAG)
    return equipstr


def createRackPhysicalDescription(rack, num):
    """
    :param rack: the rack we write
    :type rack: WhhdbRacKInstance
    """
    #TODO: This is a temporary function to handle racks fixed information are put for the rack
    #TODO: and must be configured through xml files
    # Equipement head for host
    rackName = rack.getName()
    rId = rack.getId()
    equipstr='<{} {}="{}" {}="{}" {}="{}">'.format(PHY_OP_TAG,
                                                   PHY_OP_NAME_TAG,
                                                   rackName,
                                                   PHY_OP_TYPE_TAG,
                                                   "cetracIO",
                                                   PHY_OP_NUM_TAG,
                                                   hexify(num))
    equipstr += '<{}>\n'.format(PHY_OP_CARD_L_TAG)
    equipstr += '<{} {}="{}" {}="{}" {}="{}" {}="{}"'. \
        format(PHY_OP_CARD_TAG,
               PHY_OP_CARD_NAME_TAG, rackName + "_card",
               PHY_OP_CARD_TYPE_TAG, "cetracLS",
               PHY_OP_CARD_PTP_GM_TAG,
               'No',
               PHY_OP_CARD_NUM_TAG, hex(1))
    if rId:
        equipstr += ' {}="{}"'.format(PHY_OP_CARD_ID_TAG, hex(rId))

    equipstr += '> \n'

    equipstr += '<{}>\n'.format(PHY_OP_CARD_PORT_L_TAG)
    equipstr += '<{} {}="{}_{}" {}="{}" {}="{}" {}="{}" {}="{}"' \
                ' {}="{}"/>\n'.format(PHY_OP_CARD_PORT_TAG,
                                      PHY_OP_CARD_PORT_NAME_TAG,
                                      rackName, '1_PORT_ETH',
                                      PHY_OP_CARD_PORT_BACKBONE_TAG,
                                      "No",
                                      PHY_OP_CARD_PORT_TYPE_TAG, "CETRAC",
                                      PHY_OP_CARD_PORT_OUTPUT_STATUS_TAG,
                                      "No",
                                      PHY_OP_CARD_PORT_FREQ_TAG,
                                      "0x02",
                                      PHY_OP_CARD_PORT_NUM_TAG,
                                      hexify(5))

    equipstr += '<{} {}="{}_{}" {}="{}" {}="{}" {}="{}" {}="{}"' \
                ' {}="{}"/>\n'.format(PHY_OP_CARD_PORT_TAG,
                                      PHY_OP_CARD_PORT_NAME_TAG,
                                      rackName, '1_PORT_LS',
                                      PHY_OP_CARD_PORT_BACKBONE_TAG,
                                      "No",
                                      PHY_OP_CARD_PORT_TYPE_TAG, "EXTERN",
                                      PHY_OP_CARD_PORT_OUTPUT_STATUS_TAG,
                                      "No",
                                      PHY_OP_CARD_PORT_FREQ_TAG,
                                      "0x02",
                                      PHY_OP_CARD_PORT_NUM_TAG,
                                      hexify(6))

    equipstr += '</{}>\n'.format(PHY_OP_CARD_PORT_L_TAG)
    equipstr += '</{}>\n'.format(PHY_OP_CARD_TAG)
    equipstr += '</{}>\n'.format(PHY_OP_CARD_L_TAG)
    equipstr += '</{}>\n'.format(PHY_OP_TAG)

    return equipstr


def createSwitchPhysicalDescription(switch, num, configport):
    """
    :param switch: the switch we right for
    :type switch: WhhdbCetracSwitchInstance | WhhdbOperatorInstance
    """
    # Equipement head for switch
    switchName = switch.getName()
    rId = switch.getId()
    equipstr = '<{} {}="{}" {}="{}" {}="{}">\n'.format(PHY_OP_TAG,
                                                       PHY_OP_NAME_TAG,
                                                       switchName,
                                                       PHY_OP_TYPE_TAG,
                                                       CET_SW,
                                                       PHY_OP_NUM_TAG, hexify(num))
    fullStatusMask = switch.getStatusMask()
    ptpGrandMaster = switch.getStatus()
    if fullStatusMask:
        cardLSBMask, cardMSBMask = getLsbMsb(fullStatusMask)
    else:
        cardLSBMask, cardMSBMask = None, None
    equipstr += '<{}>\n'.format(PHY_OP_CARD_L_TAG)
    # pseudo card for switch
    equipstr += '<{} {}="{}" {}="{}" {}="{}"'. \
        format(PHY_OP_CARD_TAG,
               PHY_OP_CARD_NAME_TAG, switchName+"_card",
               PHY_OP_CARD_TYPE_TAG, CET_SW_CARD_TYPE,
               PHY_OP_CARD_NUM_TAG, hex(1))
    if ptpGrandMaster == 'Yes':
        equipstr += ' {}="{}" '.format(PHY_OP_CARD_PTP_GM_TAG,
                                       ptpGrandMaster)
    if cardLSBMask:
        equipstr += ' {}="{}" '.format(PHY_OP_CARD_LSB_TAG,
                                       '0x{}'.format(cardLSBMask))
    if cardMSBMask:
        equipstr += ' {}="{}" '.format(PHY_OP_CARD_MSB_TAG,
                                       '0x{}'.format(cardMSBMask))
    if rId:
        equipstr += ' {}="{}"'.format(PHY_OP_CARD_ID_TAG, hex(rId))

    equipstr += '>\n'
    equipstr += '<{}>\n'.format(PHY_OP_CARD_PORT_L_TAG)
    Nport = 1
    # connection point description.
    for c in switch.getConnectionPoints():
        if c.getActive() == "Off":
            continue
        isDiscret = c.getCode() == WHHDB_XPOINT_DISCRET_CODE
        isFiberChannel = c.getCode() == WHHDB_XPOINT_FIBER_CHANNEL_CODE
        isAfdxConf = c.getCode() == WHHDB_XPOINT_AFDX_CODE
        is1553 = c.getCode() == WHHDB_XPOINT_1553_CODE
        isARINC825 = c.getCode() == WHHDB_XPOINT_ARINC825_CODE
        linkName = '{}.{}.{}'.format(switchName, switchName+"_card",
                                     c.getName())
        # Don't force anymore the configuration point as "CETRAC"
        # we leave the error
        if c.getPortNum() is None or c.getPortNum() <= 0:
            raise Exception("Hardware port ({}) without "
                            "Port Number".format(c.getName()))
        # PORT tag
        equipstr = createPort(equipstr, c)
        # PORT_PARAMETERS tag
        # None because there is no card so no card type in switch
        equipstr = createPortParameters(equipstr, c, CET_SW_CARD_TYPE)
        if isDiscret:
            WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
            # DISCRET_PARAMETERS tag
            equipstr = createPortDiscretParameters(equipstr, c)
        elif isFiberChannel:
            WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
            # FIBER_CHANNEL_PARAMETERS tag
            equipstr = createPortFiberParameters(equipstr, c)
        elif is1553:
            WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
            # FIBER_CHANNEL_PARAMETERS tag
            equipstr = createPort1553Parameters(equipstr, c)
        elif isAfdxConf:
            # AFDX_CONFIGURATION_PARAMETERS tag
            equipstr = createPortAfdxConf(equipstr, c)
        elif isARINC825:
            WHHDB_MAPPING_IO_CONNECTORS.append(linkName)
            # ARINC825_PARAMETERS tag
            equipstr = createPortARINC825Parameters(equipstr, c)
        equipstr += '</{}>\n'.format(PHY_OP_CARD_PORT_TAG)
        Nport += 1
    equipstr += '</{}>\n'.format(PHY_OP_CARD_PORT_L_TAG)
    equipstr += '</{}>\n'.format(PHY_OP_CARD_TAG)
    equipstr += '</{}>\n'.format(PHY_OP_CARD_L_TAG)
    equipstr += '</{}>\n'.format(PHY_OP_TAG)
    return equipstr

DFLT_PROD_APP_NAME = "defaultprodextapp"
DFLT_CONS_APP_NAME = "defaultconsextapp"


def getIoCardBindModule(instances, hostName, cardName):
    """
    return card Name bound to ioModule or None
    """
    inst = instances.get(hostName, None)
    if not inst:
        return None
    ioCardBind = inst.getIoCardBind()
    cardBind = ioCardBind.get(cardName, None)
    return cardBind


def createApplicationList(top, map):
    """
    generate xml text for application list section
    """
    bindings = top.getModuleCardBind()
    instances = {inst.getName(): inst
                 for inst in top.getOperatorInstances()}
    if len(bindings) == 0:
        return ''

    appl = '<{}>\n'.format(LOG_APP_L_TAG)
    for eqp in bindings:
        modules = bindings[eqp]
        for moduleName, cardName, modulePeriod in modules:
            hostName = '{}.{}'.format(eqp, cardName)
            if not modulePeriod:
                modulePeriod = "0"
            Period = APP_PERIODS[modulePeriod]
            moduleType = None
            for mod in map.getAllModuleDesc():
                if mod.getName() == moduleName:
                    moduleType = mod.getModuleType()
                    # if none type considered extern module
                    if not moduleType:
                        moduleType = 'Extern'
                    break
            # for IoModule replace module name by card attached
            cardBind = getIoCardBindModule(instances, eqp, cardName)
            if cardBind:
                hostName = '{}.{}'.format(eqp, cardBind)

            appl += '<{} {}="{}" {}="{}" {}="{}" {}="{}"/>\n'.format(LOG_APP_TAG,
                                                             LOG_APP_NAME_TAG,
                                                             moduleName,
                                                             LOG_APP_TYPE_TAG,
                                                             MODULE_TYPE_CONVERSION[moduleType],
                                                             LOG_APP_HOST_TAG,
                                                             hostName,
                                                             LOG_APP_PERIOD_TAG,
                                                             Period)

    appl += '</{}>\n'.format(LOG_APP_L_TAG)
    return appl
    

def createTrafficRouteList(smpctx, sys_map_mapper):
    """
    generate xml text for TR list section
    """
    # TR list section
    trl = '<{}>\n'.format(LOG_TR_L_TAG)
    smtblk = smpctx.getProjectDB().getTopHierarchy()
    for trName, tr in sys_map_mapper.getAllVirtualLinks():
        trType = tr.getVLType()
        storageMethod = tr.getVLParameter(trType, LOG_TR_STORAGE_TAG)
        if not storageMethod:
            storageMethod = DFT_STORAGE
        nbBuffer = tr.getVLParameter(trType, LOG_TR_BUFFER_TAG)
        frameSize = tr.getVLParameter(trType, LOG_TR_FRAMESIZE_TAG)
        if not frameSize:
            frameSize = DFT_FRAMESIZE
        if not nbBuffer:
            nbBuffer = DFT_BUFFER

        res = getTransmitterPortFromTR(smtblk, sys_map_mapper, tr)
        if not res:
            return
        transPortName, recPortsName = res
        prodApp = sys_map_mapper.getProdModForVlink(tr.getUUID())
        consApps = sys_map_mapper.getConsModsForVlink(tr.getUUID())

        isRouteRedundant = False
        if tr.getVLType() == WVHDB_VLINK_TYPE_AFDX:
            if (tr.getVLParameter(trType, LOG_TR_AFDX_REDNT_MGNT_TAG) == 'Yes' and
               tr.getVLParameter(trType, LOG_TR_AFDX_REDNT_VL_TAG) == 'Yes' and
               tr.getVLParameter(trType, LOG_TR_AFDX_SN_CHECK_TAG) == 'Yes'):
                isRouteRedundant = True

        trl += createTrafficRoute(sys_map_mapper, tr, storageMethod, nbBuffer,
                                  transPortName, recPortsName, frameSize,
                                  trType, prodApp, consApps)

        if isRouteRedundant:
            transPortName2 ,recPortsName2 = getRedundantPort(smtblk, sys_map_mapper,
                                                             transPortName, recPortsName)

            if not transPortName2:
                showErrorWindow('AFDX Configuration Error',
                                'Port {} has no other port to send the redundancy.', transPortName)

            if not recPortsName2[0]:
                showErrorWindow('AFDX Configuration Error',
                                'Port {} has no other port to receive the redundancy.', recPortsName[0])

            trl += createTrafficRoute(sys_map_mapper, tr, storageMethod, nbBuffer,
                                      transPortName2, recPortsName2, frameSize,
                                      trType, prodApp=prodApp, consApps=consApps)

    trl += '</{}>\n'.format(LOG_TR_L_TAG)

    return trl


def getRedundantPort(smtblk, sys_map_mapper, transPortFullName, recPortsFullName):
    # we get all the ports that are connected to help us identify the afdx redundancy port
    conns = [pt for pt in sys_map_mapper.getConections()]
    connectedPorts = []
    for conn in conns:
        eqpSName = conn.getStartObject().getName()
        cardSName = conn.getStartConnectionPoint().getCard()
        portSName = conn.getStartConnectionPoint().getName()
        portSFullName = '.'.join([eqpSName, cardSName, portSName])
        eqpEName = conn.getEndObject().getName()
        cardEName = conn.getEndConnectionPoint().getCard()
        portEName = conn.getEndConnectionPoint().getName()
        portEFullName = '.'.join([eqpEName, cardEName, portEName])
        connectedPorts.extend([portSFullName, portEFullName])

    sConnnectedPorts = set(connectedPorts)
    # get other afdx transmiter  port of the same pci_Card
    transEqpName, transCardName, transPortName = transPortFullName.split('.')
    eqps = [eqp for eqp in smtblk.getOperatorInstances()]
    transPortName2 = getOtherAfdxPort(eqps, transEqpName, transCardName,
                                      transPortName, sConnnectedPorts)
    if len(recPortsFullName) > 1:
        showErrorWindow('AFDX Configuration Error',
                        'AFDX redundancy only one receiver port is necessary.', '')

    recEqpName, recCardName, recPortName = recPortsFullName[0].split('.')

    # get other afdx port of the same pci_Card
    recPortsName2 = [getOtherAfdxPort(eqps, recEqpName, recCardName, recPortName,
                                      sConnnectedPorts)]

    return transPortName2, recPortsName2


def getOtherAfdxPort(eqps, eqpName, cardName, portName, sConnnectedPorts):

    for eqp in eqps:
        # loop to look for eqp
        if eqp.getName() == eqpName:
            # loop connection point
            for point in eqp.getConnectionPoints():
                # if we find the afdx port of the pci card
                if (cardName == point.getCard() and
                            portName == point.getName() and
                            point.getCode() == WVHDB_VLINK_TYPE_AFDX):
                    # we loop over the ports to look for the other afdx port
                    for point2 in eqp.getConnectionPoints():
                        fullportName = '.'.join([eqpName, point2.getCard(),
                                                 point2.getName()])
                        if (cardName == point2.getCard() and
                                    portName != point2.getName() and
                                    point2.getCode() == WVHDB_VLINK_TYPE_AFDX and
                                    fullportName in sConnnectedPorts):
                            return '.'.join([eqpName, cardName, point2.getName()])


def createTrafficRoute(sys_map_mapper, tr, storageMethod, nbBuffer,
                       transPortName, recPortsName, frameSize,
                       trType, prodApp,consApps):
    trl = '<{} {}="{}" {}="{}" {}="{}" {}="{}" {}="{}"'.format(LOG_TR_TAG,
                                                                LOG_TR_STORAGE_TAG,
                                                                STORAGE_METHOD_CONVERSION[storageMethod],
                                                                LOG_TR_BUFFER_TAG,
                                                                hexify(nbBuffer),
                                                                LOG_TR_TYPE_TAG,
                                                                VL_NETWORK_TYPE[trType],
                                                                LOG_TR_TRANSMITTTER_TAG,
                                                                transPortName,
                                                                LOG_TR_FRAMESIZE_TAG,
                                                                frameSize)
    # if applications add producer app
    if prodApp:
        prodAppName = prodApp
        trl += ' {}="{}"'.format(LOG_TR_PROD_APP_TAG, prodAppName)
    trl += '>\n'

    if prodApp:
        trl += createMessages(sys_map_mapper, tr)

    # receiver ports
    receiversports = ''
    trl += '<{}>\n'.format(LOG_TR_REC_PORT_L_TAG)
    for recePortName in recPortsName:
        receiversports += '<{} {}="{}"/>\n'.format(
            LOG_TR_REC_PORT_TAG,
            LOG_TR_REC_PORT_ID_TAG,
            recePortName)
    trl += receiversports
    trl += '</{}>\n'.format(LOG_TR_REC_PORT_L_TAG)
    # consumer applications
    if consApps:
        consapps = ''
        trl += '<{}>\n'.format(LOG_TR_CAPP_L_TAG)
        for consAppName in set(consApps):
            consapps += '<{} {}="{}"/>\n'.format(
                LOG_TR_CAPP_TAG,
                LOG_TR_CAPP_NAME_TAG,
                consAppName)
        trl += consapps
        trl += '</{}>\n'.format(LOG_TR_CAPP_L_TAG)

    trl += addTRParameter(tr, trType)

    # traffic route end
    trl += '</{}>\n'.format(LOG_TR_TAG)

    return trl


def createMessages(sys_map_mapper, tr):
    """
        generate xml text for Messages list section
    """
    modules = sys_map_mapper.getModulesfromVlink(tr.getUUID())
    messages = sys_map_mapper.getMessagesfromVlink(tr.getUUID())
    gbTypes = sys_map_mapper.getGlobalTypes()
    allMessages = sys_map_mapper.getAllICDMessages()
    trl = ''
    if modules:
        trl += '<{}>\n'.format(LOG_TR_MSG_LIST_TAG)
        for msgName in messages:
            for mod in modules:
                # get the producer message
                msgProducer = None
                for msg, consprod in messages[msgName]:
                    if consprod == 'P':
                        msgProducer = msg
                        break
                if not msgProducer:
                    continue
                moduleName, lclMsg = msgProducer.split('.')
                if mod.getName() == moduleName:
                    _, payloadSize = allMessages[msgName]
                    trl += '<{} {}="{}" {}="{}"'.format(LOG_TR_MSG_TAG,
                                                        LOG_TR_MSG_DATA_NAME_TAG, msgName,
                                                        LOG_TR_MSG_PAYLOAD_TAG, hex(int(payloadSize)))
                    variables = mod.getData()
                    if not variables:
                        # close and finish the tag message
                        trl += '/>\n'
                    else:
                        # close the tag message
                        trl += '>\n'
                        trl += '<{}>\n'.format(LOG_TR_VARIABLE_LIST_TAG)
                        for varName, (vType, consProd, storage, periodicity) in variables.items():
                            if consProd == 'P' and varName == lclMsg:
                                _, offset, varType = gbTypes[vType]
                                # get type if not found return int (the biggest type)
                                type_ = VAR_TYPE_CONVERSION.get(varType, 'int')
                                trl += '<{} {}="{}" {}="{}" {}="{}"/>\n'. \
                                    format(LOG_TR_VARIABLE_TAG,
                                           LOG_TR_VAR_NAME_TAG, varName,
                                           LOG_TR_VAR_OFFSET_TAG, offset,
                                           LOG_TR_VAR_TYPE_TAG, type_)
                        trl += '</{}>\n'.format(LOG_TR_VARIABLE_LIST_TAG)
                        trl += '</{}>\n'.format(LOG_TR_MSG_TAG)
                    break
                moduleName, lclMsg = msgProducer.split('.')
        trl += '</{}>\n'.format(LOG_TR_MSG_LIST_TAG)

    return trl


def getTransmitterPortFromTR(smtblk, sys_mapp, tr):
    """
    Function that return the transmitterPortName if found return None otherwise
    :param smtblk: sys_map_mapper topblock
    :param tr: traffic route
    :return: portName or None
    """
    # formatting messages. from messages files.
    portprodbind, portconsbind = getportbinddic(smtblk)
    # clean vlinkop of anu vl not binded at all
    if tr.getUUID() not in portprodbind.keys() and \
                    tr.getUUID() not in portconsbind.keys():
        showErrorWindow('Configuration Error',
                        'Traffig Route {} is not binded.', tr.getName())
        return None
    uuid = portprodbind.get(tr.getUUID(), None)
    if uuid is None:
        showErrorWindow('Configuration Error',
                        'All route should be used. \n'
                        'Traffig Route {} is not used.', tr.getName())
        return None
    sourceport = sys_mapp.getPhyPortByUUID(uuid)
    if sourceport is None:
        showErrorWindow('Configuration Error',
                        'No source port for "{}" Traffic Route',
                        tr.getName())
        return None
    #get transmitter fullName
    eqp = sys_mapp.getHostFromPortUUID(sourceport.getUUID())
    if eqp is None:
        eqp = sys_mapp.getSwitchByPortUUID(sourceport.getUUID())
        cardName = '{}_card'.format(eqp.getName())
    else:
        cardName = sourceport.getCard()
    portName = sourceport.getName()
    transmitterPortName = '{}.{}.{}'.format(eqp.getName(), cardName, portName)
    transmitterPortName = getIoEqpPortFromPortName(transmitterPortName)

    # get all receiver fullName
    receiverPorts = []
    for receport in portconsbind.get(tr.getUUID(), []):
        eqp = sys_mapp.getHostFromPortUUID(receport)
        recPort = sys_mapp.getPhyPortByUUID(receport)
        if eqp is None:
            eqp = sys_mapp.getSwitchByPortUUID(receport)
            cardName = '{}_card'.format(eqp.getName())
        else:
            cardName = recPort.getCard()
        portName = recPort.getName()
        recPortName = '{}.{}.{}'.format(eqp.getName(), cardName, portName)
        recPortName = getIoEqpPortFromPortName(recPortName)
        receiverPorts.append(recPortName)

    return transmitterPortName, receiverPorts


def getIoEqpPortFromPortName(portName):
    for c1, c2 in WHHDB_MAPPING_IO_LINKS:
        if portName == c1:
            return c2
        elif portName == c2:
            return c1
    return portName


def hex2mac(hexaMac):
    if not hexaMac:
        return DFT_MAC_ETH
    if isHexa(hexaMac):
        # skip the 0x and get each even idx in a list (idx:0,2,4,6,8,10)
        mac1 = hexaMac[2::2]
        # skip the 0x and get each odds idx in a list (idx:1,3,5,7,9,11)
        mac2 = hexaMac[3::2]
        # iterate over the two list to create a string of mac format
        macAddress = '{}:{}:{}:{}:{}:{}'.format(*['{}{}'.format(x, y)
                                                for x, y in zip(mac1, mac2)])
    else:
        macAddress = hexaMac
    return macAddress


def hex2ip(hexaIp):
    if not hexaIp:
        return DFT_IP
    if isHexa(hexaIp):
        # skip the 0x and get each even idx in a list (idx:0,2,4,6,8,10)
        ip1 = hexaIp[2::2]
        # skip the 0x and get each odds idx in a list (idx:1,3,5,7,9,11)
        ip2 = hexaIp[3::2]
        # iterate over the two list to create a string of mac format
        ipAddress = '{}.{}.{}.{}'.format(*[int('{}{}'.format(x, y), 16)
                                            for x, y in zip(ip1, ip2)])
    else:
        ipAddress = hexaIp
    return ipAddress


def addTRParameter(tr, trType):
    trl = ''
    if trType == WVHDB_VLINK_TYPE_AFDX:
        # AFDX PARAM INFO
        vlNum = tr.getVLParameter(trType, LOG_TR_AFDX_NUMBER_TAG)
        bag = tr.getVLParameter(trType, LOG_TR_AFDX_BAG_TAG)
        jitter = tr.getVLParameter(trType, LOG_TR_AFDX_JITTER_TAG)
        lmax = tr.getVLParameter(trType, LOG_TR_AFDX_LMAX_TAG)
        lmin = tr.getVLParameter(trType, LOG_TR_AFDX_LMIN_TAG)
        trl = '<{} {}="{}" {}="{}" {}="{}" {}="{}" />\n'.format(
                                    LOG_TR_DETERMINIST_TAG,
                                    LOG_TR_AFDX_BAG_TAG, hexify(bag),
                                    LOG_TR_AFDX_JITTER_TAG, hexify(jitter),
                                    LOG_TR_AFDX_LMAX_TAG, hexify(lmax),
                                    LOG_TR_AFDX_LMIN_TAG, hexify(lmin))
        snCheck = tr.getVLParameter(trType, LOG_TR_AFDX_SN_CHECK_TAG)
        redunMgnt = tr.getVLParameter(trType, LOG_TR_AFDX_REDNT_MGNT_TAG)
        redunVl = tr.getVLParameter(trType, LOG_TR_AFDX_REDNT_VL_TAG)
        priority = tr.getVLParameter(trType, LOG_TR_AFDX_PRIO_TAG)

        if not snCheck:
            snCheck = 'No'
        if not redunMgnt:
            redunMgnt = 'No'
        if not redunVl:
            redunVl = 'No'
        trl += '<{} {}="{}" {}="{}" ' \
               '{}="{}" {}="{}" {}="{}"/>\n'.format(LOG_TR_AFDX_TAG,
                                   LOG_TR_AFDX_NUMBER_TAG, hexify(vlNum),
                                   LOG_TR_AFDX_REDNT_MGNT_TAG, snCheck,
                                   LOG_TR_AFDX_REDNT_VL_TAG, redunMgnt,
                                   LOG_TR_AFDX_SN_CHECK_TAG, redunVl,
                                   LOG_TR_AFDX_PRIO_TAG, hexify(priority))
    elif trType == WVHDB_VLINK_TYPE_ETNERNET:
        # ETH PARAM INFO
        macDest = tr.getVLParameter(trType, LOG_TR_ETH_MAC_DEST_TAG)
        macSrc = tr.getVLParameter(trType, LOG_TR_ETH_MAC_SRC_TAG)
        ipSrc = tr.getVLParameter(trType, LOG_TR_ETH_IP_SRC_TAG)
        ipDest = tr.getVLParameter(trType, LOG_TR_ETH_IP_DEST_TAG)
        protocol = tr.getVLParameter(trType, LOG_TR_ETH_PROTOCOL_TAG)
        arp = tr.getVLParameter(trType, LOG_TR_ETH_ARP_TAG)
        trl = '<{} {}="{}" {}="{}" {}="{}" {}="{}" ' \
              '{}="{}" {}="{}"/>\n'.format(LOG_TR_ETH_TAG,
                                   LOG_TR_ETH_MAC_DEST_TAG, hex2mac(macDest),
                                   LOG_TR_ETH_MAC_SRC_TAG, hex2mac(macSrc),
                                   LOG_TR_ETH_IP_SRC_TAG, hex2ip(ipSrc),
                                   LOG_TR_ETH_IP_DEST_TAG, hex2ip(ipDest),
                                   LOG_TR_ETH_PROTOCOL_TAG, protocol if protocol is not None
                                                            else DFT_PROT,
                                   LOG_TR_ETH_ARP_TAG, arp)
    elif trType == WVHDB_VLINK_TYPE_ARINC825:
        # ARINC825 PARAM INFO
        id_ = tr.getVLParameter(trType, LOG_TR_ARINC825_ID_TAG)
        trl = '<{} {}="{}"/>\n'.format(LOG_TR_ARINC825_TAG,
                                       LOG_TR_ARINC825_ID_TAG, hexify(id_),
                                       )
    return trl


def getportbinddic(smtblk):
    """
    create and retern a dictionary {VLinkUUID:PhysicportUUID} from
    system maper bindings for easy acces
    """
    pbindings = smtblk.getProdBinds()
    pbinddic = {}
    for ppuuid, vluuid in pbindings:
        pbinddic[vluuid] = ppuuid

    cbindings = smtblk.getConsBinds()
    cbinddic = {}
    for ppuuid, vluuid in cbindings:
        if vluuid in cbinddic.keys():
            cbinddic[vluuid].append(ppuuid)
        else:
            cbinddic[vluuid] = [ppuuid]

    return pbinddic, cbinddic
