# ----------------------------------------------------------------------------
# - wcmod_bus_file_handler                                                   -
# - wild cruncher : file handler for importing bus in sys_map view           -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcmod_bus_file_handler
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher file handler for importing bus in sys_map view

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------

from wcr_core.common.wcmn_file_handler import WcmnMimeFileHandlerBase, WcmnFile

from silkan_common.scmn_file_path import scmn_isfile
from wcr_core.modules.wmod_application_module import wmod_getModuleForPdb
from silkan_common.scmn_log import ScmnLogger
from wcr_core.project.wprj_project import wprj_import_project, \
    wprj_createProjectDB
from wcr_core.modules.vlink.wvmod_application_module import \
    WVMOD_VLINK_APP_CODE


class WcmodVLinkFileHandler(WcmnMimeFileHandlerBase):
    def __init__(self, *mimeTypes):
        """
        :param addEditorF: Function to add project editor to application
        :type addEditorF: (WguiProjectEditor) -> _
        :param cfg: Global project editor default configuration
        :type cfg: WguiProjectEditorCfgP
        :param gContext: Global context
        :type gContext: WcorGlobalContextP
        :param actions: Action dictionary
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param mimeTypes: Mime types handled by the file handler
        :type mimeTypes: list[ScmnMimeTypeP]
        """
        super().__init__(*mimeTypes)

    def handleFile(self, path, xctx):
        if isinstance(path,str):
            file=WcmnFile(path)
        else:
            file=path
        if not scmn_isfile(file.path):
            raise ValueError('No such file: ' + file.path)

        logger=ScmnLogger()
        projectDB = self._createProjectDB(
            file, xctx.createContextWith(msgTarget=logger))
        if projectDB is not None:
            module = wmod_getModuleForPdb(projectDB)
            if not (module.code == WVMOD_VLINK_APP_CODE):
                xctx.addMessage(
                'Wrong module for the importing file'.format(module.code))
                return False
            busl=projectDB.getTopHierarchy()
            return busl

    def _createProjectDB(self, file, xctx):
        return wprj_import_project(file, xctx)