# ----------------------------------------------------------------------------
# - wcmod-project-context-api                                                -
# - wild cruncher - sys project context api                  core / sys_map  -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcmod_project_context_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher system mapper project context api

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""
from abc import abstractmethod

from wcr_core.wcor_context import WcorProjectContextP
from wcr_core.modules.system.wpmod_mapping_manager import WpmodMappingManager


# noinspection PyAbstractClass
class WcmodSystemProjectContextP(WcorProjectContextP):
    @abstractmethod
    def getHardwareFile(self):
        """
        Get the hardware source.

        :return: Hardware
        :rtype: WhhdbTopBlock | None
        """
        raise NotImplementedError()

    @abstractmethod
    def getMappingManager(self):
        """
        Get the project's mapping manager.

        :return: Mapping manager
        :rtype: WpmodMappingManager
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WcmodSystemProjectContextMP(WcmodSystemProjectContextP):
    @abstractmethod
    def setHardwareFile(self, lsim):
        """
        Set the hardware source.

        :param lsim: Hardware
        :type lsim: WhhdbTopBlock | None
        """
        raise NotImplementedError()


class NoHardwareLoadedException(Exception):
    """
    Exception raised when no simulation is found loaded in the project context
    """
    def __init__(self, errMsg):
        """
        :param errMsg: Error message
        :type errMsg: str
        """
        self.__errMsg = errMsg

    def __str__(self):
        """
        :return: Error message
        :rtype: str
        """
        return self.__errMsg