# ----------------------------------------------------------------------------
# - wpact-reload-logical-simulation-action                                   -
# - wild cruncher: reload logical simulation action  core / system / actions -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpact_reload_logical_simulation_action
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Logical simulation reload action

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.modules.simulation.db.wshdb_block import WshdbTopBlock
from wcr_core.modules.simulation.wsmod_application_module import \
    WSMOD_SIMULATION_APP_CODE
from wcr_core.modules.system.actions.wpact_action_base import WpactActionBase
from wcr_core.project.wprj_project import wprj_load_project


class _WpactReloadLogicalSimulationAction(WpactActionBase):
    """
    Project editor action to reload the logical simulation.
    """

    def __init__(self):
        super().__init__(
            'Reload Logical Simulation...',
            'Reload the logical simulation from disk')

    def _doExecute(self, actx, paramValuePairs):
        pctx = actx.getProjectContext()
        """:type: WpmodSystemProjectContextMP"""
        wpact_reloadLogicalSimulation(pctx)


def wpact_reloadLogicalSimulation(pctx):
    """
    Reload the logical simulation.

    :param pctx: Project context
    :type pctx: WpmodSystemProjectContextMP
    """
    tblk = pctx.getProjectDB().getTopHierarchy()
    """:type: WphdbTopBlock"""
    file = tblk.getLogicalSimulationFile()
    if file:
        pdb = wprj_load_project(file, pctx)
        if pdb is not None and pdb.getModule() == WSMOD_SIMULATION_APP_CODE:
            simulation = pdb.getTopHierarchy() if pdb else None
            assert isinstance(simulation, WshdbTopBlock)
            pctx.setLogicalSimulation(simulation)
            pctx.getMappingManager().setLogicalSimulation(simulation)

            return

    pctx.setLogicalSimulation(None)


#: Logical simulation reload action
WPACT_RELOAD_LOGICAL_SIMULATION_ACTION = _WpactReloadLogicalSimulationAction()
