# ----------------------------------------------------------------------------
# - wphdb-connection-point-manager                                           -
# - wild cruncher: connection point manager               core / system / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wphdb_connection_point_manager
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher connection point manager

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_operator import whdb_makeConnXYPositions
from wcr_core.db.whdb_connection_point_manager import \
    WhdbConnectionPointManagerBase, WHDB_XPOINT_DIRECTION, \
    WHDB_XPOINT_POSITION, WHDB_XPOINT_TYPE, WHDB_XPOINT_NAME, \
    WHDB_XPOINT_FACING, WhdbXPointDesc


class WphdbConnectionPointManager(WhdbConnectionPointManagerBase):
    """
    Connection point manager for states.
    """

    def __init__(self, xpntTypes=None):
        self.__xpntTypes = tuple(xpntTypes) if xpntTypes is not None else None

    def getInitialXPointSet(self):
        return []

    def getDynamicXPoints(self):
        return self.__xpntTypes

    def canBatchCreateXPoint(self, xpnts, **kwargs):
        return True

    def canInstanceCreateXPoint(self, clst, **kwargs):
        return True

    def canBatchRemoveXPoint(self, xpnts, **kwargs):
        return True

    def canInstanceRemoveXPoint(self, clst, xpnt):
        """
        :type clst: list[WhdbConnectionPointP]
        :type xpnt: dict[str, object]
        :rtype: bool
        """
        return xpnt in clst

    def createInstanceXPoint(self, clst, connF, **kwargs):

        assert WHDB_XPOINT_TYPE in kwargs
        assert WHDB_XPOINT_DIRECTION in kwargs

        xdsc = wphdb_createXPointDesc(
            ctyp=kwargs.get(WHDB_XPOINT_TYPE),
            cdir=kwargs.get(WHDB_XPOINT_DIRECTION),
            cpos=kwargs.get(WHDB_XPOINT_POSITION),
            fcng=kwargs.get(WHDB_XPOINT_FACING))
        xpnt = connF(xdsc)
        xpnt.setMeta(kwargs.get(WHDB_XPOINT_NAME) or str(xpnt.getUUID()),
                     WHDB_XPOINT_NAME)
        return xpnt

    def removeInstanceXPoint(self, clst, xpnt):
        clst.remove(xpnt)

    def setInstanceXPoints(self, clst, connF, data):
        raise Exception('Cannot set connection points')

    def areValidXPoints(self, clst):
        return True, None

    def resetXPointPositions(self, clst):
        if not isinstance(clst, list):
            clst = list(clst)
        for xpnt, (x, y, f) in zip(
                clst, whdb_makeConnXYPositions(None, len(clst))):
            xpnt.setPosition(x, y)
            xpnt.setGravity(f)


def wphdb_createXPointDesc(ctyp, cdir, name=None, cpos=None, fcng=None,
                           xpnt=None, **kwargs):
    """
    Create a connection point description.

    :param ctyp: Connection point type
    :type ctyp: str
    :param cdir: Direction
    :type cdir: int
    :param name: Name
    :type name: str | None
    :param cpos: Position
    :type cpos: (float, float) | None
    :param fcng: Facing/gravity
    :type fcng: int | None
    :param xpnt: Referenced connection point
    :type xpnt: WhdbConnectionPointP | None
    :param kwargs: Additional arguments
    :type kwargs: dict[str, object]
    :return: Newly created connection point description
    :rtype: WhdbXPointDesc
    """
    assert 'gvty' not in kwargs

    return WhdbXPointDesc(
        ctyp=ctyp, cpos=cpos or (1.0, 0.0), cdir=cdir, name=name, fcng=fcng,
        xpnt=xpnt, **kwargs)
