# ----------------------------------------------------------------------------
# - wphdb-connection-points                                                  -
# - wild cruncher: connection points                      core / system / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wphdb_connection_points
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - System connection points

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_TYPE, \
    WHDB_XPOINT_DIRECTION
from wcr_core.dbstore.wdbs_meta_type import WdbsRecordMetaType, \
    WDBS_STRING_META
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.system.db.wphdb_connection_point_base import \
    WphdbConnectionPointBase


WPHDB_XPOINT_ETHERNET_CODE = 'wcr.system.xpoint-ethernet'

WPHDB_XPOINT_ETH_KEY_INTERFACE = 'eth-interface'

WPHDB_XPOINT_ETH_CONFIG_META = WdbsRecordMetaType(
    (WPHDB_XPOINT_ETH_KEY_INTERFACE, WDBS_STRING_META,
     'Ethernet interface'),)

WPHDB_XPOINT_ETH_DFLT_ARGS = {
    WHDB_XPOINT_TYPE: WPHDB_XPOINT_ETHERNET_CODE,
    WHDB_XPOINT_DIRECTION: WDBS_XPNT_BI
}


class WphdbEthernetXPoint(WphdbConnectionPointBase):
    """
    Ethernet connection point.
    """

    def getInterface(self):
        return self._getConfig(WPHDB_XPOINT_ETH_KEY_INTERFACE)

    def getTypeName(self):
        return 'Ethernet connection point'

    def getNamePrefix(self):
        return 'eth'

    def getDescription(self, **kwargs):
        interface = self.getInterface()

        return '{} (Ethernet{})'.format(
            self.getName(),
            ', {}: {}'.format(WPHDB_XPOINT_ETH_KEY_INTERFACE, interface)
            if interface else '')
