# ----------------------------------------------------------------------------
# - wphdb-host-meta                                                          -
# - wild cruncher: host meta-data                         core / system / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wphdb_host_mote
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Host meta-data

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaField, \
    WdbsMetaDescription
from wcr_core.db.whdb_parameter import *
from wcr_core.dbstore.wdbs_meta_type import *

WPHDB_HOST_CAP_CPU_KEY = 'cpu'

WPHDB_HOST_CAP_CORES_KEY = 'cores'

WPHDB_HOST_CAP_GPU_KEY = 'gpu'

WPHDB_HOST_CAP_MEMORY_KEY = 'memory'

WPHDB_HOST_CAP_DISK_KEY = 'disk'

WPHDB_HOST_PROCESSES_KEY = 'processes'

WPHDB_HOST_PROCESS_UUID_KEY = 'uuid'

WPHDB_HOST_PROCESS_NAME_KEY = 'name'

WPHDB_HOST_MODEL_MAPS_KEY = 'models'

WPHDB_HOST_MODEL_NAME_KEY = 'name'

WPHDB_HOST_SERVICE_MAPS_KEY = 'services'

WPHDB_HOST_SERVICE_KEY = 'service'

WPHDB_HOST_SERVICE_MAP_LABEL_KEY = 'label'

WPHDB_HOST_SERVICE_MAP_PARAMS_KEY = 'params'

WPHDB_HOST_DISTRIBSVC_SCKTS_KEY = 'distrib-svcs'

WPHDB_HOST_DISTRIBSVC_KEY = 'service'

WPHDB_HOST_DISTRIBSVC_SCKT_LABEL_KEY = 'label'

WPHDB_HOST_DISTRIBSVC_SCKT_PARAMS_KEY = 'params'

WPHDB_HOST_CARDS_KEY = 'cards'

WPHDB_HOST_CARD_TYPE_KEY = 'type'

WPHDB_HOST_CARD_NAME_KEY = 'name'

WPHDB_HOST_CARD_CHANNELS_KEY = 'channels'

WPHDB_HOST_CHANNEL_NAME_KEY = 'name'

WPHDB_HOST_CHANNEL_TYPE_KEY = 'type'


#: Process model-mapping meta-data type
WPHDB_PROCESS_MODEL_META_TYPE = WdbsRecordMetaType(
    (WPHDB_HOST_MODEL_NAME_KEY, WDBS_STRING_META)
)

#: Process service meta-data type
WPHDB_PROCESS_SERVICE_META_TYPE = WdbsRecordMetaType(
    (WPHDB_HOST_SERVICE_KEY, WDBS_STRING_META),
    (WPHDB_HOST_SERVICE_MAP_LABEL_KEY,
     WDBS_STRING_META),
    (WPHDB_HOST_SERVICE_MAP_PARAMS_KEY, WHDB_PARAMETER_SET_META_TYPE)
)

#: Process distribution service meta-data type
WPHDB_PROCESS_DISTRIB_SERVICE_META_TYPE = WdbsRecordMetaType(
    (WPHDB_HOST_DISTRIBSVC_KEY, WDBS_STRING_META),
    (WPHDB_HOST_DISTRIBSVC_SCKT_LABEL_KEY,
     WDBS_STRING_META),
    (WPHDB_HOST_DISTRIBSVC_SCKT_PARAMS_KEY, WHDB_PARAMETER_SET_META_TYPE)
)

#: Process meta-data type
WPHDB_PROCESS_META_TYPE = WdbsRecordMetaType(
    (WPHDB_HOST_PROCESS_NAME_KEY, WDBS_STRING_META),
    (WPHDB_HOST_MODEL_MAPS_KEY,
     WdbsDictionaryMetaType(
         WDBS_UUID_META, WPHDB_PROCESS_MODEL_META_TYPE)),
    (WPHDB_HOST_SERVICE_MAPS_KEY,
     WdbsDictionaryMetaType(
         WDBS_UUID_META, WPHDB_PROCESS_SERVICE_META_TYPE)),
    (WPHDB_HOST_DISTRIBSVC_SCKTS_KEY,
     WdbsDictionaryMetaType(
         WDBS_UUID_META, WPHDB_PROCESS_DISTRIB_SERVICE_META_TYPE))
)

WPHDB_HOST_META_DESCRIPTION = WdbsMetaDescription(
    WhdbOperatorInstance.META,
    WdbsMetaField(WPHDB_HOST_CAP_CPU_KEY, WDBS_STRING_META),
    WdbsMetaField(WPHDB_HOST_CAP_CORES_KEY, WDBS_INTEGER_META),
    WdbsMetaField(WPHDB_HOST_CAP_GPU_KEY, WDBS_STRING_META),
    WdbsMetaField(WPHDB_HOST_CAP_MEMORY_KEY, WDBS_STRING_META),
    WdbsMetaField(WPHDB_HOST_CAP_DISK_KEY, WDBS_STRING_META),
    WdbsMetaField(
        WPHDB_HOST_PROCESSES_KEY,
        WdbsDictionaryMetaType(
            WDBS_UUID_META, WPHDB_PROCESS_META_TYPE)),
    WdbsMetaField(
        WPHDB_HOST_CARDS_KEY,
        WdbsDictionaryMetaType(
            WDBS_UUID_META,
            WdbsRecordMetaType(
                (WPHDB_HOST_CARD_NAME_KEY, WDBS_STRING_META),
                (WPHDB_HOST_CARD_TYPE_KEY, WDBS_STRING_META),
                (WPHDB_HOST_CARD_CHANNELS_KEY,
                 WdbsDictionaryMetaType(
                     WDBS_UUID_META,
                     WdbsRecordMetaType(
                         (WPHDB_HOST_CHANNEL_NAME_KEY,
                          WDBS_STRING_META),
                         (WPHDB_HOST_CHANNEL_TYPE_KEY,
                          WDBS_STRING_META)))))
        )))
