# ----------------------------------------------------------------------------
# - wphdb-meta-provider                                                      -
# - wild cruncher: meta-data provider                     core / system / db -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wphdb_meta_provider
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher system mapper meta-data provider

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_block import WHDB_TOPLEVEL_BLOCK_CODE
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.db.whdb_meta_provider import WhdbMetaProvider
from wcr_core.dbstore.wdbs_meta_description import WdbsMetaDescription, \
    WdbsMetaField
from wcr_core.modules.system.db.wphdb_block import WPHDB_CLUSTER_BLOCK_CODE, \
    WphdbClusterBlock, WphdbBlockBase, WphdbTopBlock
from wcr_core.modules.system.db.wphdb_phybus_instance \
    import WphdbPhyBusInstance
from wcr_core.modules.system.db.wphdb_connection_point_base import \
    WphdbConnectionPointBase, WPHDB_XPOINT_CONFIG_KEY
from wcr_core.modules.system.db.wphdb_host_instance import WphdbHostInstance
from wcr_core.modules.system.db.wphdb_standard_operator \
    import WPHDB_HOST_CODE, WPHDB_BUS_CODE

WPHDB_STD_OPERINST_META = {WPHDB_HOST_CODE: WphdbHostInstance.META,
                           WPHDB_BUS_CODE: WphdbPhyBusInstance.META,
                           }


class WphdbMetaProvider(WhdbMetaProvider):
    def __init__(self, xpntCfgMeta):
        """
        :param xpntCfgMeta: Connection point configuration meta-data, by code
        :type xpntCfgMeta: dict[str, WdbsMetaTypeP]
        """
        super().__init__()

        #: Connection point configuration meta-data
        self.__xpntCfgMeta = xpntCfgMeta

        #: Connection point meta-data cache (incl. base and config)
        self.__xpntMeta = {}
        """:type: dict[str, WdbsMetaTypeP]"""

    def getBlockMeta(self, btyp=None):
        if btyp == WPHDB_CLUSTER_BLOCK_CODE:
            return WphdbClusterBlock.META
        elif btyp == WHDB_TOPLEVEL_BLOCK_CODE:
            return WphdbTopBlock.META
        else:
            return WphdbBlockBase.META

    def getOperatorInstanceMeta(self, oper=None):
        opMeta = WPHDB_STD_OPERINST_META.get(oper)
        return opMeta if opMeta is not None else WhdbOperatorInstance.META

    def getConnectionPointMeta(self, xtyp=None):
        if not xtyp:
            return WphdbConnectionPointBase.META

        meta = self.__xpntMeta.get(xtyp)
        if meta:
            return meta

        cfgMeta = self.__xpntCfgMeta.get(xtyp)
        if not cfgMeta:
            return WphdbConnectionPointBase.META

        meta = WdbsMetaDescription(
            WphdbConnectionPointBase.META,
            WdbsMetaField(WPHDB_XPOINT_CONFIG_KEY, cfgMeta))
        self.__xpntMeta[xtyp] = meta
        return meta
