# ----------------------------------------------------------------------------
# - wpplg-core-install-context                                               -
# - wild cruncher: core install context                core / model / system -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpplg_core_install_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Core install context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod

from wcr_core.plugin.wplg_core_install_context import \
    WplgCoreModuleInstallContext, WplgCoreModuleInstallContextP
from wcr_core.plugin.wplg_core_install_context_factory import \
    WplgCoreInstallContextFactoryMixin
from wcr_core.plugin.wplg_install_context_factory_api import \
    WplgModuleInstallContextFactoryP


# noinspection PyAbstractClass
class WpplgCoreModuleInstallContextP(WplgCoreModuleInstallContextP):
    @abstractmethod
    def addDistributionServiceType(self, dsvc):
        """
        Add a distribution service type.

        :param dsvc: Distribution service type to be added
        :type dsvc: WpmodDistributionServiceTypeP
        """
        raise NotImplementedError()

    @abstractmethod
    def addConnectionPointType(self, xpntType):
        """
        Add a connection point type.

        :param xpntType: New connection point type to be added
        :type xpntType: WpmodConnectionPointTypeP
        """
        raise NotImplementedError()


class WpplgCoreModuleInstallContext(WplgCoreModuleInstallContext,
                                    WpplgCoreModuleInstallContextP):
    """
    System core install context.
    """

    def __init__(self, plgCode, plgName, plgDesc, module):
        """
        :param module: System application module
        :type module: WpmodSystemApplicationModuleMP
        """
        super().__init__(plgCode, plgName, plgDesc, module)

        #: Application module
        self.__module = module

        #: Distribution service types to be installed
        self.__dsvcs = []
        """:type: list[WpmodDistributionServiceTypeP]"""

        #: Connection point types to be installed
        self.__xpnts = []
        """:type: list[WpmodConnectionPointTypeP]"""

    def commit(self):
        super().commit()

        for dsvc in self.__dsvcs:
            self.__module.addDistributionServiceType(dsvc)

        for xpnt in self.__xpnts:
            self.__module.addConnectionPointType(xpnt)

    def addDistributionServiceType(self, dsvc):
        self.__dsvcs.append(dsvc)

    def addConnectionPointType(self, xpntType):
        self.__xpnts.append(xpntType)


class WppglCoreModuleInstallContextFactory(
        WplgModuleInstallContextFactoryP,
        WplgCoreInstallContextFactoryMixin):
    def __init__(self, module):
        """
        :param module: Model application module
        :type module: WpmodSystemApplicationModuleMP
        """
        self.__module = module

    def getModuleCode(self):
        return self.__module.code

    def createInstallContext(self, plgCode, plgName, plgDesc, *args, **kwargs):
        return WpplgCoreModuleInstallContext(
            plgCode, plgName, plgDesc, self.__module)
