# ----------------------------------------------------------------------------
# - wpmod-distrib-service-default-types                                      -
# - wild cruncher: default distrib service types               core / system -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpmod_distrib_service_default_types
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Default distribution service types

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
import re
from uuid import UUID
from wcr_core.common.wcmn_parameter_description import \
    WcmnParameterDescription
from wcr_core.modules.system.wpmod_distrib_service_type import \
    WpmodDistributionServiceType

WPMOD_ETH_DISTSERV_CODE = 'ethernet'
WPMOD_ETH_DISTSERV_PARAMCODE_INTERFACE = 'interface'
WPMOD_ETH_DISTSERV_PARAMCODE_BROADPORT = 'broadcastPort'
WPMOD_ETH_DISTSERV_PARAMCODE_MAXVAR = 'maxVar'

WPMOD_ETHERNET_DISTRIBUTION_SERVICE_TYPE = WpmodDistributionServiceType(
    WPMOD_ETH_DISTSERV_CODE, 'Ethernet', 'Ethernet distribution service',
    WcmnParameterDescription(WPMOD_ETH_DISTSERV_PARAMCODE_INTERFACE,
                             'Interface',
                             'Network interface to use'),
    WcmnParameterDescription(WPMOD_ETH_DISTSERV_PARAMCODE_BROADPORT,
                             'Broadcast Port',
                             'Port to be used for broadcast'),
    WcmnParameterDescription(WPMOD_ETH_DISTSERV_PARAMCODE_MAXVAR,
                             'Maximum Variables',
                             'Maximum number of variables to be handled',
                             str(100)))

UUID_REGEXP = r'[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-' \
              '[89ab][0-9a-f]{3}-[0-9a-f]{12}'


def wpmod_retrieve_conpt_ethdistsrv(distSrvSck):
    """
    Retrieve the connection point object from an ethernet distributed service
    socket

    :param distSrvSck: Distribution service socket
    :type distSrvSck: WphdbDistributionServiceSocketP
    :return: Connection point object
    :rtype: WhdbConnectionPointP | None
    """
    return wpmod_retrieve_conpt_interface(
        distSrvSck.host,
        str(distSrvSck.getSocketParameterValue(
            WPMOD_ETH_DISTSERV_PARAMCODE_INTERFACE)))


def wpmod_retrieve_conpt_interface(host, refStr):
    """
    Retrieve the connection point object from the interface reference string
    of a ethernet distribution service

    :param host: Host to which the connection point belongs
    :type: host: WphdbHostInstance
    :param refStr: Reference string to the connection point which must
        contain its UUID
    :type refStr: str
    :return: Connection point object
    :rtype: WhdbConnectionPointP | None
    """
    # Extract UUID from reference string
    conPtUUIDres = re.search(UUID_REGEXP, refStr)
    if conPtUUIDres is not None:
        uuid = UUID(hex=conPtUUIDres.group(0))

        # Compare all connection points' uuid of passed host
        for cp in host.getConnectionPoints():
            if uuid == cp.getUUID():
                return cp
    return None
