# ----------------------------------------------------------------------------
# - wpmod-distrib-service-type                                               -
# - wild cruncher: distribution service type                   core / system -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpmod_distrib_service_type
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Distribution service type

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WpmodDistributionServiceTypeP(metaclass=ABCMeta):
    """
    Distribution service type protocol.
    """

    @abstractmethod
    def getCode(self):
        """
        Get the distribution service type's code.

        :return: Code of the distribution service type
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getName(self):
        """
        Get the distribution service type's name.

        :return: Name of the distribution service type
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getDescription(self):
        """
        Get the distribution service type's description.

        :return: Code of the distribution service description
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getSocketParameters(self):
        """
        Get the distribution service type's socket parameter descriptions.

        :return: Socket parameter descriptions
        :rtype: Iterable[WcmnParameterDescriptionP]
        """
        raise NotImplementedError()

    @abstractmethod
    def getSocketParameter(self, code):
        """
        Get the socket parameter description with the given code, if it exists.

        :param code: Parameter code
        :type code: str
        :return: Socket parameter, if exists
        :rtype: WcmnParameterDescriptionP | None
        """
        raise NotImplementedError()


class WpmodDistributionServiceType(WpmodDistributionServiceTypeP):
    def __init__(self, code, name, description, *socketParams):
        """
        :param code: Code
        :type code: str
        :param name: Name
        :type name: str
        :param description: Description
        :type description: str
        :param socketParams: Socket parameter descriptions
        :type socketParams: tuple[WcmnParameterDescriptionP]
        """
        self.__code = code
        self.__name = name
        self.__description = description
        self.__socketParams = socketParams
        self.__socketParamDict = {p.getName():p for p in socketParams}

    def getCode(self):
        return self.__code

    def getName(self):
        return self.__name

    def getDescription(self):
        return self.__description

    def getSocketParameters(self):
        return self.__socketParams

    def getSocketParameter(self, code):
        return self.__socketParamDict.get(code)
