# ----------------------------------------------------------------------------
# - wpmod-project-context-api                                                -
# - wild cruncher - sys project context api                   core / system  -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpmod_project_context_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher system mapper project context api

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import abstractmethod

from wcr_core.wcor_context import WcorProjectContextP
from wcr_core.modules.system.wpmod_mapping_manager import WpmodMappingManager


# noinspection PyAbstractClass
class WpmodSystemProjectContextP(WcorProjectContextP):
    @abstractmethod
    def getLogicalSimulation(self):
        """
        Get the logical simulation.

        :return: Logical simulation
        :rtype: WshdbTopBlock | None
        """
        raise NotImplementedError()

    @abstractmethod
    def getMappingManager(self):
        """
        Get the project's mapping manager.

        :return: Mapping manager
        :rtype: WpmodMappingManager
        """
        raise NotImplementedError()

    @abstractmethod
    def getServices(self):
        """
        Get the list of known services.

        :return: List of known services
        :rtype: Iterable[(str, Iterable[WcmnParameterDescriptionP])]
        """
        raise NotImplementedError()

    @abstractmethod
    def getPhysicalBuses(self):
        """
        Get the list of physical buses.

        :return: List of known physical buses
        :rtype: list[WphdbPhyBusInstance]
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WpmodSystemProjectContextMP(WpmodSystemProjectContextP):
    @abstractmethod
    def setLogicalSimulation(self, lsim):
        """
        Set the logical simulation.

        :param lsim: Logical simulation
        :type lsim: WshdbTopBlock | None
        """
        raise NotImplementedError()


class NoSimulationLoadedException(Exception):
    """
    Exception raised when no simulation is found loaded in the project context
    """
    def __init__(self, errMsg):
        """
        :param errMsg: Error message
        :type errMsg: str
        """
        self.__errMsg = errMsg

    def __str__(self):
        """
        :return: Error message
        :rtype: str
        """
        return self.__errMsg