# ----------------------------------------------------------------------------
# - wvhdb-Vlink-instance                                                     -
# - wild cruncher: Traffic Route instances                  core / vlink / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvhdb_vlink_instance
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Traffic Route decoy instance

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore.wdbs_meta_description import *
from wcr_core.dbstore.wdbs_meta_type import *
from wcr_core.modules.sys_map.plugin.wcplg_config_generator_params import *
from wcr_core.modules.vlink.db.wvhdb_standard_operators import \
    WVHDB_VLINK, WVHDB_VLINK_CODE

WVHDB_META_VLTYPE_KEY = 'VLink'
WVHDB_VLINK_TYPES = [WVHDB_VLINK_TYPE_AFDX, WVHDB_VLINK_TYPE_CETRAC,
                     WVHDB_VLINK_TYPE_ETNERNET, WVHDB_VLINK_TYPE_FIBER,
                     WVHDB_VLINK_TYPE_DISCRETE, WVHDB_VLINK_TYPE_1553,
                     WVHDB_VLINK_TYPE_ARINC825]

"""(key_for_meta_data, label)"""

WVHDB_VLINK_TYPE_AFDX_STORAGE = (LOG_TR_STORAGE_TAG, 'Storage Method')
WVHDB_VLINK_TYPE_AFDX_BUFFERS = (LOG_TR_BUFFER_TAG, 'Nb Buffers')
WVHDB_VLINK_TYPE_AFDX_FRAMESIZE = (LOG_TR_FRAMESIZE_TAG, 'FrameSize')
WVHDB_VLINK_TYPE_AFDX_PNUMBER = (LOG_TR_AFDX_NUMBER_TAG, 'Vl Number')
WVHDB_VLINK_TYPE_AFDX_PBAG = (LOG_TR_AFDX_BAG_TAG, 'Bag')
WVHDB_VLINK_TYPE_AFDX_JITTER = (LOG_TR_AFDX_JITTER_TAG, 'Jitter')
WVHDB_VLINK_TYPE_AFDX_LMAX = (LOG_TR_AFDX_LMAX_TAG, 'Lmax')
WVHDB_VLINK_TYPE_AFDX_LMIN = (LOG_TR_AFDX_LMIN_TAG, 'Lmin')
WVHDB_VLINK_TYPE_AFDX_SN_CHECK = (LOG_TR_AFDX_SN_CHECK_TAG, 'Sn Check')
WVHDB_VLINK_TYPE_AFDX_REDNT_MGNT = (LOG_TR_AFDX_REDNT_MGNT_TAG, 'Redun Mgnt')
WVHDB_VLINK_TYPE_AFDX_REDNT_VL = (LOG_TR_AFDX_REDNT_VL_TAG, 'Redun Vl')
WVHDB_VLINK_TYPE_AFDX_PRIORITY = (LOG_TR_AFDX_PRIO_TAG, 'Priority')
WVHDB_VLINK_TYPE_AFDX_PARAMETERS = [WVHDB_VLINK_TYPE_AFDX_STORAGE,
                                    WVHDB_VLINK_TYPE_AFDX_BUFFERS,
                                    WVHDB_VLINK_TYPE_AFDX_FRAMESIZE,
                                    WVHDB_VLINK_TYPE_AFDX_PNUMBER,
                                    WVHDB_VLINK_TYPE_AFDX_PBAG,
                                    WVHDB_VLINK_TYPE_AFDX_JITTER,
                                    WVHDB_VLINK_TYPE_AFDX_LMAX,
                                    WVHDB_VLINK_TYPE_AFDX_LMIN,
                                    WVHDB_VLINK_TYPE_AFDX_PRIORITY,
                                    WVHDB_VLINK_TYPE_AFDX_SN_CHECK,
                                    WVHDB_VLINK_TYPE_AFDX_REDNT_MGNT,
                                    WVHDB_VLINK_TYPE_AFDX_REDNT_VL]

WVHDB_VLINK_TYPE_CETRAC_STORAGE = (LOG_TR_STORAGE_TAG, 'Storage Method')
WVHDB_VLINK_TYPE_CETRAC_BUFFERS = (LOG_TR_BUFFER_TAG, 'Nb Buffers')
WVHDB_VLINK_TYPE_CETRAC_FRAMESIZE = (LOG_TR_FRAMESIZE_TAG, 'FrameSize')
WVHDB_VLINK_TYPE_CETRAC_PARAMETERS = [WVHDB_VLINK_TYPE_CETRAC_STORAGE,
                                      WVHDB_VLINK_TYPE_CETRAC_BUFFERS,
                                      WVHDB_VLINK_TYPE_CETRAC_FRAMESIZE
                                      ]

WVHDB_VLINK_TYPE_DISCRETE_STORAGE = (LOG_TR_STORAGE_TAG, 'Storage Method')
WVHDB_VLINK_TYPE_DISCRETE_BUFFERS = (LOG_TR_BUFFER_TAG, 'Nb Buffers')
WVHDB_VLINK_TYPE_DISCRETE_FRAMESIZE = (LOG_TR_FRAMESIZE_TAG, 'FrameSize')
WVHDB_VLINK_TYPE_DISCRETE_PARAMETERS = [WVHDB_VLINK_TYPE_DISCRETE_STORAGE,
                                        WVHDB_VLINK_TYPE_DISCRETE_BUFFERS,
                                        WVHDB_VLINK_TYPE_DISCRETE_FRAMESIZE
                                        ]

WVHDB_VLINK_TYPE_FIBER_STORAGE = (LOG_TR_STORAGE_TAG, 'Storage Method')
WVHDB_VLINK_TYPE_FIBER_BUFFERS = (LOG_TR_BUFFER_TAG, 'Nb Buffers')
WVHDB_VLINK_TYPE_FIBER_FRAMESIZE = (LOG_TR_FRAMESIZE_TAG, 'FrameSize')
WVHDB_VLINK_TYPE_FIBER_PARAMETERS = [WVHDB_VLINK_TYPE_FIBER_STORAGE,
                                     WVHDB_VLINK_TYPE_FIBER_BUFFERS,
                                     WVHDB_VLINK_TYPE_FIBER_FRAMESIZE
                                     ]

WVHDB_VLINK_TYPE_ARINC825_STORAGE = (LOG_TR_STORAGE_TAG, 'Storage Method')
WVHDB_VLINK_TYPE_ARINC825_BUFFERS = (LOG_TR_BUFFER_TAG, 'Nb Buffers')
WVHDB_VLINK_TYPE_ARINC825_FRAMESIZE = (LOG_TR_FRAMESIZE_TAG, 'FrameSize')
WVHDB_VLINK_TYPE_ARINC825_ID = (LOG_TR_ARINC825_ID_TAG, 'arinc825_id')
WVHDB_VLINK_TYPE_ARINC825_PARAMETERS = [WVHDB_VLINK_TYPE_ARINC825_STORAGE,
                                        WVHDB_VLINK_TYPE_ARINC825_BUFFERS,
                                        WVHDB_VLINK_TYPE_ARINC825_FRAMESIZE,
                                        WVHDB_VLINK_TYPE_ARINC825_ID
                                        ]

WVHDB_VLINK_TYPE_ETNERNET_STORAGE = (LOG_TR_STORAGE_TAG, 'Storage Method')
WVHDB_VLINK_TYPE_ETNERNET_BUFFERS = (LOG_TR_BUFFER_TAG, 'Nb Buffers')
WVHDB_VLINK_TYPE_ETNERNET_FRAMESIZE = (LOG_TR_FRAMESIZE_TAG, 'FrameSize')
WVHDB_VLINK_TYPE_ETNERNET_IP_SRC = (LOG_TR_ETH_IP_SRC_TAG, 'Ip Src')
WVHDB_VLINK_TYPE_ETNERNET_IP_DEST = (LOG_TR_ETH_IP_DEST_TAG, 'Ip Dest')
WVHDB_VLINK_TYPE_ETNERNET_MAC_SRC = (LOG_TR_ETH_MAC_SRC_TAG, 'Mac Src')
WVHDB_VLINK_TYPE_ETNERNET_MAC_DST = (LOG_TR_ETH_MAC_DEST_TAG, 'Mac Dest')
WVHDB_VLINK_TYPE_ETNERNET_PROTOCOL = (LOG_TR_ETH_PROTOCOL_TAG, 'Protocol')
WVHDB_VLINK_TYPE_ETNERNET_ARP = (LOG_TR_ETH_ARP_TAG, 'Arp')
WVHDB_VLINK_TYPE_ETNERNET_PARAMETERS = [WVHDB_VLINK_TYPE_ETNERNET_STORAGE,
                                        WVHDB_VLINK_TYPE_ETNERNET_BUFFERS,
                                        WVHDB_VLINK_TYPE_ETNERNET_FRAMESIZE,
                                        WVHDB_VLINK_TYPE_ETNERNET_IP_SRC,
                                        WVHDB_VLINK_TYPE_ETNERNET_IP_DEST,
                                        WVHDB_VLINK_TYPE_ETNERNET_MAC_SRC,
                                        WVHDB_VLINK_TYPE_ETNERNET_MAC_DST,
                                        WVHDB_VLINK_TYPE_ETNERNET_PROTOCOL,
                                        WVHDB_VLINK_TYPE_ETNERNET_ARP]

WVHDB_VLINK_TYPE_1553_STORAGE = (LOG_TR_STORAGE_TAG, 'Storage Method')
WVHDB_VLINK_TYPE_1553_BUFFERS = (LOG_TR_BUFFER_TAG, 'Nb Buffers')
WVHDB_VLINK_TYPE_1553_FRAMESIZE = (LOG_TR_FRAMESIZE_TAG, 'FrameSize')
WVHDB_VLINK_TYPE_1553_PARAMETERS = [WVHDB_VLINK_TYPE_1553_STORAGE,
                                    WVHDB_VLINK_TYPE_1553_BUFFERS,
                                    WVHDB_VLINK_TYPE_1553_FRAMESIZE
                                    ]

WVHDB_VLINK_TYPES_PARAM = {
    WVHDB_VLINK_TYPE_AFDX: WVHDB_VLINK_TYPE_AFDX_PARAMETERS,
    WVHDB_VLINK_TYPE_CETRAC: WVHDB_VLINK_TYPE_CETRAC_PARAMETERS,
    WVHDB_VLINK_TYPE_ETNERNET: WVHDB_VLINK_TYPE_ETNERNET_PARAMETERS,
    WVHDB_VLINK_TYPE_DISCRETE: WVHDB_VLINK_TYPE_DISCRETE_PARAMETERS,
    WVHDB_VLINK_TYPE_FIBER: WVHDB_VLINK_TYPE_FIBER_PARAMETERS,
    WVHDB_VLINK_TYPE_1553: WVHDB_VLINK_TYPE_1553_PARAMETERS,
    WVHDB_VLINK_TYPE_ARINC825: WVHDB_VLINK_TYPE_ARINC825_PARAMETERS,
}

WVHDB_VLINK_DEST_NUMBER_KEY = 'ndest'


class WvhdbVLinkInstance(WhdbOperatorInstance):
    """
    Operator-instance for bus instances.
    """

    META = WdbsMetaDescription(
        WhdbOperatorInstance.META,
        WdbsMetaField(WVHDB_META_VLTYPE_KEY,
                      WdbsDictionaryMetaType(WDBS_STRING_META,
                                             WdbsDictionaryMetaType(
                                                 WDBS_STRING_META,
                                                 WDBS_STRING_META))),
        WdbsMetaField(WVHDB_VLINK_DEST_NUMBER_KEY,
                      WDBS_INTEGER_META)
    )

    def __init__(self, inst, wreg):
        """
        :param inst: Raw instance
        :type inst: WdbsOperatorInstanceP
        :param wreg: Wrapper registry
        :type wreg: WhdbWrapperRegistryP
        """
        assert inst.getOperator() == WVHDB_VLINK_CODE
        WhdbOperatorInstance.__init__(self, WVHDB_VLINK, inst, wreg)
        self.__modulesF = None

    def setVLType(self, vltype):
        if vltype not in WVHDB_VLINK_TYPES:
            raise Exception("Unknown VL Type :{}".format(vltype))
        actuparam = self.rawElement.getMeta(WVHDB_META_VLTYPE_KEY)
        if vltype not in actuparam.keys():
            for vlt in actuparam.keys():
                self.rawElement.setMeta(None, WVHDB_META_VLTYPE_KEY, vlt)
        self.rawElement.setMeta({vltype: {}}, WVHDB_META_VLTYPE_KEY)

    def getVLType(self):
        actuparams = self.rawElement.getMeta(WVHDB_META_VLTYPE_KEY)
        res = None
        for k in actuparams.keys():
            res = k
        return res

    def getVLParameter(self, vltype, paramkey):
        if vltype not in WVHDB_VLINK_TYPES_PARAM.keys():
            return None
        if paramkey not in [i[0] for i in WVHDB_VLINK_TYPES_PARAM[vltype]]:
            return None
        if self.rawElement.getMeta(WVHDB_META_VLTYPE_KEY, vltype, paramkey) is \
                "":
            return None

        return self.rawElement.getMeta(WVHDB_META_VLTYPE_KEY, vltype, paramkey)

    def setVLParameter(self, vltype, paramkey, value):
        """
        set the configuration for VL at configs value
        :param configs: The dictionary config.
        :type configs: {str: str}
        """
        if vltype not in WVHDB_VLINK_TYPES_PARAM.keys():
            return None
        if paramkey not in [i[0] for i in WVHDB_VLINK_TYPES_PARAM[vltype]]:
            return None

        meta = self.rawElement.getMeta(WVHDB_META_VLTYPE_KEY)
        if meta is not None:
            meta[vltype][paramkey] = value
            self.rawElement.setMeta(meta, WVHDB_META_VLTYPE_KEY)

    def setVLParameters(self, configs):
        """
        set the configuration for VL at configs value
        :param configs: The dictionary config.
        :type configs: {str: str}
        """
        VLt=self.getVLType()
        for k in configs.keys():
            if k not in [i[0] for i in WVHDB_VLINK_TYPES_PARAM[VLt]]:
                continue
            self.rawElement.setMeta(configs[k], WVHDB_META_VLTYPE_KEY, VLt, k)

    def getVLParametersKeys(self, vltype=None):
        if vltype is None:
            VLt = self.getVLType()
        else:
            VLt = vltype
        if VLt is None:
            return None
        return WVHDB_VLINK_TYPES_PARAM[VLt]

    def getVLParameters(self, vltype):
        if vltype not in WVHDB_VLINK_TYPES:
            return []
        paramk = self.getVLParametersKeys(vltype=vltype)
        paraml=[]
        for key, label in paramk:
            paraml.append((label, self.getVLParameter(vltype, key), key))
        return paraml

    def getDestNumber(self):
        """
        return the size of the Destination Number parameter
        :return:
        """
        return self.rawElement.getMeta(WVHDB_VLINK_DEST_NUMBER_KEY)

    def setDestNumber(self, ndest):
        """
        Set a new value to Destination Number parameter
        :param ndest: the new size to set
        :type ndest: int
        """
        if int(ndest) < 0:
            ndest=0
        self.rawElement.setMeta(int(ndest), WVHDB_VLINK_DEST_NUMBER_KEY)
