# ----------------------------------------------------------------------------
# - wvhdb-wrapper-registry                                                   -
# - wild cruncher: raw data wrapper registry               core / vlink / db -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvhdb_wrapper_registry
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher silkan hierarchy raw data wrapper registry

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
from wcr_core.db.whdb_wrapper_registry import WhdbWrapperRegistry
from wcr_core.db.whdb_block import WHDB_TOPLEVEL_BLOCK_CODE
from wcr_core.modules.vlink.db.wvhdb_top_block import WvhdbTopHierarchy
from wcr_core.dbstore_api.wdbs_block_api import WdbsBlockP
from wcr_core.db.whdb_block import WhdbBlock
from wcr_core.modules.vlink.db.wvhdb_operator import WVHDB_OPER

from wcr_core.db.whdb_instance import WhdbOperatorInstance
from wcr_core.dbstore_api.wdbs_instance_api import WdbsOperatorInstanceP
from wcr_core.dbstore_api.wdbs_connection_point_api import WdbsConnectionPointP
from wcr_core.modules.vlink.db.wvhdb_standard_operators import \
    WVHDB_VLINK_CODE
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import WvhdbVLinkInstance

WVHDB_STD_OPERINST_CLS = {WVHDB_VLINK_CODE: WvhdbVLinkInstance}

class WvhdbWrapperRegistry(WhdbWrapperRegistry):
    """
    Manages HDS-HSB wrappers.
    """
    def __init__(self, blockClzzs):
        """

        """
        super().__init__()

        self.__blockClasses = blockClzzs

    def createWrapper(self, obj, *args):
        if isinstance(obj, WdbsBlockP):
            if obj.getEntityType() == WHDB_TOPLEVEL_BLOCK_CODE:
                return WvhdbTopHierarchy(obj, self)
            else:
                return WhdbBlock(obj, self)
        elif isinstance(obj, WdbsOperatorInstanceP):
            return self._createOperatorInstance(obj)
        return super().createWrapper(obj, *args)


    def _createOperatorInstance(self, opInst):
        """
        Create an operator instance object based on the type of operator

        :param opInst: Raw operator instance
        :type opInst: WdbsOperatorInstanceP
        :return: Operator instance created
        :rtype: WhdbOperatorInstanceP
        """
        operator = opInst.getOperator()
        cls = WVHDB_STD_OPERINST_CLS.get(operator)

        return cls(opInst, self) if cls is not None else \
            WhdbOperatorInstance(WVHDB_OPER[operator], opInst, self)
