# ----------------------------------------------------------------------------
# - wvmod_application_module                                                 -
# - wild cruncher: application module                           core / vlink -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvmod_application_module
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher virtual link designer application module

.. moduleauthor:: philippe lambert <lambert@silkan.com>
"""
import csv
from wcr_core.db.whdb_top_level_factory import whdb_createTopLevelRawBlock
from wcr_core.modules.wmod_application_module import \
    WMOD_DEFAULT_BLOCK_TYPES, WmodApplicationModuleBase, \
    WmodApplicationModuleP, WmodApplicationModuleMP
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_VLINK

from wcr_core.db_api.whdb_block_api import WHDB_TOPLEVEL_BLOCK_CODE, \
    WHDB_DEFAULT_BLOCK_CODE
from wcr_core.modules.wmod_block_type import WmodBlockType, \
    WMOD_SYSTEM_BLOCK_FLAG

from wcr_core.modules.vlink.db.wvhdb_wrapper_registry import \
    WvhdbWrapperRegistry
from wcr_core.modules.vlink.db.wvhdb_operator import WVHDB_OPER
from wcr_core.modules.vlink.db.wvhdb_standard_operators import \
    WVHDB_DEFAULT_OPS
from wcr_core.modules.vlink.db.wvhdb_meta_provider import WvhdbMetaProvider
from wcr_core.modules.vlink.db.wvhdb_verifier import WvhdbHierarchyVerifier
from wcr_core.modules.vlink.db.wvhdb_top_block import WvhdbTopHierarchy
from wcr_core.modules.vlink.controller.wvctl_controllers import \
    WVCTL_CONTROLLER_FACTORY
from wcr_core.modules.vlink.wvmod_project_context import \
    WvmodVlinkProjectContext
from wcr_core.modules.sys_map.plugin.wcplg_config_generator_params import HW_2_VL_TYPE
from wcr_frontend.modules.vlink.gui import WVGUI_CSV_VL_NAME_KEY, \
    WVGUI_CSV_VL_TYPE_KEY, WVGUI_CSV_VL_CONFIG
from wcr_frontend import showErrorWindow
from wcr_core.db.whdb_utils import whdb_ensureUniqueName
from wcr_core.modules.vlink.db.wvhdb_standard_operators import \
    WVHDB_VLINK_CODE
from wcr_frontend.modules.vlink.gui.wvgui_scene_protocol_table import *
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import *


WVMOD_VLINK_APP_CODE = 'vlink'
WVMOD_VLINK_APP_NAME = 'Traffic Route View'

WVMOD_BLOCK_TYPES = [
    WmodBlockType(WHDB_TOPLEVEL_BLOCK_CODE, 'Top-level',
                  WvhdbTopHierarchy.META, WvhdbTopHierarchy,
                  WMOD_SYSTEM_BLOCK_FLAG)]

# noinspection PyAbstractClass
class WvmodVLinkApplicationModuleP(WmodApplicationModuleP):
    pass


# noinspection PyAbstractClass
class WvmodVLinkApplicationModuleMP(WmodApplicationModuleMP,
                                    WvmodVLinkApplicationModuleP):
    pass


class _WvmodVLinkApplicationModule(WmodApplicationModuleBase,
                                   WvmodVLinkApplicationModuleMP):
    def __init__(self, operRegistry):
        super().__init__(
            code=WVMOD_VLINK_APP_CODE,
            name=WVMOD_VLINK_APP_NAME,
            blockTypes=WVMOD_BLOCK_TYPES,
            defaultOps=WVHDB_DEFAULT_OPS,
            operRegistry=operRegistry)

    def createHierarchyVerifier(self, pctx):
        return WvhdbHierarchyVerifier(self._getDbVerifierModules())

    def getPayloadType(self):
        return WCMN_PAYLOAD_BLOCK_VLINK

    def createTopLevelBlock(self, name=None, info=None):
        blockMetas = {b.getCode(): b.getMeta() for b in self.getBlockTypes()}
        blockClzzs = {b.getCode(): b.getClass() for b in self.getBlockTypes()}
        """:type: dict[str, WdbsMetaTypeP]"""
        return wvmod_create_top_block(blockMetas, blockClzzs,
                                      name=name, info=info)

    def createProjectContext(self, gContext, projectDB, logger=None):
        return WvmodVlinkProjectContext(
            gContext, self, projectDB, logger, WVCTL_CONTROLLER_FACTORY)

    def getFileExtention(self):
        return '.tr'


def wvmod_create_top_block(blockMetas, blockClzzs,
                           name=None, info=None):
    rgty = WvhdbWrapperRegistry(blockClzzs)
    tblk = whdb_createTopLevelRawBlock(WvhdbMetaProvider(blockMetas
                                                         ), name, info)
    return rgty.createWrapper(tblk)


def wvhdb_import_csv_routes(tblk, fl, modules=None, editor=None):
    csvdesc = csv.reader(fl)
    headers = []
    csvparamdic = {}

    for vlp in csvdesc:
        if headers == []:
            headers = vlp
            continue
        for i in range(0, min(len(headers), len(vlp))):
            csvparamdic[headers[i]] = vlp[i]
        newop = tblk.createOperatorInstance(WVHDB_VLINK_CODE)
        if WVGUI_CSV_VL_NAME_KEY in csvparamdic.keys():
            name = csvparamdic[WVGUI_CSV_VL_NAME_KEY]
        else:
            name = 'default'

        newop.setName(name)
        if WVGUI_CSV_VL_TYPE_KEY in csvparamdic.keys():
            VL_TYPE = csvparamdic[WVGUI_CSV_VL_TYPE_KEY]
            newop.setVLType(VL_TYPE)
            for hkey in csvparamdic.keys():
                if hkey != WVGUI_CSV_VL_TYPE_KEY and \
                                hkey != WVGUI_CSV_VL_NAME_KEY:
                    prmKey = WVGUI_CSV_VL_CONFIG.get(hkey, None)
                    if prmKey:
                        newop.setVLParameters({prmKey: csvparamdic[hkey]})
                    else:
                        showErrorWindow('Import Error',
                                        'Traffic Route key "{}" does not exist.', hkey)

            whdb_ensureUniqueName(newop, tblk)
            if modules:
                sceneItem = editor.scene.addFunctionItem(newop)
                modules[VL_TYPE].addRow(sceneItem)


#: HDS Application module singleton
WVMOD_VLINK_APP_MODULE = _WvmodVLinkApplicationModule(WVHDB_OPER)
