# ----------------------------------------------------------------------------
# - wvmod-mapping-manager                                                    -
# - wild cruncher - virtual link mapping manager                core / vlink -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvmod_mapping_manager
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher Virtual link model mapping manager

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""
from wcr_core.modules.icd.db.wihdb_module_instance \
    import WihdbModuleInstance
from wcr_core.db.whdb_connection import WhdbConnection, whdb_getOtherObject
from silkan_common.scmn_log import ScmnLogger
from wcr_core.project.wprj_project import wprj_import_project
from wcr_core.common.wcmn_file_handler import WcmnFile


class WvmodMappingManager(object):
    """
    The mapping manager assists in accessing element of logic bus source
    referencing elements by they're UUIDs
    """

    def __init__(self, tblk, xctx):
        """
        :param tblk: Vlink hierarchy top-level
        :type tblk: WvhdbTopBlock
        """

        self.__xctx = xctx

        #: Vlink hierarchy top-level
        self.__topBlock = tblk

        #: ICD hierarchy top-level
        self.__icd = None

        #: Mappring from UUID to icd Modules
        self.__globaleTypes = []

        #: Messages topblock dictionnary.
        self.__messages = {}
        self.__messagesBinding = {}

    def setICDfiletb(self, icdblk, prefix=None):
        """
        set a the icd source.

        :param icdblk: ICD source
        :type icdblk: WihdbTopBlock | None
        """
        self.__icd = [(icdblk, prefix)]
        self._initmessages(icdblk)
        self.__globaleTypes = icdblk.getGlobalTypes()

    def resetMessages(self):
        """
        set all the messages related variables to they empty state.
        """
        self.__messages = {}
        self.__messagesBinding = {}
        self.__icd = None

    def _initmessages(self, icdtblk):
        """
        get all the messages defined in icd view.
        """
        self.__messages = icdtblk.getAllMessages()
        self.__messagesBinding = icdtblk.getAllMessageBindings()

    def formatName(self, inst, prefix=None):
        """
        return a formated name for an instance given.
        """
        if not prefix or not isinstance(prefix, str):
            return inst.getName()
        return prefix+'-'+inst.getName()

    def getAllMessages(self):
        """
        return the list of the messages contained inside every icd
        files imported by the icd set in the mapper
        """
        return self.__messages

    def getAllMessagesBinding(self):
        """
        return the list of the messages binding contained inside every
        files imported by the icd set in the mapper
        """
        return self.__messagesBinding

    def getGlobalTypes(self):
        """
        return the list of the global types defined in the icd view
        """
        return self.__globaleTypes
