# ----------------------------------------------------------------------------
# - wvmod-project-context                                                    -
# - wild cruncher - virtual link context                       core / vlink  -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvmod_project_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher Traffic Route project context

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""

from wcr_core.modules.vlink.controller.wvctl_controllers import \
    WvctlControllerFactory
from wcr_core.project.wprj_project_context import WprjProjectContext
from wcr_core.modules.vlink.wvmod_mapping_manager import WvmodMappingManager
from wcr_core.modules.vlink.wvmod_bus_file_handler \
    import WvmodMessageFileHandler
from silkan_common.scmn_file_path import scmn_abspath

class WvmodVlinkProjectContext(WprjProjectContext):
    """
    Default project context implementation.
    """

    def __init__(self, gContext, module, projectDB, logger=None, ctrlfct=None):
        """
        :param gContext: Global context
        :type gContext: WcorGlobalContextP
        :param module: Application module
        :type module: WpmodSystemApplicationModuleP
        :param projectDB: Project database
        :type projectDB: WprjProjectDatabase
        :param logger: Logger
        :type logger: ScmnLoggerP
        """
        super().__init__(gContext, module, projectDB, logger)
        if ctrlfct is None :
            self.__ctrlFy = WvctlControllerFactory()
        else:
            self.__ctrlFy = ctrlfct

        projectDB.getTopHierarchy().setSourcePath(projectDB.path)

        # #: Mapping Manager
        # # noinspection PyTypeChecker
        self.__mappingManager = WvmodMappingManager(
            projectDB.getTopHierarchy(), self)

        self.SideUpdatefunc=None

    def getMappingManager(self):
        return self.__mappingManager

    def getControllerFactory(self):
        return self.__ctrlFy

    def setsideupdatefunc(self, func):
        self.SideUpdatefunc = func

    def updateMessages(self, path):
        """
        Add the path file to the Mapping manager don't Unbind invalid
        binding let by this change and don't change the Topblock bus reference
        """
        abspath = scmn_abspath(path)
        # Recuperation des messages
        handler = WvmodMessageFileHandler(self.getProjectDB().getModule())
        icd = handler.handleFile(abspath, self)
        if not icd:
            raise ValueError("Wrong source file for Traffic Route Source.")
            # error
        icd.setSourcePath(abspath)
        self.getMappingManager().setICDfiletb(icd)
