# ----------------------------------------------------------------------------
# - wplg-install-context                                                     -
# - wild cruncher: install context                             core / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wplg_install_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Install context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_mime_type import scmn_register_mime_type, \
    scmn_add_reader, ScmnMimeTypeP
from wcr_core.plugin.wplg_install_context_api import *


# noinspection PyAbstractClass
class WplgInstallContextBaseP(WplgInstallContextP):
    """
    Install context base class API.
    """

    @abstractmethod
    def commit(self):
        """
        Commit the recorded install-steps to the system/module.
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WplgInstallContextBase(WplgInstallContextBaseP):
    def __init__(self, plgCode, plgName, plgDesc):
        """
        :param plgCode: Plugin code
        :type plgCode: str
        :param plgName: Plugin name
        :type plgName: str
        :param plgDesc: Plugin description
        :type plgDesc: str
        """
        self.__plgCode = plgCode
        self.__plgName = plgName
        self.__plgDescription = plgDesc

    def getPluginCode(self):
        return self.__plgCode

    def getPluginName(self):
        return self.__plgName

    def getPluginDescription(self):
        return self.__plgDescription

    def registerMimeType(self, mime):
        """
        :type mime: ScmnMimeTypeP
        """
        scmn_register_mime_type(mime)

    def registerFileReader(self, handler, *mimes):
        for mime in mimes:
            if isinstance(mime, ScmnMimeTypeP):
                mime = mime.mimeType
            scmn_add_reader(mime, handler)


# noinspection PyAbstractClass
class WplgPlatformInstallContextBase(WplgInstallContextBase,
                                     WplgPlatformInstallContextP):
    def __init__(self, plgCode, plgName, plgDesc):
        """
        :param plgCode: Plugin code
        :type plgCode: str
        :param plgName: Plugin name
        :type plgName: str
        :param plgDesc: Plugin description
        :type plgDesc: str
        """
        super().__init__(plgCode, plgName, plgDesc)


# noinspection PyAbstractClass
class WplgModuleInstallContextBase(WplgInstallContextBase,
                                   WplgModuleInstallContextP):
    def __init__(self, plgCode, plgName, plgDesc):
        """
        :param plgCode: Plugin code
        :type plgCode: str
        :param plgName: Plugin name
        :type plgName: str
        :param plgDesc: Plugin description
        :type plgDesc: str
        """
        super().__init__(plgCode, plgName, plgDesc)
