# ----------------------------------------------------------------------------
# - wplg-install-context-api                                                 -
# - wild cruncher: install context api                         core / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wplg_install_context_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Install context API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WplgInstallContextP(metaclass=ABCMeta):
    """
    Install context procotol.

    Install contexts are used by plugins etc. to install their components into
    the system and/or respective module.
    Install contexts are created for every plugin installation separately by
    the corresponding install manager.
    """
    @abstractmethod
    def registerMimeType(self, mime):
        """
        Register a new MIME-type with the system.

        :param mime: MIME-type
        :type mime: ScmnMimeTypeP
        """
        raise NotImplementedError()

    @abstractmethod
    def registerFileReader(self, handler, *mimes):
        """
        Register a new file handler with the system.

        :param handler: File handler
        :type handler: WcmnFileHandler
        :param mimes: MIME-types handled by the file handler
        :type mimes: tuple[ScmnMimeTypeP]
        """
        raise NotImplementedError()

    @abstractmethod
    def getComponentCode(self):
        """
        Get the code for the component handled by the install context,
        e.g. 'core'.

        :return: Component code
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getPluginCode(self):
        """
        Get the code of the plugin the install context was created for.

        :return: Plugin code
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getPluginName(self):
        """
        Get the name of the plugin the install context was created for.

        :return: Plugin name
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def getPluginDescription(self):
        """
        Get the description of the plugin the install context was created for.

        :return: Plugin description
        :rtype: str
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WplgPlatformInstallContextP(WplgInstallContextP):
    """
    Platform install context protocol.

    Platform install contexts are used for non-module-specific installations.
    """
    pass


# noinspection PyAbstractClass
class WplgModuleInstallContextP(WplgInstallContextP):
    """
    Module install context protocol.

    Module install contexts are used for module-specific installations.
    """
    pass
