# ----------------------------------------------------------------------------
# - wplg-install-context-factory-api                                         -
# - wild cruncher: install context factory api                 core / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wplg_install_context_factory_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Install context factory API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WplgInstallContextFactoryP(metaclass=ABCMeta):
    """
    Install context factory protocol.

    Install context factory are responsible for creating install contexts.
    """

    @abstractmethod
    def getComponentCode(self):
        """
        Get the code for the component handled by the install context,
        e.g. 'core'.

        :return: Component code
        :rtype: str
        """
        raise NotImplementedError()

    @abstractmethod
    def createInstallContext(self, plgCode, plgName, plgDesc, *args, **kwargs):
        """
        Create a new install context for a plugin installation.

        :param plgCode: Plugin code
        :type plgCode: str
        :param plgName: Plugin name
        :type plgName: str
        :param plgDesc: Plugin description
        :type plgDesc: str
        :return: Newly created install context
        :rtype: WplgInstallContextBaseP
        """
        raise NotImplementedError()


# noinspection PyAbstractClass
class WplgPlatformInstallContextFactoryP(WplgInstallContextFactoryP):
    """
    Platform install context factory protocol.

    Platform install context factories are responsible for creating contexts
    for platform, i.e. non-module-specific, installations.
    """
    pass


# noinspection PyAbstractClass
class WplgModuleInstallContextFactoryP(WplgInstallContextFactoryP):
    """
    Module install context factory protocol.

    Module install context factories are responsible for creating contexts for
    module-specific installations.
    """
    @abstractmethod
    def getModuleCode(self):
        """
        Get the code of the associated module.

        :return: Module code
        :rtype: str
        """
        raise NotImplementedError()
