# ----------------------------------------------------------------------------
# - wplg-install-context-registry                                            -
# - wild cruncher: install context registry                   core / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wplg_install_context_registry
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Instnall context registry

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.plugin.wplg_install_context_factory_api import \
    WplgModuleInstallContextFactoryP


class WplgInstallContextRegistry(object):
    """
    Install context factory registry.

    The install context factory registry maintains a list of all available
    install context factories, and can provide the factory for a given
    component and possibly module, if it exists.
    """

    def __init__(self):
        self.__factories = {}
        """:type: dict[(str, str | None), WplgInstallContextFactoryP]"""

    def registerInstallContextFactory(self, factory):
        """
        Register an install context factory.

        :param factory: Install context factory to be regestired
        :type factory: WplgInstallContextFactoryP
        """
        comp = factory.getComponentCode()
        if isinstance(factory, WplgModuleInstallContextFactoryP):
            module = factory.getModuleCode()
        else:
            module = None
        code = comp, module

        if code in self.__factories:
            raise ValueError(
                'Factory for {} already registered: {}'.format(
                    code, self.__factories.get(code)))
        #else:
        #    raise ValueError(
        #        'Manager for {} not yet registered: {}'.format(
        #            code, self.__managers.get(code)))

        self.__factories[code] = factory

    def getInstallContextFactory(self, component, module=None):
        """
        Get the install context factory for the given component,
        and possibly module.

        If no module is specified, the platform install context factory
        for the given component will be returned.

        :param component: Component code
        :type component: str
        :param module: Module code
        :type module: str | None
        :return: Context factory, if found
        :rtype: WplgInstallContextFactoryP | None
        """
        return self.__factories.get((component, module))


#: Global install context registry
WPLG_INSTALL_CONTEXT_REGISTRY = WplgInstallContextRegistry()
