# ----------------------------------------------------------------------------
# - wplg-xml-reader-plugin                                                   -
# - wild cruncher: xml reader plugin                           core / plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wplg_xml_reader_plugin
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - XML reader plugin

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from importlib import import_module
from xml.etree.ElementTree import ElementTree

from silkan_common.scmn_file_path import scmn_listdir, scmn_isfile, \
    scmn_abspath, scmn_path_dir
from wcr_core.plugin.wplg_installer import WplgPluginDescriptor
from wcr_core.plugin.wplg_plugin_reader_plugin_api import \
    WplgPluginReaderPluginP


WCR_PLUGIN_ROOT_TAG = 'wcr-plugin'


class WplgXmlPluginReaderPlugin(WplgPluginReaderPluginP):
    """
    Plugin-reader plugin to read plugins from XML descriptor files with
    code-base installers.
    """

    def readAll(self, path, installer):
        if scmn_isfile(path):
            self.readPlugin(path, installer)
        else:
            for f in scmn_listdir(path, fullpath=True,
                                  pattern=r'^\w+_plugin.xml$'):
                self.readPlugin(f, installer)

    def readPlugin(self, file, installer):
        tree = ElementTree(file=file)
        root = tree.getroot()

        if root.tag != WCR_PLUGIN_ROOT_TAG:
            return

        plgCode = root.get('code')

        nameNode = root.find('name')
        plgName = nameNode.text if nameNode is not None else None

        descNode = root.find('description')
        plgDesc = descNode.text if descNode is not None else None

        installs = []
        paths = []
        pinsts, ppths = self._readPythonInstalls(root)
        if pinsts:
            installs.extend(pinsts)
            if ppths:
                base = scmn_path_dir(file)
                paths.extend(scmn_abspath(p, base=base) for p in ppths)

        plgDescriptor = WplgPluginDescriptor(
            plgCode, plgName, plgDesc, installs, paths)

        installer.installPlugin(plgDescriptor)

    def _readPythonInstalls(self, root):
        # noinspection PyShadowingNames
        def instF(ictx, ipkg, ifnc):
            mdl = import_module(ipkg)
            installF = getattr(mdl, ifnc)
            installF(ictx)

        installs = []
        instNodes = root.find('installs')
        if instNodes is None or len(instNodes) == 0:
            return None, None

        path = instNodes.get('path')

        for instNode in instNodes.findall('install'):
            comp = instNode.get('component')
            module = instNode.get('module')
            inst = instNode.text
            ipkg, ifnc = inst.rsplit('.', 1)
            ipkg = ipkg.strip()
            ifnc = ifnc.strip()

            installs.append((comp, module, instF, ipkg, ifnc))

        return installs, [path] if path else None
