# ----------------------------------------------------------------------------
# - wprj-project-context                                                     -
# - wild cruncher - default project context                   core / project -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wprj_project_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher default project context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.controller.wctl_default_controllers import \
    WCTL_DEFAULT_CONTROLLER_FACTORY
from wcr_core.wcor_context import WcorProjectContextP
from wcr_core.project.wprj_project import WprjProjectDatabase
from silkan_common.scmn_log import ScmnLogger


class WprjProjectContext(WcorProjectContextP):
    """
    Default project context implementation.
    """

    def __init__(self, gContext, module, projectDB, logger=None, ctrlFy=None):
        """
        :param gContext: Global context
        :type gContext: WcorGlobalContextP
        :param module: Application module
        :type module: WmodApplicationModuleP
        :param projectDB: Project database
        :type projectDB: WprjProjectDatabase
        :param logger: Logger
        :type logger: ScmnLoggerP
        """
        #: Global context
        self.__gContext = gContext
        #: Module
        self.__module = module
        #: Project database
        self.__projectDB = projectDB
        #: Project logger
        self.__logger = logger or ScmnLogger()
        self.__controlFcty = ctrlFy or WCTL_DEFAULT_CONTROLLER_FACTORY

    def getModule(self):
        return self.__module

    def addMessage(self, mtxt, mcmp=None, mcat=None, mlvl=None, trce=None):
        self.__logger.addMessage(mtxt, mcmp, mcat, mlvl, trce)
        self.__gContext.addMessage(mtxt, mcmp, mcat, mlvl, trce)

    def getMessageSource(self):
        return self.__logger

    def getProjectName(self):
        return self.__projectDB.getProjectName()

    def getProjectDB(self):
        return self.__projectDB

    def getProjectWorkspace(self):
        return self.__gContext.getWorkspaceManager().getProjectWorkspace(
            self.__projectDB.getProjectName())

    def getConfigurationValues(self):
        cnvs = dict(self.__gContext.getConfigurationValues())
        pcnf = self.__projectDB.tops.getLocalConfiguration()
        cnvs.update({k: r.getValue() for k, r in pcnf if r.getValue()})
        return cnvs

    def getConfigurationValue(self, key):
        pcnf = self.__projectDB.tops.getLocalConfiguration()
        pval = pcnf.getRecordValue(key)
        return pval if pval else self.__gContext.getConfigurationValue(key)

    def handleFile(self, path):
        return self.__gContext.handleFile(path)

    def getRecentlyOpenedFiles(self):
        return self.__gContext.getRecentlyOpenedFiles()

    def addRecentlyOpenedFile(self, path):
        return self.__gContext.addRecentlyOpenedFile(path)

    def createContextWith(self, msgTarget=None):
        return _WprjProjectContextWrapper(self, msgTarget)

    def getGlobalContext(self):
        return self.__gContext

    def getControllerFactory(self):
        return self.__controlFcty


class _WprjProjectContextWrapper(WcorProjectContextP):
    """
    Project context implementation for amended contexts.
    """

    def __init__(self, delegate, msgTarget=None):
        """
        :param delegate: Delegate context for default functionality
        :type delegate: WcorProjectContextP
        :param msgTarget: Additional message target
        :type msgTarget: ScmnMessageTargetP | None
        """
        #: Delegate context
        self.__delegate = delegate
        """:type: WcorProjectContextP"""

        #: Additional message target
        self.__msgTarget = msgTarget

    def getModule(self):
        return self.__delegate.getModule()

    def getProjectDB(self):
        return self.__delegate.getProjectDB()

    def getProjectWorkspace(self):
        return self.__delegate.getProjectWorkspace()

    def addMessage(self, mtxt, mcmp=None, mcat=None, mlvl=None, trce=None):
        if self.__msgTarget is not None:
            self.__msgTarget.addMessage(mtxt, mcmp, mcat, mlvl, trce)

        return self.__delegate.addMessage(mtxt, mcmp, mcat, mlvl, trce)

    def getConfigurationValues(self):
        return self.__delegate.getConfigurationValues()

    def getConfigurationValue(self, key):
        return self.__delegate.getConfigurationValue(key)

    def getMessageSource(self):
        return self.__delegate.getMessageSource()

    def handleFile(self, path):
        return self.__delegate.handleFile(path)

    def getRecentlyOpenedFiles(self):
        return self.__delegate.getRecentlyOpenedFiles()

    def addRecentlyOpenedFile(self, path):
        return self.__delegate.addRecentlyOpenedFile(path)

    def getGlobalContext(self):
        return self.__delegate.getGlobalContext()

    def createContextWith(self, msgTarget=None):
        return _WprjProjectContextWrapper(self, msgTarget)

    def getProjectName(self):
        return self.__delegate.getProjectName()

    def getControllerFactory(self):
        return self.__delegate.getControllerFactory()
