# ----------------------------------------------------------------------------
# - wcli-context                                                             -
# - wild cruncher:cli context                                 frontend / cli -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcli_context
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher console execution context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.db.whdb_path_manager import WhdbPathManager
from wcr_core.modules.wmod_application_module import wmod_getModule
from wcr_core.wcor_context import WcorProjectContextP
from wcr_core.project.wprj_project import WprjProjectDatabase


class WcliContext(WcorProjectContextP):
    """
    Context for execution of REPL/CLI commands.
    """

    def __init__(self, shellCtx):
        """
        :param shellCtx: Shell context
        :type shellCtx: WcorGlobalContextP
        """
        self.__appContext = shellCtx
        self.__projectDB = None
        """:type: WprjProjectDatabase | None"""
        self.__pathManager = None

    def setProjectDB(self, projectDB):
        """
        Set the current project database.

        :param projectDB: The new project database
        :type projectDB: WprjProjectDatabase
        """
        self.__projectDB = projectDB
        if projectDB:
            self.__pathManager = WhdbPathManager(projectDB.getTopHierarchy())
        else:
            self.__pathManager = None

    def unsetProjectDB(self):
        """
        Unset the current project database.
        """
        self.__projectDB = None

    def getGlobalContext(self):
        return self.__appContext

    def getModule(self):
        pdb = self.__projectDB
        return wmod_getModule(pdb.getModule()) if pdb else None

    def getProjectName(self):
        return self.getProjectDB().getProjectName()

    def getProjectDB(self):
        return self.__projectDB

    def getProjectWorkspace(self):
        pnam = self.getProjectName()
        return self.__appContext.getWorkspaceManager(). \
            getProjectWorkspace(pnam)

    def getConfigurationValues(self):
        cnvs = dict(self.__appContext.getConfigurationValues())
        pcnf = self.__projectDB.tops.getLocalConfiguration()
        cnvs.update({k: r.getValue() for k, r in pcnf if r.getValue()})
        return cnvs

    def getConfigurationValue(self, key):
        pcnf = self.__projectDB.tops.getLocalConfiguration()
        pval = pcnf.getRecordValue(key)
        return pval if pval else self.__appContext.getConfigurationValue(key)

    def handleFile(self, path):
        return self.__appContext.handleFile(path)

    def addMessage(self, mtxt, mcmp=None, mcat=None, mlvl=None, trce=None):
        return self.__appContext.addMessage(mtxt, mcmp, mcat, mlvl, trce)

    def getMessageSource(self):
        return self.__appContext.getMessageSource()

    def createContextWith(self, msgTarget=None):
        return self

    def getRecentlyOpenedFiles(self):
        return self.__appContext.getRecentlyOpenedFiles()

    def addRecentlyOpenedFile(self, path):
        return self.__appContext.addRecentlyOpenedFile(path)

    def getControllerFactory(self):
        raise NotImplementedError()
