# ----------------------------------------------------------------------------
# - app-was-cmnln                                                            -
# - wild cruncher:command line main file                      frontend / cli -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------
import argparse
import os
import re
import sys

# noinspection PyUnresolvedReferences
import wcr_core.modules.model.wmmod_setup
# noinspection PyUnresolvedReferences
import wcr_core.modules.statemachine.wfmod_setup
# noinspection PyUnresolvedReferences
import wcr_core.modules.simulation.wsmod_setup
# noinspection PyUnresolvedReferences
import wcr_core.modules.system.wpmod_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.message.gui.wmegui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.bus.gui.wbgui_setup

# noinspection PyUnresolvedReferences
import wcr_core.xmlio.wxio_xtr01
# noinspection PyUnresolvedReferences
import wcr_core.xmlio.wxio_xtr02

from wcr_frontend.cli.wcli_actions import *
from wcr_frontend.cli.wcli_repl import WcliCommandLine
from wcr_core.wcor_global_context import WcorGlobalContext
from wcr_core.db.whdb_top_level_factory import whdb_setImmutableDb
from wcr_core.common.wcmn_file_path import wcmn_path_data
from wcr_core.plugin.wplg_install_context_registry import \
    WPLG_INSTALL_CONTEXT_REGISTRY
from wcr_core.plugin.wplg_installer import WplgInstaller
from wcr_core.plugin.wplg_plugin_reader import WplgPluginReader
from wcr_core.plugin.wplg_xml_reader_plugin import WplgXmlPluginReaderPlugin
from wcr_core.plugin.wplg_library_reader_plugin import WplgLibraryReaderPlugin


_HISTFILE = scmn_path_join(wcmn_path_data(), 'console-history')


def _wcli_parseArguments():
    parser = argparse.ArgumentParser(description='Launch Wild Cruncher 2.0')
    parser.add_argument('file', metavar='FILE', type=str, nargs='?',
                        help='project file to be opened initially')
    parser.add_argument('-l', '--lib', dest='libraries', action='append',
                        metavar='library', help='operator libraries to load')
    parser.add_argument('-p', '--plugin', dest='plugins', action='append',
                        metavar='plugin', help='plugin to load')
    parser.add_argument('-s', '--stacktrace', dest='stacktrace',
                        action='store_true', default=False,
                        help='show stacktrace on errors')
    parser.add_argument('-m', '--mute-deprecated-ops',
                        dest='mute_deprecated_ops', action='store_true',
                        default=False,
                        help='mute deprecated operator warnings')
    parser.add_argument('-i', '--immutable-database',
                        dest='immutable_database', action='store_true',
                        default=False,
                        help='use immutable database (experimental)')

    return parser.parse_args()


def _wcli_loadPlugins():
    plgInstaller = WplgInstaller(WPLG_INSTALL_CONTEXT_REGISTRY)
    plgReader = WplgPluginReader(plgInstaller)
    plgReader.addReaderPlugin(WplgXmlPluginReaderPlugin())
    plgReader.addReaderPlugin(WplgLibraryReaderPlugin())
    if args.plugins:
        for plugin in args.plugins:
            plgReader.readAll(plugin)
    envPlgs = os.environ.get('WCR_PLUGINS')
    if envPlgs:
        for plg in envPlgs.split(os.pathsep):
            plgReader.readAll(plg)


# noinspection PyShadowingNames
def _wcli_registerCommands(cmdl, context):
    loadCmd = WcliLoadCommand(context)
    cmdl.addCommand(loadCmd)
    cmdl.addCommand(WcliLoadRecentCommand(loadCmd, context))
    cmdl.addCommand(WcliImportCommand(context))
    cmdl.addCommand(WcliSaveCommand(context))
    cmdl.addCommand(WcliSaveAsCommand(context))
    cmdl.addCommand(WcliDbInfoCommand(context))
    cmdl.addCommand(WcliLogCommand(context))
    cmdl.addCommand(WcliDbVerifyCommand(context))
    cmdl.addCommand(WcliGenerateCommand(context))
    cmdl.addCommand(WcliCompileCommand(context))
    cmdl.addCommand(WcliExecuteCommand(context))
    cmdl.addCommand(WcliCleanCommand(context))
    cmdl.addCommand(WcliWorkspaceCommand(context))
    cmdl.addCommand(WcliInitCallbacksCommand(context))
    cmdl.addCommand(WcliPropagateTypeCommand(context))
    cmdl.addCommand(WcliGUIGenerateCommand(context))


def _splitLines(inStr):
    parts = re.split('[\n;]', inStr)
    return filter(lambda l: bool(l), parts)


if __name__ == '__main__':
    args = _wcli_parseArguments()

    if args.immutable_database:
        whdb_setImmutableDb()

    actx = WcorGlobalContext()

    _wcli_loadPlugins()

    if args.mute_deprecated_ops:
        global WMOD_MUTE_DEPRECATED_OPS
        WMOD_MUTE_DEPRECATED_OPS = True

    cmdl = WcliCommandLine(prompt='wcr > ', histFile=_HISTFILE,
                           strace=args.stacktrace)

    context = WcliContext(actx)
    _wcli_registerCommands(cmdl, context)

    lines = _splitLines(sys.stdin.read()) if not sys.stdin.isatty() else None
    if lines:
        cmdl.evalAll(lines)
    else:
        if args.file:
            WcliLoadCommand(context).execute({}, [args.file])
        cmdl.repl()
