# ----------------------------------------------------------------------------
# - wgui-action-execution-context                                            -
# - wild cruncher: action executoin context                   frontend / gui -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_action_execution_context
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - GUI action execution context

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from PyQt4.QtCore import QObject
from PyQt4.QtGui import QApplication

from silkan_common.scmn_log import SCMN_LOG_ERROR
from wcr_frontend.gui.wgui_action_execution_context_api import \
    WguiActionExecutionContextP
from wcr_frontend.gui.wgui_protocols import QObject_Meta_M
from wcr_frontend.gui.wgui_utils import WguiCommandThread, \
    WguiCommandRunResultHandlerP
from wcr_frontend.gui.wgui_widgets import WguiCommandOutput


# noinspection PyAbstractClass
class WguiActionExecutionContextMixin(WguiActionExecutionContextP):
    def statusMessage(self, msg):
        self.addMessage(msg, 'cockpit')

    def executeCommand(self, cmndCode, cmndTitle, func, successF=None,
                       errorF=None):
        def mkOutputWidget():
            return WguiCommandOutput(), cmndTitle

        editor = self.getEditor()
        """:type: WguiProjectEditor"""
        output = editor.getOrCreateBottomWidget(cmndCode, mkOutputWidget)
        """:type: WguiCommandOutput"""
        pctx = editor.projectContext.createContextWith(msgTarget=output)
        resultBridge = _WguiCommandRunResultHandler(
            pctx, cmndTitle, successF=successF, errorF=errorF)
        output.clear()

        # noinspection PyArgumentList
        WguiCommandThread(
            func, output, resultBridge, QApplication.instance()).start()


# noinspection PyAbstractClass
class WguiActionExecutionContextBase(WguiActionExecutionContextMixin):
    def __init__(self, xctx, prmValSrc, parent):
        """
        :param xctx: Context
        :type xctx: WcorContextP
        :param prmValSrc: Parametetr value source
        :type prmValSrc: WactParameterValueSourceP
        """
        self.__xctx = xctx
        self.__prmValSrc = prmValSrc
        self.__parent = parent

    def getValue(self, parameter):
        return self.__prmValSrc.getValue(parameter)

    def addMessage(self, mtxt, mcmp=None, mcat=None, mlvl=None, trce=None):
        self.__xctx.addMessage(
            mtxt, mcmp=mcmp, mcat=mcat, mlvl=mlvl, trce=trce)

    def getParent(self):
        return self.__parent


class _WguiCommandRunResultHandler(QObject, WguiCommandRunResultHandlerP,
                                   metaclass=QObject_Meta_M):
    def __init__(self, pctx, title, successF=None, errorF=None):
        """
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param title: Title (for log messages etc.)
        :type title: str
        :param successF: Function to execute after successful execution
        :type successF: () -> _
        :param errorF: Function to execute after failed execution
        :type errorF: (Exception) -> _
        """
        QObject.__init__(self)

        self.__pctx = pctx
        self.__tite = title
        self.__successF = successF
        self.__errorF = errorF

    def success(self):
        if self.__successF:
            self.__successF()

    def error(self, excp, trce):
        handled = False
        if self.__errorF:
            handled = self.__errorF(excp)
        if not handled:
            self.__pctx.addMessage(
                '{c}-command failed: {e}'.format(c=self.__tite, e=excp),
                mcmp='cockpit', mcat=SCMN_LOG_ERROR, trce=trce)
