# ----------------------------------------------------------------------------
# - wgui-connection-props-dialog                                             -
# - wild cruncher - cockpit : wire properties dialog widgets                 -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2013 clemens oertel                                   -
# - copyright (c) 2013-2013 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_connection_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wire properties dialog

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from PyQt4.QtGui import QLineEdit, QVBoxLayout, QLabel

from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesDialog, \
    WguiPropertiesWidgetP, \
    WguiMultiPropertiesWidgetP


class WguiConnectionPropertiesDialog(WguiPropertiesDialog):
    """
    Properties dialog for wire objects.
    """

    def __init__(self, conn, updateF, parent=None):
        """
        :param conn: the connection object
        :type conn: WhdbConnectionP
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param parent: Parent widget
        """
        # base initialization
        super().__init__('Connection Properties', updateF, parent=parent)

        connWdgt = _WguiConnectionProperties(conn)
        # noinspection PyNoneFunctionAssignment
        typeWdgt = self._createConnectionTypePropertiesWidget(conn)
        if typeWdgt:
            generalWdgt = WguiMultiPropertiesWidgetP(connWdgt, typeWdgt)
        else:
            generalWdgt = connWdgt

        self.setTabbedPropertyWidgets(
            ('General', generalWdgt),
            *self._createAdditionalConnectionPropertiesWidget(conn))

    # noinspection PyMethodMayBeStatic
    def _createConnectionTypePropertiesWidget(self, conn):
        return None

    # noinspection PyMethodMayBeStatic
    def _createAdditionalConnectionPropertiesWidget(self, conn):
        return []


class _WguiConnectionProperties(WguiPropertiesWidgetP):
    """
    Properties editing widget for a wire's base information.
    """

    def __init__(self, conn, parent=None):
        """
        :param conn: Connection object
        :type conn: WhdbConnectionP
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Wire object
        self.__connection = conn

        #: Name field
        self.__nameF = QLineEdit()
        #: Info field
        self.__infoF = QLineEdit()

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        """
        Create the layout for the widget.
        Should only be called once.

        :return: Layout
        :rtype: QLayout
        """
        layout = QVBoxLayout()
        layout.addWidget(QLabel('Wire Name:'))
        layout.addWidget(self.__nameF)
        layout.addWidget(QLabel('Wire Information'))
        layout.addWidget(self.__infoF)
        layout.addStretch(1)
        return layout

    def updateFields(self):
        self.__nameF.setText(self.__connection.getName())
        self.__infoF.setText(self.__connection.getInfo())

    def save(self):
        self.__connection.setName(self.__nameF.text())
        self.__connection.setInfo(self.__infoF.text())
