# ----------------------------------------------------------------------------
# - wgui-editor-api                                                          -
# - wild cruncher: editor api                                 frontend / gui -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_editor_api
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Editor API

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


#: Left/right side identifiers
WGUI_EDITOR_LEFT_SIDE, WGUI_EDITOR_RIGHT_SIDE = range(2)


class WguiEditorInstTargetP(metaclass=ABCMeta):
    """
    Protocol for editor widget installation targets.
    """

    @property
    @abstractmethod
    def sideButtonToggled(self):
        """
        Get the signal emitting whenever a side button has been toggled.

        :return: Signal emitting whenever a side button has been toggled
        :rtype: pyqtBoundSignal[str, bool]
        """
        raise NotImplementedError()

    @abstractmethod
    def installSideButton(self, code, label, side, toggle=False):
        """
        Create a button with the given label.

        :param code: Button code (for identifying it in signals)
        :type code: str
        :param label: Label
        :type label: str
        :param side: Side to install the button on (left/right)
        :type side: int
        :param toggle: Whether to create button as a toggle button
        :type toggle: bool
        :return: Button created
        :rtype: WguiSidebarButtonP
        """
        raise NotImplementedError()

    @abstractmethod
    def insertMainWidget(self, code, widget):
        """
        Install a widget into the main area.

        :param code: Code of the widget
        :type code: str
        :param widget: Widget to install
        :type widget: QWidget
        """
        raise NotImplementedError()
