#!/usr/bin/env python
# ----------------------------------------------------------------------------
# - wgui-main                                                                -
# - wild cruncher - main starter                                             -
# ----------------------------------------------------------------------------
# - author    (c) 2012-2012 amaury darsch                                    -
# - author    (c) 2013-2014 clemens oertel                                   -
# - copyright (c) 2012-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

# ----------------------------------------------------------------------------
# - import section                                                           -
# ----------------------------------------------------------------------------
import argparse
import os
import sys

from PyQt5 import QtWidgets
from PyQt5 import QtCore

# noinspection PyUnresolvedReferences
# import wcr_frontend.modules.model.gui.wmgui_setup
# noinspection PyUnresolvedReferences
# import wcr_frontend.modules.simulation.gui.wsgui_setup
# noinspection PyUnresolvedReferences
# import wcr_frontend.modules.statemachine.gui.wfgui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.system.gui.wpgui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.icd.gui.wigui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.message.gui.wmegui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.bus.gui.wbgui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.vlink.gui.wvgui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.hardware.gui.whgui_setup
# noinspection PyUnresolvedReferences
import wcr_frontend.modules.sys_map.gui.wcgui_setup

# noinspection PyUnresolvedReferences
# import wcr_backend.wbke_setup

# noinspection PyUnresolvedReferences
import wcr_core.xmlio.wxio_xtr01
# noinspection PyUnresolvedReferences
import wcr_core.xmlio.wxio_xtr02

from wcr_core.wcor_global_context import WcorGlobalContext
from wcr_frontend.gui.wgui_main_window import WguiMainWindow
from wcr_core.db.whdb_top_level_factory import whdb_setImmutableDb
from wcr_core.plugin.wplg_install_context_registry import \
    WPLG_INSTALL_CONTEXT_REGISTRY
from wcr_core.plugin.wplg_installer import WplgInstaller
from wcr_core.plugin.wplg_plugin_reader import WplgPluginReader
from wcr_core.plugin.wplg_xml_reader_plugin import WplgXmlPluginReaderPlugin
from wcr_core.plugin.wplg_library_reader_plugin import WplgLibraryReaderPlugin
from silkan_common import __version__

# ----------------------------------------------------------------------------
# - main section                                                             -
# ----------------------------------------------------------------------------

def _wgui_parseArguments():
    parser = argparse.ArgumentParser(description='Launch Dezygner ' + __version__)
    parser.add_argument('files', metavar='FILE', type=str, nargs='*',
                        help='project files to be opened initially')
    parser.add_argument('-l', '--lib', dest='libraries', action='append',
                        metavar='library', help='operator libraries to load')
    parser.add_argument('-p', '--plugin', dest='plugins', action='append',
                        metavar='plugin', help='plugin to load')
    parser.add_argument('-f', '--force-close', dest='force_close',
                        action='store_true', default=False,
                        help='do not confirm application close')
    parser.add_argument('-i', '--immutable-database',
                        dest='immutable_database', action='store_true',
                        default=False,
                        help='use immutable database (experimental)')

    return parser.parse_known_args()


def _wgui_loadPlugins(args):
    plgInstaller = WplgInstaller(WPLG_INSTALL_CONTEXT_REGISTRY)
    plgReader = WplgPluginReader(plgInstaller)
    plgReader.addReaderPlugin(WplgXmlPluginReaderPlugin())
    plgReader.addReaderPlugin(WplgLibraryReaderPlugin())
    if args.plugins:
        for plugin in args.plugins:
            plgReader.readAll(plugin)
    envPlgs = os.environ.get('WCR_PLUGINS')
    if envPlgs:
        for plg in envPlgs.split(os.pathsep):
            plgReader.readAll(plg)


def _wgui_showMainWindow(context):
    # create the window with the context
    win = WguiMainWindow(context)
    # win.setAttribute(QtCore.Qt.WA_DeleteOnClose)
    # resize and show
    win.resize(1024, 768)
    win.show()
    win.raise_()

def main():
    args, otherArgs = _wgui_parseArguments()

    if args.immutable_database:
        whdb_setImmutableDb()

    # bind the qt application
    app = QtWidgets.QApplication(otherArgs)
    # create the shell context
    actx = WcorGlobalContext()

    _wgui_loadPlugins(args)

    # create the window with the context
    _wgui_showMainWindow(actx)

    # open files given on command line
    for arg in args.files:
        actx.handleFile(arg)
    # main application loop
    sys.exit(app.exec_())

if __name__ == '__main__':
    import faulthandler
    faulthandler.enable()
    main()
