# ----------------------------------------------------------------------------
# - wgui-props-dialog-factory-api                                            -
# - wild cruncher - cockpit: properties dialog box factory protocol          -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_props_dialog_factory_api
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher properties dialog box factory protocol

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from abc import ABCMeta, abstractmethod


class WguiPropertiesDialogFactoryP(metaclass=ABCMeta):
    """
    Protocol for properties dialog factories.
    """

    @property
    @abstractmethod
    def projectPropertiesChangedS(self):
        """
        Get the signal to notify about project property changes.

        :return: Signal to notify about project property changes
        :rtype: pyqtBoundSignal
        """
        raise NotImplementedError()

    @abstractmethod
    def createProjectPropertiesDialog(self, parent=None):
        """
        Create a project properties dialog.

        :param parent: Parent widget
        :return: Newly created project properties dialog
        :rtype: WguiPropertiesDialog | None
        """
        raise NotImplementedError()

    @abstractmethod
    def createBlockInstancePropertiesDialog(self, inst, selectXpnt=None,
                                            parent=None):
        """
        Create a block instance properties dialog.

        :param inst: Block instance to create properties dialog for
        :type inst: WhdbBlockInstanceP
        :param selectXpnt: Connection point to show initially, if any
        :type selectXpnt: WhdbConnectionPointP | None
        :param parent: Parent widget
        :return: Newly created block instance properties dialog
        :rtype: WguiPropertiesDialog
        """
        raise NotImplementedError()

    @abstractmethod
    def createModelInstancePropertiesDialog(self, inst, selectXpnt=None,
                                            parent=None):
        """
        Create a model instance properties dialog.

        :param inst: Model instance to create properties dialog for
        :type inst: WhdbModelInstanceP
        :param selectXpnt: Connection point to show initially, if any
        :type selectXpnt: WhdbConnectionPointP | None
        :param parent: Parent widget
        :return: Newly created model instance properties dialog
        :rtype: WguiPropertiesDialog
        """
        raise NotImplementedError()

    @abstractmethod
    def createOperatorInstancePropertiesDialog(self, inst, selectXpnt=None,
                                               parent=None):
        """
        Create a operator instance properties dialog.

        :param inst: Operator instance to create properties dialog for
        :type inst: WhdbOperatorInstanceP
        :param selectXpnt: Connection point to show initially, if any
        :type selectXpnt: WhdbConnectionPointP | None
        :param parent: Parent widget
        :return: Newly created operator instance properties dialog
        :rtype: WguiPropertiesDialog
        """
        raise NotImplementedError()

    @abstractmethod
    def createConnectionPropertiesDialog(self, conn, parent=None):
        """
        Create a connection properties dialog.

        :param conn: Connection to create properties dialog for
        :type conn: WhdbConnectionP
        :param parent: Parent widget
        :return: Newly created connection properties dialog
        :rtype: WguiPropertiesDialog
        """
        raise NotImplementedError()

    @abstractmethod
    def createShapePropertiesDialog(self, shape, parent=None):
        """
        Create a shape properties dialog.

        :param shape: Shape to create properties dialog for
        :type shape: WhdbShapeP
        :param parent: Parent widget
        :return: Newly created shape properties dialog
        :rtype: WguiPropertiesDialog
        """
        raise NotImplementedError()

    @abstractmethod
    def createButtonPropertiesDialog(self, button, parent=None):
        """
        Create a button properties dialog.

        :param button: Button to create properties dialog for
        :type button: WhdbButtonP
        :param parent: Parent widget
        :return: Newly created button properties dialog
        :rtype: WguiPropertiesDialog
        """
        raise NotImplementedError()
