# ----------------------------------------------------------------------------
# - wgui-thumbnail                                                           -
# - wild cruncher: Project thumbnail display                                 -
# ----------------------------------------------------------------------------
# - author    (c) 2013-2013 clemens oertel                                   -
# - copyright (c) 2013-2013 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_thumbnail
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher project thumbnail display

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""

from PyQt4.QtCore import QRectF, Qt
from PyQt4.QtGui import QGraphicsView, QTransform, QSizePolicy, QPainter, \
    QColor


_TARGET_COLOUR = QColor(95, 129, 77)


class WguiThumbnail(QGraphicsView):
    def __init__(self, scene, view, parent=None):
        """
        :type scene: WguiGraphicsScene
        :param view: Graphics view to display viewport for
        :type view: WguiGraphicsView
        :param parent: Parent widget
        """
        super().__init__(scene, parent)

        #: Graphics scene
        self.__scene = scene
        #: Graphics view to display viewport for
        self.__view = view

        self.setInteractive(False)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setRenderHints(QPainter.Antialiasing | QPainter.TextAntialiasing |
                            QPainter.SmoothPixmapTransform |
                            QPainter.HighQualityAntialiasing)

        self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Fixed)
        self.setMaximumWidth(200)

        self.__scene.sceneRectChanged.connect(self._resize)
        self.__view.verticalScrollBar().valueChanged.connect(
            self._viewportChanged)
        self.__view.horizontalScrollBar().valueChanged.connect(
            self._viewportChanged)

        self._resize()

    def _viewportChanged(self, *_):
        """
        Callback function to be called whenever the primary view changes.
        """
        self.viewport().update()

    def paintEvent(self, event):
        super().paintEvent(event)

        painter = QPainter()
        painter.begin(self.viewport())
        painter.setPen(_TARGET_COLOUR)

        sr = self.__scene.sceneRect()
        vc = self.__view.viewCenter()
        vr = self.__view.viewRectangle()
        size = self.viewRectangle()
        width = self.viewport().width() * (sr.width() / size.width())
        height = self.viewport().height() * (sr.height() / size.height())
        xoff = ((size.width() - sr.width()) / 2) / sr.width()
        yoff = ((size.height() - sr.height()) / 2) / sr.height()

        cx = round(width * ((vc.x() - sr.x()) / sr.width() + xoff))
        cy = round(height * ((vc.y() - sr.y()) / sr.height() + yoff))
        crx = round(width * ((vr.x() - sr.x()) / sr.width() + xoff))
        cry = round(height * ((vr.y() - sr.y()) / sr.height() + yoff))
        crw = round(width * vr.width() / sr.width()) - 1
        crh = round(height * vr.height() / sr.height()) - 1

        painter.drawLine(cx, 0, cx, cry)
        painter.drawLine(cx, cry + crh, cx, self.height())
        painter.drawLine(0, cy, crx, cy)
        painter.drawLine(crx + crw, cy, self.width(), cy)
        painter.drawRect(crx, cry, crw, crh)
        painter.end()

    def _resize(self, *_):
        """
        Callback function to be called whenever the scene rectangle changes.
        """
        sceneRect = self.__scene.sceneRect()

        width = self.viewport().width()
        height = self.viewport().height()
        if width <= 0 or height <= 0: return

        wf = sceneRect.width() / width
        hf = sceneRect.height() / height
        if wf <= 0 or hf <= 0: return

        if self.height() != self.width():
            self.setFixedHeight(self.width())

        if width > sceneRect.width() and height > sceneRect.height():
            self.resetTransform()
            self.ensureVisible(sceneRect)
        else:
            zoom = 1 / max(wf, hf)
            self.centerOn(sceneRect.center())
            self.setTransform(QTransform().scale(zoom, zoom))

    def viewRectangle(self):
        """
        Provides the current view area in scene coordinates.

        :return: View area
        :rtype: QRectF
        """
        return self.mapToScene(self.viewport().geometry()).boundingRect()
