# ----------------------------------------------------------------------------
# - wgui-gui-install-context-factory                                         -
# - wild cruncher: gui install context factory         frontend / gui_plugin -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_gui_context_factory_manager
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - GUI install context factory

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_frontend.gui_plugin.wgui_gui_install_context import *
from wcr_core.plugin.wplg_install_context_factory_api import \
    WplgPlatformInstallContextFactoryP, WplgModuleInstallContextFactoryP, \
    WplgInstallContextFactoryP


# noinspection PyAbstractClass
class WguiGuiInstallContextFactoryMixin(WplgInstallContextFactoryP):
    def getComponentCode(self):
        return WGUI_GUI_INSTALL_CODE


class WguiGuiPlatformInstallContextFactory(
        WplgPlatformInstallContextFactoryP,
        WguiGuiInstallContextFactoryMixin):
    """
    GUI platform install context factory.
    """
    def createInstallContext(self, plgCode, plgName, plgDesc, *args, **kwargs):
        return WguiGuiPlatformInstallContext(plgCode, plgName, plgDesc)


class WguiGuiModuleInstallContextFactory(
        WplgModuleInstallContextFactoryP,
        WguiGuiInstallContextFactoryMixin):
    """
    GUI module install context factory.
    """
    def __init__(self, module):
        """
        :param module: GUI module
        :type module: WguiCockpitSystemModuleMP
        """
        self.__module = module

    def getModuleCode(self):
        return self.__module.code

    def createInstallContext(self, plgCode, plgName, plgDesc, *args, **kwargs):
        return WguiGuiModuleInstallContext(
            plgCode, plgName, plgDesc, self.__module)
