# ----------------------------------------------------------------------------
# - bus-gui-cstcx                                                            -
# - wild cruncher:bus design cockpit system contex                           -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbgui_cstcx
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wcs cockpit system contex

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
# noinspection PyUnresolvedReferences
# import wcr_frontend.modules.model.gui.wmgui_icons

from wcr_frontend.modules.bus.gui.wbgui_controller_value_provider import \
    WbguiControllerValueProvider

from wcr_core.common.wcmn_protocols import WcmnReleasableP
from wcr_frontend.gui.wgui_scene_view_mouse_handler import \
    WguiDefaultSceneViewMouseHandler
from wcr_frontend.modules.bus.gui.wbgui_scene import WbguiGraphicsScene
from wcr_frontend.gui.wgui_gui_module import \
    WguiCockpitSystemContextBase, WguiCockpitSystemModuleBase
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler
from wcr_frontend.modules.bus.gui.wbgui_scene_context_menu_factory import \
    WbguiSceneContextMenuFactory
from wcr_core.modules.bus.wbmod_application_module import \
    WBMOD_BUS_APP_CODE, WBMOD_BUS_APP_MODULE
from wcr_frontend.modules.bus.gui.wbgui_props_dialog_factory import \
    WbguiPropertiesDialogFactory
from wcr_frontend.modules.bus.gui.wbgui_scene_item_factory import \
    WbguiSceneItemFactory

from wcr_frontend.modules.bus.gui.wbgui_messages_library_tree import \
    WbguiMessageLibraryTree
from wcr_frontend.modules.bus.gui.wbgui_message_binding_table import \
    WbguiMessageBindingTable

from wcr_frontend.modules.bus import _WBGUI_ICONS


class _WbguiCockpitSystemContext(WguiCockpitSystemContextBase):
    def __init__(self, cmod, pctx, pcfg, actions, actx, parent):
        """
        :param cmod: Cockpit system module
        :type cmod: WguiCockpitSystemModuleP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param pcfg: Project editor configuration
        :type pcfg: WguiProjectEditorCfgP
        :param actions: Actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(cmod)

        modActions = list(WBGUI_COCKPIT_SYSTEM_MODULE.getActions())
        """:type: list[(WactActionP, str, str, tuple[str])]"""

        self.__ctrlValuePrv = WbguiControllerValueProvider(pctx)
        self.__sceneItmFy = WbguiSceneItemFactory(WBMOD_BUS_APP_MODULE)
        self.__scene = WbguiGraphicsScene(
            pctx.getProjectDB(), pcfg, actions, pctx, self, actx, parent)
        self.__propDiagF = WbguiPropertiesDialogFactory(
            pctx, pcfg, self.__scene, actions, parent)
        self.__clpbrdHandler = WguiClipboardHandler(
            self.__scene, cmod.getAppModule())
        self.__cntxtMenuF = WbguiSceneContextMenuFactory(
            self.__scene, actions, self.__clpbrdHandler, pctx,
            self.__propDiagF)
        self.__mouseHandler = WguiDefaultSceneViewMouseHandler(
            pctx, self.__propDiagF, self.__scene, self.__cntxtMenuF)

        self.__sidebarModules = self._createSidebarModules(
            pctx, self.__scene, self.__cntxtMenuF, modActions, parent)

    def release(self):
        for _, _, smod in self.__sidebarModules:
            if isinstance(smod, WcmnReleasableP):
                smod.release()

        for item in [self.__scene, self.__propDiagF, self.__clpbrdHandler,
                     self.__cntxtMenuF, self.__mouseHandler,
                     self.__sidebarModules]:
            if isinstance(item, WcmnReleasableP):
                item.release()

        self.__scene = None
        self.__propDiagF = None
        self.__clpbrdHandler = None
        self.__cntxtMenuF = None
        self.__mouseHandler = None
        self.__sidebarModules = []
        self.__ctrlValuePrv = None

        super().release()

    def _createSidebarModules(self, pctx, scene, cntxtMenuF, actions, parent):
        modules = []

        # portTree = WbguiPortLibraryTree(pctx, self)
        # modules.append(('portstree', 'Ports Library', portTree))
        lsTreeObsActions = (
            a for a, _, _, fs in actions
            if WBMOD_BUS_APP_CODE in fs)
        lsimTree = WbguiMessageLibraryTree(pctx, self, scene, lsTreeObsActions)
        modules.append(('messagetree', 'Messages Library', lsimTree))

        bndg = WbguiMessageBindingTable(pctx, scene, parent)
        modules.append(('messagebindings', 'Message binding', bndg))

        return modules

    def getExcludedSystemToolboxItems(self):
        return []#XPNT_ITEM_ID,CNCT_ITEM_ID]

    def getHighlightingModules(self):
        return []

    def getDirectoryDescription(self, level):
        return 'project' if level == 0 else 'sched. group'

    def getScene(self):
        return self.__scene

    def getSceneViewMouseHandler(self):
        return self.__mouseHandler

    def getClipboardHandler(self):
        return self.__clpbrdHandler

    def getSceneItemFactory(self):
        return self.__sceneItmFy

    def getContextMenuFactory(self):
        return self.__cntxtMenuF

    def getSidebarModules(self):
        return iter(self.__sidebarModules)

    def getPropertiesDialogFactory(self):
        return self.__propDiagF

    def getControllerValueProvider(self):
        return self.__ctrlValuePrv


class _WbguiCockpitSystemModule(WguiCockpitSystemModuleBase):
    def __init__(self):
        super().__init__(WBMOD_BUS_APP_MODULE)

    def createContext(self, pctx, pcfg, actions, actx, parent):
        return _WbguiCockpitSystemContext(
            self, pctx, pcfg, actions, actx, parent)

    def getIconForCode(self, code, iconType):
        return _WBGUI_ICONS.get(code) or \
            super().getIconForCode(code, iconType)


#: GUI Cockpit system cotnext singleton
WBGUI_COCKPIT_SYSTEM_MODULE = _WbguiCockpitSystemModule()
