# ----------------------------------------------------------------------------
# - wbgui-Ports-library-tree                                                 -
# - wild cruncher: Ports library database tree browser                       -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan                                           -
# ----------------------------------------------------------------------------

"""
.. module::       wpgui_logical_simulation_tree
   :platform:     Unix
   :synopsis:     Wild Cruncher bus Portss tree browser

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.action.wact_action_api import WactActionObserverP, \
    WACT_ACTION_EXECUTED_EVENT
from wcr_core.common.wcmn_protocols import WcmnReleasableP
# from wcr_core.db_api.whdb_instance_api import WhdbInstanceP
# from wcr_frontend.gui.wgui_db_tree import WguiDbTreeItemFactory, \
#     WguiDbTreeModelManager, WguiDbTreeSortFilter
from wcr_frontend.gui.wgui_tree import WguiTree, WguiTreeModelManager
from wcr_frontend.gui.wgui_tree_model import WguiFilterModel, \
    WguiSimpleTreeItemFactory, WguiTreeItemFactoryP, WguiTreeItemBase, WguiTreeModel
from wcr_frontend.gui.wgui_protocols import WguiEmptyContextMenuFactory, \
    QWidget_Meta_M
from wcr_frontend.gui.wgui_utils import wgui_getOpenFile
from wcr_core.db.whdb_block_proxy import WhdbEntityContainerProxy
from wcr_frontend.modules.bus.gui.wbgui_import_port_file_handler import \
    WbguiPortImportFileHandler
from wcr_core.modules.bus.db.wbhdb_top_block import BBM_MODULE
from wcr_core.project.wprj_xmlio import WPRJ_HARDWARE_SILKAN_XML_MIME_TYPE

import os

from wcr_core.db.whdb_block import WhdbBlock


from PyQt4.QtCore import Qt
from PyQt4.QtGui import QHBoxLayout, QPushButton


class WbguiPortLibraryTree(WguiTree, WactActionObserverP,
                              WcmnReleasableP, metaclass=QWidget_Meta_M):
    def __init__(self, pctx, cctx, actions=None):
        """
        :param pctx: Project context
        :type pctx: WpmodSystemProjectContextP
        :param cctx: Cockpit system context
        :type cctx: WguiCockpitSystemContextP
        :param actions: Action to observe for update
        :type actions: Iterable[WactActionP] | None
        """
        self.__topLevel = WhdbEntityContainerProxy()
        self.__pctx = pctx
        self.__actions = list(actions) if actions else []

        # item factory
        self.__itemFc = _WbguiPortsTreeItemFactory()

        # WguiTreeModelManager
        modelMngr = WbguiPortsTreeModelManager(
            self.__pctx.getProjectDB().getTopHierarchy().getPorts(),
            self.__itemFc)

        #QAbstractItemModel
        model = WguiFilterModel(modelMngr.model)
        model.setFilterKeyColumn(1)


        #: Collapse-all button
        self.__impButton = QPushButton('Import')

        self.__updateButton = QPushButton('Update')
        self.__updateButton.setEnabled(False)

        self.__updateAllButton = QPushButton('Update All')
        self.__updateAllButton.setEnabled(False)

        #tree init
        super().__init__(model, modelMngr, WguiEmptyContextMenuFactory())

        for action in self.__actions:
            if action:
                action.registerActionObserver(self)

    def _createLayout(self):
        """
        Create the layout.

        :return: Layout
        :rtype: QLayout
        """
        layout = super()._createLayout()

        btnLayout = QHBoxLayout()
        btnLayout.addWidget(self.__impButton)
        btnLayout.addWidget(self.__updateButton)
        btnLayout.addWidget(self.__updateAllButton)
        layout.addLayout(btnLayout)

        return layout

    def _initElements(self):
        """
        Initialize the UI elements as needed.
        """
        super()._initElements()
        #link to import action
        self.__impButton.clicked.connect(self._importPorts)
        self.filterF.setPlaceholderText(
            'Filter filename (regular expression)')
        # link to update action
        # self.__updateButton.clicked.connect(None)
        # link to update all action
        # self.__updateAllButton.clicked.connect(None)

    def release(self):
        for action in self.__actions:
            if action:
                action.unregisterActionObserver(self)
        self.__actions = []
        self.__topLevel.release()
        self.__pctx = None
        super().release()

    def actionNotify(self, actx, action, event, *args, **kwargs):
        if event & WACT_ACTION_EXECUTED_EVENT:
            self.updateTree()

    def updateTree(self):
        pctx = self.__pctx
        if pctx is None:
            return
        if self.__pctx.getProjectDB().getTopHierarchy().getPorts() == {}:
            for f in self.__pctx.getProjectDB().getTopHierarchy().\
                    getPortsFiles():
                self._updatePorts(f)

        super().updateTree()

    def _updatePorts(self, path):
        # Recuperation des Ports
        handler = WbguiPortImportFileHandler(
            *self.__pctx.getProjectDB().getModule())
        itop, name = handler.handleFile(path, self.__pctx)
        if not itop:
            raise Exception("No Ports to be found in {}".format(path))
        if name == '':
            name = 'file:' + os.path.basename(path)
        msgs = self.__pctx.getProjectDB().getTopHierarchy().getPorts()

        # we only want Ports that are used in modules,
        # so we iter over all connection and get the start connection name
        # which correspond to the Ports name
        fmsgs = []
        for conn in itop.getConnections():
            m = conn.getStartConnectionPoint()
            fmsgs.append(m)
        # fill msgs by file dictionary
        msgs[path] = (os.path.basename(path),
                      (itop, name),
                      {BBM_MODULE: fmsgs})

        # saving Ports at right place
        self.__pctx.getProjectDB().getTopHierarchy().setPorts(msgs)
        return name

    def _importPorts(self):
        #Selection d'un fichier
        selected_path = wgui_getOpenFile("Import Ports from icd", self,
                                         WPRJ_HARDWARE_SILKAN_XML_MIME_TYPE)
        if selected_path:
            name = self._updatePorts(selected_path)
            self.__pctx.getProjectDB().getTopHierarchy().\
                addPortsFile(selected_path,name)
            # super().updateTree()
            self.updateTree()
            #update tree to make new Ports apear


class WbguiTreeModel(WguiTreeModel):

    def __init__(self, itemFc):
        super().__init__(itemFc)

    def setSourceModel(self, model):
        None

    def setDynamicSortFilter(self, stat):
        None

    def invalidate(self):
        None


class WbguiPortsTreeModelManager(WguiTreeModelManager):
    """
    Tree-model manager for database trees.
    """

    def __init__(self, ports, itemFy):
        """
        :param tblk: Database top-level block
        :type tblk: WhdbEntityContainerP
        :param itemFy: Tree-item factory
        :type itemFy: WguiTreeItemFactoryP
        """
        def itemIdF(i):
            if i is None:
                return None
            return i.getUUID()
        super(WbguiPortsTreeModelManager, self).__init__(itemIdF, itemFy)

        self.__dtb = ports

    def populateModel(self, fromScratch = True):
        """
        :type fromScratch: bool
        """
        def addFiles(block, path):
            for k in block.keys():
                path1 = list(path)
                f = block[k]
                msgs = f[2]
                f[1][0].setName(f[1][1])
                self.itemAdded(f[1][0], path)
                path1.append(f[1][0])
                # will be modified at the integration of whitebox Ports
                for bbm in msgs[BBM_MODULE]:
                    self.itemAdded(bbm, path1)

        if fromScratch:
            self.itemClear()
        addFiles(self.__dtb, [])

    def itemClear(self):
        """
        Remove all the items. respecting leaforder
        """
        for itemKey, itemVal in self.getmap().items():
            #we delet only "_WbguiMessageFileTreeItem" constituing the first
            # level of the tree.
            if isinstance(itemVal, _WbguiPortsFileTreeItem):
                self.model.removeItem(itemVal)
                del itemVal
        self.getmap().clear()


# noinspection PyAbstractClass
class _WbguiTreeItemBase(WguiTreeItemBase):
    """
    State-machine Datamodel tree item.
    """

    def __init__(self, parentItem=None):
        super().__init__(parentItem)

    def setData(self, data, column, role=None):
        pass

    def getColumnCount(self):
        return 3

    @property
    def flags(self):
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    # @abstractmethod
    def createDetailUI(self):
        """
        Create the detail-view UI.

        :return: Detail-view UI
        :rtype: QWidget
        """
        raise NotImplementedError()


class _WbguiPortsRootTreeItem(_WbguiTreeItemBase):
    def __init__(self):
        super().__init__()

    def getData(self, column, role=None):
        if role == Qt.DisplayRole:
            if column == 0:
                return 'Type'
            elif column == 1:
                return 'Name'
            elif column == 2:
                return 'Protocol'
        return None

    def createDetailUI(self):
        return None


class _WbguiPortsFileTreeItem(_WbguiTreeItemBase):
    def __init__(self, itemData, parentItem=None):

        self.data = itemData

        super().__init__(parentItem)

    def getData(self, column, role=None):
        if role == Qt.DisplayRole:
            if column == 0:
                return 'File'
            elif column == 1:
                return self.data.getName()
            elif column == 2:
                return None
        return None

    def createDetailUI(self):
        return None


class _WbguiPortsTreeItem(_WbguiTreeItemBase):
    def __init__(self, itemData, parentItem=None):

        self.data = itemData

        super().__init__(parentItem)

    def getData(self, column, role=None):
        if role == Qt.DisplayRole:
            if column == 0:
                return 'Ports'
            elif column == 1:
                return self.data.getName()
            elif column == 2:
                return self.data.getCode()
        return None

    def createDetailUI(self):
        return None


class _WbguiPortsTreeItemFactory(WguiTreeItemFactoryP):

    def createTreeItem(self, itemData, parentItem, **kwargs):
        if not parentItem:
            return _WbguiPortsRootTreeItem(), None, None
        if not itemData:
            return None, None, None
        #WARNING whdbBlock are used cause it is the type of topblock of
        # Ports... it coul'd be more specified.
        if isinstance(itemData, WhdbBlock):
            return _WbguiPortsFileTreeItem(itemData, parentItem), \
                   parentItem, None
        return _WbguiPortsTreeItem(itemData, parentItem), parentItem, None
