# ----------------------------------------------------------------------------
# - whgui-host-connection-points-props-widget                                -
# - wild cruncher - cockpit : connection point properties dialog widgets     -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_host_connection_points_props_widget
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher connection point properties dialog widgets

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""
from collections import OrderedDict
from functools import partial
import re

from PyQt4.QtCore import Qt, QAbstractTableModel
from PyQt4.QtGui import QPushButton, QComboBox, QSortFilterProxyModel, \
    QLineEdit, QAbstractItemView, QWidget, QVBoxLayout, QLabel, \
    QTableView, QHeaderView, QHBoxLayout, QSplitter, QItemDelegate, QMessageBox

from wcr_frontend.gui.wgui_props_dialog_widget import \
    WguiXPointPropertiesBase, WguiXPointPropsTextField, \
    WguiXPointPropsDeleteButton, WGUI_NAME_PLACEHOLDER, WGUI_XPNT_CODE, \
    WguiXPointPropsROLookupField, WguiXPointPropsFieldInstance, \
    WguiXPointPropsField
from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_NAME, \
    WHDB_XPOINT_TYPE
from wcr_core.modules.hardware.db.whhdb_connection_point_base import \
    WHHDB_XPOINT_CONFIG_KEY

from wcr_core.modules.hardware.db.whhdb_connection_point_base_meta import *
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_NAMES, WHHDB_XPOINT_DEFAULT_CODE, WHHDB_XPOINT_FIBER_CHANNEL_CODE, \
    WHHDB_XPOINT_DISCRET_CODE, WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_CETRAC_CODE
from wcr_core.modules.hardware.db.whhdb_Host_instance import \
    WhhdbHostInstance
from wcr_frontend import isNameValid, showErrorWindow, NameFormatMessageError
from wcr_core.db.whdb_utils import whdb_getNextName
from wcr_frontend.modules.hardware.gui.whgui_Operator_connection_points_props_widget import \
    _WHGUI_XPOINT_INFO, WguiXPointPropsComboField, _WHGUI_WGUI_CONF_OPTIONS, _WHGUI_COL_CONF,\
    _WHGUI_BASE_HIDDEN_OPTION, _WHGUI_DEFAULT_HIDDEN_OPTIONS, _WhguiXPointConfigDataAdapter,\
    WHHDB_XPOINT_DISCRET_KEY, WHHDB_DISCRET_KEYS, WHHDB_FIBER_KEYS, WHHDB_AFDX_KEYS, WHDB_STATUS_MASK, \
    WHHDB_XPOINT_FIBER_CHANNEL_KEY, WHHDB_XPOINT_AFDX_KEY, _WhguiXPointConfigTableModel, \
    _WHGUI_COL_STATUS, WHHDB_1553_KEYS, getAllHiddenOptions
from wcr_frontend.modules.hardware.gui.whgui_connection_parameter_edition_delegate \
    import WhguiConnectionParameterBindDelegate

_WHGUI_XPNT_CONFIG = '__config'
WHHDB_REPORT_STATUS = 'Status'
_WHHDB_HOST_CETRAC_KEY = WhhdbHostInstance.WHHDB_HOST_CETRAC_KEY


_WHGUI_AFDX_HIDDEN_OPTION = [WHHDB_PORT_AFDX_DELAY_KEY,
                             WHHDB_PORT_AFDX_HPBS_KEY,
                             WHHDB_PORT_AFDX_LPBS_KEY]

# noinspection PyAbstractClass
class WhguiHostXPointProperties(WguiXPointPropertiesBase):
    def __init__(self, instance, pctx, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        self.__module = pctx.getModule()
        """:type: WpmodSystemApplicationModuleP"""
        self.__xpntTypeNames = {
            x.getCode(): x.getName() for x in self.__module.getConnectionPointTypes()}
        self.__xpntCfgMetas = self.getCfgMeta()
        super(WhguiHostXPointProperties, self).__init__(instance, parent=parent)
        self.table = None
        #: Configuration table view
        self.__card_tab = _WhguiPCICardTableInstance(pctx,instance,parent=self)

        self.__ConnectionUpdateF = ConverifF

        if self.__class__ is WhguiHostXPointProperties:
            self._finishInitialization()
        self.instance.restoreCards()

        self.table = self.ui.xpntTableF

    @property
    def cardTab(self):
        return self.__card_tab

    def _loadData(self):
        if self.table:
            for row in range(self.table.rowCount()):
                comboConf = self.table.cellWidget(row, _WHGUI_COL_CONF)
                metaData = self.model.getValue(row + 1, _WHGUI_XPNT_CONFIG)
                connType = metaData['type']
                idx = comboConf.findText(connType)
                comboConf.setCurrentIndex(idx)
                comboConf.currentIndexChanged.connect(partial(self.updatePortField,
                                                              row + 1, comboConf))
                editStatus = self.table.cellWidget(row, _WHGUI_COL_STATUS)
                statusMask = metaData[WHHDB_PORT_STATUS_MASK_KEY]
                editStatus.setText(statusMask)
                editStatus.textChanged.connect(partial(self.updatePortStatusMask,
                                                       row + 1, editStatus))
            # update view of first row when card selected
            comboConf = self.table.cellWidget(0, _WHGUI_COL_CONF)
            self.updatePortField(1, comboConf)

    def getCfgMeta(self, isDiscret=None, isFiber=None, isAfdx=None):
        dflt = {}
        for x in self.__module.getConnectionPointTypes():
            cfgMeta = x.getConfigMeta()
            disMeta = x.getConfigDisMeta()
            fiberMeta = x.getConfigFiberMeta()
            afdxMeta = x.getConfigAfdxMeta()
            if isDiscret:
                dflt[x.getCode()] = cfgMeta + disMeta
            elif isFiber:
                dflt[x.getCode()] = cfgMeta + fiberMeta
            elif isAfdx:
                dflt[x.getCode()] = cfgMeta + afdxMeta
            else:
                dflt[x.getCode()] = cfgMeta + disMeta + fiberMeta + afdxMeta
        return dflt

    def updateFields(self):
        self.__card_tab.resettab()
        super().updateFields()

    def createFields(self):
        return {
            WHDB_XPOINT_NAME: WguiXPointPropsTextField(
                WHDB_XPOINT_NAME, 'Conn. Points', WGUI_NAME_PLACEHOLDER),
            _WHGUI_XPNT_CONFIG: WguiXPointPropsField(_WHGUI_XPNT_CONFIG,
                                                     'Information',
                                                     WhguiXPointConfigFieldInstance,
                                                     self.__xpntCfgMetas),
            _WHGUI_XPOINT_INFO: WguiXPointPropsComboField(
                _WHGUI_XPOINT_INFO, 'Config', _WHGUI_WGUI_CONF_OPTIONS),
            WHDB_STATUS_MASK: WguiXPointPropsTextField(
                WHDB_STATUS_MASK, 'status mask'),
                }

    def getFieldOrder(self):
        return [WHDB_XPOINT_NAME, _WHGUI_XPNT_CONFIG, _WHGUI_XPOINT_INFO]

    def getStretchFields(self):
        return {_WHGUI_XPNT_CONFIG}

    def getButtons(self):
        def add():
            curcard = self.__card_tab.selectedCard(role=Qt.DisplayRole)
            if curcard is None:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Information)
                msg.setText("Select a card to add a port")
                msg.setWindowTitle("Add port Warning")
                msg.exec_()
                return
            selIdx = self.__card_tab.pcicTblW.selectedIndexes()[0]
            ctyp = WHHDB_XPOINT_CETRAC_CODE

            cardType = self.instance.getCardType(curcard)
            if cardType == _WHHDB_HOST_CETRAC_KEY:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Information)
                msg.setText("A port can't be added to a CetraCPCI card.")
                msg.setWindowTitle("Add port Warning")
                msg.exec_()
                return

            if ctyp:
                meta = self.__xpntCfgMetas.get(ctyp)
                confbase=meta.getEmptyValue()
                confbase[WHHDB_TYPE_KEY] = ctyp
                confbase[WHHDB_PORT_CARD_KEY] = curcard
                name = 'Port_{}'.format(ctyp)
                names = [ptx[WHDB_XPOINT_NAME] for ptx in self.model.getValues()]
                connName = whdb_getNextName(name, names)
                portNumbers = [int(ptx[_WHGUI_XPNT_CONFIG][WHHDB_PORT_NUM_KEY])
                               for ptx in self.model.getValues()]
                portNum = max(portNumbers) + 1 if portNumbers else 1
                confbase[WHHDB_PORT_NUM_KEY] = portNum
                self.model.addXPoint(**{
                    WHDB_XPOINT_NAME: connName,
                    WHDB_XPOINT_TYPE: ctyp,
                    _WHGUI_XPNT_CONFIG: confbase})
            self.save()
            self.__card_tab.pcicTblW.setCurrentIndex(selIdx)

        def deleteRow():
            curcard = self.__card_tab.selectedCard(role=Qt.DisplayRole)
            if curcard is None:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Information)
                msg.setText("Select a card to delete a port")
                msg.setWindowTitle("Delete port Warning")
                msg.exec_()
                return
            cardType = self.instance.getCardType(curcard)
            if cardType == _WHHDB_HOST_CETRAC_KEY:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Information)
                msg.setText("Port can't be deleted from a CetraCPCI card.")
                msg.setWindowTitle("Add port Warning")
                msg.exec_()
                return
            selIdx = self.__card_tab.pcicTblW.selectedIndexes()[0]
            xpntTable = self.fields[_WHGUI_XPNT_CONFIG].configTblM
            idx = xpntTable.getLastIndex()
            if idx <= 0:
                return None
            self.model.removeXPoint(idx)
            self.save()
            self.__card_tab.pcicTblW.setCurrentIndex(selIdx)


        addBtn = QPushButton('Add')
        delBtn = QPushButton('Delete')
        addBtn.clicked.connect(add)
        delBtn.clicked.connect(deleteRow)

        return [addBtn, delBtn]

    def resetModel(self, model, instance):
        model.setXPoints(
            {WGUI_XPNT_CODE: x,
             WHDB_XPOINT_NAME: x.getName(),
             WHDB_XPOINT_TYPE: x.getCode(),
             _WHGUI_XPNT_CONFIG:
                 OrderedDict(x.rawElement.getMeta(WHHDB_XPOINT_CONFIG_KEY))}
            for x in instance.getCardConnectionPoints(
                self.__card_tab.selectedCard()))
        self._loadData()

    def createXPoint(self, idx, model, instance):
        return instance.createConnectionPoint(
            ctyp=model.getValue(idx, WHDB_XPOINT_TYPE),
            name=model.getValue(idx, WHDB_XPOINT_NAME),)

    def updateXPoint(self, idx, model, xpnt):
        if not xpnt:
            return None
        name = model.getValue(idx, WHDB_XPOINT_NAME)
        if name:
            xpnt.setName(name)

        metaKey = None
        confCombo = self.table.cellWidget(idx-1, 2)
        if confCombo is None:
            return None
        confValue = confCombo.currentText()
        allMetaData = model.getValue(idx, _WHGUI_XPNT_CONFIG)

        def removeMetaFromKeys(meta, rmKeys):
            tmpMeta = OrderedDict()
            for k in rmKeys:
                if k in meta.keys():
                    tmpMeta[k] = meta[k]
            return tmpMeta

        metaData = removeMetaFromKeys(allMetaData, CONF_KEYS)
        metaData2 = None

        if confValue == WHHDB_XPOINT_AFDX_CODE:
            metaKey = WHHDB_XPOINT_AFDX_KEY
            metaData2 = removeMetaFromKeys(allMetaData, WHHDB_AFDX_KEYS)

        if metaData2:
            # update discret or fiber data
            xpnt.rawElement.setMeta(metaData2, metaKey)
        # update Normal data
        xpnt.rawElement.setMeta(metaData,
                                WHHDB_XPOINT_CONFIG_KEY)

    def updatePortField(self, row, comboConf):
        res = self.fields[_WHGUI_XPNT_CONFIG].configTblM.activeXPointChanged(row)
        if res:
            connType = comboConf.currentText()
            metaData = self.model.getValue(row, _WHGUI_XPNT_CONFIG)
            metaData['type'] = connType
            self.model.setValue(row, _WHGUI_XPNT_CONFIG, metaData)

    def updateHiddenOption(self, opt):
        if opt in WHHDB_XPOINT_AFDX_CODE:
            hiddenOptions = getAllHiddenOptions(opt) + _WHGUI_AFDX_HIDDEN_OPTION
        else:
            hiddenOptions = _WHGUI_DEFAULT_HIDDEN_OPTIONS

        return hiddenOptions

    def updatePortStatusMask(self, row, statusEdit):
        statusMask = statusEdit.text()
        metaData = self.model.getValue(row, _WHGUI_XPNT_CONFIG)
        metaData[WHHDB_PORT_STATUS_MASK_KEY] = statusMask
        self.model.setValue(row, _WHGUI_XPNT_CONFIG, metaData)

    def _createLayout(self):
        container = QWidget()
        container_Layout = QVBoxLayout()
        splitprinc = QSplitter()
        splitprinc.setOrientation(Qt.Vertical)

        up = QWidget()
        up.setLayout(self.__card_tab.createLayout())
        splitprinc.addWidget(up)

        down = QWidget()
        down.setLayout(super()._createLayout())
        container_Layout.addWidget(down)
        container.setLayout(container_Layout)
        splitprinc.addWidget(container)
        layout = QHBoxLayout()
        layout.addWidget(splitprinc)
        # layout.setMargin(2)

        return layout

    def save(self):
        self.__card_tab.syncCards()

        curcard = self.__card_tab.selectedCard(role=Qt.DisplayRole)
        cardportuuidl=[x.getUUID() for x in \
                       self.instance.getCardConnectionPoints(curcard)]
        contoverif=[]
        contabelem=self.model.getValues()
        for con in contabelem:
            if '__xpnt' in con.keys():
                if con['__xpnt'] in self.instance.getConnectionPoints():
                    contoverif.append(con['__xpnt'])

        for x in self.instance.getConnectionPoints():
            if not x.getUUID() in cardportuuidl:
                self.model.addXPoint(
                      **{WGUI_XPNT_CODE: x,
                       WHDB_XPOINT_NAME: x.getName(),
                       WHDB_XPOINT_TYPE: x.getEntityType(),
                       _WHGUI_XPNT_CONFIG:
                           OrderedDict(x.rawElement.getMeta(
                               WHHDB_XPOINT_CONFIG_KEY))})

        #update ports
        super(WhguiHostXPointProperties, self).save()
        # Now all port not binded with a card are the new port so we binding
        # them with the selected card.
        self.instance.bindFreePorts(curcard)

        # Identify card not in card table as to be deleted
        instcards = [(c,t,s) for c,t,_,s,_,_ in self.instance.getPCICardsDescription()]
        tabcards = [(c,t,s) for c,t,s,_,_,_,_ in self.__card_tab.getCards()]
        s_tabcards = set(tabcards)
        s_instcards = set(instcards)
        missing_cards = s_tabcards.symmetric_difference(s_instcards)
        for c in missing_cards:
            self.instance.markPCICard(c[0])
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Information)
            msg.setText("Card is missing a name !")
            msg.setWindowTitle("Save Card Warning")
            msg.exec_()
            return True
        # Clearing the deleted card buffer and ports binded to it
        self.instance.delCardandPorts()

        # assign cardName to each port and check cetracPCI card port number
        for cardName in self.instance.getCards():
            for conn in self.instance.getCardConnectionPoints(cardName):
                if self.instance.getCardType(cardName) == WHHDB_XPOINT_CETRAC_CODE:
                    if conn.getPortNum() < 6:
                        msg = QMessageBox()
                        msg.setIcon(QMessageBox.Information)
                        msg.setText("Card '{}' : port number from 1 to 5 are"
                                    " reserved for CETRAC_PCI Card."
                                    .format(cardName))
                        msg.setWindowTitle("Save Card Warning")
                        msg.exec_()
                        return True
                conn.setCard(cardName)

        self.resetModel(self.model, self.instance)

        self.__card_tab.resettab()

        if self.__ConnectionUpdateF is not None:
            self.__ConnectionUpdateF(contoverif)

        if not self.checkPortNumberConsistency():
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Information)
            msg.setText("Port consistency error, "
                        "multiple ports have the same port number.")
            msg.setWindowTitle("Save Ports Warning")
            msg.exec_()
            return True
        hasError = self.checkStatusMask()
        if hasError:
            return hasError

    def checkStatusMask(self):
        for cardName in self.instance.getCards():
            statusMask = self.instance.getCardStatus(cardName)
            if len(statusMask) > 0 and len(statusMask) != 16:
                showErrorWindow('Configuration Error',
                                'Status report mask mandatory size is 16,'
                                'Card has a bad size of {}.',
                                len(statusMask))
                return True
            for conn in self.instance.getCardConnectionPoints(cardName):
                connStatus = conn.getStatusMask()
                if len(connStatus) > 0 and len(connStatus) != 16:
                    showErrorWindow('Configuration Error',
                                    'Status report mask mandatory size is 16,'
                                    ' port {} has a bad size of {}.',
                                    conn.getName(), len(connStatus))
                    return True
        return False




class _WhguiXPointHostConfigTableModel(_WhguiXPointConfigTableModel):
    def __init__(self, model, context, cfgAdapter, parent=None):
        """
        :param model: Connection points data model
        :type model: WguiXPointPropsModelP
        :param cfgAdapter: Configuration data adapter
        :type cfgAdapter: _WhguiXPointConfigDataAdapter
        :param parent: Parent item
        """
        super().__init__(model, context, cfgAdapter, parent)

        #: Connection points data model
        self.__model = model
        #: Connection points data context
        self.__context = context
        #: Configuration data adapter
        self.__cfgAdapter = cfgAdapter

    def getFlags(self, flgs, index):

        cardTab = self.__context.cardTab
        indexes = cardTab.getSelectedIndexes()
        if len(indexes) == 0:
            return flgs | Qt.ItemIsEditable
        cardIdx = indexes[0]
        cardInfo = cardTab.getCards()[cardIdx.row()]
        # if port is added we don't receive info from selection
        if not cardInfo:
            idx = self.getLastIndex()
            xpnt = self.__model.getValuesForIndex(idx)[_WHGUI_XPNT_CONFIG]
            cardName = xpnt['cardName']
            cardInfo = cardTab.getCard(cardName)
        cardType = cardInfo[1]
        # for CETRAC PCI CARD ONLY
        if cardType == _WHHDB_HOST_CETRAC_KEY:
            # Fix the port Number only
            if index.row() == 0:
                return flgs & Qt.ItemIsEditable
            else:
                return flgs | Qt.ItemIsEditable
        if index.column() == 1:
            return flgs | Qt.ItemIsEditable
        else:
            return flgs & ~Qt.ItemIsEditable


class WhguiXPointConfigFieldInstance(WguiXPointPropsFieldInstance):
    def __init__(self, code, label, context, xpntCfgMetas):
        """
        :param code: Value code for the field
        :type code: str
        :param label: Label for the field
        :type label: str
        :param context: Field installation context
        :type context: WguiXPointPropsFieldCtx
        """
        super().__init__(code, label)

        self.__context = context

        #: Configuration data adapter
        self.__cfgAdapter = _WhguiXPointConfigDataAdapter(
            context.model, xpntCfgMetas)

        #: Configuration

        #: Configuration table view
        self.__configTblF = QTableView()
        #: Configuration table model
        self.__configTblM = _WhguiXPointHostConfigTableModel(
            context.model, context, self.__cfgAdapter)
        self.__configTblDelegate = \
            WhguiConnectionParameterBindDelegate(sorted(WHHDB_XPOINT_NAMES))
        #: Configuration table filter proxy model
        self.__ctblFilter = QSortFilterProxyModel(self.__configTblM)
        self.__ctblFilter.setFilterCaseSensitivity(Qt.CaseInsensitive)
        #: Configuration table filter input
        self.__configFilterF = QLineEdit()

        self.__auxWidget = None

    def _createAuxWidget(self):
        self.__ctblFilter.setSourceModel(self.__configTblM)
        self.__configTblF.setModel(self.__ctblFilter)
        self.__configTblF.setSelectionMode(QAbstractItemView.NoSelection)
        self.__configTblF.horizontalHeader().setResizeMode(
            0, QHeaderView.ResizeToContents)
        self.__configTblF.horizontalHeader().setResizeMode(
            1, QHeaderView.Stretch)
        self.__configTblF.setItemDelegate(self.__configTblDelegate)
        self.__configFilterF.setPlaceholderText('Filter configuration keys')

        self.__context.activeXPointChanged.connect(
            self.__configTblM.activeXPointChanged)
        self.__configFilterF.textChanged.connect(
            self.__ctblFilter.setFilterFixedString)

        wdgt = QWidget()
        layout = QVBoxLayout()
        layout.addWidget(
            QLabel('Configuration values for the selected connection point:'))
        layout.addWidget(self.__configTblF)
        layout.addWidget(self.__configFilterF)
        wdgt.setLayout(layout)
        return wdgt

    @property
    def configTblM(self):
        return self.__configTblM

    def installWidget(self, index, widgetF, textF):
        # noinspection PyShadowingNames
        def formatConfig():
            cfg = self.__context.model.getValue(index, _WHGUI_XPNT_CONFIG)
            """:type: dict[str, str]"""
            if not cfg:
                return ''
            else:
                # set default Values
                if not cfg[WHHDB_PORT_BBONE_KEY]:
                    cfg[WHHDB_PORT_BBONE_KEY] = 'No'
                if not cfg[WHHDB_PORT_STATUS_MONITOR_KEY]:
                    cfg[WHHDB_PORT_STATUS_MONITOR_KEY] = 'Off'
                if not cfg[WHHDB_PORT_SPEED_KEY]:
                    cfg[WHHDB_PORT_SPEED_KEY] = '1G'
                if not cfg[WHHDB_PORT_NUM_KEY]:
                    cfg[WHHDB_PORT_NUM_KEY] = '1'
                if not cfg[WHHDB_PORT_PTP_ACTIVE_KEY]:
                    cfg[WHHDB_PORT_PTP_ACTIVE_KEY] = 'Off'
            return '; '.join(
                '{}: {}'.format(self.__cfgAdapter.getLabel(index, k) or k,
                                cfg.get(k))
                for k in self.__cfgAdapter.getKeys(index) if cfg.get(k))

        def configChanged(idx, code):
            try:
                if idx == index and code == self.code:
                    textUpdateF(formatConfig())
            except:
                pass

        textUpdateF = textF(formatConfig())
        self.__context.model.valueChanged.connect(configChanged)

    def getAuxWidget(self):
        if not self.__auxWidget:
            self.__auxWidget = self._createAuxWidget()
        return self.__auxWidget


class _WhguiPCICardTableInstance(QWidget):
    def __init__(self, pctx, instance, parent=None):
        """
        :param pctx: Project context
        :type pctx:
        :param instance: The instance edited
        :param parent: Parent widget
        """
        super().__init__(parent)
        self._Inst = instance
        self.__parent = parent

        """:type: WpmodSystemApplicationModuleP"""
        if self.__class__ is _WhguiPCICardTableInstance:
            self._xpntTypeNames = WHHDB_XPOINT_NAMES
            self._CardTypesF = self._Inst.getTypes()

        # : Project context
        self.__pctx = pctx
        self.__pcicTblW = QTableView(self)
        self._pcictypF = lambda a,b: a

        self._initModels()
        self._initElements()
        self._loadFromData()

    @property
    def pcicTblM(self):
        return self.__pcicTblM

    @property
    def pcicTblW(self):
        return self.__pcicTblW

    @property
    def pcicItmDlgt(self):
        return self.__pcicItmDlgt

    def _initModels(self):
        self.__pcicTblM = _WhguiPCICardTableInstanceModel(self._xpntTypeNames,
                                                          self.formatcards,
                                                          parent=self)
        self.__pcicItmDlgt = _WhguiPCICardBindDelegate(self._CardTypesF,
                                                       self._pcictypF,
                                                       parent=self)

    def _initElements(self):
        #: Add entry button
        self.__addButton = QPushButton('Add')
        #: Delete entry button
        self.__deleteButton = QPushButton('Delete')
        self.__pcicTblW.setModel(self.pcicTblM)
        self.__pcicTblW.horizontalHeader().setResizeMode(1)
        self.__pcicTblW.setMinimumWidth(500)
        self.__pcicTblW.setEditTriggers(QAbstractItemView.AllEditTriggers)
        self.__pcicTblW.setItemDelegate(self.pcicItmDlgt)

        self.__addButton.clicked.connect(self._addEntry)
        self.__deleteButton.clicked.connect(self._deleteSelEntries)

        self.__pcicTblW.selectionModel(). \
            selectionChanged.connect(self.selecChanged)

    def resettab(self):
        """
        reset self.__pcicTblM
        """
        self.pcicTblM.deleteRows(range(0, self.pcicTblM.rowCount()))
        self._Inst.restoreCards()
        self._loadFromData()

    def _loadFromData(self):
        """
        Import information from HostInst to fill pcicTblM
        """
        #need to initialise tab with instanceinfo
        fullinfo = self._Inst.getPCICardsDescription()
        for card, typ, bind, ptp, status, skew in fullinfo:
            self.pcicTblM.addRow(Cardid=card, Card=card, Ctyp=typ, Ports=bind,
                                 Ptp=ptp, Status=status, Skew=skew)

    def getCards(self):
        """
        return the description of all card describes in the Table
        """
        return self.pcicTblM.getFullCards()

    def getCard(self, cardName):
        cards = self.getCards()
        for cardInfo in cards:
            if cardInfo[0] == cardName:
                return cardInfo

    def formatcards(self, data, column, colhead):
        """
        :param data: data to format
        :type data: Cardname | Cardtype | [PortUUID]
        """
        header_size = len(_WhguiPCICardTableInstanceModel.HEADERS)
        if data is None:
            return ''
        if column == 0:
            return data
        elif column == 1:
            return data
        elif column == 2:
            return data
        elif column == 3:
            return data
        elif column == 4:
            return data
        elif 4 < column < (header_size + len(self._xpntTypeNames)):
            dtplist = self._Inst.getConnectionDistributionTypeByUUID(data)
            for dpt in dtplist.keys():
                if colhead in dpt:
                    return str(dtplist[dpt])
            return '0'
        else:
            return 'No formating available'

    def createLayout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.__pcicTblW)

        btnLayout = QHBoxLayout()
        btnLayout.addStretch(1)
        btnLayout.addWidget(self.__addButton)
        btnLayout.addWidget(self.__deleteButton)
        layout.addLayout(btnLayout)

        return layout

    def _addEntry(self):
        """
        Add entry in the model
        """
        self.pcicTblM.addRow()

    def _deleteSelEntries(self):
        """
        Delete selected entries from the model
        """
        selIdx = self.__pcicTblW.selectedIndexes()
        """:type: list[QModelIndex]"""
        self.pcicTblM.deleteRows([idx.row() for idx in selIdx])
        rl=[]
        for idx in selIdx:
            rl.append(idx.row())
        rowlist = sorted(set(rl))
        if self.pcicTblM.rowCount()>0 and rowlist:
            if rowlist[0] >= self.pcicTblM.rowCount():
                self.__pcicTblW.setCurrentIndex(self.pcicTblM.
                                                index(self.pcicTblM.rowCount()-1,0))
            else:
                self.__pcicTblW.setCurrentIndex(self.pcicTblM.
                                                index(rowlist[0], 0))

    def syncCards(self):
        datas=[]
        datatab = self.pcicTblM.getFullCards()
        for data in datatab:
            cardname, cardtype, ptp, status, skew, bindings, carid = data
            """data = (cardname, cardtype, bindings, cardid)"""
            if data is not None and carid is '' and cardname is not None \
                    and (cardtype is not None or bindings is not None):
                self._Inst.setPCICard(cardname)
                self._Inst.setPCICardType(cardname, cardtype)
                self._Inst.setPCICardPtpMaster(cardname, ptp)
                self._Inst.setPCICardStatus(cardname, status)
                self._Inst.setPCICardMaxSkew(cardname, skew)
            elif carid is not None:
                datas.append(data)
        hcards=[(n, t, b, n) for n,t,b,_,_,_ in
                self._Inst.getPCICardsDescription()]
        for dat in datas:
            # since cardid is the original name of the card it is use to detect
            # name edition
            if dat[6] in [cid for _,_,_,cid in hcards]:
                self._Inst.replaceCard(dat[6], [dat[0], dat[1], dat[2],
                                                     dat[3], dat[4], dat[5]])

    def removecard(self, cardn):
        """
        add card with name cardn sit the card to delete when save
        """
        self._Inst.markPCICard(cardn)

    def selectedCard(self, selectignor=False, role=Qt.UserRole):
        lrow = []
        for index in self.getSelectedIndexes():
            lrow.append(index.row())
        ulrow = set(lrow)
        if len(ulrow) == 1 and not selectignor:
            return self.pcicTblM.data(self.pcicTblM.index(lrow[0],0),
                                      role=role)
        else :
            return None

    def getSelectedIndexes(self):
        return self.__pcicTblW.selectionModel().selectedIndexes()

    def selecChanged(self, newselec, oldselect):
        """
        Signal handler : when selection change reset connection editor with
        the param of the new card selected
        """
        for idx in self.__parent.model.getIndices():
            xpnt = self.__parent.model.getValue(idx, WGUI_XPNT_CODE)
            self.__parent.updateXPoint(idx, self.__parent.model, xpnt)
        self.__parent.resetModel(self.__parent.model,
                                 self.__parent.instance)


class _WhguiPCICardTableInstanceModel(QAbstractTableModel):
    """
    Bus Binding table model.
    """

    TITLE_NAME = 'Card Name'
    TITLE_TYPE = 'Card Type'
    PTP_MASTER = 'PTP Master'
    CARD_STATUS = 'Status'
    MAX_SKEW = 'Max Skew'
    HEADERS = [TITLE_NAME, TITLE_TYPE, PTP_MASTER, CARD_STATUS, MAX_SKEW]

    def __init__(self, xpntTypeNames, formatF, parent=None):
        """
        :param XpntTypeNames: The Names of Xpoint of the mule
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.__parent = parent
        for pt in xpntTypeNames:
            if not pt in self.HEADERS:
                self.HEADERS.append(pt)

        self.__formatF = formatF

        #: Entries as list of tuples of
        #  port, message and type
        self.__entries = []
        """:type: list[(Cardname,Cardtype,[PortUUID])]"""
        self._update()

    @property
    def entries(self):
        return self.__entries

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
               0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if 0 <= column <= self.columnCount():
            if column < 5:
                info = self.__entries[row][column]
            else:
                info = self.__entries[row][5]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return self.__formatF(info, column, self.HEADERS[column])
            elif role == Qt.UserRole:
                return self.__entries[row][6]#info
        return None

    def setData(self, index, data, role=None):
        assert role == Qt.UserRole
        self._setCardEntry(index, data)

    def _setCardEntry(self, index, data):
        card, ctype, ptp, status, skew, bind, cardid = self.__entries[index.row()]
        if index.column() == 0:
            actualentry = (data, ctype, ptp, status, skew, bind, cardid)
        elif index.column() == 1:
            actualentry = (card, data, ptp, status, skew, bind, cardid)
        elif index.column() == 2:
            actualentry = (card, ctype, data, status, skew, bind, cardid)
        elif index.column() == 3:
            actualentry = (card, ctype, ptp, data, skew, bind, cardid)
        elif index.column() == 4:
            actualentry = (card, ctype, ptp, status, data, bind, cardid)
        self.__entries[index.row()] = actualentry

    def getFullCards(self):
        return self.__entries

    def addRow(self, Cardid='', Card='DefaultCard', Ctyp='External', Ports=None, Ptp='No',
               Status=None, Skew=None):
        """
        Add a row at the end of the list (append)

        :param Card: The card name inside the host
        :type Card: str | None
        :param Ctyp: Type of the card
        :type Ctyp: str | None
        :param Ports: ports uuid binded to card
        :type Ports: [uuid] | None
        :return: The appended row index
        :rtype: int
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())

        self.__entries.append((Card, Ctyp, Ptp, Status, Skew, Ports, Cardid))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)

        card, ctype, _, _, _, ports, cardid = self.__entries[rowIndex]
        if card is not None and (ctype is not None or ports is not None):
            try:
                # Unbound function of message
                self.__parent.removecard(self.__entries[rowIndex][0])
                pass
            except ValueError:
                pass

        self.__entries.pop(rowIndex)

        self.endRemoveRows()
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def flags(self, index):
        flgs = super().flags(index)
        cardTypeIdx = 1
        # if cardtype is CetraCpci
        if self.__entries[index.row()][cardTypeIdx] == _WHHDB_HOST_CETRAC_KEY:
            if index.column() < 5:
                flgs |= Qt.ItemIsEditable
            return flgs
        elif index.column() < 4:
            flgs |= Qt.ItemIsEditable
        return flgs


class _WhguiPCICardBindDelegate(QItemDelegate):
    """
    Delegate for bus items (both physical and logical)
    """

    def __init__(self, CardTyp, formatF, parent=None):
        """
        :type CardTyp: () -> list[CardTyp]
        :type formatF: (elem,column) -> str
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)
        self.formatF = formatF
        self.__CardTyp = CardTyp

    def createEditor(self, parent, option, index):
        """
        Create a editors for card table the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """

        def save(*_):
            self.setModelData(editor, index.model(), index)

        editor = None
        col = index.column()
        # id col = 0 is str for cardname
        if col == 0:
            editor = QLineEdit(parent)
        # if col =  combobox with ReportStatus
        elif col == 2:
            editor = QComboBox(parent)
            editor.addItem('No', 'No')
            editor.addItem('Yes', 'Yes')
        elif col == 3:
            editor = QLineEdit(parent)
        elif col == 4:
            editor = QLineEdit(parent)
        # else Non editable.

        return editor

    def setEditorData(self, editor, index):
        """
        :type editor: QComboBox | QLineEdit
        :type index: QModelIndex
        """
        data = index.model().data(index, Qt.EditRole)
        if isinstance(editor, QComboBox):
            for index in range(editor.count()):
                if editor.itemData(index, Qt.UserRole) == data:
                    editor.setCurrentIndex(index)
                    break
            else:
                editor.setCurrentIndex(0)
        elif isinstance(editor,QLineEdit):
            editor.setText(data)

    def setModelData(self, editor, model, index):
        """
        :type editor: QComboBox | QLineEdit
        :type model: QAbstractItemModel
        :type index: QModelIndex
        """
        if isinstance(editor, QComboBox):
            model.setData(index, wgui_qcb_get_data(editor), Qt.UserRole)
        elif isinstance(editor, QLineEdit):
            if index.column() == 3:
                if editor.text():
                    if re.match("^[0-9A-Fa-f]+$", editor.text()) is None:
                        showErrorWindow("Status Error", "Value '{}' is incorrect,"
                                                        " status can only be hexadecimal.",
                                        editor.text())
                    else:
                        model.setData(index, editor.text(), Qt.UserRole)
            if index.column() == 4:
                if editor.text():
                    if re.match("^[0-9]+$", editor.text()) is None:
                        showErrorWindow("Max Skew", "Value '{}' is incorrect,"
                                                    " needs to be a decimal value.",
                                        editor.text())
                    else:
                        model.setData(index, editor.text(), Qt.UserRole)
            if index.column() == 0:
                if isNameValid(editor.text()):
                    model.setData(index, editor.text(), Qt.UserRole)
                else:
                    showErrorWindow("Name Error", NameFormatMessageError,
                                    editor.text())
