# ----------------------------------------------------------------------------
# - whgui-Host-instance-props-dialog                                         -
# - wild cruncher - cockpit : Host properties dialog widgets                 -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbgui_Host_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher Host instance properties dialog

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from PyQt4.QtGui import QComboBox, QVBoxLayout, QLabel

from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog
from \
wcr_frontend.modules.hardware.gui.whgui_Host_connection_points_props_widget\
    import WhguiHostXPointProperties

from wcr_frontend.modules.hardware.gui.whgui_Operator_Modules_Io_props_widget \
    import WhguiOperatorModulesIoProperties

class WhguiHostPropertiesDialog(
    WguiOperatorInstancePropertiesDialog):
    _TYPE_NAME = 'Host Instance'

    _SHOW_OPERATOR_WIDGET = False

    def __init__(self, inst, pctx, updateF, selectXpnt=None, parent=None,
                 ConF=None):
        """
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: () -> _
        :param parent: Parent widget
        """
        self.__connectionF = ConF
        super().__init__(inst, pctx, updateF, selectXpnt, parent)


    def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
        return []#_WhguiPCICardProperties(inst)]

    # noinspection PyMethodMayBeStatic
    def _createAdditionalPropertiesWidgets(self, inst, pctx):
        return []

    def _createConnectionPointsPropertiesWidget(self, inst, pctx):
        return 'Cards', \
               WhguiHostXPointProperties(inst, pctx,
                                         ConverifF=self.__connectionF)

    def _createModuleIOPropertiesWidget(self, inst, pctx):
        return 'Modules Io', \
               WhguiOperatorModulesIoProperties(inst, pctx,
                                         ConverifF=self.__connectionF)


class _WhguiHostProperties(WguiPropertiesWidgetP):
    """
    Property widget for PCI card capabilities.
    """

    def __init__(self, card, parent=None):
        """
        :param card: PCI card instance
        :type card: WhhdbBlackBoxMessageInstance
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: bus instance
        self.__card = card

        #: Type field
        self.__typeF = QComboBox()
        self.__typeF.setEditable(False)

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        """
        Create the layout.

        :return: Layout
        :rtype: QLayout
        """
        layout = QVBoxLayout()
        layout.addWidget(QLabel('Type:'))
        self.__typeF.addItem('')
        for t in self.__card.getTypes():
            self.__typeF.addItem(t)
        layout.addWidget(self.__typeF)
        return layout

    def updateFields(self):
        if self.__card.getType():
            self.__typeF.setCurrentIndex(
                self.__typeF.findText(self.__card.getType()))
        else:
            self.__typeF.setCurrentIndex(0)

    def save(self):
        self.__card.setType(self.__typeF.currentText())
        self.updateFields()
