# ----------------------------------------------------------------------------
# - whgui-io-card-instance-props-dialog                                      -
# - wild cruncher - cockpit : Io card roperties dialog widgets               -
# ----------------------------------------------------------------------------
# - author    (c) 2017-2017 achraf mahti                                     -
# - copyright (c) 2017-2017 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_Io_Card_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher Io Card instance properties dialog

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""
import operator
from PyQt4.QtGui import QHeaderView, QComboBox
from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog, _WguiParametersProperties
from wcr_frontend.gui.wgui_props_dialog_widget import *


class _WguiCardParametersProperties(_WguiParametersProperties):
    def __init__(self, inst, parent=None):
        """
        :param inst: instance
        :type inst: WhdbOperatorInstanceP
        :param parent: Parent widget
        """
        # base initialization
        self.header = []
        super().__init__(inst, parent)
        #: Parameter table

    def updateFields(self):
        table = self.table
        params = self.instance.parameters
        if not params:
            return
        nbRow = len(params)
        # all channels have same column
        nbColumn = len(params['channel1'])
        table.clear()
        table.setColumnCount(nbColumn)
        table.setRowCount(nbRow)
        self.instance.getParameters()
        for channel, row in sorted(params.items()):
            rowNumber = int(channel.strip('channel'))
            for col, param in enumerate(sorted(row.values())):
                self.header.append(param.getName())
                self._addParameter(param, rowNumber-1, col)
        table.setHorizontalHeaderLabels(self.header)
        table.resizeColumnsToContents()
        width = table.horizontalHeader().length()
        table.setMinimumWidth(width)
        self.table.horizontalHeader().setResizeMode(QHeaderView.Stretch)

    def _addParameter(self, param, row, col):
        """
        Add a new parameter line in the parameter table.

        :param param: The parameter to add
        :type param: WhdbParameterP
        :param row: Row number of the new row
        :type row: int
        """
        table = self.table
        value = param.getValue()
        type = param.getDictValue('type')
        # defines a combo box
        if type == 'combo':
            values = param.getDictValue('values').split(';')
            comboBox = QComboBox()
            comboBox.addItems(values)
            idx = comboBox.findText(value)
            comboBox.setCurrentIndex(idx)
            table.setCellWidget(row, col, comboBox)
        else:
            item = QTableWidgetItem(str(value) if value is not None else '')
            if type == 'constant':
                item.setFlags(item.flags() & Qt.ItemIsEditable)
            table.setItem(row, col, item)

    def save(self):
        table = self.table
        card = self.instance
        for row in range(table.rowCount()):
            for col in range(1, table.columnCount()):
                item = table.item(row, col)
                # if value field
                if item:
                    value = item.text()
                else:
                    # get value from combobox
                    item = table.cellWidget(row, col)
                    value = item.currentText()
                chanName = 'channel{}'.format(row+1)
                paramName = self.header[col]
                card.setChannelParamValue(chanName, paramName, value)


class WhguiIoCardPropertiesDialog(WguiOperatorInstancePropertiesDialog):
    _TYPE_NAME = 'Io Card Instance'

    _SHOW_OPERATOR_WIDGET = False

    PARAM_TAB = _WguiCardParametersProperties

    def __init__(self, inst, pctx, updateF, selectXpnt=None, parent=None,
                 ConF=None):
        """
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: () -> _
        :param parent: Parent widget
        """
        # show the card type
        self._TYPE_NAME = 'Card {}'.format(inst)
        self.__connectionF = ConF
        self.opened = False
        super().__init__(inst, pctx, updateF, selectXpnt, parent)
        # widgets.append(('Parameters', _WguiParametersProperties(inst)))
        # widgets.append(('Appearance', WguiFlipRotationProperties(inst)))


    def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
        return []

    # noinspection PyMethodMayBeStatic
    def _createAdditionalPropertiesWidgets(self, inst, pctx):
        return []

    def _createConnectionPointsPropertiesWidget(self, inst, pctx):
        return None

    def exec_(self):
        self.opened = True
        super().exec()

    def closeEvent(self, event):
        self.opened = False

    def getOpened(self):
        return self.opened
