# ----------------------------------------------------------------------------
# - whgui-operator-connection-points-props-widget                            -
# - wild cruncher - cockpit : connection point properties dialog widgets     -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_operator_connection_points_props_widget
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher connection point properties dialog widgets

.. moduleauthor:: Philippe lambert <lambert@silkan.com>
"""
from collections import OrderedDict
from functools import partial

from PyQt4.QtCore import Qt, QAbstractTableModel
from PyQt4.QtGui import QComboBox, QSortFilterProxyModel, \
    QLineEdit, QAbstractItemView, QWidget, QVBoxLayout, QLabel, \
    QTableView, QHeaderView

from wcr_frontend.gui.wgui_props_dialog_widget import \
    WguiXPointPropertiesBase, WguiXPointPropsTextField, \
    WguiXPointPropsDeleteButton, WGUI_NAME_PLACEHOLDER, WGUI_XPNT_CODE, \
    WguiXPointPropsROLookupField, WguiXPointPropsFieldInstance, \
    WguiXPointPropsField, WguiXPointPropsComboField
from wcr_core.modules.hardware.db.whhdb_connection_point_base_meta import *
from wcr_core.db.whdb_connection_point_manager import WHDB_XPOINT_NAME, \
    WHDB_XPOINT_TYPE
from wcr_core.modules.hardware.db.whhdb_connection_point_base import \
    WHHDB_XPOINT_CONFIG_KEY, WHHDB_XPOINT_DISCRET_KEY,\
    WHHDB_XPOINT_FIBER_CHANNEL_KEY, WHHDB_XPOINT_AFDX_KEY, WHHDB_XPOINT_ARINC825_KEY,\
    WHHDB_XPOINT_1553_KEY
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_NAMES, WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_ETHERNET_CODE, \
    WHHDB_XPOINT_CETRAC_CODE, WHHDB_XPOINT_DISCRET_CODE, WHHDB_XPOINT_ARINC825_CODE,\
    WHHDB_XPOINT_FIBER_CHANNEL_CODE, WHHDB_XPOINT_1553_CODE, WHHDB_MODULES_IO_NAMES
from wcr_frontend.modules.hardware.gui.whgui_connection_parameter_edition_delegate \
    import WhguiConnectionParameterBindDelegate
from wcr_frontend import showErrorWindow


_WHGUI_XPNT_CONFIG = '__config'
_WHGUI_XPOINT_INFO = '__info'
WHDB_STATUS_MASK = 'status mask'
_WHGUI_WGUI_CONF_OPTIONS = [(WHHDB_XPOINT_CETRAC_CODE, WHHDB_XPOINT_CETRAC_CODE),
                            (WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_AFDX_CODE),
                            (WHHDB_XPOINT_ETHERNET_CODE, WHHDB_XPOINT_ETHERNET_CODE),
                            ]

_WHGUI_COL_CONF = 2
_WHGUI_COL_STATUS = 3

_WHGUI_BASE_HIDDEN_OPTION = [WHHDB_PORT_CARD_KEY, WHHDB_TYPE_KEY,
                             WHHDB_PORT_ACTIVE_KEY,
                             WHHDB_PORT_STATUS_MASK_KEY]

_WHGUI_AFDX_HIDDEN_OPTION = [WHHDB_PORT_AFDX_NET_ID_KEY,
                             WHHDB_PORT_AFDX_INTERFACE_ID_KEY,
                             WHHDB_PORT_AFDX_SIDE_ID_KEY,
                             WHHDB_PORT_AFDX_LOCATION_ID_KEY]

_WHGUI_DEFAULT_HIDDEN_OPTIONS = _WHGUI_BASE_HIDDEN_OPTION + WHHDB_FIBER_KEYS + \
                                WHHDB_DISCRET_KEYS + WHHDB_AFDX_KEYS +\
                                WHHDB_1553_KEYS


_WHGUI_ALL_PORT_OPTIONS = {'base': _WHGUI_BASE_HIDDEN_OPTION,
                           WHHDB_XPOINT_FIBER_CHANNEL_CODE: WHHDB_FIBER_KEYS,
                           WHHDB_XPOINT_AFDX_CODE: WHHDB_AFDX_KEYS,
                           WHHDB_XPOINT_1553_CODE: WHHDB_1553_KEYS,
                           WHHDB_XPOINT_DISCRET_CODE: WHHDB_DISCRET_KEYS,
                           WHHDB_XPOINT_ARINC825_CODE: WHHDB_ARINC825_KEYS}


def getAllHiddenOptions(ioCode):
    tmpKeys = []
    for code, keys in _WHGUI_ALL_PORT_OPTIONS.items():
        if ioCode != code:
            tmpKeys.extend(keys)
    return tmpKeys


# noinspection PyAbstractClass
class WhguiOperatorInstanceXPointProperties(WguiXPointPropertiesBase):
    def __init__(self, instance, pctx, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        self.__module = pctx.getModule()
        """:type: WpmodSystemApplicationModuleP"""
        self.__xpntTypeNames = {
            x.getCode(): x.getName() for x in self.__module.getConnectionPointTypes()}
        self.__xpntCfgMetas = self.getCfgMeta()

        # init table necessary for
        self.table = None
        self._configStatus = QComboBox()
        self._configStatus.addItem('No', 'No')
        self._configStatus.addItem('Yes', 'Yes')

        super(WhguiOperatorInstanceXPointProperties, self). \
            __init__(instance, parent=parent)

        #: Configuration table view
        self.labelMask = QLabel('Status Mask: ')
        self._configStatusMask = QLineEdit()
        if self.__class__ is WhguiOperatorInstanceXPointProperties:
            self._finishInitialization()
        self.table = self.ui.xpntTableF

        self.__ConnectionUpdateF = ConverifF
        self._loadFromData()

    def getCfgMeta(self, isDiscret=None, isFiber=None, isAfdx=None):
        dflt = {}
        for x in self.__module.getConnectionPointTypes():
            cfgMeta = x.getConfigMeta()
            disMeta = x.getConfigDisMeta()
            fiberMeta = x.getConfigFiberMeta()
            afdxMeta = x.getConfigAfdxMeta()
            if isDiscret:
                dflt[x.getCode()] = cfgMeta + disMeta
            elif isFiber:
                dflt[x.getCode()] = cfgMeta + fiberMeta
            elif isAfdx:
                dflt[x.getCode()] = cfgMeta + afdxMeta
            else:
                dflt[x.getCode()] = cfgMeta + disMeta + fiberMeta + afdxMeta
        return dflt

    def _loadFromData(self):
        """
        Import information from Switch Status
        """
        #need to initialise tab with instanceinfo
        statusMask = self.instance.getStatusMask()
        if statusMask:
            self._configStatusMask.setText(statusMask)
        status = self.instance.getStatus()
        if status is None:
            status = 'No'
        statusIdx = self._configStatus.findText(status)
        self._configStatus.setCurrentIndex(statusIdx)
        if self.table:
            for row in range(self.table.rowCount()):
                comboConf = self.table.cellWidget(row, _WHGUI_COL_CONF)
                metaData = self.model.getValue(row+1, _WHGUI_XPNT_CONFIG)
                connType = metaData['type']
                idx = comboConf.findText(connType)
                comboConf.setCurrentIndex(idx)
                comboConf.currentIndexChanged.connect(partial(self.updatePortField,
                                                              row+1, comboConf))

                editStatus = self.table.cellWidget(row, _WHGUI_COL_STATUS)
                statusMask = metaData[WHHDB_PORT_STATUS_MASK_KEY]
                editStatus.setText(statusMask)

    def createFields(self):
        return {
            WHDB_XPOINT_NAME: WguiXPointPropsTextField(
                WHDB_XPOINT_NAME, 'Name', WGUI_NAME_PLACEHOLDER),
            _WHGUI_XPNT_CONFIG: WguiXPointPropsField(
                _WHGUI_XPNT_CONFIG, 'Information',
                WhguiXPointConfigFieldInstance, self.__xpntCfgMetas),
            _WHGUI_XPOINT_INFO: WguiXPointPropsComboField(
                _WHGUI_XPOINT_INFO, 'Config', _WHGUI_WGUI_CONF_OPTIONS),
            WHDB_STATUS_MASK: WguiXPointPropsTextField(
                WHDB_STATUS_MASK, 'status mask'),
        }

    def getFieldOrder(self):
        return [WHDB_XPOINT_NAME, _WHGUI_XPNT_CONFIG, _WHGUI_XPOINT_INFO]

    def getStretchFields(self):
        return {_WHGUI_XPNT_CONFIG}

    def getButtons(self):
        label = QLabel('Ptp GrandMaster: ')
        return [label, self._configStatus, self.labelMask, self._configStatusMask]

    def resetModel(self, model, instance):
        model.setXPoints(
            {WGUI_XPNT_CODE: x,
             WHDB_XPOINT_NAME: x.getName(),
             WHDB_XPOINT_TYPE: x.getEntityType(),
             _WHGUI_XPNT_CONFIG:
                 OrderedDict(x.rawElement.getMeta(WHHDB_XPOINT_CONFIG_KEY))}
            for x in instance.getConnectionPoints())
        self._loadFromData()

    def createXPoint(self, idx, model, instance):
        pass

    def updateXPoint(self, idx, model, xpnt):
        if not xpnt:
            return None
        name = model.getValue(idx, WHDB_XPOINT_NAME)
        if name:
            xpnt.setName(name)

        metaKey = None
        confCombo = self.table.cellWidget(idx-1, _WHGUI_COL_CONF)
        statusEdit = self.table.cellWidget(idx - 1, _WHGUI_COL_STATUS)
        statusMask = statusEdit.text()
        confValue = confCombo.currentText()
        allMetaData = model.getValue(idx, _WHGUI_XPNT_CONFIG)

        def removeMetaFromKeys(meta, rmKeys):
            tmpMeta = OrderedDict()
            for k in rmKeys:
                if k in meta.keys():
                    tmpMeta[k] = meta[k]
            return tmpMeta

        metaData = removeMetaFromKeys(allMetaData, CONF_KEYS)
        metaData2 = None

        if confValue == WHHDB_XPOINT_DISCRET_CODE:
            metaKey = WHHDB_XPOINT_DISCRET_KEY
            metaData2 = removeMetaFromKeys(allMetaData, WHHDB_DISCRET_KEYS)
        elif confValue == WHHDB_XPOINT_FIBER_CHANNEL_CODE:
            metaKey = WHHDB_XPOINT_FIBER_CHANNEL_KEY
            metaData2 = removeMetaFromKeys(allMetaData, WHHDB_FIBER_KEYS)
        elif confValue == WHHDB_XPOINT_ARINC825_CODE:
            metaKey = WHHDB_XPOINT_ARINC825_KEY
            metaData2 = removeMetaFromKeys(allMetaData, WHHDB_ARINC825_KEYS)
        elif confValue == WHHDB_XPOINT_AFDX_CODE:
            metaKey = WHHDB_XPOINT_AFDX_KEY
            metaData2 = removeMetaFromKeys(allMetaData, WHHDB_AFDX_KEYS)
        elif confValue == WHHDB_XPOINT_1553_CODE:
            metaKey = WHHDB_XPOINT_1553_KEY
            metaData2 = removeMetaFromKeys(allMetaData, WHHDB_1553_KEYS)

        if metaData2:
            # update discret or fiber data
            xpnt.rawElement.setMeta(metaData2, metaKey)
        # update Normal data
        xpnt.rawElement.setMeta(metaData,
                                WHHDB_XPOINT_CONFIG_KEY)
        # update the status mask
        xpnt.setStatusMask(statusMask)

    def updateHiddenOption(self, opt):

        if opt in WHHDB_XPOINT_AFDX_CODE:
            hiddenOptions = getAllHiddenOptions(opt) + _WHGUI_AFDX_HIDDEN_OPTION
        else:
            hiddenOptions = _WHGUI_DEFAULT_HIDDEN_OPTIONS

        return hiddenOptions

    def updatePortField(self, row, comboConf):
        self.fields[_WHGUI_XPNT_CONFIG].configTblM.activeXPointChanged(row)
        connType = comboConf.currentText()
        metaData = self.model.getValue(row, _WHGUI_XPNT_CONFIG)
        metaData['type'] = connType
        self.model.setValue(row, _WHGUI_XPNT_CONFIG, metaData)

    def save(self):
        contoverif = []
        contabelem = self.model.getValues()
        for con in contabelem:
            if '__xpnt' in con.keys():
                if con['__xpnt'] in self.instance.getConnectionPoints():
                    contoverif.append(con['__xpnt'])
        status = self._configStatus.currentText()
        self.instance.setStatus(status)
        statusMask = self._configStatusMask.text()
        self.instance.setStatusMask(statusMask)

        #update ports
        super(WhguiOperatorInstanceXPointProperties, self).save()
        hasError = self.checkStatusMask()

        if hasError:
            return hasError

        self.resetModel(self.model, self.instance)

        if self.__ConnectionUpdateF is not None:
            self.__ConnectionUpdateF(contoverif)

    def checkStatusMask(self):
        for conn in self.instance.getConnectionPoints():
            connStatus = conn.getStatusMask()
            if len(connStatus) > 0 and len(connStatus) != 16:
                showErrorWindow('Configuration Error',
                                'Status report mask mandatory size is 16,'
                                ' port {} has a bad size of {}.',
                                conn.getName(), len(connStatus))
                return True
        statusMask = self._configStatusMask.text()
        if len(statusMask) > 0 and len(statusMask) != 16:
            showErrorWindow('Configuration Error',
                            'Status report mask mandatory size is 16,'
                            'Switch has a bad size of {}.',
                            len(statusMask))
            return True
        return False


class _WhguiXPointConfigDataAdapter(object):
    def __init__(self, model, xpntCfgMetas):
        self.__model = model
        self.__xpntCfgMetas = xpntCfgMetas

    def _getMeta(self, index):
        """
        Get the configuration meta-data for the connection point at the given
        index.

        :param index: Index
        :type index: int
        :return: Configuration meta-data
        :rtype: WdbsRecordMetaTypeP
        """
        ctyp = self.__model.getValue(index, WHDB_XPOINT_TYPE)
        """:type: str"""
        return self.__xpntCfgMetas.get(ctyp)

    def getKeys(self, index):
        """
        Get the configuration keys for the connection point at the given index.

        :param index: Index
        :type index: int
        :return: Configuration keys
        :rtype: Iterable[str]
        """
        return self._getMeta(index).getKeys()

    def getLabel(self, index, key):
        """
        Get the label for the configuration key for the connection point
        at the given index.

        :param index: Index
        :type index: int
        :param key: Configuration key
        :type key: str
        :return: Label
        :rtype: str
        """
        return self._getMeta(index).getLabel(key)


class _WhguiXPointConfigTableModel(QAbstractTableModel):
    HEADERS = ['field', 'data']

    def __init__(self, model, context, cfgAdapter, parent=None):
        """
        :param model: Connection points data model
        :type model: WguiXPointPropsModelP
        :param cfgAdapter: Configuration data adapter
        :type cfgAdapter: _WhguiXPointConfigDataAdapter
        :param parent: Parent item
        """
        super().__init__(parent)

        #: Connection points data model
        self.__model = model
        #: Connection points data context
        self.__context = context
        #: Configuration data adapter
        self.__cfgAdapter = cfgAdapter

        #: Current connection point index
        self.__index = -1
        #: Current configuration keys
        self.__cfgKeys = []

    @property
    def model(self):
        return self.__model

    @property
    def context(self):
        return self.__context

    @property
    def cfgKeys(self):
        return self.__cfgKeys

    def _getCfgDict(self, idx=None):
        """
        Get the configuration dictionary of the connection point at the given
        index.

        :param idx: Index of the connection point (current if `None`)
        :type idx: int | None
        :return: Configuration dictionary
        :rtype: dict[str, str]
        """
        if idx is None:
            idx = self.__index
        if idx is None or idx < 0:
            return {}

        return self.__model.getValue(idx, _WHGUI_XPNT_CONFIG)

    def _getCfgValue(self, key):
        """
        Get the configuration value for the given key for the current
        connection point.

        :param key: Key
        :type key: str
        :return: Configuration value
        :rtype: str
        """
        return self._getCfgDict().get(key)

    def _setCfgValue(self, key, value):
        """
        Set the configuration value for the current connection point for the
        given key to the given value.

        :param key: Key
        :type key: str
        :param value: Value
        :type value: str
        """
        idx = self.__index
        if idx is None or idx < 0:
            return

        cdct = self.__model.getValue(idx, _WHGUI_XPNT_CONFIG)
        """:type: dict[str, str]"""
        cdct[key] = value
        self.__model.setValue(idx, _WHGUI_XPNT_CONFIG, cdct)

    def activeXPointChanged(self, index):
        """
        Callback for changes in the currently active connection point.

        :param index: Index of the currently active connection point
        :type index: int
        """
        self.beginResetModel()
        if self.__index > 0:
            xpnt = self.__model.getValuesForIndex(self.__index)[WGUI_XPNT_CODE]
            self.__context.updateXPoint(self.__index, self.__model, xpnt)
        self.__index = index
        if index > 0:
            try:
                self.__cfgKeys = list(self.__cfgAdapter.getKeys(index))
            except KeyError:
                self.endResetModel()
                return False
            self._hideOptions(index)
        else:
            self.__cfgKeys = []

        self.endResetModel()
        return True

    def _hideOptions(self, index):
        """
        function that hide the option that is not necessary and update the data
        :param index:
        :return:
        """
        connType = None
        conf = self.__model.getValuesForIndex(index)[_WHGUI_XPNT_CONFIG]
        xpnt = self.__model.getValuesForIndex(index)[WGUI_XPNT_CODE]
        if self.__context.table:
            # update data on selecting row
            comboConf = self.__context.table.cellWidget(index - 1, _WHGUI_COL_CONF)
            if not comboConf:
                self.endResetModel()
                return
            connType = comboConf.currentText()
        else:
            metaData = self.__model.getValue(index, _WHGUI_XPNT_CONFIG)
            connType = metaData['type']
        # update data on opening the window
        if connType == WHHDB_XPOINT_AFDX_CODE:
            confValues = xpnt._getConfigAfdx()
            if confValues:
                conf.update(confValues)

        hidOptions = self.__context.updateHiddenOption(connType)
        # hide options depending on port type
        for hiddenOpt in hidOptions:
            if hiddenOpt in self.__cfgKeys:
                self.__cfgKeys.remove(hiddenOpt)

    def getLastIndex(self):
        return self.__index

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__cfgKeys)

    def columnCount(self, parent=None, *args, **kwargs):
        return 2

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
               0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def data(self, index, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            key = self.__cfgKeys[index.row()]
            if index.column() == 0:
                return self.__cfgAdapter.getLabel(self.__index, key)
            else:
                return self._getCfgValue(key)
        return None

    def setData(self, index, value, role=None):
        if role == Qt.EditRole:
            if index.column() == 1:
                key = self.__cfgKeys[index.row()]
                self._setCfgValue(key, value)
                return True
        return False

    def flags(self, index):
        flgs = super().flags(index)
        return self.getFlags(flgs, index)

    def getFlags(self, flgs, index):
        # fix the values for port number and isBackbone
        if index.column() == 1:
            # Fix the port Number only
            if index.row() == 0:
                return flgs & Qt.ItemIsEditable
            else:
                return flgs | Qt.ItemIsEditable
        elif index.column() == 0:
            return flgs & ~Qt.ItemIsEditable


class WhguiXPointConfigFieldInstance(WguiXPointPropsFieldInstance):
    def __init__(self, code, label, context, xpntCfgMetas):
        """
        :param code: Value code for the field
        :type code: str
        :param label: Label for the field
        :type label: str
        :param context: Field installation context
        :type context: WguiXPointPropsFieldCtx
        """
        super().__init__(code, label)

        self.__context = context

        #: Configuration data adapter
        self.__cfgAdapter = _WhguiXPointConfigDataAdapter(
            context.model, xpntCfgMetas)

        #: Configuration

        #: Configuration table view
        self.__configTblF = QTableView()
        #: Configuration table model
        self.__configTblM = _WhguiXPointConfigTableModel(
            context.model, context, self.__cfgAdapter)
        self.__configTblDelegate = \
            WhguiConnectionParameterBindDelegate(sorted(WHHDB_XPOINT_NAMES))

        #: Configuration table filter proxy model
        self.__ctblFilter = QSortFilterProxyModel(self.__configTblM)
        self.__ctblFilter.setFilterCaseSensitivity(Qt.CaseInsensitive)
        #: Configuration table filter input
        self.__configFilterF = QLineEdit()
        self.__auxWidget = None

    def _createAuxWidget(self):
        self.__ctblFilter.setSourceModel(self.__configTblM)
        self.__configTblF.setModel(self.__ctblFilter)
        self.__configTblF.setSelectionMode(QAbstractItemView.NoSelection)
        self.__configTblF.horizontalHeader().setResizeMode(
            0, QHeaderView.ResizeToContents)
        self.__configTblF.horizontalHeader().setResizeMode(
            1, QHeaderView.Stretch)
        self.__configTblF.setItemDelegate(self.__configTblDelegate)
        self.__configFilterF.setPlaceholderText('Filter configuration keys')

        self.__context.activeXPointChanged.connect(
            self.__configTblM.activeXPointChanged)
        self.__configFilterF.textChanged.connect(
            self.__ctblFilter.setFilterFixedString)

        wdgt = QWidget()
        layout = QVBoxLayout()
        layout.addWidget(
            QLabel('Configuration values for the selected connection point:'))
        layout.addWidget(self.__configTblF)
        layout.addWidget(self.__configFilterF)
        wdgt.setLayout(layout)
        return wdgt

    @property
    def configTblM(self):
        return self.__configTblM

    def installWidget(self, index, widgetF, textF):
        # noinspection PyShadowingNames
        def formatConfig():
            cfg = self.__context.model.getValue(index, _WHGUI_XPNT_CONFIG)
            """:type: dict[str, str]"""
            if not cfg:
                return ''

            return '; '.join(
                '{}: {}'.format(self.__cfgAdapter.getLabel(index, k) or k,
                                cfg.get(k))
                for k in self.__cfgAdapter.getKeys(index) if cfg.get(k))

        def configChanged(idx, code):
            if idx == index and code == self.code:
                try:
                    textUpdateF(formatConfig())
                except RuntimeError:
                    pass


        textUpdateF = textF(formatConfig())
        self.__context.model.valueChanged.connect(configChanged)

    def getAuxWidget(self):
        if not self.__auxWidget:
            self.__auxWidget = self._createAuxWidget()
        return self.__auxWidget
