# ----------------------------------------------------------------------------
# - hardware-gui-cstcx                                                       -
# - wild cruncher:hardware design cockpit system contex                      -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_cstcx
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wcs cockpit system contex

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
# noinspection PyUnresolvedReferences
from wcr_core.common.wcmn_protocols import WcmnReleasableP
from wcr_core.modules.hardware.db.whhdb_standard_operators import \
    WHHDB_HOST_CODE, WHHDB_CETRAC_SWITCH_CODE, WHHDB_RACK_IO_CODE, \
    WHHDB_AFDX_CARD_CODE, WHHDB_CETRAC_CARD_CODE, WHHDB_ETHERNET_CARD_CODE, \
    WHHDB_FIBER_CARD_CODE, WHHDB_DISCRET_CARD_CODE, WHHDB_1553_CARD_CODE, \
    WHHDB_ARINC825_CARD_CODE
from wcr_core.modules.hardware.whmod_application_module import \
    WHMOD_HARDWARE_APP_MODULE
from wcr_core.modules.message.wmemod_application_module import *
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler
from wcr_frontend.gui.wgui_db_tree import wgui_createDbTree, \
    WguiDbTreeItemFactory
from wcr_frontend.gui.wgui_gui_module import \
    WguiCockpitSystemContextBase, WguiCockpitSystemModuleBase
from wcr_frontend.gui.wgui_scene_item_ids import XPNT_ITEM_ID
from wcr_frontend.gui.wgui_scene_view_mouse_handler import \
    WguiDefaultSceneViewMouseHandler
from wcr_frontend.modules.hardware.gui.whgui_props_dialog_factory import \
    WhguiPropertiesDialogFactory
from wcr_frontend.modules.hardware.gui.whgui_scene import WhguiGraphicsScene
from wcr_frontend.modules.hardware.gui.whgui_scene_context_menu_factory import \
    WhguiSceneContextMenuFactory
from wcr_frontend.modules.hardware.gui.whgui_scene_item_factory import \
    WhguiSceneItemFactory

#TODO: get proper icones for modules
_WHGUI_ICONS = {WHHDB_HOST_CODE: ':/img/wpgui_pHost.png',
                WHHDB_RACK_IO_CODE: ':/img/wpgui_Rack.png',
                WHHDB_CETRAC_SWITCH_CODE: ':/img/wpgui_phbus.png',
                WHHDB_AFDX_CARD_CODE: ':/img/wpgui_card.png',
                WHHDB_CETRAC_CARD_CODE: ':/img/wpgui_card.png',
                WHHDB_ETHERNET_CARD_CODE: ':/img/wpgui_card.png',
                WHHDB_FIBER_CARD_CODE: ':/img/wpgui_module.png',
                WHHDB_DISCRET_CARD_CODE: ':/img/wpgui_module.png',
                WHHDB_1553_CARD_CODE: ':/img/wpgui_module.png',
                WHHDB_ARINC825_CARD_CODE: ':/img/wpgui_module.png',
                }

class _WhguiCockpitSystemContext(WguiCockpitSystemContextBase):
    def __init__(self, cmod, pctx, pcfg, actions, actx, parent):
        """
        :param cmod: Cockpit system module
        :type cmod: WguiCockpitSystemModuleP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param pcfg: Project editor configuration
        :type pcfg: WguiProjectEditorCfgP
        :param actions: Actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(cmod)

        modActions = list(WHGUI_COCKPIT_SYSTEM_MODULE.getActions())
        """:type: list[(WactActionP, str, str, tuple[str])]"""

        self.__sceneItmFy = WhguiSceneItemFactory(WHMOD_HARDWARE_APP_MODULE)
        self.__scene =WhguiGraphicsScene(#) WguiGraphicsScene(
            pctx.getProjectDB(), pcfg, actions, pctx, self, actx,
            cmod.getAppModule(), parent)
        self.__propDiagF = WhguiPropertiesDialogFactory(
            pctx, pcfg, self.__scene, actions, parent)
        self.__clpbrdHandler = WguiClipboardHandler(
            self.__scene, cmod.getAppModule())
        self.__cntxtMenuF = WhguiSceneContextMenuFactory(
            self.__scene, actions, self.__clpbrdHandler, pctx,
            self.__propDiagF)
        self.__mouseHandler = WguiDefaultSceneViewMouseHandler(
            pctx, self.__propDiagF, self.__scene, self.__cntxtMenuF)

        self.__sidebarModules = self._createSidebarModules(
            pctx, self.__scene, self.__cntxtMenuF, modActions, parent)

    def release(self):
        for _, _, smod in self.__sidebarModules:
            if isinstance(smod, WcmnReleasableP):
                smod.release()

        for item in [self.__scene, self.__propDiagF, self.__clpbrdHandler,
                     self.__cntxtMenuF, self.__mouseHandler,
                     self.__sidebarModules]:
            if isinstance(item, WcmnReleasableP):
                item.release()

        self.__scene = None
        self.__propDiagF = None
        self.__clpbrdHandler = None
        self.__cntxtMenuF = None
        self.__mouseHandler = None
        self.__sidebarModules = []

        super().release()

    def _createSidebarModules(self, pctx, scene, cntxtMenuF, actions, parent):
        modules = []

        dbTree = wgui_createDbTree(
            pctx.getProjectDB().getTopHierarchy(), scene,
            WguiDbTreeItemFactory(self), cntxtMenuF)
        modules.append(('dbtree', 'Database', dbTree))

        return modules

    def getExcludedSystemToolboxItems(self):
        return [XPNT_ITEM_ID,]

    def getHighlightingModules(self):
        return []

    def getDirectoryDescription(self, level):
        return 'project' if level == 0 else 'sched. group'

    def getScene(self):
        return self.__scene

    def getSceneViewMouseHandler(self):
        return self.__mouseHandler

    def getClipboardHandler(self):
        return self.__clpbrdHandler

    def getSceneItemFactory(self):
        return self.__sceneItmFy

    def getContextMenuFactory(self):
        return self.__cntxtMenuF

    def getSidebarModules(self):
        return iter(self.__sidebarModules)

    def getPropertiesDialogFactory(self):
        return self.__propDiagF


class _WhguiCockpitSystemModule(WguiCockpitSystemModuleBase):
    def __init__(self):
        super().__init__(WHMOD_HARDWARE_APP_MODULE)

    def createContext(self, pctx, pcfg, actions, actx, parent):
        return _WhguiCockpitSystemContext(
            self, pctx, pcfg, actions, actx, parent)

    def getIconForCode(self, code, iconType):
        return _WHGUI_ICONS.get(code) or \
            super().getIconForCode(code, iconType)


#: GUI Cockpit system cotnext singleton
WHGUI_COCKPIT_SYSTEM_MODULE = _WhguiCockpitSystemModule()
