# ----------------------------------------------------------------------------
# - whgui-props-dialog-factory                                               -
# - wild cruncher - cockpit: properties dialog box factory                   -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_props_dialog_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher properties dialog box factory

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_frontend.gui.wgui_props_dialog_factory import \
    WguiPropertiesDialogFactory
from wcr_core.modules.hardware.db.whhdb_standard_operators import WHHDB_HOST, \
    WHHDB_CETRAC_SWITCH, WHHDB_RACK, IO_CARDS_CODE, WHHDB_KAZYMIR_FIBER_SWITCH,\
    WHHDB_KAZYMIR_CETRAC_SWITCH, WHHDB_KAZYMIR_SWITCH
from wcr_frontend.modules.hardware.gui.whgui_Host_instance_props_dialog \
    import WhguiHostPropertiesDialog
from \
wcr_frontend.modules.hardware.gui.whgui_Cetrac_Switch_instance_props_dialog\
    import WhguiCetracSwitchPropertiesDialog
from \
wcr_frontend.modules.hardware.gui.whgui_kazymir_switch_instance_props_dialog\
    import WhguiCetracKazymirPropertiesDialog
from wcr_frontend.modules.hardware.gui.whgui_rack_instance_props_dialog\
    import WhguiRackPropertiesDialog
from wcr_frontend.modules.hardware.gui.whgui_Operator_Instance_props_dialog \
    import WhguiOperatorInstancePropertiesDialog
from wcr_frontend.modules.hardware.gui.whgui_Io_Card_instance_props_dialog \
    import WhguiIoCardPropertiesDialog
from wcr_core.modules.hardware.db.whhdb_operator_instance \
    import WhhdbOperatorInstance, WHHDB_OPER_SWITCH_TYPE
from wcr_core.modules.hardware.db.whhdb_iocard_instance import \
    WhhdbIoCardInstance

WHHDB_STD_OPERINST_PROPDLG = {WHHDB_HOST: WhguiHostPropertiesDialog,
                              WHHDB_RACK: WhguiRackPropertiesDialog,
                              WHHDB_CETRAC_SWITCH:
                                  WhguiCetracSwitchPropertiesDialog,
                              WHHDB_KAZYMIR_FIBER_SWITCH:
                                  WhguiCetracKazymirPropertiesDialog,
                              WHHDB_KAZYMIR_CETRAC_SWITCH:
                                  WhguiCetracKazymirPropertiesDialog,
                              WHHDB_KAZYMIR_SWITCH:
                                  WhguiCetracKazymirPropertiesDialog,
                              WhhdbIoCardInstance:
                                  WhguiCetracSwitchPropertiesDialog
                              }

class WhguiPropertiesDialogFactory(WguiPropertiesDialogFactory):
    def __init__(self, prjContext, config, scene, actions, parent=None):
        """
        :param prjContext: Project context
        :type prjContext: WcorProjectContextP
        :param config: Project editor configuration
        :type config: WguiProjectEditorCfgP
        :param scene: Graphic scene adapter
        :type scene: WguiSceneAdapterP
        :param actions: GUI actions
        :type actions: dict[str, WactActionP]
        :param parent: Parent widget
        """
        self.__scene = scene
        super().__init__(prjContext, config, scene, actions, parent)
        self.converifF=scene.verifyConnectionbyPorts


    def _createOperatorInstancePropertiesDialog(self, inst, pctx, updateF,
                                                selectXpnt, parent):
        # check io card
        if isinstance(inst, WhhdbIoCardInstance):
            if inst.op in IO_CARDS_CODE:
                propDlg = WhguiIoCardPropertiesDialog
        else:
            op = inst.getOperator()
            propDlg = WHHDB_STD_OPERINST_PROPDLG.get(op)
        if propDlg == WhguiRackPropertiesDialog:
            return propDlg(inst, pctx, updateF, self.__scene, selectXpnt,
                           parent, self.converifF)
        if propDlg is not None:
            return propDlg(inst, pctx, updateF, selectXpnt, parent,
                           self.converifF)
        elif isinstance(inst, WhhdbOperatorInstance) and \
                        inst.getOperatorMetaType() == WHHDB_OPER_SWITCH_TYPE:
            return WhguiOperatorInstancePropertiesDialog(inst, pctx,
                                                         updateF, selectXpnt,
                                                         parent, self.converifF)
        else:
            return super()._createOperatorInstancePropertiesDialog(
                inst, pctx, updateF, selectXpnt, parent)
