# ----------------------------------------------------------------------------
# - whgui-scene-item-factory                                                 -
# - wild cruncher: scene item factory                hardware cockpit system -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       whgui_scene_item_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher hardware scene item factory

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from PyQt4.QtGui import QColor
from PyQt4.QtCore import Qt
from wcr_frontend.gui.wgui_scene_item import WguiXPointRectPositionProvider,\
    WGUI_CONNECTION_COLOUR, WGUI_CONNECTION_STYLE, _WGUI_CONN_BGRND
from wcr_frontend.gui.wgui_scene_item_factory import WguiDefaultSceneItemFactoy
from wcr_core.db_api.whdb_instance_api import WhdbBlockInstanceP
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_CETRAC_CODE, WHHDB_XPOINT_DISCRET_CODE, \
    WHHDB_XPOINT_ETHERNET_CODE, WHHDB_XPOINT_FIBER_CHANNEL_CODE, WHHDB_XPOINT_1553_CODE,\
    WHHDB_XPOINT_ARINC825_CODE
from wcr_core.modules.hardware.db.whhdb_generic_Host_instance import WhhdbGenericHostInstance

IO_COLOR = QColor(244, 134, 66)

WHGUI_COLOR_CON_POINT_DIC = {WHHDB_XPOINT_AFDX_CODE: QColor(50, 50, 50),
                             WHHDB_XPOINT_CETRAC_CODE: QColor(50, 200, 50),
                             WHHDB_XPOINT_ETHERNET_CODE: QColor(50, 50, 200),
                             WHHDB_XPOINT_DISCRET_CODE: IO_COLOR,
                             WHHDB_XPOINT_FIBER_CHANNEL_CODE: IO_COLOR,
                             WHHDB_XPOINT_ARINC825_CODE: IO_COLOR,
                             WHHDB_XPOINT_1553_CODE: IO_COLOR,}

BANDWIDTH_XPOINT_SPEED_0_0_1 = '10M'
BANDWIDTH_XPOINT_SPEED_0_1 = '100M'
BANDWIDTH_XPOINT_SPEED_1 = '1G'
BANDWIDTH_XPOINT_SPEED_10 = '10G'

BANDWIDTH_XPOINT_ARINC825_SPEED_83_33 = '83.33'
BANDWIDTH_XPOINT_ARINC825_SPEED_125 = '125'
BANDWIDTH_XPOINT_ARINC825_SPEED_250 = '250'
BANDWIDTH_XPOINT_ARINC825_SPEED_500 = '500'
BANDWIDTH_XPOINT_ARINC825_SPEED_1000 = '1000'

WHGUI_STYLE_CON_DIC = {BANDWIDTH_XPOINT_SPEED_0_0_1: Qt.DotLine,
                       BANDWIDTH_XPOINT_SPEED_0_1: Qt.DashDotLine,
                       BANDWIDTH_XPOINT_SPEED_1: Qt.DashLine,
                       BANDWIDTH_XPOINT_SPEED_10: Qt.SolidLine}

WHGUI_ERROR_CON_COLOR = QColor(200, 30, 30)
WHGUI_CONN_BGRND = Qt.lightGray

class WhguiSceneItemFactory(WguiDefaultSceneItemFactoy):
    """
    Message scene item factory.
    """

    def __init__(self, appModule):
        """
        :param appModule: Application module
        :type appModule: WmmodModelApplicationModuleP
        """
        super().__init__(WguiXPointRectPositionProvider())
        self.__instTooltipF = whgui_instance_tooltip_func(appModule)

    def createBlockInstanceSceneItem(self, obj, *args, **kwargs):
        item = super().createBlockInstanceSceneItem(obj, *args, **kwargs)
        item.setTooltipF(self.__instTooltipF)
        return item

    def createOperatorInstanceSceneItem(self, obj, *args, **kwargs):
        item = super().createOperatorInstanceSceneItem(obj, *args, **kwargs)
        item.setTooltipF(self.__instTooltipF)
        return item

    def _updateTooltip(self):
        if self.__tooltipF:
            self.setToolTip(self.__tooltipF(self.getDataObject()))

    def createConnectionPoint(self, conn, item, *args, **kwargs):
        conpoint = super().createConnectionPoint(conn,item,*args,**kwargs)
        if conpoint.getDataObject().getActive() == "Off":
            conpoint.setbackgroundcolor(WHGUI_CONN_BGRND)
        else:
            conpoint.setbackgroundcolor(WHGUI_COLOR_CON_POINT_DIC.get(
                conn.getCode(), _WGUI_CONN_BGRND))

            tooltip = "<b>{}</b> ({})".format(conn.getName(), conn.getCode())
            tooltip += '< br / > <i>{}</i>'.format(conn.getCard())
            conpoint.setToolTip(tooltip)

        return conpoint

    def createConnectionSceneItem(self, obj, *args, **kwargs):
        connection=super().createConnectionSceneItem(obj,*args,**kwargs)
        if obj.getStartConnectionPoint().getCode() == \
                obj.getEndConnectionPoint().getCode():
            connection.setConnColour(WHGUI_COLOR_CON_POINT_DIC.get(
                obj.getStartConnectionPoint().getCode(),
                WGUI_CONNECTION_COLOUR))
        else:
            if obj.getStartConnectionPoint().getCode() == WHHDB_XPOINT_CETRAC_CODE:
                connection.setConnColour(WHGUI_COLOR_CON_POINT_DIC.get(
                    obj.getEndConnectionPoint().getCode(),
                    WGUI_CONNECTION_COLOUR))
                return connection
            elif obj.getEndConnectionPoint().getCode() == WHHDB_XPOINT_CETRAC_CODE:
                connection.setConnColour(WHGUI_COLOR_CON_POINT_DIC.get(
                    obj.getStartConnectionPoint().getCode(),
                    WGUI_CONNECTION_COLOUR))
                return connection
            else:
                connection.setConnColour(WHGUI_ERROR_CON_COLOR)
        connection.setConnStyle(WHGUI_STYLE_CON_DIC.get(
            obj.getStartConnectionPoint().getSpeed(),
            WGUI_CONNECTION_STYLE))
        return connection


def whgui_instance_tooltip_func(appModule):
    """
    :param appModule: Application module
    :type appModule: WmmodModelApplicationModuleP
    :rtype: (WhdbInstanceP) -> str
    """
    def tooltip(instance):
        ttip = '<b>{}</b>'.format(instance.getName())
        if isinstance(instance, WhdbBlockInstanceP):
            block = instance.getBlock()
            btyp = appModule.getBlockType(block.getEntityType())
            if btyp:
                bnam = btyp.getName()
                if bnam:
                    ttip += '<br/><i>{}</i>'.format(bnam.title())

        if isinstance(instance, WhhdbGenericHostInstance):
            cards = instance.getCards()
            for card in cards:
                ttip += '<br/><i>{}</i>'.format(card)

            cards = getattr(instance, "getIoCards", None)
            if cards and len(cards()) > 0:
                ttip += '<br/>---IO---'
                for card in cards():
                    ttip += '<br/><i>{}</i>'.format(card)

        if instance.getInfo():
            ttip += '<br/>' + instance.getInfo()

        return ttip
    return tooltip
