# ----------------------------------------------------------------------------
# - wigui-avail-module-table                                                      -
# - wild cruncher:icd available module widget                                   -
# ----------------------------------------------------------------------------
# - author    (c) 2017-2017 achraf mahti                                     -
# - copyright (c) 2017-2017 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wigui_message_table
   :platform:     Unix, Darwing, Windows
   :synopsis:     Wild Cruncher silkan icd message table widget

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""

import csv
from PyQt4.QtCore import QAbstractTableModel, Qt
from PyQt4.QtGui import QWidget, QItemDelegate, QVBoxLayout, QLineEdit, \
    QHBoxLayout, QPushButton, QLabel
from wcr_frontend.modules.icd.gui.wigui_table import WiguiTable
from wcr_core.common.wcmn_file_handler import WcmnFile
from wcr_frontend.gui.wgui_utils import wgui_getOpenFile
# from wcr_frontend.modules.icd.gui.wigui_module_creation_props_dialog \
#  import WiguiModuleInstanciateDialog


class WiguiAvailModuleTable(WiguiTable):
    """
    available module table widget.
    """

    def __init__(self, pctx, scene, parent=None):
        """
        :param pctx: Project context
        :type pctx: WpmodSystemProjectContextP
        :param scene: Graphics scene (for current block and change signals)
        :type scene: WguiGraphicsScene
        :param parent: Parent widget
        """
        super().__init__(pctx, scene, parent)
        self.scene = scene
        #: Bus binding table model
        self.gblTblM = _WiguiAvailModuleTableModel(parent=self)
        #: Creating editor for element in table
        self.gblDlgt = _WiguiAvailModuleDelegate()
        #: Add entry button
        self.__importButton = QPushButton('Import')
        #: Delete entry button
        self.__exportButton = QPushButton('Export')
        #: Creation entry button
        self.__createButton = QPushButton('Add')

        self._initElements()
        self.setLayout(self._createLayout())
        self._loadFromData()

    def _initElements(self):
        super()._initElements()
        self.__importButton.clicked.connect(self._importModule)
        self.__exportButton.clicked.connect(self._exportModule)
        self.__createButton.clicked.connect(self._createModule)

    def _loadFromData(self):
        for oper in self.scene.projOperators:
            self.gblTblM.addRow(oper)
        super()._loadFromData()

    def _createLayout(self):
        layout = super()._createLayout()
        btnLayout = QHBoxLayout()
        layout.addLayout(btnLayout)
        btnLayout.addWidget(self.__importButton)
        btnLayout.addWidget(self.__exportButton)
        btnLayout.addWidget(self.__createButton)
        filtrLayout = QVBoxLayout()
        layout.addLayout(filtrLayout)
        return layout

    def getOpenFile(self):
        # paramType = parameter.getParamType()
        """:type: WactActionOpenFileParameterTypeP"""
        path = wgui_getOpenFile('Select Import file',
                                self.parent(), None)
        if not path:
            return None
        return WcmnFile(path)

    def _importModule(self):
        """
        Import model from csv file in icd view
        """
        # TODO: call function that open window to import module selected modules
        # path = self.getOpenFile()
        # if path:
        #     with open(path) as importFile:
        #         importReader = csv.reader(importFile)
        #
        # print(path)
        # self.gblTblM.addRow()

    def _exportModule(self):
        """
        Export model to csv file
        """
        # TODO: call function that open window to export module selected module
        pass

    def _createModule(self):
        """
        create model to csv file
        """
        self.scene.createOperator()


class _WiguiAvailModuleTableModel(QAbstractTableModel):
    """
    Available module table model.
    """
    TITLE_NAME = 'Modules'

    HEADERS = [TITLE_NAME]

    def __init__(self, parent=None):
        """
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.__parent = parent

        #: Entries as list of tuples of
        #  name and message
        self.__entries = []

        self._update()

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def _doUpdate(self):
        pass

    def beginResetModel(self):
        super().beginResetModel()

    def endResetModel(self):
        self._doUpdate()
        super().endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
               0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def addRow(self, variable=None):
        """
        Add a row at the end of the list (append)

        :param variable: global variable name
        :type variable: Str
        :param varType: variable type
        :type varType: str
        :return: The appended row index
        :rtype: int
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())

        self.__entries.append(variable)

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices, Unbind=True):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row, Unbind=Unbind)

    def deleteRow(self, rowIndex, Unbind=True):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)

        name = self.__entries[rowIndex]
        if name is not None and Unbind:
            try:
                self.__parent.unbindPortProd(name)
                pass
            except ValueError:
                pass

        self.__entries.pop(rowIndex)

        self.endRemoveRows()
        self._update()

    def data(self, index, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            row = index.row()
            column = index.column()
            if 0 <= column < 2:
                info = self.__entries[row]
                return info
        return None

    # def setData(self, index, data, role=None):
    #     """
    #     :type index: QModelIndex
    #     :type data: object
    #     """
    #     assert role == Qt.EditRole or role == Qt.DisplayRole
    #     scene = self.__parent.scene
    #     oper = scene.projOperators[index.data()]
    #     instanciate = WiguiModuleInstanciateDialog(pctx=scene.projectCtx,
    #                                                oper=oper, parent=scene)
    #     instanciate.exec_()
    #     self._setDataEntry(index, instanciate.getName())

    def _setDataEntry(self, index, data):
        nl = None
        if index.column() == 0:
            nl = data
        self.__entries[index.row()] = nl

    def flags(self, index):
        """
        set flags fot able
        """
        flgs = super().flags(index)
        #allowing editing
        flgs |= Qt.ItemIsEditable
        return flgs

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)


class _WiguiAvailModuleDelegate(QItemDelegate):
    """
    Delegate for Message table
    """

    def __init__(self, parent=None):
        """
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)

    def createEditor(self, parent, option, index):
        """
        Create a label containing for the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """
        editor = QLabel(parent)
        return editor