# ----------------------------------------------------------------------------
# - icd-gui-cstcx                                                            -
# - wild cruncher:icd design cockpit system contex                           -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wigui_cstcx
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wcs cockpit system contex

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
# noinspection PyUnresolvedReferences
# import wcr_frontend.modules.model.gui.wmgui_icons

from wcr_core.modules.message.wmemod_application_module import *
from wcr_frontend.gui.wgui_scene_item_ids import XPNT_ITEM_ID, \
    CNCT_ITEM_ID
from wcr_core.common.wcmn_protocols import WcmnReleasableP
from wcr_frontend.gui.wgui_scene_view_mouse_handler import \
    WguiDefaultSceneViewMouseHandler
from wcr_frontend.modules.icd.gui.wigui_scene import WiguiGraphicsScene
from wcr_frontend.modules.icd.gui.wigui_global_table import WiguiGlobalTable
from wcr_frontend.modules.icd.gui.wigui_module_table import WiguiModuleTable
from wcr_frontend.modules.icd.gui.wigui_binding_table import WiguiBindingTable
from wcr_frontend.modules.icd.gui.wigui_avail_module_table import WiguiAvailModuleTable
from wcr_frontend.gui.wgui_gui_module import \
    WguiCockpitSystemContextBase, WguiCockpitSystemModuleBase
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler

from wcr_frontend.modules.icd.gui.wigui_scene_context_menu_factory import \
    WiguiSceneContextMenuFactory
from wcr_core.modules.icd.wimod_application_module import WIMOD_ICD_APP_MODULE
from wcr_frontend.modules.icd.gui.wigui_props_dialog_factory import \
    WiguiPropertiesDialogFactory
from wcr_frontend.modules.icd.gui.wigui_scene_item_factory import \
    WiguiSceneItemFactory
from wcr_frontend.modules.icd import _WIGUI_ICONS


class _WiguiCockpitSystemContext(WguiCockpitSystemContextBase):
    def __init__(self, cmod, pctx, pcfg, actions, actx, parent):
        """
        :param cmod: Cockpit system module
        :type cmod: WguiCockpitSystemModuleP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param pcfg: Project editor configuration
        :type pcfg: WguiProjectEditorCfgP
        :param actions: Actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(cmod)

        modActions = list(WIGUI_COCKPIT_SYSTEM_MODULE.getActions())
        """:type: list[(WactActionP, str, str, tuple[str])]"""

        self.__sceneItmFy = WiguiSceneItemFactory(WIMOD_ICD_APP_MODULE)
        self.__scene = WiguiGraphicsScene(
            pctx.getProjectDB(), pcfg, actions, pctx, self, actx,
            WIMOD_ICD_APP_MODULE, parent)
        self.__propDiagF = WiguiPropertiesDialogFactory(
            pctx, pcfg, self.__scene, actions, parent)
        self.__clpbrdHandler = WguiClipboardHandler(
            self.__scene, cmod.getAppModule())
        self.__cntxtMenuF = WiguiSceneContextMenuFactory(
            self.__scene, actions, self.__clpbrdHandler, pctx,
            self.__propDiagF)
        self.__mouseHandler = WguiDefaultSceneViewMouseHandler(
            pctx, self.__propDiagF, self.__scene, self.__cntxtMenuF)

        self.__sidebarModules = self._createSidebarModules(
            pctx, self.__scene, self.__cntxtMenuF, modActions, parent)

    def release(self):
        for _, _, smod, _ in self.__sidebarModules:
            if isinstance(smod, WcmnReleasableP):
                smod.release()

        for item in [self.__scene, self.__propDiagF, self.__clpbrdHandler,
                     self.__cntxtMenuF, self.__mouseHandler,
                     self.__sidebarModules]:
            if isinstance(item, WcmnReleasableP):
                item.release()

        self.__scene = None
        self.__propDiagF = None
        self.__clpbrdHandler = None
        self.__cntxtMenuF = None
        self.__mouseHandler = None
        self.__sidebarModules = []

        super().release()

    def _createSidebarModules(self, pctx, scene, cntxtMenuF, actions, parent):
        modules = []
        # add available module
        availModules = WiguiAvailModuleTable(pctx, scene, parent)
        modules.append(('availmodule', 'Available modules', availModules, ('#43c0cc', '#6ee9f4')))
        # add module dictionary
        moduleDict = WiguiModuleTable(pctx, scene, parent)
        modules.append(('module', 'Module dictionary', moduleDict, ('#5F8122', '#C2D622')))
        # add binding dictionary
        pBdtbl = WiguiBindingTable(pctx, scene, parent)
        modules.append(('bind', 'binding dictionary', pBdtbl, ('#43ef8d', '#76f2ab')))
        # add global dictionary
        globalDict = WiguiGlobalTable(pctx, scene, parent)
        modules.append(('global', 'Global dictionary', globalDict, ('#30ccac', '#59eacc')))

        return modules

    def getExcludedSystemToolboxItems(self):
        return [XPNT_ITEM_ID,CNCT_ITEM_ID]

    def getHighlightingModules(self):
        return []

    def getDirectoryDescription(self, level):
        return 'project' if level == 0 else 'sched. group'

    def getScene(self):
        return self.__scene

    def getSceneViewMouseHandler(self):
        return self.__mouseHandler

    def getClipboardHandler(self):
        return self.__clpbrdHandler

    def getSceneItemFactory(self):
        return self.__sceneItmFy

    def getContextMenuFactory(self):
        return self.__cntxtMenuF

    def getSidebarModules(self):
        return iter(self.__sidebarModules)

    def getPropertiesDialogFactory(self):
        return self.__propDiagF

    def isICDProject(self):
        return True


class _WiguiCockpitSystemModule(WguiCockpitSystemModuleBase):
    def __init__(self):
        super().__init__(WIMOD_ICD_APP_MODULE)

    def createContext(self, pctx, pcfg, actions, actx, parent):
        return _WiguiCockpitSystemContext(
            self, pctx, pcfg, actions, actx, parent)

    def getIconForCode(self, code, iconType):
        return _WIGUI_ICONS.get(code) or \
            super().getIconForCode(code, iconType)


#: GUI Cockpit system cotnext singleton
WIGUI_COCKPIT_SYSTEM_MODULE = _WiguiCockpitSystemModule()
