# ----------------------------------------------------------------------------
# - wigui-module-creation-props-dialog                                       -
# - wild cruncher - cockpit : graphic scene widget                           -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan                                           -
# ----------------------------------------------------------------------------

"""
.. module::       wigui_module_creation_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wigui graphics scene editor

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""

from PyQt4.QtGui import QDialogButtonBox, QWidget, QVBoxLayout

from wcr_frontend.modules.icd.gui.wigui_operator_props_dialog import \
    WiguiOperatorInstanciateDialog
from wcr_frontend.modules.icd.gui.wigui_creation_message_props_dialog import \
    _WiguiModuleMessageInstanciate
from wcr_frontend.modules.icd.gui.wigui_creation_type_props_dialog import \
    _WiguiMessageTypeInstanciate


class WiguiModuleInstanciateDialog(WiguiOperatorInstanciateDialog):
    _TYPE_NAME = 'Module Creation'

    _SHOW_OPERATOR_WIDGET = False

    def __init__(self, pctx, selectXpnt=None, oper=None, parent=None):
        """
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: () -> _
        :param parent: Parent widget
        """
        self.scene = parent
        super().__init__(pctx, selectXpnt, oper=oper)

    def _createAdditionalInstancePropertiesWidgets(self, pctx):
        return []

    # noinspection PyMethodMayBeStatic
    def _createAdditionalPropertiesWidgets(self, pctx):
        return [("Message",
                 _WiguiModuleMessageInstanciate(self.instWdgt, pctx, self.scene,
                                                self)),
                 ]

    def _createConnectionPointsPropertiesWidget(self, pctx):
        return None

    def _setMainLayout(self, widgets, widget):
        """
        Set the dialog's property widgets.

        :param widgets: Property widgets, and their titles
        :type widgets: Iterable[(str, WguiPropertiesWidgetP)]
        :param widget: Wrapper widget containing the property widgets
        :type widget: QWidget
        """
        self.__widgets = [widget for _, widget in widgets]

        layout = QVBoxLayout()
        layout.addWidget(widget)
        buttons = QDialogButtonBox(QDialogButtonBox.Ok |
                                   QDialogButtonBox.Cancel)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        self.okStateChanged.connect(
            buttons.button(QDialogButtonBox.Ok).setEnabled)
        layout.addWidget(buttons)
        # create the main layout
        self.setLayout(layout)

    def accept(self):
        hasErr = self._save()
        if not hasErr:
            super().accept()

    def _apply(self):
        """
        Apply the current changes, and leave the dialog window open.
        """
        self._save()
        self._updateFields()

    def _reset(self):
        """
        Reset the dialog window.
        """
        self._updateFields()

    def _save(self):
        """
        Save all changes.
        """
        hasErr = False
        for widget in self.__widgets:
            widgErr = widget.save()
            if widgErr:
                hasErr = True

        return hasErr

    def _updateFields(self):
        """
        Reset all fields to their original state.
        """
        for widget in self.__widgets:
            widget.updateFields()


