# ----------------------------------------------------------------------------
# - wbgui-blbxmessage-instance-props-dialog                                  -
# - wild cruncher - cockpit :                                                -
# -         Black box message properties dialog widgets                      -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wbgui_blbxmessage_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance properties dialog

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from collections import Counter
from PyQt4.QtCore import Qt, QAbstractTableModel
from PyQt4.QtGui import QLabel, QComboBox, QMessageBox, QPushButton,\
    QLineEdit, QAbstractItemView, QWidget, QVBoxLayout, \
    QTableView, QHBoxLayout, QItemDelegate, QSplitter

from wcr_core.db.whdb_utils import whdb_getNextName
from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog
from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend import isNameValid, showErrorWindow, NameFormatMessageError

_WIDB_VARIABLE_NAME = 'name'
_WIDB_OFFSET_SIZE = 'offset'
_WIDB_VARIABLE_TYPE = 'type'
_WIDB_VARIABLE_MSG = 'MessageBind'


class WiguiInstanceModuleProperties(WguiPropertiesWidgetP):
    """
        Configuration widget for a  data.
        """

    def __init__(self, typeName, inst, tophm, parent=None):
        """
        :param typeName: Type name of the displayed element
        :type typeName: str
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param tophm: Hierarchy top level
        :type tophm: WhdbBlock
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Type name
        self.__typeName = typeName
        #: instance object
        self.__instance = inst
        #: Hierarchy top level
        self.__tophm = tophm

        #: Name field
        self.__nameF = QLineEdit()
        self.__nameF.setFixedWidth(400)
        #: Info field
        self.__moduleType = QLineEdit()
        self.__moduleType.setEnabled(False)
        self.__moduleType.setFixedWidth(100)

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        container = QWidget()
        layout = QVBoxLayout()
        layout.addWidget(self.__nameF)
        layout.addWidget(container)
        layout.addWidget(QLabel('Module type:'))
        layout.addWidget(self.__moduleType)
        layout.addStretch(1)
        return layout

    def updateFields(self):
        inst = self.__instance
        self.__nameF.setText(inst.getName())
        self.__moduleType.setText(inst.getModuleType())

    def save(self):
        inst = self.__instance
        if isNameValid(self.__nameF.text()):
            inst.setName(self.__nameF.text())
        else:
            showErrorWindow("Name Error", NameFormatMessageError,
                            self.__nameF.text())

        allEqpNames = tuple(e.getName() for e in self.__tophm.getOperatorInstances())
        counter = Counter(allEqpNames)
        commons = counter.most_common()
        if commons[0][1] > 1:
            showErrorWindow("Duplicate Name Error", "The name {} is already used for another equipement!",
                            commons[0][0])
            return True


class WiguiModulePropertiesDialog(WguiOperatorInstancePropertiesDialog):
        _TYPE_NAME = 'Module Instance '

        _SHOW_OPERATOR_WIDGET = False
        GENERAL_TAB = WiguiInstanceModuleProperties

        def __init__(self, inst, pctx, updateF, selectXpnt=None, parent=None):
            """
            :param inst: the instance object
            :type inst: WhdbInstanceP
            :param pctx: Project context
            :type pctx: WcorProjectContextP
            :param updateF: Callback function to notify about saves
            :type updateF: () -> _
            :param parent: Parent widget
            """
            super().__init__(inst, pctx, updateF, selectXpnt, parent)

        def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
            return []

        # noinspection PyMethodMayBeStatic
        def _createAdditionalPropertiesWidgets(self, inst, pctx):
            return [("Module dictionary", _WiguiModuleMessageInstanciate(inst, pctx))]

        def _createConnectionPointsPropertiesWidget(self, inst, pctx):
            return None


# noinspection PyAbstractClass
class _WiguiModuleMessageInstanciate(QWidget):
    GLB_COL = 0
    LOC_COL = 1

    def __init__(self, instance, pctx, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.instance = instance
        self.__module = pctx.getModule()
        """:type: WpmodSystemApplicationModuleP"""
        self.__xpntTypeNames = ()
        self.messages = self.instance.getData()
        self.__topBlock = pctx.getProjectDB().getTopHierarchy()
        # init table necessary for
        self.__msgTblW = QTableView(self)
        self._initModels()
        self._initElements()
        self.createLayout()

        self._loadFromData()

    @property
    def msgTblM(self):
        return self.__msgTblM

    @property
    def msgTblW(self):
        return self.__msgTblW

    @property
    def msgDlgt(self):
        return self.__msgDlgt

    def _initModels(self):
        self.__msgTblM = _WhguiMessageTableInstanceModel(self.formatMsgs,
                                                         parent=self)
        self.__msgDlgt = _WhguiMessageBindDelegate(parent=self)

    def _initElements(self):
        self.__msgTblW.setModel(self.msgTblM)
        self.__msgTblW.horizontalHeader().setResizeMode(1)
        self.__msgTblW.setMaximumWidth(500)
        self.__msgTblW.setItemDelegate(self.__msgDlgt)

    def _loadFromData(self):
        for msg, (type_, consprod, storage, period) in self.messages.items():
            self.__msgTblM.addRow(type_, msg, consprod, storage, period)

    def resettab(self):
        """
        reset self.__pcicTblM
        """
        self.msgTblM.deleteRows(range(0, self.msgTblM.rowCount()))

    def getMsgs(self):
        """
        return the description of all card describes in the Table
        """
        return self.msgTblM.getFullMessages()

    def formatMsgs(self, data, column):
        """
        :param data: data to format
        :type data: Cardname | Cardtype | [PortUUID]
        """
        if data is None:
            return ''
        return data

    def createLayout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.__msgTblW)

        btnLayout = QHBoxLayout()
        btnLayout.addStretch()
        layout.addLayout(btnLayout)

        self.setLayout(layout)

        return layout

    def _addEntry(self):
        """
        Add entry in the model
        """
        self.msgTblM.addRow()

    def _deleteSelEntries(self):
        """
        Delete selected entries from the model
        """
        selIdx = self.msgTblW.selectedIndexes()
        """:type: list[QModelIndex]"""
        self.msgTblM.deleteRows([idx.row() for idx in selIdx])
        rl = []
        for idx in selIdx:
            rl.append(idx.row())
        rowlist = sorted(set(rl))
        if self.msgTblM.rowCount() > 0 and rowlist:
            if rowlist[0] >= self.pcicTblM.rowCount():
                self.msgTblW.setCurrentIndex(self.msgTblM.
                                             index(self.msgTblM.rowCount() - 1, 0))
            else:
                self.msgTblW.setCurrentIndex(self.msgTblM.
                                             index(rowlist[0], 0))

    def removeMsg(self, glbMsg):
        """
        remove message and variables associated to it
        """
        if glbMsg in self.__parent.msg2variable:
            del self.__parent.msg2variable[glbMsg]
        self.__parent.resetModel(self.__parent.model, glbMsg)

    def selectedMsg(self, selectignor=False, role=Qt.UserRole):
        lrow = []
        for index in self.getSelectedIndexes():
            lrow.append(index.row())
        ulrow = set(lrow)
        if len(ulrow) == 1 and not selectignor:
            return self.msgTblM.data(self.msgTblM.index(lrow[0], 0),
                                     role=role)
        else:
            return None

    def getSelectedIndexes(self):
        return self.msgTblW.selectionModel().selectedIndexes()

    def save(self):
        pass


class _WhguiMessageTableInstanceModel(QAbstractTableModel):
    """
    icd Binding table model.
    """
    TITLE_TYPE = 'Type'
    TITLE_NAME = 'Name'
    TITLE_CONSPROD = 'Con/Prod'
    TITLE_STORAGE = 'Storage'
    TITLE_PERIOD = 'Period'
    HEADERS = [TITLE_TYPE, TITLE_NAME, TITLE_CONSPROD, TITLE_STORAGE, TITLE_PERIOD]

    def __init__(self, formatF, parent=None):
        """
        :param XpntTypeNames: The Names of Xpoint of the mule
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.__parent = parent
        self.__formatF = formatF

        #: Entries as list of tuples of
        #  port, message and type
        self.__entries = []
        self._update()

    @property
    def entries(self):
        return self.__entries

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
                                    0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if 0 <= column <= self.columnCount():
            info = self.__entries[row][column]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return self.__formatF(info, column)
        return None

    def setData(self, index, data, role=None):
        assert role == Qt.UserRole
        self._setMessageEntry(index, data)

    def _setMessageEntry(self, index, data):
        type_, msg, consprod, storage, period = self.__entries[index.row()]
        if index.column() == 0:
            actualentry = (data, msg, consprod, storage, period)
        if index.column() == 1:
            actualentry = (type_, data, consprod, storage, period)
        elif index.column() == 2:
            actualentry = (type_, msg, data, storage, period)
        elif index.column() == 3:
            actualentry = (type_, msg, consprod, data, period)
        elif index.column() == 4:
            actualentry = (type_, msg, consprod, storage, data)
        self.__entries[index.row()] = actualentry

    def checkUnicityMsg(self, msg, row):
        for i, (entry) in enumerate(self.__entries):
            if i != row:
                # check uniqueness on msg name only
                if msg == entry[0]:
                    return False
        return True

    def getFullMessages(self):
        return self.__entries

    def addRow(self, type_, msgGbl, consprod, storage, period):
        """
        Add a row at the end of the list (append)

        :param msgGbl: The message name global
        :type msgGbl: str | None
        :param msgLcl: The message name local
        :type msgLcl: str | None
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())
        self.__entries.append((type_, msgGbl, consprod, storage, period))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)

        msgGbl, msgLcl = self.__entries[rowIndex]
        if msgGbl is not None or msgLcl is not None:
            try:
                # Unbound function of message
                self.__parent.removeMsg(msgGbl)
                pass
            except ValueError:
                pass
        self.__entries.pop(rowIndex)
        self.endRemoveRows()
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def flags(self, index):
        flgs = super().flags(index)
        return flgs


class _WhguiMessageBindDelegate(QItemDelegate):
    """
    Delegate for bus items (both physical and logical)
    """

    def __init__(self, parent=None):
        """
        :type CardTyp: () -> list[CardTyp]
        :type formatF: (elem,column) -> str
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)

    def createEditor(self, parent, option, index):
        """
        Create a editors for card table the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """

        def save(*_):
            self.setModelData(editor, index.model(), index)

        editor = QLabel(parent)

        return editor

    def setEditorData(self, editor, index):
        """
        :type editor: QComboBox | QLineEdit
        :type index: QModelIndex
        """
        data = index.model().data(index, Qt.EditRole)
        editor.setText(data)

    def setModelData(self, editor, model, index):
        """
        :type editor: QComboBox | QLineEdit
        :type model: QAbstractItemModel
        :type index: QModelIndex
        """
        if editor.text():
            model.setData(index, editor.text(), Qt.UserRole)
