# ----------------------------------------------------------------------------
# - wigui-module-table                                                       -
# - wild cruncher:icd module dictionary widget                               -
# ----------------------------------------------------------------------------
# - author    (c) 2017-2017 achraf mahti                                     -
# - copyright (c) 2017-2017 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wigui_module_table
   :platform:     Unix, Darwing, Windows
   :synopsis:     Wild Cruncher silkan icd module dictionary widget

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""

from PyQt4.QtCore import QAbstractTableModel, Qt
from PyQt4.QtGui import QWidget, QItemDelegate, QLabel, QHeaderView, \
    QAbstractItemView, QTableView
from wcr_frontend.modules.icd.gui.wigui_table import WiguiTable
from wcr_core.modules.icd.db.wihdb_module_instance import WihdbModuleInstance


class WiguiModuleTable(WiguiTable):
    """
    Module dictionary table widget.
    """

    def __init__(self, pctx, scene, parent=None):
        """
        :param pctx: Project context
        :type pctx: WpmodSystemProjectContextP
        :param scene: Graphics scene (for current block and change signals)
        :type scene: WguiGraphicsScene
        :param parent: Parent widget
        """
        super().__init__(pctx, scene, parent)

        #: formating function for element ot table
        self.__formatF = lambda o, c: self.formatF(o, c)
        #: Bus binding table model
        self.gblTblM = _WiguiModuleTableModel(self.__formatF,
                                              scene, parent=self)

        #: Consumer or Producer
        self.__consprod = ['C', 'P']
        #: Creating function geting all available modules
        self.__moduleVar = []
        #: Creating function geting all available Types
        self.__varType = []
        #: Creating editor for element in table
        self.gblDlgt = _WiguiModuleDelegate(self.__formatF)

        self._initElements()
        self.setLayout(self._createLayout())
        self._loadFromData()

    def _initElements(self):
        table = self.getGblTblW()
        table.setModel(self.gblTblM)
        nb_col = self.gblTblM.columnCount()
        for col in range(nb_col):
            table.horizontalHeader().setResizeMode(
            col, QHeaderView.ResizeToContents)
        table.setEditTriggers(QAbstractItemView.AllEditTriggers)
        #: delegate classe for edition in table
        table.setItemDelegate(self.gblDlgt)
        table.horizontalHeader().setResizeMode(
            2, QHeaderView.Stretch)

    def _loadFromData(self):
        super()._loadFromData()


class _WiguiModuleTableModel(QAbstractTableModel):
    """
    Module Dictionary table model.
    """
    TITLE_VAR_TYPE = 'Type'
    TITLE_VAR_CONSPROD = 'Cons/Prod'
    TITLE_VARIABLE = 'Module message'
    TITLE_VAR_STORAGE = 'Storage'
    TITLE_VAR_PERIOD = 'Period'

    HEADERS = [TITLE_VAR_TYPE, TITLE_VAR_CONSPROD, TITLE_VARIABLE,
               TITLE_VAR_STORAGE, TITLE_VAR_PERIOD]

    def __init__(self, formatfunc, scene, parent=None):
        """
        :param parent: Parent widget
        """
        super().__init__(parent)

        self.__parent=parent
        self.__scene = scene

        #: Entries as list of tuples of
        #  cons/prod, module var
        self.__entries = []
        """:type: list[]"""

        self.__formatF = formatfunc

        scene.sceneItemSelected.connect(self.populateTable)
        scene.GlobalMsgChanged.connect(self.updateModule)
        self._update()

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def doUpdate(self):
        pass

    def beginResetModel(self):
        super().beginResetModel()

    def endResetModel(self):
        self.doUpdate()
        super().endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
               0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def updateModule(self):
        for inst in self.__scene.getSceneItems():
            modInst = getattr(inst, "getDataObject", None)
            if modInst:
                item = modInst()
                if not isinstance(item, WihdbModuleInstance):
                    continue
                datas = item.getData()
                from PyQt4.QtCore import pyqtRemoveInputHook
                import ipdb
                pyqtRemoveInputHook()
                ipdb.set_trace()


    def populateTable(self, instances=None):
        self.deleteRows(range(self.rowCount()))
        if not instances:
            return
        for inst in instances:
            modInst = getattr(inst, "getDataObject", None)
            if modInst:
                item = modInst()
                if not isinstance(item, WihdbModuleInstance):
                    continue
                datas = item.getData()
                for msg, (type_, consprod, storage, period) in datas.items():
                    modMsg = '{}.{}'.format(item.getName(), msg)
                    self.addRow(type_, consprod, modMsg, storage, period)

    def addRow(self, type_=None, consprod=None, moduleMsg=None, storage=None, period=None):
        """
        Add a row at the end of the list (append)

        :param consprod: if variable is Producer or Consumer
        :type consprod: str (P or C)
        :param moduleMsg: message name in module
        :type moduleMsg: str
        :param varType: variable's type
        :type varType: str
        :return: The appended row index
        :rtype: int
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())

        self.__entries.append((type_, consprod, moduleMsg, storage, period))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices, Unbind=True):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row, Unbind=Unbind)

    def deleteRow(self, rowIndex, Unbind=True):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)

        self.__entries.pop(rowIndex)

        self.endRemoveRows()
        self._update()

    def data(self, index, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            row = index.row()
            column = index.column()
            info = self.__entries[row][column]
            return info
        return None

    def _setDataEntry(self, index, data):
        type_, consprod, moduleMsg, storage, period = self.__entries[index.row()]

        nl = ()
        if index.column() == 0:
            nl = (data, consprod, moduleMsg, storage, period)
        elif index.column() == 1:
            nl = (type_, data, moduleMsg, storage, period)
        elif index.column() == 2:
            nl = (type_, consprod, data, storage, period)
        elif index.column() == 3:
            nl = (type_, consprod, moduleMsg, data, period)
        elif index.column() == 4:
            nl = (type_, consprod, moduleMsg, storage, data)
        self.__entries[index.row()] = nl

    def flags(self, index):
        flgs = super().flags(index)
        #: set table as editable
        flgs |= Qt.ItemIsEditable
        return flgs

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)


class _WiguiModuleDelegate(QItemDelegate):
    """
    Delegate for module dictionary table
    """

    def __init__(self, formatF, parent=None):
        """
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)
        self.formatF = formatF

    def createEditor(self, parent, option, index):
        """
        Create a label containing the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """
        editor = QLabel(parent)
        return editor