# ----------------------------------------------------------------------------
# - wigui-operator-props-dialog                                       -
# - wild cruncher - cockpit : graphic scene widget                           -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan                                           -
# ----------------------------------------------------------------------------

"""
.. module::       wigui_operator_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wigui graphics scene editor

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""


from PyQt4.QtGui import QLabel, QLineEdit, QVBoxLayout, QComboBox

from wcr_frontend.gui.wgui_operator_instance_props_dialog import _WguiParametersProperties
from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP, \
    WguiFlipRotationProperties, \
    WguiPropertiesDialog, WguiXPointProperties, WguiMultiPropertiesWidgetP
from wcr_frontend import isNameValid, showErrorWindow, NameFormatMessageError
from wcr_core.modules.icd.db.wihdb_wrapper_registry import \
    create_new_imported_module

class WiguiOperatorInstanciateDialog(WguiPropertiesDialog):
    """
    The WguiOperatorInstancePropertiesDialog class is a dialog design to
    access and manipulates operator instance data. This include name,
    info and connection points.
    """

    _TYPE_NAME = 'Operator Instanciator'

    _SHOW_OPERATOR_WIDGET = True

    PARAM_TAB = _WguiParametersProperties

    # noinspection PyTypeChecker
    def __init__(self, pctx, projOper, updateF=None, selectXpnt=None, oper=None, parent=None):
        """
        :param inst: Instance
        :type inst: WhdbOperatorInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param selectXpnt: Initially selected connection point, if any
        :type selectXpnt: WhdbConnectionPointP
        :param parent: Parent widget
        """
        super().__init__(self._TYPE_NAME, updateF, parent=parent)

        tophm = pctx.getProjectDB().getTopHierarchy()

        instWdgt = WguiOperatorBaseProperties(self._TYPE_NAME, tophm, projOper, oper)
        self.instWdgt = instWdgt
        addiWdgts = self._createAdditionalInstancePropertiesWidgets(pctx)
        if addiWdgts:
            # noinspection PyTypeChecker
            generalWdgts = [instWdgt] + addiWdgts
        else:
            generalWdgts = [instWdgt]

        widgets = [('General', WguiMultiPropertiesWidgetP(*generalWdgts))]

        widgets.extend(self._createAdditionalPropertiesWidgets(pctx))

        xpntWdgt = self._createConnectionPointsPropertiesWidget(pctx)
        if xpntWdgt:
            widgets.append(xpntWdgt)


        # noinspection PyUnresolvedReferences
        self.setTabbedPropertyWidgets(
            *widgets,
            activate=xpntWdgt[1] if selectXpnt and xpntWdgt else None)


    def showOperatorWidget(self):
        return True

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createAdditionalInstancePropertiesWidgets(self, pctx):
        return []

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createAdditionalPropertiesWidgets(self, pctx):
        return []

    # noinspection PyMethodMayBeStatic
    def _createConnectionPointsPropertiesWidget(self, pctx):
        pass


class WguiOperatorBaseProperties(WguiPropertiesWidgetP):
    """
    Configuration widget to create an icd operator base data.
    """

    def __init__(self, typeName, tophm, projOper, oper=None, parent=None):
        """
        :param typeName: Type name of the displayed element
        :type typeName: str
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param tophm: Hierarchy top level
        :type tophm: WhdbBlock
        :param parent: Parent widget
        """
        super().__init__(parent)
        # no operator at the begining
        self.__operator = oper

        self.__projOperators = projOper
        #: Type name
        self.__typeName = typeName
        #: Hierarchy top level
        self.__tophm = tophm

        #: Name field
        # if operator exist it's name can't be changed
        self.__nameF = QLineEdit()
        if oper:
            self.__nameF.setText(oper.getName())
            self.__nameF.setReadOnly(True)
        self.__moduleType = QComboBox()
        self.setLayout(self._createLayout())

    @property
    def moduleName(self):
        return self.__nameF.text()

    @property
    def moduleType(self):
        return self.__moduleType.currentText()

    @property
    def operator(self):
        return self.__operator

    def setOperator(self, op):
        self.__operator = op

    def updateFields(self):
        pass

    def _createLayout(self):
        typeName = self.__typeName
        layout = QVBoxLayout()
        layout.addWidget(QLabel(typeName + ' Name:'))
        layout.addWidget(self.__nameF)
        layout.addWidget(QLabel('Module Type:'))
        self.__moduleType.addItems(('CetraC', 'Extern'))
        layout.addWidget(self.__moduleType)
        layout.addStretch(1)
        return layout

    def save(self):
        mod_name = self.moduleName
        if not isNameValid(self.__nameF.text()):
            showErrorWindow("Name Error", NameFormatMessageError,
                            self.__nameF.text())
            return True
        if mod_name in self.__projOperators.keys():
            showErrorWindow("Module import Error",
                            "Module with name \"{}\" already defined", mod_name)
            return True


        WIHDB_MODULE_CODE = 'wcr.icd.module.{}'.format(mod_name)
        oper = create_new_imported_module(WIHDB_MODULE_CODE)
        self.setOperator(oper)
