# ----------------------------------------------------------------------------
# - mess-gui-cstcx                                                           -
# - wild cruncher:message design cockpit system contex                       -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmegui_cstcx
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wcs cockpit system contex

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
# noinspection PyUnresolvedReferences
# import wcr_frontend.modules.model.gui.wmgui_icons

from wcr_core.modules.message.wmemod_application_module import *
from wcr_frontend.gui.wgui_scene_item_ids import XPNT_ITEM_ID, \
    CNCT_ITEM_ID
from wcr_core.common.wcmn_protocols import WcmnReleasableP
from wcr_frontend.gui.wgui_scene_view_mouse_handler import \
    WguiDefaultSceneViewMouseHandler
from wcr_frontend.gui.wgui_scene import WguiGraphicsScene
from wcr_frontend.gui.wgui_workspace_tree import WguiWorkspaceTree
from wcr_frontend.gui.wgui_gui_module import \
    WguiCockpitSystemContextBase, WguiCockpitSystemModuleBase
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler
from wcr_frontend.gui.wgui_db_tree import wgui_createDbTree, \
    WguiDbTreeItemFactory
from wcr_frontend.modules.message.gui.wmegui_scene_context_menu_factory import \
    WmeguiSceneContextMenuFactory
from wcr_core.modules.message.wmemod_application_module import \
    WMEMOD_MESSAGE_APP_MODULE
from wcr_frontend.modules.message.gui.wmegui_props_dialog_factory import \
    WmeguiPropertiesDialogFactory
from wcr_frontend.modules.message.gui.wmegui_scene_item_factory import \
    WmeguiSceneItemFactory
from wcr_core.modules.message.db.wmehdb_standard_operators import \
    WMEHDB_WBMESSAGE_CODE

from wcr_frontend.modules.message.gui.wmegui_datas_library_tree import \
    WmeguiDataLibraryTree
from wcr_frontend.modules.message.gui.wmegui_message_binding_table import \
    WmeguiMessageBindingTable


#TODO: get proper icones for modules
_WMEGUI_ICONS = {WMEHDB_WBMESSAGE_CODE: ':/img/wpgui_phost.png'}



class _WmeguiCockpitSystemContext(WguiCockpitSystemContextBase):
    def __init__(self, cmod, pctx, pcfg, actions, actx, parent):
        """
        :param cmod: Cockpit system module
        :type cmod: WguiCockpitSystemModuleP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param pcfg: Project editor configuration
        :type pcfg: WguiProjectEditorCfgP
        :param actions: Actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(cmod)

        modActions = list(WMEGUI_COCKPIT_SYSTEM_MODULE.getActions())
        """:type: list[(WactActionP, str, str, tuple[str])]"""

        self.__sceneItmFy = WmeguiSceneItemFactory(WMEMOD_MESSAGE_APP_MODULE)
        self.__scene = WguiGraphicsScene(
            pctx.getProjectDB(), pcfg, actions, pctx, self, actx, parent)
        self.__propDiagF = WmeguiPropertiesDialogFactory(
            pctx, pcfg, self.__scene, actions, parent)
        self.__clpbrdHandler = WguiClipboardHandler(
            self.__scene, cmod.getAppModule())
        self.__cntxtMenuF = WmeguiSceneContextMenuFactory(
            self.__scene, actions, self.__clpbrdHandler, pctx,
            self.__propDiagF)
        self.__mouseHandler = WguiDefaultSceneViewMouseHandler(
            pctx, self.__propDiagF, self.__scene, self.__cntxtMenuF)

        self.__sidebarModules = self._createSidebarModules(
            pctx, self.__scene, self.__cntxtMenuF, modActions, parent)

    def release(self):
        for _, _, smod in self.__sidebarModules:
            if isinstance(smod, WcmnReleasableP):
                smod.release()

        for item in [self.__scene, self.__propDiagF, self.__clpbrdHandler,
                     self.__cntxtMenuF, self.__mouseHandler,
                     self.__sidebarModules]:
            if isinstance(item, WcmnReleasableP):
                item.release()

        self.__scene = None
        self.__propDiagF = None
        self.__clpbrdHandler = None
        self.__cntxtMenuF = None
        self.__mouseHandler = None
        self.__sidebarModules = []

        super().release()

    def _createSidebarModules(self, pctx, scene, cntxtMenuF, actions, parent):
        modules = []

        dbTree = wgui_createDbTree(
            pctx.getProjectDB().getTopHierarchy(), scene,
            WguiDbTreeItemFactory(self), cntxtMenuF)
        modules.append(('dbtree', 'Database', dbTree))

        lsTreeObsActions = (
            a for a, _, _, fs in actions
            if WMEMOD_MESSAGE_APP_CODE in fs)
        lsimTree = WmeguiDataLibraryTree(
            pctx, self, lsTreeObsActions)
        modules.append(('datatree', 'Data Library', lsimTree))

        bndg = WmeguiMessageBindingTable(pctx, scene, parent)
        modules.append(('messagebindings', 'Message binding', bndg))

        return modules

    def getExcludedSystemToolboxItems(self):
        return [XPNT_ITEM_ID,CNCT_ITEM_ID]

    def getHighlightingModules(self):
        return []

    def getDirectoryDescription(self, level):
        return 'project' if level == 0 else 'sched. group'

    def getScene(self):
        return self.__scene

    def getSceneViewMouseHandler(self):
        return self.__mouseHandler

    def getClipboardHandler(self):
        return self.__clpbrdHandler

    def getSceneItemFactory(self):
        return self.__sceneItmFy

    def getContextMenuFactory(self):
        return self.__cntxtMenuF

    def getSidebarModules(self):
        return iter(self.__sidebarModules)

    def getPropertiesDialogFactory(self):
        return self.__propDiagF


class _WmeguiCockpitSystemModule(WguiCockpitSystemModuleBase):
    def __init__(self):
        super().__init__(WMEMOD_MESSAGE_APP_MODULE)

    def createContext(self, pctx, pcfg, actions, actx, parent):
        return _WmeguiCockpitSystemContext(
            self, pctx, pcfg, actions, actx, parent)

    def getIconForCode(self, code, iconType):
        return _WMEGUI_ICONS.get(code) or \
            super().getIconForCode(code, iconType)


#: GUI Cockpit system cotnext singleton
WMEGUI_COCKPIT_SYSTEM_MODULE = _WmeguiCockpitSystemModule()
