# ----------------------------------------------------------------------------
# - wmegui-scene-item-factory                                                -
# - wild cruncher: scene item factory                 message cockpit system -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2013-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmegui_scene_item_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher model scene item factory

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
# from wcr_core.db_api.whdb_instance_api import WhdbBlockInstanceP
from wcr_frontend.gui.wgui_scene_item import WguiXPointRectPositionProvider
from wcr_frontend.gui.wgui_scene_item_factory import WguiDefaultSceneItemFactoy

class WmeguiSceneItemFactory(WguiDefaultSceneItemFactoy):
    """
    Message scene item factory.
    """

    def __init__(self, appModule):
        """
        :param appModule: Application module
        :type appModule: WmmodModelApplicationModuleP
        """
        super().__init__(WguiXPointRectPositionProvider())

        self.__instTooltipF = wmegui_instance_tooltip_func(appModule)

    def createBlockInstanceSceneItem(self, obj, *args, **kwargs):
        item = super().createBlockInstanceSceneItem(obj, *args, **kwargs)
        item.setTooltipF(self.__instTooltipF)
        return item


def wmegui_instance_tooltip_func(appModule):
    """
    :param appModule: Application module
    :type appModule: WmmodModelApplicationModuleP
    :rtype: (WhdbInstanceP) -> str
    """
    def tooltip(instance):
        ttip = '<b>{}</b>'.format(instance.getName())

        if isinstance(instance, WhdbBlockInstanceP):
            block = instance.getBlock()
            btyp = appModule.getBlockType(block.getEntityType())
            if btyp:
                bnam = btyp.getName()
                if bnam:
                    ttip += '<br/><i>{}</i>'.format(bnam.title())

        if instance.getInfo():
            ttip += '<br/>' + instance.getInfo()

        return ttip

    return tooltip
