# ----------------------------------------------------------------------------
# - wfgui-clipboard                                                          -
# - wild cruncher: clipboard handler           frontend / statemachine / gui -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfgui_clipboard
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - Clipboard handler

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_log import SCMN_LOG_WARN
from wcr_core.dbstore_api.wdbs_instance_api import WdbsBlockInstanceP, \
    WdbsOperatorInstanceP
from wcr_core.modules.statemachine.db.wfhdb_standard_operators import \
    WFHDB_XBT_PROXY_OP_CODES
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler


class WfguiClipboardHandler(WguiClipboardHandler):
    def __init__(self, cntxt, scene, module):
        """
        :param cntxt: Context
        :type cntxt: WcorContextP
        :param scene: Graphics scene to copy selection from/paste to
        :type scene: WguiGraphicsScene
        :param module: Application module
        :type module: WmodApplicationModuleP
        """
        super().__init__(scene, module)

        self.__context = cntxt

    def _copyBlock(self, block):
        if self._containsXbtItems(block):
            self.__context.addMessage(
                'Could not copy items because of XBT elements',
                'cockpit', SCMN_LOG_WARN)
            return
        super()._copyBlock(block)

    def _containsXbtItems(self, block):
        """
        Whether the block contains any XBT items, recursively.

        :param block: Block to test for XBT items
        :type block: WdbsBlockP
        :return: True if any XBT items where found in the block or in any of
                 its child blocks
        :rtype: bool
        """
        for inst in block.getInstances():
            if isinstance(inst, WdbsOperatorInstanceP):
                if inst.getOperator() in WFHDB_XBT_PROXY_OP_CODES:
                    return True
            elif isinstance(inst, WdbsBlockInstanceP):
                if self._containsXbtItems(inst.getBlock()):
                    return True
        return False
