# ----------------------------------------------------------------------------
# - wfgui-highlight-module                                                   -
# - wild cruncher: item highlighters                   state machine cockpit -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfgui_highlight_module
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher state machine cockpit item highlighters

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from PyQt4.QtCore import Qt

from wcr_core.modules.statemachine.db.wfhdb_transition import WfhdbTransition
from wcr_core.modules.statemachine.db.wfhdb_xbt_utils import \
    wfhdb_is_xbt_transition
from wcr_frontend.gui.wgui_highlight_module import WguiHighlightModule, \
    WguiToggleHighlightModule


class WfguiEventItemsHighlightModule(WguiToggleHighlightModule):
    """
    Highlighting module for events.

    If enabled, all elements that are affected by the selected event will be
    highlighted.
    """

    def __init__(self, itemsPrv, scene):
        """
        :param itemsPrv: Event items provider
        :type itemsPrv: WfguiItemSelectionProviderP
        :param scene: Graphics scene
        :type scene: WguiGraphicsScene
        """
        self.__itemsPrv = itemsPrv
        self.__scene = scene
        self.__eventItems = set()
        self.__highlightingEnabled = False

        itemsPrv.selectedItemsChanged.connect(self._itemsChanged)

    def enableHighlighting(self, enable):
        self.__highlightingEnabled = enable
        self._update()
        self.__scene.update()

    def _itemsChanged(self):
        self._update()
        self.__scene.update()

    def _update(self):
        if self.__highlightingEnabled:
            self.__eventItems = set(
                i for i, _ in self.__itemsPrv.getSelectedItems())
        else:
            self.__eventItems = set()

    @property
    def identifier(self):
        return 'event'

    @property
    def description(self):
        return 'Highlight items affected by currently selected event'

    def highlight(self, item):
        if not self.__highlightingEnabled:
            return None

        if item in self.__eventItems:
            return Qt.cyan, 3, None
        else:
            return None, None, None

    def providesHighlight(self, hType):
        return hType == self.HLGHT_LINE_COLOUR or \
            hType == self.HLGHT_LINE_WIDTH


class WfguiTriggerMissingHighlightModule(WguiHighlightModule):
    """
    Highlighting module for transitions with missing triggers.

    If enabled, all transitions that have no trigger condition or event set
    will be highlighted.
    """

    def __init__(self, xbtMngr, scene):
        """
        :param xbtMngr: XBT Manager
        :type xbtMngr: WfhdbXbtManager
        :param scene: Graphics scene
        :type scene: WguiGraphicsScene
        """
        self.__xbtMngr = xbtMngr
        self.__scene = scene

    @property
    def identifier(self):
        return 'missing-trigger'

    @property
    def description(self):
        return 'Highlight transitions without trigger event/condition'

    def highlight(self, item):
        if not isinstance(item, WfhdbTransition):
            return None

        if wfhdb_is_xbt_transition(item):
            item = self.__xbtMngr.getXbtTransition(item)

        if item.getTriggerCondition() or item.getTriggerEvent():
            return None, None, None
        else:
            return Qt.red, None, None

    def providesHighlight(self, hType):
        return hType == self.HLGHT_LINE_COLOUR or \
            hType == self.HLGHT_LINE_WIDTH
