# ----------------------------------------------------------------------------
# - wmgui-project-props-dialog                                               -
# - wild cruncher - cockpit: project properties dialog widgets               -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 philippe lambert                                 -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmgui_project_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher project properties dialog

.. moduleauthor:: Philippe Lambert <Philippe.Lambert@silkan.com>
"""

from wcr_frontend.gui.wgui_project_props_dialog import \
    WguiProjectPropertiesDialog


from wcr_frontend.modules.statemachine.gui.wfgui_datamodel_props_widget \
    import WfguiDatamodelProperties
from wcr_core.modules.statemachine.db.wfhdb_protocols import WFHDB_DATA_ID_KEY


class WfguiProjectPropertiesDialog(WguiProjectPropertiesDialog):
    """
    The WfguiProjectPropertiesDialog class is a dialog design to access
    and manipulate project metadata, project editor preferences and project
    configuration settings.
    """

    def __init__(self, prjContext, config, updateF, parent=None):
        """
        :param prjContext: Project context
        :type prjContext: WcorProjectContextP
        :param config: Project editor configuration
        :type config: WguiProjectEditorCfg
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param parent: Parent widget
        """
        super().__init__(prjContext, config, updateF, parent=parent)

    def _createProjectPropertiesTabs(self, prjContext):
        return [('Datamodel',
                 _WfhdbDataModelProperty(
                     prjContext.getProjectDB().getTopHierarchy()))]


class _WfhdbDataModelProperty(WfguiDatamodelProperties):
    def __init__(self, inst, parent=None):
        """
        :param inst: State instance
        :type inst: WfhdbInstanceP
        :param parent: Parent widget
        """
        #: State instance
        self.__instance = inst

        super().__init__(parent)

    def _getDatas(self):
        od = self.__instance.getDataModel(self.__instance.rawElement)
        l = []
        for k in od.keys():
            Datdic={WFHDB_DATA_ID_KEY: k}
            Datdic.update(od[k])
            l.append(("Data", Datdic))
        return l

    def _setDatas(self, execs):
        add_data = {}
        for dat in execs:
            Datadesc = dat[1]
            datkey = Datadesc.pop(WFHDB_DATA_ID_KEY)
            add_data[datkey] = Datadesc
        old_data = self.__instance.getDataModel(self.__instance.rawElement)
        for oldk in old_data.keys():
            if oldk not in add_data.keys():
                self.__instance.removeFromDataModel(self.__instance.rawElement,
                                                    oldk)
        self.__instance.setDataModel(self.__instance.rawElement, add_data)