# ----------------------------------------------------------------------------
# - wfgui-props-dialog-factory                                               -
# - wild cruncher - cockpit: properties dialog box factory                   -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       flm_gui_pdgfc
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher properties dialog box factory

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.modules.statemachine.db.wfhdb_xbt_utils import \
    wfhdb_is_xbt_state, wfhdb_is_xbt_transition
from wcr_frontend.gui.wgui_props_dialog_factory import \
    WguiPropertiesDialogFactory
from wcr_frontend.modules.statemachine.gui.wfgui_instance_props_dialog import \
    WfguiInstancePropertiesDialog
from wcr_frontend.modules.statemachine.gui.wfgui_transition_props_dialog \
    import WfguiTransitionPropertiesDialog
from wcr_frontend.modules.statemachine.gui.wfgui_project_props_dialog import \
    WfguiProjectPropertiesDialog


class WfguiPropertiesDialogFactory(WguiPropertiesDialogFactory):
    def __init__(self, prjContext, config, scene, actions, cctx, parent=None):
        """
        :param prjContext: Project context
        :type prjContext: WfmodStatemachineProjectContextP
        :param config: Project editor configuration
        :type config: WguiProjectEditorCfgP
        :param scene: Graphic scene adapter
        :type scene: WguiSceneAdapterP
        :param actions: GUI actions
        :type actions: dict[str, WactActionP]
        :param cctx: Cockpit contex
        :type cctx: WfguiCockpitSystemContextP
        :param parent: Parent widget
        """
        super().__init__(prjContext, config, scene, actions, parent)

        self.__projectCtx = prjContext
        self.__context = cctx
        self.__scene = scene

    def _createProjectPropertiesDialog(self, prjContext, config, updateF,
                                       parent):
        return WfguiProjectPropertiesDialog(
            prjContext, config, updateF, parent=parent)

    def _createBlockInstancePropertiesDialog(self, inst, pctx, updateF,
                                             selectXpnt, parent):
        return WfguiInstancePropertiesDialog(
            inst, pctx, updateF, parent=parent)

    def _createModelInstancePropertiesDialog(self, inst, pctx, updateF,
                                             selectXpnt, parent):
        return WfguiInstancePropertiesDialog(
            inst, pctx, updateF, parent=parent)

    def _createOperatorInstancePropertiesDialog(self, inst, pctx, updateF,
                                                selectXpnt, parent):
        if wfhdb_is_xbt_state(inst):
            return self.createXbtPropertiesDialog(inst)
        return WfguiInstancePropertiesDialog(
            inst, pctx, updateF, parent=parent)

    def _createConnectionPropertiesDialog(self, conn, updateF, parent):
        if wfhdb_is_xbt_transition(conn):
            return self.createXbtPropertiesDialog(conn)

        return WfguiTransitionPropertiesDialog(conn, updateF, parent)

    def createXbtPropertiesDialog(self, stateOrTrans, parent=None):
        def save():
            for item in [strn, sstt, estt, etrn]:
                sitm = self.__scene.getSceneItem(item)
                if sitm:
                    sitm.updateItem()
                    self.__scene.sceneItemChangedS.emit(item)

        xmng = self.__projectCtx.getXbtManager()
        _, strn, sstt, estt, etrn, *_ = xmng.getXbtElements(stateOrTrans)

        return WfguiTransitionPropertiesDialog(strn, save, parent)

    def _createInstancePostSaveHook(self, *_):
        def hook(*_):
            self.__context.refreshEvents()

        return hook

    def _createConnectionPostSaveHook(self, *_):
        def hook(*_):
            self.__context.refreshEvents()

        return hook
