# ----------------------------------------------------------------------------
# - wfgui-scene-context-menu-factory                                         -
# - wild cruncher - cockpit: scene-based context menu factory                -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfgui_scene_context_menu_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher scene-based context menu factory

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from functools import partial
from PyQt4 import QtGui
from PyQt4.QtGui import QAction
from wcr_core.modules.statemachine.db.wfhdb_xbt_utils import \
    wfhdb_is_xbt_state, wfhdb_is_xbt_entity
from wcr_frontend.gui.wgui_scene_context_menu_factory import \
    WguiSceneContextMenuFactory
from wcr_frontend.modules.statemachine.gui.wfgui_xbt_utils import \
    wfgui_jump_to_sister_state


class WfguiSceneContextMenuFactory(WguiSceneContextMenuFactory):
    """
    Context menu generate based on the WCR graphics scene.
    """

    def __init__(self, scene, actions, clpbrd, pctx, pdgf):
        """
        :param scene: Graphics scene
        :type scene: WguiSceneAdapterP
        :param actions: Action dictionary
        :type actions: dict[str, WactActionP]
        :param clpbrd: Clipboard handler
        :type clpbrd: WguiClipboardHandler
        :param pctx: Project context
        :type pctx: WfmodStatemachineProjectContextP
        :param pdgf: Properties dialog factory
        :type pdgf: WguiPropertiesDialogFactoryP
        """
        super().__init__(scene, actions, clpbrd, pctx, pdgf)

        self.__scene = scene
        self.__projectCtx = pctx
        self.__clipboard = clpbrd
        self.__propDiagFy = pdgf

    def _canCopyCutPaste(self, entity):
        return not wfhdb_is_xbt_entity(entity)

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createOperatorInstanceContextMenuActions(self, inst, menu, parent):
        """
        :type inst: WhdbOperatorInstanceP
        :type menu: QMenu
        :type parent: QWidget
        :rtype: list[QAction]
        """
        if wfhdb_is_xbt_state(inst):
            # noinspection PyArgumentList
            return [
                QAction(
                    'Go to other end', menu,
                    statusTip='Go to the other end of this cross-block'
                              ' transition',
                    triggered=partial(self._goToCrossBlockSibling, inst))]
        return None

    def _goToCrossBlockSibling(self, inst):
        """
        :type inst: WfhdbXbtTargetProxyInstance | WfhdbXbtSourceProxyInstance
        :rtype: bool
        """
        return wfgui_jump_to_sister_state(
            inst, self.__projectCtx, self.__scene)

    def createViewContextMenu(self, parent, scenePos=None):
        def paste():
            self.__clipboard.paste(scenePos)

        menu = QtGui.QMenu(parent)
        menu.addSeparator()
        # noinspection PyArgumentList
        menu.addAction(QAction('Paste', menu, triggered=paste,
                               enabled=bool(self.__clipboard.canPaste())))

        if len(self.__scene.blockInstancePath) > 0:
            dialogF = self.__propDiagFy.createBlockInstancePropertiesDialog
            # noinspection PyArgumentList,PyUnresolvedReferences
            menu.addAction(
                QAction(
                    '&Properties...', menu,
                    statusTip='Manage instance properties',
                    triggered=self._propertiesDialogAction(
                        dialogF, self.__scene.blockInstancePath[-1],
                        parent=parent)))
        else:
            dialogF = self.__propDiagFy.createProjectPropertiesDialog
            menu.addAction(
                QAction(
                    '&Properties...', menu,
                    statusTip='Manage instance properties',
                    triggered=self._propertiesDialogAction(dialogF,
                                                           parent=parent)))
        return menu