# ----------------------------------------------------------------------------
# - wfgui-scene-item-factory                                                 -
# - wild cruncher: scene item factory                  state machine cockpit -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher TBD_MODULE_DESC

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from wcr_core.modules.statemachine.db.wfhdb_instance import \
    WfhdbXbtSourceProxyInstance, WfhdbXbtTargetProxyInstance
from wcr_core.modules.statemachine.db.wfhdb_xbt_utils import wfhdb_is_xbt_state
from wcr_frontend.gui.wgui_scene_item import WguiXPointCenterPositionProvider
from wcr_frontend.gui.wgui_scene_item_factory import WguiDefaultSceneItemFactoy
from wcr_frontend.modules.statemachine.gui.wfgui_scene_item import *
from wcr_frontend.modules.statemachine.gui.wfgui_utils import \
    wfgui_block_state_tooltip, wfgui_state_tooltip, wfgui_transition_tooltip
from wcr_frontend.modules.statemachine.gui.wfgui_xbt_utils import \
    wfgui_xbt_tooltip_f


class WfguiSceneItemFactory(WguiDefaultSceneItemFactoy):
    """
    Model scene item factory.
    """

    def __init__(self, pctx):
        super().__init__(WguiXPointCenterPositionProvider())

        self.__projectCtx = pctx

    # noinspection PyMethodMayBeStatic
    def createBlockInstanceSceneItem(self, obj, *args, **kwargs):
        item = WfguiInstanceSceneItem(
            obj, self.createConnectionPoint, self.positionProvider,
            *args, **kwargs)
        item.setTooltipF(wfgui_block_state_tooltip)
        return item

    def createOperatorInstanceSceneItem(self, obj, *args, **kwargs):
        if isinstance(obj, WfhdbXbtSourceProxyInstance) or \
                isinstance(obj, WfhdbXbtTargetProxyInstance):
            item = WfguiXbtInstanceSceneItem(
                obj, self.createConnectionPoint, self.positionProvider,
                self.__projectCtx, *args, **kwargs)
            # noinspection PyTypeChecker
            item.setTooltipF(wfgui_xbt_tooltip_f(self.__projectCtx))
        else:
            item = WfguiOperatorInstanceSceneItem(
                obj, self.createConnectionPoint, self.positionProvider,
                *args, **kwargs)
            item.setTooltipF(wfgui_state_tooltip)
        return item

    def createConnectionSceneItem(self, obj, *args, **kwargs):
        """
        :type obj: WhdbConnectionP
        :rtype: WguiConnectionSceneItem
        """
        item = super().createConnectionSceneItem(obj, *args, **kwargs)

        if wfhdb_is_xbt_state(obj.getStartObject()) or \
                wfhdb_is_xbt_state(obj.getEndObject()):
            item.setTooltipF(wfgui_xbt_tooltip_f(self.__projectCtx))
        else:
            item.setTooltipF(wfgui_transition_tooltip)
        return item

    def createConnectionPoint(self, obj, item, *args, **kwargs):
        return WfguiConnectionPointSceneItem(
            obj, item, self.positionProvider, *args, **kwargs)
