# ----------------------------------------------------------------------------
# - wfgui-transition-props-dialog                                            -
# - wild cruncher - cockpit : transition properties dialog widgets           -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfgui_transition_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher transition properties dialog

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from PyQt5 import QtGui
from PyQt5.QtWidgets import QLabel, QLineEdit

from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend.gui.wgui_connection_props_dialog import \
    WguiConnectionPropertiesDialog
from wcr_frontend.modules.statemachine.gui.wfgui_executables_props_widget \
    import WfguiExecutableProperties


class WfguiTransitionPropertiesDialog(WguiConnectionPropertiesDialog):
    """
    Properties dialog for transition objects.
    """

    def __init__(self, trans, updateF, parent=None):
        """
        :param trans: Transition
        :type trans: wfhdb_transition.WfhdbTransition
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param parent: Parent widget
        """
        # base initialization
        super().__init__(trans, updateF, parent)

    def _createConnectionTypePropertiesWidget(self, conn):
        return _WfguiTransitionProperties(conn)

    # noinspection PyMethodMayBeStatic
    def _createAdditionalConnectionPropertiesWidget(self, conn):
        return [('Execution', _WfguiTransitionExecutionProperties(conn))]


class _WfguiTransitionProperties(WguiPropertiesWidgetP):
    """
    Properties editing widget for a transition's properties.
    """

    def __init__(self, trans, parent=None):
        """
        :param trans: Transition
        :type trans: wfhdb_transition.WfhdbTransition
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Transition
        self.__trans = trans

        #: Event trigger field
        self.__eventF = QLineEdit()
        #: Condition trigger field
        self.__condF = QLineEdit()

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        """
        Create the layout for the widget.
        Should only be called once.

        :return: Layout
        :rtype: QLayout
        """
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(QLabel('<b>Event</b>'))
        layout.addWidget(self.__eventF)
        layout.addWidget(QLabel('<b>Condition<b>'))
        layout.addWidget(self.__condF)

        return layout

    def updateFields(self):
        self.__eventF.setText(self.__trans.getTriggerEvent())
        self.__condF.setText(self.__trans.getTriggerCondition())

    def save(self):
        self.__trans.setTriggerCondition(self.__condF.text())
        self.__trans.setTriggerEvent(self.__eventF.text())


class _WfguiTransitionExecutionProperties(WfguiExecutableProperties):
    def __init__(self, trans, parent=None):
        """
        :param trans: Transition
        :type trans: wfhdb_transition.WfhdbTransition
        :param parent: Parent widget
        """
        #: Transition
        self.__trans = trans

        super().__init__(parent)

    def _getExecutables(self):
        return self.__trans.getExecutable()

    def _setExecutables(self, execs):
        self.__trans.setExecutable(execs)
