# ----------------------------------------------------------------------------
# - wfgui-xbt-utils                                                          -
# - wild cruncher: xbt utility functions       frontend / statemachine / gui -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wfgui_xbt_utils
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - XBT utility functions

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from silkan_common.scmn_log import SCMN_LOG_WARN
from wcr_core.modules.statemachine.db.wfhdb_instance import \
    WfhdbXbtTargetProxyInstance
from wcr_core.modules.statemachine.db.wfhdb_xbt_utils import \
    WfhdbXbtException
from wcr_frontend.modules.statemachine.gui.wfgui_utils import \
    wfgui_transition_tooltip


def wfgui_xbt_tooltip_f(pctx):
    """
    Generate the tooltip text for the given state.

    :param pctx: State to generate tooltip text for
    :type pctx: WfmodStatemachineProjectContextP
    :return: Tooltip text for the given state
    :rtype: (object) -> str
    """
    def tooltip(xbtObj):
        xmng = pctx.getXbtManager()

        try:
            sobj, strn, sstt, estt, etrn, eobj, spth, epth = \
                xmng.getXbtElements(xbtObj)
        except WfhdbXbtException:
            return '<b>INVALID XBT TRANSITION</b>'

        if xbtObj is sstt or xbtObj is strn:
            snam = sobj.getName()
            enmp = '.'.join(p.getName() for p in epth)
            enam = (enmp + '.' if enmp else '') + eobj.getName()
        else:
            snmp = '.'.join(p.getName() for p in spth)
            snam = (snmp + '.' if snmp else '') + sobj.getName()
            enam = eobj.getName()

        return wfgui_transition_tooltip(strn, snam, enam)

    return tooltip


def wfgui_jump_to_sister_state(state, pctx, scene):
    """
    Jump to the sister-state of the given XBT-state.

    :param state: XBT-state
    :type state: WfhdbXbtSourceProxyInstance | WfhdbXbtTargetProxyInstance
    :param pctx: Project context
    :type pctx: WfmodStatemachineProjectContextP
    :param scene: Graphics scene
    :type scene: WguiSceneAdapterP
    :return: True iff jump was made
    :rtype: bool
    """
    try:
        sister, path = pctx.getXbtManager().getSisterState(state)
        if sister:
            scene.setHierarchy(path)
            scene.selectEntity(sister)
            return True
    except WfhdbXbtException:
        pass
    pctx.addMessage(
        'Could not jump to other end; sister proxy state may be missing.',
        'cockpit', SCMN_LOG_WARN)
    return False
