# ----------------------------------------------------------------------------
# - wcgui-bind-verification-dialog                                           -
# - wild cruncher - cockpit: bind verification result dialog                 -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_bind_verification_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher binding verification result dialog

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from PyQt4.QtCore import Qt

from wcr_core.action.wact_action_api import WactActionObserverP, \
    WACT_ACTION_EXECUTION_FAILED_EVENT
from wcr_frontend.gui.wgui_db_elements_dialog import WguiElementsDialogBase
from wcr_core.modules.sys_map.actions.wcact_map_checker_action import \
    WchdbBindsException
from wcr_frontend.gui.wgui_project_editor import WguiProjectEditor


class WcguiBindingVerificationDialog(WguiElementsDialogBase):
    """
    Binding verification result dialog.
    """

    def __init__(self, projectDB, scene, items=None, parent=None):
        """
        :param projectDB: Project Database
        :type projectDB: WprjProjectDatabase
        :param scene: Graphic scene
        :type scene: WguiGraphicsScene
        :param items: Initial type propagation result items, as list of:
                      error message, affected elements, path, and add'l data
        :type items: list[(str, list[WdbsEntityP], list[WdbsBlockInstanceP],
                           object)]
        :param parent: Parent widget
        """
        super().__init__(projectDB, scene, closeIfOk=True,
                         items=items, parent=parent)

        #: The current project database
        self.__projectDB = projectDB
        #: The graphics scene
        self.__scene = scene
        #: Database verifier

        self.setWindowTitle('Binding Verification')


class WcguiBindingVerifierFailureHandler(WactActionObserverP):
    """
    Binding Verifier failure handler.
    """

    def handle(self, exception, projectCtx, scene, parent):
        """
        Handle a binding verification failure.

        :param: Exception
        :type exception: WchdbBindsException
        :param projectCtx: Project context
        :type projectCtx: WcorProjectContextP
        :param scene: Graphics scene
        :type scene: WguiGraphicsScene
        """
        items = [(str(exception), [], [], None)]

        """[(str, list[WdbsEntityP], list[WdbsBlockInstanceP],object)]"""

        dialog = WcguiBindingVerificationDialog(
            projectCtx.getProjectDB(), scene, items, parent)
        dialog.show()
        dialog.exec_()

    def actionNotify(self, actx, action, event, *args, **kwargs):
        """
        :type actx: WguiActionExecutionContextP
        """
        if event == WACT_ACTION_EXECUTION_FAILED_EVENT and args:
            excp = args[0]
            if isinstance(excp, WchdbBindsException):
                projectCtx = actx.getProjectContext()
                editor = actx.getEditor()

                if projectCtx and isinstance(editor, WguiProjectEditor):
                    self.handle(excp, projectCtx, editor.scene, editor)
