# ----------------------------------------------------------------------------
# - wcgui-instance-props-dialog                                              -
# - wild cruncher - cockpit : card properties dialog widgets                 -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance properties dialog

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""
import os

from wcr_core.db.whdb_utils import whdb_getNextName
from wcr_core.modules.vlink.db.wvhdb_standard_operators import WVHDB_OPER, \
    WVHDB_VLINK_CODE
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import *
from wcr_core.modules.vlink.wvmod_application_module import WVMOD_VLINK_APP_MODULE
from wcr_core.modules.sys_map.db.wchdb_route_generation_container import \
    RouteGenerationContainer
from wcr_core.modules.sys_map.plugin.wcplg_config_generator_params import HW_2_VL_TYPE
from wcr_core.common.wcmn_file_handler import WcmnFile
from wcr_core.modules.hardware.db.whhdb_connection_points import WHHDB_MODULES_IO_NAMES

from wcr_frontend.gui.wgui_props_dialog_widget import *
from wcr_core.modules.sys_map.plugin.wcplg_config_generator_params import *
from wcr_frontend.modules.sys_map.gui.wcgui_route_generation_helper import \
    WiguiTypeInstanciateDialog, _WCGUI_PORTS_CONS, _WCGUI_CARDS_CONS
from wcr_frontend import isNameValid, showErrorWindow, NameFormatMessageError


class RoutesGenerationException(Exception):
    pass


class WcguiAutoBindModules:
    """
        class to create routes automatically and helps binding them
    """

    def __init__(self, pctx, scene, tableWidget, tableModel,
                 tableProducer, tableConsumer):
        self.__scene = scene
        self.__pctx = pctx
        self.__top = pctx.getProjectDB().getTopHierarchy()
        self.__tableW = tableWidget
        self.__tableM = tableModel
        self.__tableProd = tableProducer
        self.__tableCons = tableConsumer
        self.__moduleBindings = self.__top.getModuleCardBind()
        self.__map = self.__pctx.getMappingManager()
        self.__bindedMsg = self.__map.logicTop.getAllMessageDictBindings()
        self.__msgData = self.__map.logicTop.getAllMessages()
        self.__modulesBound = self.__scene.getModulesBound()
        self.__generatedRoutes = {}
        self.__hblock = self.__map.harware
        vlblok = self.__top.getVLinkBlock()

        if vlblok:
            self.__vlTop = vlblok
        else:
            vlFileName = 'TrAuto.tr.xml'
            dirname = os.path.dirname(self.__top.getHardwareFile().path)
            vlfullpath = os.path.join(dirname, vlFileName)
            self.__vlTop = WVMOD_VLINK_APP_MODULE.createTopLevelBlock()
            self.__vlTop.setSourcePath(vlfullpath)
            self.__vlTop.formatToSave(vlfullpath)
            self.__top.setVLinkFile(WcmnFile(vlfullpath))
            self.__top.setVLinkBlock(self.__vlTop)

        self.__isHelperNeeded = False
        self.__moduleInProcessBinding = None
        self.__routeContainers = []

    def computeMapping(self):
        """
            create routes auto using the module/host binding
            :return:
        """
        # iter over list of binded message defined in icd views
        for message in sorted(self.__bindedMsg):
            # find eqpProducer and moduleProducer of the message
            messageProducer, moduleProducer = self.findProducer(message)
            # find eqpConsumer and moduleConsumer of the message
            messageConsumers, modulesConsumer = self.findConsumers(message)
            if messageProducer and messageConsumers:
                # if message already bound to a route there is no need to create a route
                if message in self.__tableM.getMessagesName():
                    continue
                # the route if the algo can't guess info from system
                # instantiate a routeContainer to help the user generate
                routeContainer = self.createRouteContainer(message,
                                                           messageProducer,
                                                           moduleProducer,
                                                           messageConsumers,
                                                           modulesConsumer)
                # try to find the ports producer/consumer
                routProtocols = self.getPortFromCard(routeContainer, messageProducer, messageConsumers)
                # if hosts have only 1 port per card it should be easy and
                # protocol should be returned
                # print((message, routProtocols, messageConsumers))
                if None not in routProtocols:
                    if len(set(routProtocols)) > 1:
                        raise RoutesGenerationException('Multiple protocls detected {}'.format(set(routProtocols)))
                    self.createRoute(routProtocols[0], message, routeContainer=routeContainer)
                # else we store routeContainer to be used for guessing route info
                else:
                    # print((message, messageProducer, messageConsumers))
                    self.__routeContainers.append(routeContainer)
                    self.__isHelperNeeded = True
                    # print('---------------------------------------------------')
        # if helperFlag is up we open a window to ask user to fill missing data
        if self.__isHelperNeeded:
            instanciate = WiguiTypeInstanciateDialog(pctx=self.__pctx,
                                                     scene=self.__scene,
                                                     routes=self.__routeContainers,
                                                     bindingModuleClz=self)
            instanciate.show()
            instanciate.exec_()
        else:
            showErrorWindow("Route Creation",
                            'All messages are bound to a route.', None)

    def createRouteContainer(self, message, messageProducer, moduleProducer,
                             messageConsumers, modulesConsumer):
        """
            instantiate RouteGenerationContainer with data
            :param message: str
            :param messageProducer: str
            :param moduleProducer: str
            :param messageConsumers: [str]
            :param modulesConsumer: [str]
            :return: RouteGenerationContainer
        """
        routeContainer = RouteGenerationContainer()
        routeContainer.setMessage(message)
        routeContainer.setEquipmentProducer(messageProducer)
        routeContainer.setModuleProducer(moduleProducer)
        routeContainer.setEquipmentsConsumer(messageConsumers)
        routeContainer.setModulesConsumer(modulesConsumer)
        routeContainer.setNbConsumer(len(set(modulesConsumer)))
        return routeContainer

    def findProducer(self, message):
        """
            find eqpProducer and moduleProducer from the messageName
            :param message: str
            :return: (str, str)
        """
        producer = self.__bindedMsg[message]['P']
        # print((message, producer))
        producerModule = producer.split('.')[0]
        producerEqp = self.__modulesBound.get(producerModule, None)
        if producerEqp:
            # print(self.__moduleBindings[producerEqp])
            # print('------------------------------------------')
            bindings = self.__moduleBindings.get(producerEqp, None)
            # module already bound to a card (e.g host has only 1 card)
            if bindings:
                for modName, card, modPeriod in bindings:
                    if modName == producerModule:
                        # print((message, modName, card, modPeriod))
                        return ['.'.join([producerEqp, card])], modName
            # else return all the cards
            else:
                producerEqps = []
                for scenItem in self.__scene.getSceneItems():
                    item = scenItem.getDataObject()
                    if item.getName() == producerEqp:
                        for card in item.getCards():
                            producerEqps.append('.'.join([producerEqp, card]))
                return producerEqps, producerModule
        return [None], None

    def findConsumers(self, message):
        """
            find eqpProducer and moduleProducer from the messageName
            :param message: str
            :return: ([str], [str])
        """
        consumers = self.__bindedMsg[message]['C']
        messageConsumers = []
        modules = []
        for consumer in consumers:
            consumerModule = consumer.split('.')[0]
            consumerEqp = self.__modulesBound.get(consumerModule, None)
            if consumerEqp:
                # print(self.__moduleBindings[producerEqp])
                # print('------------------------------------------')
                bindings = self.__moduleBindings.get(consumerEqp, None)
                # module already bound to a card (e.g host has only 1 card)
                if bindings:
                    for modName, card, modPeriod in bindings:
                        if modName == consumerModule:
                            # print((message, modName, card, modPeriod))
                            messageConsumers.append('.'.join([consumerEqp, card]))
                            modules.append(modName)
                else:
                    for scenItem in self.__scene.getSceneItems():
                        item = scenItem.getDataObject()
                        if item.getName() == consumerEqp:
                            for card in item.getCards():
                                messageConsumers.append('.'.join([consumerEqp, card]))
                                modules.append(consumerModule)
        return messageConsumers, modules

    def getPortFromCard(self, routeContainer, eqpCardProducer, eqpCardConsumers):
        """
            Try to guess the protocol to create a new route by using the
            eqpCardProducer/consumers by identifying the ports of the cards,
            their number and protocol
            :param routeContainer: RouteGenerationContainer
            :param eqpCardProducer: str
            :param eqpCardConsumers: [str]
            :return: protocol: str or None
        """
        for item in self.__hblock.getInstances():
            for eqpProd in eqpCardProducer:
                if not eqpProd:
                    continue
                eqpName, cardName = eqpProd.split('.')
                if item.getName() == eqpName:
                    eqpPorts = tuple(item.getConnectionPoints())
                    if len(eqpPorts) == 0:
                        raise RoutesGenerationException('The producer equipment "{}" does not '
                                                        'contain any port.'.format(eqpName))
                    if len(eqpPorts) == 1:
                        portProducer = eqpPorts[0]
                        if self.__hblock.checkportIsConnected(portProducer.getUUID()):
                            return self.getPortConsumer(routeContainer, [portProducer], eqpCardConsumers)
                    elif len(eqpPorts) > 1:
                        ports = self.getConnectedCardPorts(item, cardName)
                        if len(ports) == 0:
                            raise RoutesGenerationException('The producer card "{}" does not '
                                                            'contain any port.'.format(eqpCardProducer))
                        elif len(ports) == 1:
                            portProducer = ports[0]
                            return self.getPortConsumer(routeContainer, [portProducer], eqpCardConsumers)
                        elif len(ports) > 1:
                            #-------- try to see if the eqp consumer has only
                            # one port or a port of only one protocol
                            return self.getPortConsumer(routeContainer, ports, eqpCardConsumers)
        return [None]

    def getPortConsumer(self, routeContainer, producerPorts, eqpCardConsumers):
        """
            Try to guess the protocol to create a new route by using the
            eqpCardProducer/consumers by finding the consumer ports
            related to the producer eqp.card.port
            :param routeContainer: RouteGenerationContainer
            :param producerPorts: str
            :param eqpCardConsumers: [str]
            :return: protocol: str or None
        """
        protocols = []

        for eqpCardConsumer in eqpCardConsumers:
            for item in self.__hblock.getInstances():
                eqpName, cardName = eqpCardConsumer.split('.')

                if item.getName() == eqpName:
                    eqpPorts = tuple(item.getConnectionPoints())
                    if len(eqpPorts) == 0:
                        raise RoutesGenerationException('The consumer equipment "{}" does not '
                                                        'contain any port.'.format(eqpName))
                    if len(eqpPorts) == 1:
                        protocols.append(self.guessProducterConsumerProtocol(routeContainer, producerPorts, eqpPorts))
                    elif len(eqpPorts) > 1:
                        ports = self.getConnectedCardPorts(item, cardName)
                        if len(ports) == 0:
                            raise RoutesGenerationException('The consumer card "{}" does not '
                                                            'contain any port.'.format(eqpCardConsumer))
                        elif len(ports) >= 1:
                            protocols.append(self.guessProducterConsumerProtocol(routeContainer, producerPorts, ports))
        return protocols

    def guessProducterConsumerProtocol(self, routeContainer, producerPorts, consumerPorts):
        protocols = []
        portProducerConsumer = None
        for pport in producerPorts:
            for cport in consumerPorts:
                if pport.getCode() == cport.getCode():
                    protocols.append(pport.getCode())
                    portProducerConsumer = (pport, cport)
        if len(set(protocols)) == 0:
            pass
            # print('Can\'t guess protocol, no protocol found matching producer '
            #       'do you want to use a CetraC route ?')
        elif len(set(protocols)) > 1:
            for pport in producerPorts:
                routeContainer.addPortProducer(pport)
            for cport in consumerPorts:
                routeContainer.addPortConsumer(cport)
            self.__isHelperNeeded = True
        else:
            pport, cport = portProducerConsumer
            routeContainer.addPortProducer(pport)
            routeContainer.addPortConsumer(cport)
            return protocols[0]

        return None

    def getConnectedCardPorts(self, item, cardName):
        ports = []
        for port in item.getConnectionPoints():
            portCardName = port.getCard()
            if port.getCode() in WHHDB_MODULES_IO_NAMES:
                ports.append(port)
            elif (portCardName == cardName and
                    self.__hblock.checkportIsConnected(port.getUUID())):
                ports.append(port)
        return ports

    def createRoute(self, protocol, messageName, producers=None, consumers=None, routeContainer=None):
        if messageName not in self.__tableM.getMessagesName():
            routeName = '{}_RouteAuto'.format(protocol)
            generatedRoutes = [inst.getName() for inst in self.__vlTop.getInstances()]
            routeName = whdb_getNextName(routeName,
                                         generatedRoutes)
            oper = WVHDB_OPER[WVHDB_VLINK_CODE]
            newVl = self.__vlTop.createOperatorInstance(oper, name=routeName)
            proto = HW_2_VL_TYPE[protocol]
            newVl.setVLType(proto)
            msgType, msgSize = self.__msgData[messageName]
            prod = self.__bindedMsg[messageName]['P']
            modProd, msgProd = prod.split('.')
            cons = self.__bindedMsg[messageName]['C']
            modCons = []
            msgCons = []
            for c in cons:
                modC, msgC = c.split('.')
                modCons.append(modC)
                msgCons.append(msgC)
            data = self.__map.logicTop.getMessageByModule(modProd, msgProd)
            if data:
                _, _, storage, period = data
            else:
                storage = ''
                period = ''
            message = {'name': messageName,
                       'type': msgType,
                       'size': msgSize,
                       'moduleProducer': modProd,
                       'moduleConsumer': '.'.join(modCons),
                       'storage': storage,
                       'period': period}
            newVl.addVLMessage(message)
            self.setDefaultVlValue(newVl, protocol)
            self.__tableM.addRow(newVl, newVl.getVLMessagesName())
            self.fillProducerConsumerTables(messageName, consumers, producers,
                                            routeName, newVl, routeContainer)

    def setDefaultVlValue(self, vl, protocol):
        vl.setVLParameter(protocol, LOG_TR_STORAGE_TAG,   'Queue')
        vl.setVLParameter(protocol, LOG_TR_BUFFER_TAG,    '0xF')
        vl.setVLParameter(protocol, LOG_TR_FRAMESIZE_TAG, '0x5EE')
        if protocol == WVHDB_VLINK_TYPE_ETNERNET:
            vl.setVLParameter(protocol, LOG_TR_ETH_IP_SRC_TAG,   '0.0.0.0')
            vl.setVLParameter(protocol, LOG_TR_ETH_IP_DEST_TAG,  '255.255.255.255')
            vl.setVLParameter(protocol, LOG_TR_ETH_MAC_SRC_TAG,  '00:00:00:00:00:00')
            vl.setVLParameter(protocol, LOG_TR_ETH_MAC_DEST_TAG, 'FF:FF:FF:FF:FF:FF')
            vl.setVLParameter(protocol, LOG_TR_ETH_PROTOCOL_TAG, 'UDP')
            vl.setVLParameter(protocol, LOG_TR_ETH_ARP_TAG,      'No')
            vl.setVLParameter(protocol, LOG_TR_ETH_DETER_TAG,    'No')
        elif protocol == WVHDB_VLINK_TYPE_AFDX:
            vlNumber = self.calculateNextNumber(protocol, LOG_TR_AFDX_NUMBER_TAG)
            vl.setVLParameter(protocol, LOG_TR_AFDX_NUMBER_TAG, hex(vlNumber))
            vl.setVLParameter(protocol, LOG_TR_AFDX_BAG_TAG,        '0x1')
            vl.setVLParameter(protocol, LOG_TR_AFDX_JITTER_TAG,     '0x1')
            vl.setVLParameter(protocol, LOG_TR_AFDX_LMAX_TAG,       '0x5EE')
            vl.setVLParameter(protocol, LOG_TR_AFDX_LMIN_TAG,       '0x45')
            vl.setVLParameter(protocol, LOG_TR_AFDX_PRIO_TAG,       'Low')
            vl.setVLParameter(protocol, LOG_TR_AFDX_SN_CHECK_TAG,   'No')
            vl.setVLParameter(protocol, LOG_TR_AFDX_REDNT_MGNT_TAG, 'No')
            vl.setVLParameter(protocol, LOG_TR_AFDX_REDNT_VL_TAG,   'No')
        elif protocol == WVHDB_VLINK_TYPE_ARINC825:
            idx = self.calculateNextNumber(protocol, LOG_TR_ARINC825_ID_TAG)
            vl.setVLParameter(protocol, LOG_TR_ARINC825_ID_TAG, idx)

    def calculateNextNumber(self, protocol, param):
        numberMax = 0
        for vl in self.__vlTop.getInstances():
            currentNumber = vl.getVLParameter(protocol, param)
            if currentNumber:
                if int(currentNumber, 16) > numberMax:
                    numberMax = int(currentNumber, 16)
        return numberMax + 1

    def fillProducerConsumerTables(self, messageName, consumers, producers,
                                   routeName, newVl, routeContainer=None):
            if consumers and producers:
                eqpCard, port = producers[messageName]
                eqp, card = eqpCard.split('.')
                if port.getCode() in WHHDB_MODULES_IO_NAMES:
                    card = port.getIoCard()
                portName = '{}.{}.{}'.format(eqp, card, port.getName())
                self.__tableProd._addEntry((portName, port), (routeName, newVl))
                self.__tableProd.bindPortProd((portName, port), (routeName, newVl))
                for data in consumers[messageName]:
                    consPort = data[_WCGUI_PORTS_CONS]
                    consEqpCard = data[_WCGUI_CARDS_CONS]
                    consEqp, card = consEqpCard.split('.')
                    if consPort.getCode() in WHHDB_MODULES_IO_NAMES:
                        card = consPort.getIoCard()
                    portName = '{}.{}.{}'.format(consEqp, card, consPort.getName())
                    self.__tableCons._addEntry((portName, consPort), (routeName, newVl))
                    self.__tableCons.bindPorts((portName, consPort), (routeName, newVl))
            elif routeContainer:
                # if route container is fed it means route was easily found
                # so there should be only one elem per eqp/port
                if (len(routeContainer.getEquipmentProducer()) > 1 or
                        len(routeContainer.getPortProducer()) > 1 or
                        len(routeContainer.getEquipmentsConsumer()) > 1 or
                        len(routeContainer.getPortConsumer()) > 1):
                    print(len(routeContainer.getEquipmentProducer()))
                    print(len(routeContainer.getPortProducer()))
                    print(len(routeContainer.getEquipmentsConsumer()))
                    print(len(routeContainer.getPortConsumer()))
                    raise RoutesGenerationException("Can't use Routecontainer if more than on elem for port/eqp")
                eqpProd = [eqp for eqp in routeContainer.getEquipmentProducer()][0]
                portProd = [pport for pport in routeContainer.getPortProducer()][0]
                eqpCons = [eqp for eqp in routeContainer.getEquipmentsConsumer()][0]
                portCons = [pport for pport in routeContainer.getPortConsumer()][0]
                portName = '{}.{}'.format(eqpProd, portProd.getName())
                self.__tableProd._addEntry((portName, portProd), (routeName, newVl))
                self.__tableProd.bindPortProd((portName, portProd), (routeName, newVl))
                portName = '{}.{}'.format(eqpCons, portCons.getName())
                self.__tableCons._addEntry((portName, portCons), (routeName, newVl))
                self.__tableCons.bindPorts((portName, portCons), (routeName, newVl))
