# ----------------------------------------------------------------------------
# - wcgui-instance-props-dialog                                      -
# - wild cruncher - cockpit : card properties dialog widgets               -
# ----------------------------------------------------------------------------
# - author    (c) 2018-2018 achraf mahti                                     -
# - copyright (c) 2018-2018 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance properties dialog

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""
import re
from PyQt4.QtCore import Qt, QAbstractTableModel
from PyQt4.QtGui import QHBoxLayout, QComboBox, QLabel, QVBoxLayout, QLineEdit,\
                        QWidget, QPushButton, QItemDelegate, QTableView, QHeaderView, \
                        QColor, QBrush
from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog, _WguiParametersProperties
from wcr_frontend.gui.wgui_props_dialog_widget import *
from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data
from wcr_core.modules.sys_map.db.wchdb_Switch_lure_instance import WchdbSwitchLureInstance
from wcr_core.modules.sys_map.db.wchdb_Host_lure_instance import WchdbHostLureInstance
from wcr_frontend import isNameValid, showErrorWindow, NameFormatMessageError

WCGUI_APP_PERIOD = ('0', '1', '2', '4', '8', '16', '32', '64', '128', '256')

# noinspection PyAbstractClass
class _WcguiModuleInstanciate(QWidget):
    GLB_COL = 0
    LOC_COL = 1

    def __init__(self, instance, pctx, scene, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.__scene = scene
        self.instance = instance
        self.eqpName = instance.getName()
        self.__module = pctx.getModule()
        """:type: WpmodSystemApplicationModuleP"""
        self.__xpntTypeNames = ()
        self.__bindings = []
        self.__iocards = self.instance.getIoCards()
        if isinstance(self.instance, WchdbSwitchLureInstance):
            self.__cards = []
        else:
            self.__cards = self.instance.getCards()
        self._allCards = self.__cards + self.__iocards
        self.__topBlock = pctx.getProjectDB().getTopHierarchy()
        self.__mapper = pctx.getMappingManager()
        # init table necessary for
        self.__modulesTblW = QTableView(self)
        self._initModels()
        self._initElements()
        self.createLayout()
        self._loadFromData()

    @property
    def topBlock(self):
        return self.__topBlock

    @property
    def modulesTblM(self):
        return self.__modulesTblM

    @property
    def modulesTblW(self):
        return self.__modulesTblW

    @property
    def modulesDlgt(self):
        return self.__modulesDlgt

    def _initModels(self):
        self.__modulesTblM = _WcguiModuleTableInstanceModel(self.formatModules,
                                                            parent=self)
        self.__modulesDlgt = _WcguiModuleBindDelegate(mapper=self.__mapper,
                                                      cards=self._allCards,
                                                      parent=self)

    def _initElements(self):
        #: Add entry button
        self.__addButton = QPushButton('Add')
        #: Delete entry button
        self.__deleteButton = QPushButton('Delete')
        self.__modulesTblW.setModel(self.modulesTblM)
        self.__modulesTblW.horizontalHeader().setResizeMode(0, QHeaderView.ResizeToContents)
        self.__modulesTblW.horizontalHeader().setResizeMode(1, QHeaderView.Stretch)
        self.__modulesTblW.horizontalHeader().setResizeMode(2, QHeaderView.ResizeToContents)
        self.__modulesTblW.setMaximumWidth(500)
        self.__modulesTblW.setItemDelegate(self.__modulesDlgt)

        self.__deleteButton.clicked.connect(self._deleteSelEntries)

    def resettab(self):
        """
        reset self.__modulesTblM
        """
        self.modulesTblM.deleteRows(range(0, self.modulesTblM.rowCount()))
        self._loadFromData()

    def _loadFromData(self):
        if isinstance(self.instance, WchdbHostLureInstance):
            self.modulesTblM.populateTable(self.__topBlock.getEqpModuleCardBind(self.eqpName),
                                           self.instance.getModules(),
                                           self.instance.getCards())

    def getModules(self):
        """
        return the description of all card describes in the Table
        """
        return self.modulesTblM.getFullBindings()

    def formatModules(self, data, column):
        """
        :param data: data to format
        :type data: Cardname | Cardtype | [PortUUID]
        """
        if data is None:
            return ''
        else:
            return data

    def createLayout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.__modulesTblW)

        btnLayout = QHBoxLayout()
        btnLayout.addWidget(self.__deleteButton)
        btnLayout.addStretch()
        layout.addLayout(btnLayout)

        self.setLayout(layout)

        return layout

    def _addEntry(self):
        """
        Add entry in the model
        """
        self.modulesTblM.addRow()

    def _deleteSelEntries(self):
        """
        Delete selected entries from the model
        """
        selIdx = self.modulesTblW.selectedIndexes()
        """:type: list[QModelIndex]"""
        self.modulesTblM.deleteRows([idx.row() for idx in selIdx])
        rl = []
        for idx in selIdx:
            rl.append(idx.row())
        rowlist = sorted(set(rl))
        if self.modulesTblM.rowCount() > 0 and rowlist:
            if rowlist[0] >= self.modulesTblM.rowCount():
                self.modulesTblW.setCurrentIndex(self.modulesTblM.
                                                index(self.modulesTblM.rowCount() - 1, 0))
            else:
                self.modulesTblW.setCurrentIndex(self.modulesTblM.
                                                index(rowlist[0], 0))

    def removeModule(self, module_):
        """
        remove module binding associated to the card
        """
        self.instance.removeModule(module_)
        self.__scene.removeModuleBound(module_)
        self.__scene.ModuleUnbound.emit(module_)

    def selectedMsg(self, selectignor=False, role=Qt.UserRole):
        lrow = []
        for index in self.getSelectedIndexes():
            lrow.append(index.row())
        ulrow = set(lrow)
        if len(ulrow) == 1 and not selectignor:
            return self.modulesTblM.data(self.modulesTblM.index(lrow[0], 0),
                                      role=role)
        else:
            return None

    def getSelectedIndexes(self):
        return self.modulesTblW.selectionModel().selectedIndexes()

    def save(self):
        self.__topBlock.resetEqpModuleCardBind(self.eqpName)
        modules = [mod for mod, _, _ in self.getModules()]
        # cleanup in reverse or else some elements will be skipped
        for modName in self.instance.getModules()[::-1]:
            if modName not in modules:
                self.removeModule(modName)
        for module, card, period in self.getModules():
            self.__topBlock.addModuleCardBind(self.eqpName, module, card, period)


class _WcguiModuleTableInstanceModel(QAbstractTableModel):
    """
    icd Binding table model.
    """

    TITLE_MODULES = 'Modules'
    TITLE_CARDS = 'Cards'
    TITLE_PERIOD = 'ModulePeriod (ms)'
    HEADERS = [TITLE_MODULES, TITLE_CARDS, TITLE_PERIOD]

    def __init__(self, formatF, parent=None):
        """
        :param XpntTypeNames: The Names of Xpoint of the mule
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.__parent = parent
        self.__formatF = formatF

        #: Entries as list of tuples of
        #  port, message and type
        self.__entries = []
        self._update()

    @property
    def entries(self):
        return self.__entries

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
                                    0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if 0 <= column <= self.columnCount():
            info = self.__entries[row][column]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return self.__formatF(info, column)
            if role == Qt.BackgroundRole:
                self.isEditable(index, row)
                return QBrush(self.color_back)
        return None

    def setData(self, index, data, role=None):
        assert role == Qt.UserRole
        self._setMessageEntry(index, data)

    def _setMessageEntry(self, index, data):
        module, card, period = self.__entries[index.row()]
        if index.column() == 0:
            if not self.checkUnicityMsg(data, index.row()):
                return
            actualentry = (data, card, period)
        elif index.column() == 1:
            actualentry = (module, data, period)
        elif index.column() == 2:
            actualentry = (module, card, data)
        self.__entries[index.row()] = actualentry

    def checkUnicityMsg(self, msg, row):
        for i, (entry) in enumerate(self.__entries):
            if i != row:
                # check uniqueness on msg name only
                if msg == entry[0]:
                    return False
        return True

    def getFullBindings(self):
        return self.__entries

    def populateTable(self, moduleBindings, unboundModules, cards):
        self.deleteRows(range(self.rowCount()))
        boundModules = []
        for moduleName, cardName, period in moduleBindings:
            boundModules.append(moduleName)
            self.addRow(moduleName, cardName, period)
        for modName in unboundModules:
            if modName not in boundModules and modName not in self.getModules():
                if len(cards) == 1:
                    card = cards[0]
                else:
                    card = ''
                self.addRow(modName, card, '1')

    def getModules(self):
        return [modName for modName, _, _ in self.__entries]

    def addRow(self, moduleName='', card='', period=''):
        """
        Add a row at the end of the list (append)

        :param module: The module name
        :type module: str | None
        :param card: The card name
        :type card: str | None
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())
        # help uniqueness by auto increment name
        self.__entries.append((moduleName, card, period))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)
        self.__entries.pop(rowIndex)
        self.endRemoveRows()
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def isEditable(self, index, row):
        msg = index.sibling(row, 0).data()
        type_ = index.sibling(row, 1).data()
        size = index.sibling(row, 2).data()
        if not msg or not type_ or not size:
            self.color_back = QColor(255, 160, 160)
            return True
        else:
            self.color_back = Qt.white
            return False

    def flags(self, index):
        flgs = super().flags(index)
        row = index.row()
        self.isEditable(index, row)
        flgs |= ~Qt.ItemIsEditable
        return flgs


class _WcguiModuleBindDelegate(QItemDelegate):
    """
    Delegate for message items
    """

    def __init__(self, mapper, cards, parent=None):
        """
        :type CardTyp: () -> list[CardTyp]
        :type formatF: (elem,column) -> str
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)
        self.__mapper = mapper
        self.__cards = cards
        self.__top = parent.topBlock

    def createEditor(self, parent, option, index):
        """
        Create a editors for message table the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """

        def save(*_):
            self.setModelData(editor, index.model(), index)

        allModules = [mod.getName() for mod in self.__mapper.getAllModuleDesc()]
        availModules = ['']
        availModules.extend(allModules)
        # remove the module from avail module if it's already assigned somewhere
        allBindindedModules = self.__top.getModuleCardBind()
        for eqpName in allBindindedModules:
            for moduleName, card, _ in allBindindedModules[eqpName]:
                if moduleName in availModules:
                    availModules.remove(moduleName)
        col = index.column()
        # id col = 0 is str for cardname
        if col == 0:
            editor = QComboBox(parent)
            for mod in availModules:
                editor.addItem(mod, mod)
        # message storage method
        elif col == 1:
            editor = QComboBox(parent)
            for card in self.__cards:
                editor.addItem(card, card)
        # message storage method
        elif col == 2:
            editor = QComboBox(parent)

            for card in WCGUI_APP_PERIOD:
                editor.addItem(card, card)

        return editor

    def setEditorData(self, editor, index):
        """
        :type editor: QComboBox | QLineEdit
        :type index: QModelIndex
        """
        data = index.model().data(index, Qt.EditRole)
        if isinstance(editor, QComboBox):
            for index in range(editor.count()):
                if editor.itemData(index, Qt.UserRole) == data:
                    editor.setCurrentIndex(index)
                    break
            else:
                editor.setCurrentIndex(0)
        elif isinstance(editor, QLineEdit):
            editor.setText(data)

    def setModelData(self, editor, model, index):
        """
        :type editor: QComboBox | QLineEdit
        :type model: QAbstractItemModel
        :type index: QModelIndex
        """
        if isinstance(editor, QComboBox):
            model.setData(index, wgui_qcb_get_data(editor), Qt.UserRole)


class WguiCardInstanceBaseProperties(WguiPropertiesWidgetP):
    """
    Configuration widget for a model-/function-instance's base data.
    """

    def __init__(self, typeName, inst, tophm, parent=None):
        """
        :param typeName: Type name of the displayed element
        :type typeName: str
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param tophm: Hierarchy top level
        :type tophm: WhdbBlock
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Type name
        self.__typeName = typeName
        #: instance object
        self.__instance = inst
        #: Hierarchy top level
        self.__tophm = tophm

        #: Name field
        self.__nameF = QLineEdit()
        self.__nameF.setFixedWidth(400)
        self.__nameF.setEnabled(False)
        #: Info field
        self.__infoF = QLineEdit()
        self.__infoF.setFixedWidth(400)
        self.__infoF.setEnabled(False)

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        typeName = self.__typeName
        container = QWidget()
        layout = QVBoxLayout()
        layout.addWidget(QLabel(typeName + ' Name:'))
        layout.addWidget(self.__nameF)
        layout.addWidget(container)
        layout.addWidget(QLabel(typeName + ' Information:'))
        layout.addWidget(self.__infoF)
        layout.addStretch(1)
        return layout

    def updateFields(self):
        inst = self.__instance
        self.__nameF.setText(inst.getName())
        self.__infoF.setText(inst.getInfo())

    def save(self):
        pass


class WcguiCardPropertiesDialog(WguiOperatorInstancePropertiesDialog):
    _TYPE_NAME = 'Properties'

    _SHOW_OPERATOR_WIDGET = False

    GENERAL_TAB = WguiCardInstanceBaseProperties

    def __init__(self, inst, pctx, updateF, scene, selectXpnt=None, parent=None,
                 ConF=None):
        """
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: () -> _
        :param parent: Parent widget
        """
        # show the card type
        code = inst.getOperator().getCode().split('.')[-1].strip('_lure')
        self._TYPE_NAME = 'Eqp {}'.format(code)
        self.__connectionF = ConF
        self.opened = False
        self.scene = scene
        super().__init__(inst, pctx, updateF, selectXpnt, parent)

    def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
        return []

    # noinspection PyMethodMayBeStatic
    def _createAdditionalPropertiesWidgets(self, inst, pctx):
        return [("Module binding", _WcguiModuleInstanciate(inst, pctx, self.scene,
                                                self))]

    def _createConnectionPointsPropertiesWidget(self, inst, pctx):
        return None

    def exec_(self):
        self.opened = True
        super().exec()

    def closeEvent(self, event):
        self.opened = False

    def getOpened(self):
        return self.opened

    def _setMainLayout(self, widgets, widget):
        """
        Set the dialog's property widgets.

        :param widgets: Property widgets, and their titles
        :type widgets: Iterable[(str, WguiPropertiesWidgetP)]
        :param widget: Wrapper widget containing the property widgets
        :type widget: QWidget
        """
        self.__widgets = [widget for _, widget in widgets]

        layout = QVBoxLayout()
        layout.addWidget(widget)
        buttons = QDialogButtonBox(QDialogButtonBox.Ok |
                                   QDialogButtonBox.Cancel)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        self.okStateChanged.connect(
            buttons.button(QDialogButtonBox.Ok).setEnabled)
        layout.addWidget(buttons)
        # create the main layout
        self.setLayout(layout)

    def _apply(self):
        """
        Apply the current changes, and leave the dialog window open.
        """
        self._save()
        self._updateFields()

    def _reset(self):
        """
        Reset the dialog window.
        """
        self._updateFields()

    def _save(self):
        """
        Save all changes.
        """
        hasErr = False
        for widget in self.__widgets:
            widgErr = widget.save()
            if widgErr:
                hasErr = True

        return hasErr

    def _updateFields(self):
        """
        Reset all fields to their original state.
        """
        for widget in self.__widgets:
            widget.updateFields()