# ----------------------------------------------------------------------------
# - wcgui-props-dialog-factory                                               -
# - wild cruncher - cockpit: properties dialog box factory                   -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_props_dialog_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher properties dialog box factory

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from wcr_core.modules.sys_map.db.wchdb_standard_operators import WCHDB_HOST_LURE_CODE, \
    WCHDB_RACK_LURE_CODE, WCHDB_SWITCH_LURE_CODE
from wcr_frontend.gui.wgui_props_dialog_factory import \
    WguiPropertiesDialogFactory
from wcr_frontend.modules.sys_map.gui.wcgui_instance_props_dialog import WcguiCardPropertiesDialog

WCHDB_STD_OPERINST_PROPDLG = {WCHDB_HOST_LURE_CODE:
                                  WcguiCardPropertiesDialog,
                              WCHDB_RACK_LURE_CODE:
                                  WcguiCardPropertiesDialog,
                              WCHDB_SWITCH_LURE_CODE:
                                  WcguiCardPropertiesDialog,
                              }

class WcguiPropertiesDialogFactory(WguiPropertiesDialogFactory):
    def __init__(self, prjContext, config, scene, actions, parent=None):
        """
        :param prjContext: Project context
        :type prjContext: WcorProjectContextP
        :param config: Project editor configuration
        :type config: WguiProjectEditorCfgP
        :param scene: Graphic scene adapter
        :type scene: WguiSceneAdapterP
        :param actions: GUI actions
        :type actions: dict[str, WactActionP]
        :param parent: Parent widget
        """
        super().__init__(prjContext, config, scene, actions, parent)
        self.scene = scene

    def _createOperatorInstancePropertiesDialog(self, inst, pctx, updateF,
                                                selectXpnt, parent):
        code = inst.getOperator().getCode()
        propDlg = WCHDB_STD_OPERINST_PROPDLG.get(code)
        if propDlg is not None:
            return propDlg(inst, pctx, updateF, self.scene, selectXpnt, parent)
        else:
            return super()._createOperatorInstancePropertiesDialog(
                inst, pctx, updateF, selectXpnt, parent)