# ----------------------------------------------------------------------------
# - wcgui-route-generation-helper                                            -
# - wild cruncher - cockpit : route generation helper                        -
# ----------------------------------------------------------------------------
# - author    (c) 2019-2019 achraf mahti                                     -
# - copyright (c) 2019-2019 silkan                                           -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_route_generation_helper
   :platform:     Unix, Windows

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""
from collections import OrderedDict
from functools import partial
from operator import methodcaller

from PyQt4.QtCore import Qt, QAbstractTableModel
from PyQt4.QtGui import QHBoxLayout, QComboBox, QVBoxLayout, QLineEdit, \
                        QBrush, QDialogButtonBox, QWidget, QLabel, \
                        QItemDelegate, QTableView, QHeaderView, QColor
from wcr_frontend.gui.wgui_props_dialog_widget import *
from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data
from wcr_frontend import isNameValid, showErrorWindow, NameFormatMessageError
from wcr_core.db.whdb_utils import whdb_getNextName


WCGUI_APP_PERIOD = ('0', '1', '2', '4', '8', '16', '32', '64', '128', '256')
_WCGUI_MODULES_CONS = 'Mod Consumer'
_WCGUI_CARDS_CONS = 'Eqp Consumer'
_WCGUI_PORTS_CONS = 'Ports Consumer'


# noinspection PyAbstractClass
class _WcguiRouteAssignationHelper(QWidget):

    def __init__(self, pctx, scene, routes, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.parent = parent
        self.__scene = scene
        self.__routes = routes
        self.__module = pctx.getModule()
        """:type: WpmodSystemApplicationModuleP"""
        self.__xpntTypeNames = ()
        self.__bindings = []
        self.__topBlock = pctx.getProjectDB().getTopHierarchy()
        self.__mapper = pctx.getMappingManager()
        # init table necessary for
        self.__modulesTblW = QTableView(self)
        self._initModels()
        self._initElements()
        # self.createLayout()
        self._loadFromData()

    @property
    def topBlock(self):
        return self.__topBlock

    @property
    def modulesTblM(self):
        return self.__modulesTblM

    @property
    def modulesTblW(self):
        return self.__modulesTblW

    @property
    def modulesDlgt(self):
        return self.__modulesDlgt

    def _initModels(self):
        self.__modulesTblM = _WcguiRouteAssignationTableInstanceModel(self.formatModules,
                                                                      parent=self)
        self.__modulesDlgt = _WcguiRouteAssignationDelegate(mapper=self.__mapper,
                                                            routes=self.__routes,
                                                            parent=self)

    def _initElements(self):
        #: Add entry button
        self.__modulesTblW.setModel(self.modulesTblM)
        # self.__modulesTblW.horizontalHeader().setResizeMode(0, QHeaderView.Stretch)
        self.__modulesTblW.horizontalHeader().setResizeMode(1, QHeaderView.Stretch)
        self.__modulesTblW.horizontalHeader().setResizeMode(2, QHeaderView.Stretch)
        self.__modulesTblW.horizontalHeader().setResizeMode(3, QHeaderView.Stretch)
        self.__modulesTblW.horizontalHeader().setResizeMode(4, QHeaderView.Stretch)

        # self.__modulesTblW.setMaximumWidth(1000)
        # self.__modulesTblW.setMinimumHeight(700)
        self.__modulesTblW.setMinimumHeight(300)
        self.__modulesTblW.setItemDelegate(self.__modulesDlgt)
        self.__modulesTblW.selectionModel(). \
            selectionChanged.connect(self.__modulesTblM.selecChanged)

    def resettab(self):
        """
        reset self.__modulesTblM
        """
        self.modulesTblM.deleteRows(range(0, self.modulesTblM.rowCount()))
        self._loadFromData()

    def _loadFromData(self):

        self.modulesTblM.populateTable(self.__routes)

    def getBindings(self):
        """
        return the description of all card describes in the Table
        """
        return self.modulesTblM.getFullBindings()

    def formatModules(self, data, column):
        """
        :param data: data to format
        :type data: Cardname | Cardtype | [PortUUID]
        """
        if data is None:
            return ''
        elif getattr(data, 'getName', None):
            return data.getName()
        else:
            return data

    def createLayout(self):
        layout = QVBoxLayout()
        layout.addWidget(self.__modulesTblW)

        btnLayout = QHBoxLayout()
        btnLayout.addStretch()
        layout.addLayout(btnLayout)
        layout.addStretch(1)

        self.setLayout(layout)

        return layout

    def _addEntry(self):
        """
        Add entry in the model
        """
        self.modulesTblM.addRow()

    def _deleteSelEntries(self):
        """
        Delete selected entries from the model
        """
        selIdx = self.modulesTblW.selectedIndexes()
        """:type: list[QModelIndex]"""
        self.modulesTblM.deleteRows([idx.row() for idx in selIdx])
        rl = []
        for idx in selIdx:
            rl.append(idx.row())
        rowlist = sorted(set(rl))
        if self.modulesTblM.rowCount() > 0 and rowlist:
            if rowlist[0] >= self.modulesTblM.rowCount():
                self.modulesTblW.setCurrentIndex(self.modulesTblM.
                                                index(self.modulesTblM.rowCount() - 1, 0))
            else:
                self.modulesTblW.setCurrentIndex(self.modulesTblM.
                                                index(rowlist[0], 0))

    def removeModule(self, module_):
        """
        remove module binding associated to the card
        """
        self.instance.removeModule(module_)
        self.__scene.removeModuleBound(module_)
        self.__scene.ModuleUnbound.emit(module_)

    def selectedMsg(self, selectignor=False, role=Qt.UserRole):
        lrow = []
        for index in self.getSelectedIndexes():
            lrow.append(index.row())
        ulrow = set(lrow)
        if len(ulrow) == 1 and not selectignor:
            return self.modulesTblM.data(self.modulesTblM.index(lrow[0], 0),
                                      role=role)
        else:
            return None

    def getSelectedIndexes(self):
        return self.modulesTblW.selectionModel().selectedIndexes()

    def save(self):
        pass


# noinspection PyAbstractClass
class _WcguiRouteAssignationInstantiate(WguiXPointPropertiesBase):

    def __init__(self, pctx, scene, routes, bindingModuleClz, parent=None, ConverifF=None):
        """
        :param instance: The instance object
        :type instance: WhhdbHostInstance
        :param pctx: Project context
        :type pctx: WhmodHardwareProjectContextP
        :param parent: Parent widget
        """
        self.__module = pctx.getModule()
        self.__bindingModuleClz = bindingModuleClz
        self.__routes = routes
        self.__scene = scene
        """:type: WpmodSystemApplicationModuleP"""
        self.__topBlock = pctx.getProjectDB().getTopHierarchy()
        self.__hblock = pctx.getMappingManager().harware
        self.__consumers = {}
        self.warningText = []
        self.warningTextLabel = QLabel()
        for route in self.__routes:
            nbRoute = route.getNbConsumer()
            msg = route.getMessage()
            self.__consumers[msg] = [{_WCGUI_MODULES_CONS: None,
                                      _WCGUI_CARDS_CONS: None,
                                      _WCGUI_PORTS_CONS: None} for _ in range(nbRoute)]
        # init table necessary for
        self.table = None

        super(_WcguiRouteAssignationInstantiate, self). \
            __init__(None, parent=parent)

        self.__message_tab = _WcguiRouteAssignationHelper(pctx, scene, routes,
                                                          parent=self)

        #: Configuration table view
        if self.__class__ is _WcguiRouteAssignationInstantiate:
            self._finishInitialization()
        self.table = self.ui.xpntTableF
        self.table.setFixedWidth(500)

        self.__ConnectionUpdateF = ConverifF

    @property
    def consumers(self):
        return self.__consumers

    @property
    def producers(self):
        return {msgName: (eqp, port)
                for (msgName, mod, eqp, port, per) in self.__message_tab.getBindings()}

    def _loadConsumer(self, msg, eqpsSet, portsSet):
        if self.table:
            for row in range(self.table.rowCount()):
                comboModule = self.table.cellWidget(row, 0)
                comboEqp = self.table.cellWidget(row, 1)
                comboPort = self.table.cellWidget(row, 2)

                comboModule.currentIndexChanged.connect(partial(self.updateModuleField, row,
                                                                comboModule, comboEqp,
                                                                comboPort, msg,
                                                                eqpsSet, portsSet))

                comboEqp.currentIndexChanged.connect(partial(self.updateEqpField, row,
                                                             comboEqp, comboPort, msg))

                comboPort.currentIndexChanged.connect(partial(self.updatePortField, row,
                                                              comboPort, msg))
                # make sure that the other combo box feed data related to module when opening
                modName = self.__consumers[msg][row][_WCGUI_MODULES_CONS]
                eqpName = self.__consumers[msg][row][_WCGUI_CARDS_CONS]
                port = self.__consumers[msg][row][_WCGUI_PORTS_CONS]
                idxMod = comboModule.findText(modName)
                if modName is None:
                    idxMod = -1
                comboModule.setCurrentIndex(idxMod)
                idxEqp = comboEqp.findText(eqpName)
                if eqpName is None:
                    idxEqp = 0
                comboEqp.setCurrentIndex(idxEqp)
                if port:
                    idxPort = comboPort.findText(port.getName())
                    if not idxPort:
                        idxPort = 0
                else:
                    idxPort = 0
                comboPort.setCurrentIndex(idxPort)

    def updateFields(self):
        pass

    def createFields(self):
        modules = set()
        eqps = set()
        ports = set()
        for route in sorted(self.__routes, key=methodcaller('getMessage')):
            for module_ in route.getModulesConsumer():
                modules.add((module_, module_))
            for eqp in route.getEquipmentsConsumer():
                eqps.add((eqp, eqp))
            for port in route.getPortConsumer():
                ports.add((port, port.getName()))

        return {
            _WCGUI_MODULES_CONS: WguiXPointPropsComboField(
                _WCGUI_MODULES_CONS, _WCGUI_MODULES_CONS, modules),
            _WCGUI_CARDS_CONS: WguiXPointPropsComboField(
                _WCGUI_CARDS_CONS, _WCGUI_PORTS_CONS, eqps),
            _WCGUI_PORTS_CONS: WguiXPointPropsComboField(
                _WCGUI_PORTS_CONS, _WCGUI_PORTS_CONS, ports),
        }

    def getFieldOrder(self):
        return [_WCGUI_MODULES_CONS, _WCGUI_CARDS_CONS, _WCGUI_PORTS_CONS]

    def getStretchFields(self):
        return {_WCGUI_CARDS_CONS}

    def getButtons(self):
        pass

    def createXPoint(self, idx, model, instance):
        pass

    def resetModel(self, model, idx):
        if idx is None:
            model.setXPoints({})
            return
        routes = sorted(self.__routes, key=methodcaller('getMessage'))
        msg = routes[idx].getMessage()
        modules = {}
        modulesSet = set()
        for module_ in routes[idx].getModulesConsumer():
            modules[module_] = module_
            modulesSet.add((module_, module_))
        eqps = {}
        eqpsSet = set()
        for eqp in routes[idx].getEquipmentsConsumer():
            eqps[eqp] = eqp
            eqpsSet.add((eqp, eqp))
        ports = {}
        portsSet = set()
        for port in routes[idx].getPortConsumer():
            ports[port.getName()] = port
            portsSet.add((port.getName(), port))
        # first filter for modules
        self.fields[_WCGUI_MODULES_CONS].setOptions(modulesSet)

        model.setXPoints(
            {_WCGUI_MODULES_CONS: OrderedDict(modules),
             _WCGUI_CARDS_CONS: OrderedDict(eqps),
             _WCGUI_PORTS_CONS: OrderedDict(ports),}
            for _ in range(routes[idx].getNbConsumer())
        )

        self._loadConsumer(msg, eqpsSet, portsSet)

    def updateModuleField(self, row, comboModule, comboEqp, comboPort, msg,
                          eqpsSet, portsSet):
        modName = comboModule.currentText()
        eqpName = self.__scene.getModulesBound().get(modName, None)
        if eqpName:
            eqpsFiltered = self.filterEqp(eqpsSet, eqpName)

            self.setComboData(comboEqp, eqpsFiltered)
            self.__consumers[msg][row][_WCGUI_MODULES_CONS] = modName
            self.updateEqpField(row, comboEqp, comboPort, msg)
        else:
            self.setComboData(comboEqp, eqpsSet)
            self.setComboData(comboPort, portsSet)

    def updateEqpField(self, row, comboEqp, comboPort, msg):
        eqp = comboEqp.currentText()
        if eqp:
            eqpName, carName = eqp.split('.')
            portFiltered = self.filterPort(eqpName, targetCard=carName)
            self.setComboData(comboPort, portFiltered)
            self.__consumers[msg][row][_WCGUI_CARDS_CONS] = eqp

    def updatePortField(self, row, comboPort, msg):
        idx = comboPort.currentIndex()
        port = comboPort.itemData(idx)
        self.__consumers[msg][row][_WCGUI_PORTS_CONS] = port

    def setComboData(self, combo, data):
        for row in reversed(range(combo.count())):
            combo.removeItem(row)
        for dat, dat1 in data:
            combo.addItem(dat, dat1)

    def filterEqp(self, eqps, tragetEpq):
        eqpsFiltered = []
        for eqp, _ in eqps:
            eqpName, carName = eqp.split('.')
            if tragetEpq == eqpName:
                eqpsFiltered.append((eqp, eqp))
        return eqpsFiltered

    def filterPort(self, tragetEpq, targetCard=None):
        portsFiltered = []
        for item in self.__hblock.getInstances():
            if item.getName() == tragetEpq:
                for port in item.getConnectionPoints():
                    portName = port.getName()
                    if targetCard:
                        if port.getCard() == targetCard:
                            portsFiltered.append((portName, port))
                    else:
                        portsFiltered.append((portName, port))
                for port in item.getAllIoConnectionPoints():
                    portName = port.getName()
                    if targetCard:
                        if port.getIoCard() == targetCard:
                            portsFiltered.append((portName, port))
                    else:
                        portsFiltered.append((portName, port))

        return portsFiltered

    def updateXPoint(self, idx=None, model=None):
        pass

    def _createLayout(self):
        container = QWidget()
        container_Layout = QHBoxLayout()
        splitprinc = QSplitter()
        splitprinc.setOrientation(Qt.Horizontal)

        left = QWidget()
        left.setLayout(self.__message_tab.createLayout())
        left.setMinimumWidth(650)
        splitprinc.addWidget(left)

        right = QWidget()
        lefLayout = super()._createLayout()

        right.setLayout(lefLayout)

        container_Layout.addWidget(right)
        container.setLayout(container_Layout)
        splitprinc.addWidget(container)
        layout = QVBoxLayout()
        hLayout = QHBoxLayout()
        hLayout.addWidget(splitprinc)
        layout.addLayout(hLayout)
        layout.addWidget(self.warningTextLabel)
        layout.setMargin(2)

        return layout

    def save(self):
        errorMessage = []
        for route in self.__routes:
            msg = route.getMessage()
            protocol, protocolError, errMsg = self.guessProtocolFromBinding(route)
            if protocolError:
                errorMessage.append(errMsg)
            if protocol:
                self.__bindingModuleClz.createRoute(protocol, msg, self.producers,
                                                    self.consumers)

        if len(errorMessage) > 0:
            showErrorWindow("Protocol Coherence", 'Some conflict were not solved.', None)
            return True

    def guessProtocolFromBinding(self, route):
        msg = route.getMessage()
        allBinds = self.__message_tab.getBindings()
        protocols = set()
        errMsg = self.checkCoherency(msg, route)
        if errMsg:
            return None, True, errMsg
        for msgBind, modProducer, eqpProducer, portProducer, period in allBinds:
            if msgBind == msg:
                eqp, card = eqpProducer.split('.')
                self.__topBlock.addModuleCardBind(eqp, modProducer,
                                                  card, period=period)
                for dataConsumer in self.__consumers[msg]:
                    modConsumer = dataConsumer[_WCGUI_MODULES_CONS]
                    eqpConsumer = dataConsumer[_WCGUI_CARDS_CONS]
                    portConsumer = dataConsumer[_WCGUI_PORTS_CONS]
                    protocols.add(portConsumer.getCode())
                    eqpCons, cardCons = eqpConsumer.split('.')
                    self.__topBlock.addModuleCardBind(eqpCons, modConsumer,
                                                      cardCons, period=period)
        if len(protocols) == 0:
            errMsg = 'No protocol was found for Message {}.'.format(msg)
            return None, True, errMsg
        else:
            return list(protocols)[0], False, None

    def checkCoherency(self, msg, route=None):
        allBinds = self.__message_tab.getBindings()
        errMsg = None
        allModCons = []
        for msgBind, modProducer, eqpProducer, portProducer, _ in allBinds:
            if msgBind == msg:
                if not eqpProducer:
                    errMsg = 'Message: {} have no producer equipment declared.'.format(msg)
                    self.warningText.append(errMsg)
                    # return errMsg
                if not portProducer:
                    errMsg = 'Message: {} have no producer port declared.'.format(msg)
                    self.warningText.append(errMsg)
                    # return errMsg
                for dataConsumer in self.__consumers[msg]:
                    modConsumer = dataConsumer[_WCGUI_MODULES_CONS]
                    eqpConsumer = dataConsumer[_WCGUI_CARDS_CONS]
                    portConsumer = dataConsumer[_WCGUI_PORTS_CONS]
                    if not modConsumer:
                        errMsg = 'Message: {} have no consumer module declared.'.format(msg)
                        self.warningText.append(errMsg)
                        return errMsg
                    if not eqpConsumer:
                        errMsg = 'Message: {} have no consumer equipment declared.'.format(msg)
                        self.warningText.append(errMsg)
                        return errMsg
                    if not portConsumer:
                        errMsg = 'Message: {} have no consumer port declared.'.format(msg)
                        self.warningText.append(errMsg)
                        return errMsg
                    if portConsumer.getCode() != portProducer.getCode():
                        errMsg = 'Message: {} have consumer and producer port' \
                                 ' with different protocol.'.format(msg)
                        self.warningText.append(errMsg)
                    allModCons.append(modConsumer)
        if route:
            modConsumers = route.getModulesConsumer()
            if set(allModCons) != set(modConsumers):
                errMsg = 'Message: {} have some module that are not consumed,' \
                         ' all modules need to be bound.'.format(msg)
                self.warningText.append(errMsg)

        return errMsg


class _WcguiRouteAssignationTableInstanceModel(QAbstractTableModel):
    """
    Routes assignation table model.
    """
    TITLE_MESSAGE = 'Message'
    TITLE_MODULES_PROD = 'Mod Producer'
    TITLE_CARDS_PROD = 'Eqp Producer'
    TITLE_PORT_PROD = 'Port Producer'
    TITLE_PERIOD = 'Mod Period (ms)'

    HEADERS = [TITLE_MESSAGE, TITLE_MODULES_PROD, TITLE_CARDS_PROD,
               TITLE_PORT_PROD, TITLE_PERIOD]

    def __init__(self, formatF, parent=None):
        """
        :param XpntTypeNames: The Names of Xpoint of the mule
        :param parent: Parent widget
        """
        super().__init__(parent)
        self.__parent = parent
        self.__gParent = parent.parent
        self.__formatF = formatF

        #: Entries as list of tuples of
        #  port, message and type
        self.__entries = []
        self._update()

    @property
    def entries(self):
        return self.__entries

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
                                    0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if 0 <= column <= self.columnCount():
            info = self.__entries[row][column]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return self.__formatF(info, column)
            if role == Qt.BackgroundRole:
                self.isCorrect(index, row)
                return QBrush(self.color_back)
        return None

    def setData(self, index, data, role=None):
        assert role == Qt.UserRole
        self._setMessageEntry(index, data)

    def _setMessageEntry(self, index, data):
        msg, modProd, cardProd, portProd, period = self.__entries[index.row()]

        if index.column() == 0:
            actualentry = (data, modProd, cardProd, portProd, period)
        elif index.column() == 1:
            actualentry = (msg, cardProd, portProd, period)
        elif index.column() == 2:
            actualentry = (msg, modProd, data, portProd, period)
        elif index.column() == 3:
            actualentry = (msg, modProd, cardProd, data, period)
        elif index.column() == 4:
            actualentry = (msg, modProd, cardProd, portProd, data)
        self.__entries[index.row()] = actualentry

    def checkUnicityMsg(self, msg, row):
        for i, (entry) in enumerate(self.__entries):
            if i != row:
                # check uniqueness on msg name only
                if msg == entry[0]:
                    return False
        return True

    def getFullBindings(self):
        return self.__entries

    def populateTable(self, routes):
        for routeContain in sorted(routes, key=methodcaller('getMessage')):
            msg = routeContain.getMessage()
            modProd = routeContain.getModuleProducer()
            eqpProd = routeContain.getEquipmentProducer()
            if len(eqpProd) == 1:
                eqpProd = eqpProd[0]
            portProd = routeContain.getPortProducer()
            if len(portProd) == 1:
                portProd = portProd[0]
            else:
                portProd = None
            period = '1'
            self.addRow(msg, modProd, eqpProd, period, portProd)

    def getModules(self):
        return [modName for modName, _, _ in self.__entries]

    def addRow(self, msg, modProd, eqpProd, period, prodPort=None):
        """
        Add a row at the end of the list (append)

        :param module: The module name
        :type module: str | None
        :param card: The card name
        :type card: str | None
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())
        # help uniqueness by auto increment name
        self.__entries.append((msg, modProd, eqpProd, prodPort, period))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)
        self.__entries.pop(rowIndex)
        self.endRemoveRows()
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def isCorrect(self, index, row):
        msg = index.sibling(row, 0).data()
        if self.__gParent.checkCoherency(msg):
            self.color_back = QColor(255, 160, 160)
        else:
            self.color_back = Qt.white

    def checkAllCoherence(self):
        self.__gParent.warningText = []
        self.__gParent.warningTextLabel.setText('')
        for msg in self.getFullBindings():
            self.__gParent.checkCoherency(msg[0])
        self.__gParent.warningTextLabel.setText('\n'.join(self.__gParent.warningText))

    def flags(self, index):
        flgs = super().flags(index)
        row = index.row()
        col = index.column()
        if col in (0, 1):
            flgs |= ~Qt.ItemIsEditable
        else:
            flgs |= Qt.ItemIsEditable
        # self.isCorrect(index, row)
        self.checkAllCoherence()
        return flgs

    def selecChanged(self, newselec, oldselect):
        """
        Signal handler : when selection change reset connection editor with
        the param of the new card selected
        """
        if len(newselec.indexes()) == 0:
            # self.updateLastSelection()  <<<<<< to save data
            self.__gParent.resetModel(self.__gParent.model, None)
            return
        idx = newselec.indexes()[0]
        row = idx.row()
        self.__gParent.resetModel(self.__gParent.model, row)


class _WcguiRouteAssignationDelegate(QItemDelegate):
    """
    Delegate for message items
    """

    def __init__(self, mapper, routes, parent=None):
        """
        :type CardTyp: () -> list[CardTyp]
        :type formatF: (elem,column) -> str
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(parent)
        self.__mapper = mapper
        self.__routes = routes
        self.__top = parent.topBlock

    def createEditor(self, parent, option, index):
        """
        Create a editors for message table the selected item

        :type parent: QWidget
        :type option: QStyleOptionViewItem
        :type index: QModelIndex
        """

        def save(*_):
            self.setModelData(editor, index.model(), index)

        col = index.column()
        row = index.row()
        if col == 2:
            eqpProd = self.__routes[row].getEquipmentProducer()
            editor = QComboBox(parent)
            for eqp in eqpProd:
                editor.addItem(eqp, eqp)
        elif col == 3:
            portProd = self.__routes[row].getPortProducer()
            editor = QComboBox(parent)
            for port in portProd:
                editor.addItem(port.getName(), port)
        elif col == 4:
                editor = QComboBox(parent)
                for period in WCGUI_APP_PERIOD:
                    editor.addItem(period, period)
        else:
            editor = QLineEdit(parent)
        return editor

    def setEditorData(self, editor, index):
        """
        :type editor: QComboBox | QLineEdit
        :type index: QModelIndex
        """
        data = index.model().data(index, Qt.EditRole)
        if isinstance(editor, QComboBox):
            for index in range(editor.count()):
                if editor.itemData(index, Qt.UserRole) == data:
                    editor.setCurrentIndex(index)
                    break
            else:
                editor.setCurrentIndex(0)
        elif isinstance(editor, QLineEdit):
            if isinstance(data, set) or isinstance(data, list):
                if len(data) == 0:
                    data = data[0]
                else:
                    data = ', '.join(data)
            editor.setText(data)

    def setModelData(self, editor, model, index):
        """
        :type editor: QComboBox | QLineEdit
        :type model: QAbstractItemModel
        :type index: QModelIndex
        """
        if isinstance(editor, QComboBox):
            model.setData(index, wgui_qcb_get_data(editor), Qt.UserRole)
        elif isinstance(editor, QLineEdit):
            model.setData(index, editor.text(), Qt.UserRole)


class WiguiTypeInstanciateDialog(WguiPropertiesDialog):
    """
    The WguiOperatorInstancePropertiesDialog class is a dialog design to
    access and manipulates operator instance data. This include name,
    info and connection points.
    """

    _TYPE_NAME = 'Route Creation Helper'

    _SHOW_OPERATOR_WIDGET = True

    # noinspection PyTypeChecker
    def __init__(self, pctx, scene, routes, bindingModuleClz, updateF=None, selectXpnt=None, parent=None):
        """
        :param inst: Instance
        :type inst: WhdbOperatorInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param selectXpnt: Initially selected connection point, if any
        :type selectXpnt: WhdbConnectionPointP
        :param parent: Parent widget
        """
        super().__init__(self._TYPE_NAME, updateF, parent=parent)
        self.setFixedWidth(1200)
        # self.setMaximumWidth(1100)
        self.setMinimumHeight(500)
        instWdgt = _WcguiRouteAssignationInstantiate(pctx, scene, routes, bindingModuleClz)
        self.instWdgt = instWdgt
        addiWdgts = self._createAdditionalInstancePropertiesWidgets(pctx)
        if addiWdgts:
            # noinspection PyTypeChecker
            generalWdgts = [instWdgt] + addiWdgts
        else:
            generalWdgts = [instWdgt]

        widgets = [('User Input', WguiMultiPropertiesWidgetP(*generalWdgts))]

        widgets.extend(self._createAdditionalPropertiesWidgets(pctx))

        xpntWdgt = self._createConnectionPointsPropertiesWidget(pctx)
        if xpntWdgt:
            widgets.append(xpntWdgt)


        # noinspection PyUnresolvedReferences
        self.setTabbedPropertyWidgets(
            *widgets,
            activate=xpntWdgt[1] if selectXpnt and xpntWdgt else None)


    def showOperatorWidget(self):
        return True

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createAdditionalInstancePropertiesWidgets(self, pctx):
        return []

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createAdditionalPropertiesWidgets(self, pctx):
        return []

    # noinspection PyMethodMayBeStatic
    def _createConnectionPointsPropertiesWidget(self, pctx):
        pass

    def _setMainLayout(self, widgets, widget):
        """
        Set the dialog's property widgets.

        :param widgets: Property widgets, and their titles
        :type widgets: Iterable[(str, WguiPropertiesWidgetP)]
        :param widget: Wrapper widget containing the property widgets
        :type widget: QWidget
        """
        self.__widgets = [widget for _, widget in widgets]

        layout = QVBoxLayout()
        layout.addWidget(widget)
        buttons = QDialogButtonBox(QDialogButtonBox.Ok |
                                   QDialogButtonBox.Cancel)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        self.okStateChanged.connect(
            buttons.button(QDialogButtonBox.Ok).setEnabled)
        layout.addWidget(buttons)
        # create the main layout
        self.setLayout(layout)

    def accept(self):
        hasErr = self._save()
        if not hasErr:
            super().accept()

    def _apply(self):
        """
        Apply the current changes, and leave the dialog window open.
        """
        self._save()
        self._updateFields()

    def _reset(self):
        """
        Reset the dialog window.
        """
        self._updateFields()

    def _save(self):
        """
        Save all changes.
        """
        hasErr = False
        for widget in self.__widgets:
            widgErr = widget.save()
            if widgErr:
                hasErr = True

        return hasErr

    def _updateFields(self):
        """
        Reset all fields to their original state.
        """
        for widget in self.__widgets:
            widget.updateFields()