# ----------------------------------------------------------------------------
# - wcgui-route-viewer-dialog                                                -
# - wild cruncher - cockpit : route viwer dialog                             -
# ----------------------------------------------------------------------------
# - author    (c) 2019-2019 achraf mahti                                     -
# - copyright (c) 2019-2019 silkan                                           -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_route_viewer_dialog
   :platform:     Unix, Windows

.. moduleauthor:: achraf mahti <mahti@silkan.net>
"""
import os
from PyQt5.QtWidgets import QVBoxLayout,QDialogButtonBox
from wcr_frontend.gui.wgui_props_dialog_widget import *


class WiguiRouteViewerDialog(WguiPropertiesDialog):
    """
    The WguiOperatorInstancePropertiesDialog class is a dialog design to
    access and manipulates operator instance data. This include name,
    info and connection points.
    """

    _TYPE_NAME = 'Route Viewer'

    _SHOW_OPERATOR_WIDGET = True

    # noinspection PyTypeChecker
    def __init__(self, pctx, editor, filepath=None, updateF=None, parent=None):
        """
        :param inst: Instance
        :type inst: WhdbOperatorInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param selectXpnt: Initially selected connection point, if any
        :type selectXpnt: WhdbConnectionPointP
        :param parent: Parent widget
        """
        super().__init__(self._TYPE_NAME, updateF, parent=parent)
        self.saved = False
        editor.setMinimumWidth(1100)
        editor.setMinimumHeight(600)
        addiWdgts = self._createAdditionalInstancePropertiesWidgets(pctx)
        if addiWdgts:
            # noinspection PyTypeChecker
            generalWdgts = [editor] + addiWdgts
        else:
            generalWdgts = [editor]
        self.instWdgt = editor
        widgets = [(filename, WguiMultiPropertiesWidgetP(*generalWdgts))]

        widgets.extend(self._createAdditionalPropertiesWidgets(pctx))

        xpntWdgt = self._createConnectionPointsPropertiesWidget(pctx)
        if xpntWdgt:
            widgets.append(xpntWdgt)


        # noinspection PyUnresolvedReferences
        self.setTabbedPropertyWidgets(*widgets)

        self.opened= False


    def showOperatorWidget(self):
        return True

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createAdditionalInstancePropertiesWidgets(self, pctx):
        return []

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createAdditionalPropertiesWidgets(self, pctx):
        return []

    # noinspection PyMethodMayBeStatic
    def _createConnectionPointsPropertiesWidget(self, pctx):
        pass

    def _setMainLayout(self, widgets, widget):
        """
        Set the dialog's property widgets.

        :param widgets: Property widgets, and their titles
        :type widgets: Iterable[(str, WguiPropertiesWidgetP)]
        :param widget: Wrapper widget containing the property widgets
        :type widget: QWidget
        """
        self.__widgets = [widget for _, widget in widgets]

        layout = QVBoxLayout()
        layout.addWidget(widget)
        buttons = QDialogButtonBox(QDialogButtonBox.Ok |
                                   QDialogButtonBox.Cancel)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        self.okStateChanged.connect(
            buttons.button(QDialogButtonBox.Ok).setEnabled)
        layout.addWidget(buttons)
        # create the main layout
        self.setLayout(layout)

    def accept(self):
        hasErr = self._save()
        if not hasErr:
            super().accept()
        self.saved = True
        self.opened = False

    def reject(self):
        super().reject()
        self.opened = False
        self.saved = False

    def _apply(self):
        """
        Apply the current changes, and leave the dialog window open.
        """
        self._save()
        self._updateFields()

    def _reset(self):
        """
        Reset the dialog window.
        """
        self._updateFields()

    def _save(self):
        """
        Save all changes.
        """
        hasErr = False

        for widget in self.__widgets:
            widgErr = widget.save()
            if widgErr:
                hasErr = True

        return hasErr

    def _updateFields(self):
        """
        Reset all fields to their original state.
        """
        for widget in self.__widgets:
            widget.updateFields()

    def exec_(self):
        self.opened = True
        super().exec()

    def closeEvent(self, event):
        self.opened = False

    def getOpened(self):
        return self.opened