# ----------------------------------------------------------------------------
# - wcgui-scene-item-factory                                                 -
# - wild cruncher: scene item factory                 sys_map cockpit system -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_scene_item_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher hardware scene item factory

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_frontend.gui.wgui_scene_item import WguiXPointRectPositionProvider, \
    WGUI_CONNECTION_COLOUR, WGUI_CONNECTION_STYLE, _WGUI_CONN_BGRND
from wcr_frontend.gui.wgui_scene_item_factory import WguiDefaultSceneItemFactoy
from wcr_core.db_api.whdb_instance_api import WhdbBlockInstanceP
from wcr_core.modules.sys_map.db.wchdb_Host_lure_instance import \
    WchdbHostLureInstance
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_CETRAC_CODE
from wcr_core.modules.sys_map.db.wchdb_rack_lure_instance import \
    WchdbRackLureInstance
from wcr_frontend.modules.hardware.gui.whgui_scene_item_factory import \
    WHGUI_COLOR_CON_POINT_DIC, WHGUI_STYLE_CON_DIC, WHGUI_ERROR_CON_COLOR,\
    WHGUI_CONN_BGRND


class WcguiSceneItemFactory(WguiDefaultSceneItemFactoy):
    """
    System Mapper scene item factory.
    """

    def __init__(self, appModule, mapper):
        """
        :param appModule: Application module
        :type appModule: WmmodModelApplicationModuleP
        """
        super().__init__(WguiXPointRectPositionProvider())

        self.__instTooltipF = wcgui_instance_tooltip_func(appModule, mapper=mapper)

        self.__mapper = mapper

    def createOperatorInstanceSceneItem(self, obj, *args, **kwargs):
        item = super().createOperatorInstanceSceneItem(obj, *args, **kwargs)
        item.setTooltipF(self.__instTooltipF)
        return item

    def createConnectionPoint(self, conn, item, *args, **kwargs):
        conpoint = super().createConnectionPoint(conn, item, *args, **kwargs)

        if self.__mapper.getPhyPortByUUID(conn.getUUIDSrc()):
            hpoint = self.__mapper.getPhyPortByUUID(conn.getUUIDSrc())
            hpCode = hpoint.getCode()
            if hpoint.getActive() == "Off":
                color = WHGUI_CONN_BGRND
            else:
                color = WHGUI_COLOR_CON_POINT_DIC.get(hpCode, _WGUI_CONN_BGRND)
            conpoint.setbackgroundcolor(color)
        return conpoint

    def createConnectionSceneItem(self, obj, *args, **kwargs):
        connection = super().createConnectionSceneItem(obj, *args, **kwargs)
        hpscode = self.__mapper.getPhyPortByUUID(obj.getStartConnectionPoint().
                                                getUUIDSrc()).getCode()
        hpecode = self.__mapper.getPhyPortByUUID(obj.getEndConnectionPoint().
                                                 getUUIDSrc()).getCode()
        hpsspeed = self.__mapper.getPhyPortByUUID(obj.getStartConnectionPoint().
                                                 getUUIDSrc()).getSpeed()

        if hpscode == hpecode or (
                WHHDB_XPOINT_CETRAC_CODE in (hpscode, hpecode)):
            connection.setConnColour(WHGUI_COLOR_CON_POINT_DIC.get(
                hpscode, WGUI_CONNECTION_COLOUR))
        else:
            connection.setConnColour(WHGUI_ERROR_CON_COLOR)
        connection.setConnStyle(WHGUI_STYLE_CON_DIC.get(
            hpsspeed, WGUI_CONNECTION_STYLE))



        return connection


def wcgui_instance_tooltip_func(appModule, mapper=None):
    """
    :param appModule: Application module
    :type appModule: WmmodModelApplicationModuleP
    :rtype: (WhdbInstanceP) -> str
    """
    def tooltip(instance):
        ttip = '<b>{}</b>'.format(instance.getName())
        moduleBind = mapper._WcmodMappingManager__topBlock.getModuleCardBind()
        if isinstance(instance, WhdbBlockInstanceP):
            block = instance.getBlock()
            btyp = appModule.getBlockType(block.getEntityType())
            if btyp:
                bnam = btyp.getName()
                if bnam:
                    ttip += '<br/><i>{}</i>'.format(bnam.title())
        if (isinstance(instance, WchdbHostLureInstance) or
                isinstance(instance, WchdbRackLureInstance)):
            cards = instance.getCards()
            ttip += '<br/>---- Cards ----</i>'
            for card in cards:
                ttip += '<br/><i>{}</i>'.format(card)
        modules = moduleBind.get(instance.getName(), None)
        if modules:
            ttip += '<br/>---- Modules ----</i>'
            for modName, cardName, _ in modules:
                ttip += '<br/>{}   ({})'.format(modName, cardName)

        if instance.getInfo():
            ttip += '<br/>' + instance.getDescription()

        return ttip

    return tooltip
