# ----------------------------------------------------------------------------
# - wcgui-scene                                                              -
# - wild cruncher - cockpit: system mapping graphics view                    -
# ----------------------------------------------------------------------------
# - author    (c) 2019-2019 achraf mahti                                     -
# - copyright (c) 2019-2019 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wgui_scene_view
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher graphics view

.. moduleauthor:: achraf mahti <achraf@cetrac.io>
"""
from uuid import UUID

from wcr_frontend.gui.wgui_scene_view import WguiGraphicsView
from wcr_frontend.gui.wgui_scene_item_ids import *
from wcr_frontend.gui.wgui_scene_item import *
from wcr_frontend.gui.wgui_utils import WGUI_VL_PRODUCER_MIME_TYPE,\
                                        WGUI_VL_CONSUMER_MIME_TYPE

WCGUI_MODE_PRODUCER = 'Producer'
WCGUI_MODE_CONSUMER = 'Consumer'

class WcguiGraphicsView(WguiGraphicsView):

    def __init__(self, pctx, cctx, mouseHandler, scene):
        super().__init__(pctx, cctx, mouseHandler, scene)

        self.__mode = WCGUI_MODE_PRODUCER

    def dragEnterEvent(self, event):
        if (event.mimeData().hasFormat(WGUI_VL_PRODUCER_MIME_TYPE) or
                event.mimeData().hasFormat(WGUI_VL_CONSUMER_MIME_TYPE)):
            self.scene()._highlightsConnectedConnectorPoint()
            event.acceptProposedAction()

    def dropEvent(self, event):
        if event.mimeData().hasFormat(WGUI_VL_PRODUCER_MIME_TYPE):
            routes = self.getDataRoutes(event, WGUI_VL_PRODUCER_MIME_TYPE)
            self.dropProducerRoutes(event, routes)
        elif event.mimeData().hasFormat(WGUI_VL_CONSUMER_MIME_TYPE):
            routes = self.getDataRoutes(event, WGUI_VL_CONSUMER_MIME_TYPE)
            if len(routes) > 0:
                self.dropConsumerRoute(event, routes)

    def getDataRoutes(self, event, mode):
        data = event.mimeData().data(mode)
        if not data or data.isEmpty():
            return
        strData = str(data, 'ASCII')
        vlBlock = self.scene().top.getVLinkBlock()
        routes = []
        for uuidstr in strData.split(', '):
            routes.append(vlBlock.getRouteFromUUID(UUID(uuidstr)))
        return routes

    def dropProducerRoutes(self, event, routes):
        pos = self.mapToScene(event.pos())
        if self.scene().createProducerBind(pos, routes):
            event.acceptProposedAction()

    def dropConsumerRoute(self, event, routes):
        pos = self.mapToScene(event.pos())
        if self.scene().createConsumerBind(pos, routes):
            event.acceptProposedAction()

