# ----------------------------------------------------------------------------
# - sys-map-gui-cstcx                                                        -
# - wild cruncher:sys_map design cockpit system contex                       -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_cstcx
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher wcs cockpit system contex

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_frontend.gui.wgui_scene_item_ids import XPNT_ITEM_ID, \
    CNCT_ITEM_ID
from wcr_core.common.wcmn_protocols import WcmnReleasableP
from wcr_frontend.gui.wgui_scene_view_mouse_handler import \
    WguiDefaultSceneViewMouseHandler
from wcr_frontend.gui.wgui_gui_module import \
    WguiCockpitSystemContextBase, WguiCockpitSystemModuleBase
from wcr_frontend.gui.wgui_clipboard import WguiClipboardHandler
from wcr_frontend.modules.sys_map.gui.wcgui_scene_context_menu_factory import \
    WcguiSceneContextMenuFactory
from wcr_core.modules.sys_map.wcmod_application_module import \
    WCMOD_SYSTEM_MAP_APP_MODULE
from wcr_frontend.modules.sys_map.gui.wcgui_props_dialog_factory import \
    WcguiPropertiesDialogFactory
from wcr_frontend.modules.sys_map.gui.wcgui_scene_item_factory import \
    WcguiSceneItemFactory

from wcr_frontend.modules.sys_map.gui.wcgui_vlink_table import \
    WcguiVLinkViewTable
from wcr_frontend.modules.sys_map.gui.wcgui_vlink_prod_binding_table import \
    WcguiVLProdBindingTable
from wcr_frontend.modules.sys_map.gui.wcgui_vlink_cons_binding_table import \
    WcguiVLConsBindingTable
from wcr_frontend.modules.sys_map.gui.wcgui_view_filter import sysmapFilter
from wcr_core.modules.sys_map.actions.wcact_map_checker_action import \
    WCACT_ACTION_BIND_VERIFY
from wcr_frontend.modules.sys_map.gui.wcgui_bind_verification_dialog import \
    WcguiBindingVerifierFailureHandler
from wcr_frontend import showErrorWindow
from wcr_frontend.modules.sys_map.gui.wcgui_scene import WcguiGraphicsScene
from wcr_frontend.modules.icd import _WIGUI_ICONS

_WCGUI_ICONS = _WIGUI_ICONS


class _WcguiCockpitSystemContext(WguiCockpitSystemContextBase):
    def __init__(self, cmod, pctx, pcfg, actions, actx, parent):
        """
        :param cmod: Cockpit system module
        :type cmod: WguiCockpitSystemModuleP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param pcfg: Project editor configuration
        :type pcfg: WguiProjectEditorCfgP
        :param actions: Actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WguiActionExecutionContextP
        :param parent: Parent widget
        :type parent: QWidget
        """
        super().__init__(cmod)

        modActions = list(WCGUI_COCKPIT_SYSTEM_MODULE.getActions())
        """:type: list[(WactActionP, str, str, tuple[str])]"""
        self.__filter = sysmapFilter()
        self.__actx = actx
        self.__sceneItmFy = WcguiSceneItemFactory(WCMOD_SYSTEM_MAP_APP_MODULE,
                                                  pctx.getMappingManager())
        self.__scene = WcguiGraphicsScene(
            pctx.getProjectDB(), pcfg, actions, pctx, self, actx,
            WCMOD_SYSTEM_MAP_APP_MODULE, parent)
        self.__propDiagF = WcguiPropertiesDialogFactory(
            pctx, pcfg, self.__scene, actions, parent)
        self.__clpbrdHandler = WguiClipboardHandler(
            self.__scene, cmod.getAppModule())
        self.__cntxtMenuF = WcguiSceneContextMenuFactory(
            self.__scene, actions, self.__clpbrdHandler, pctx,
            self.__propDiagF, self.updateSides)
        self.__mouseHandler = WguiDefaultSceneViewMouseHandler(
            pctx, self.__propDiagF, self.__scene, self.__cntxtMenuF)

        self.__sidebarModules = self._createSidebarModules(
            pctx, self.__scene, self.__cntxtMenuF, modActions, parent)

        #giving pctx scene interaction functions
        pctx.setaddOperInstguifunc(self.__scene.addFunctionItem)
        pctx.setaddConnectionguifunc(self.__scene.addConnectionItem)
        pctx.setremovefunc(self.removeElement)
        pctx.setsideupdatefunc(self.updateSides)

    def release(self):
        for _, _, smod in self.__sidebarModules:
            if isinstance(smod, WcmnReleasableP):
                smod.release()

        for item in [self.__scene, self.__propDiagF, self.__clpbrdHandler,
                     self.__cntxtMenuF, self.__mouseHandler,
                     self.__sidebarModules]:
            if isinstance(item, WcmnReleasableP):
                item.release()

        self.__scene = None
        self.__propDiagF = None
        self.__clpbrdHandler = None
        self.__cntxtMenuF = None
        self.__mouseHandler = None
        self.__sidebarModules = []

        super().release()

    def _createSidebarModules(self, pctx, scene, cntxtMenuF, actions, parent):
        modules = []

        #add logical module.
        pBdtbl = WcguiVLProdBindingTable(pctx, scene,self.__filter, parent)
        cBdtbl = WcguiVLConsBindingTable(pctx, scene,self.__filter, parent)
        logic = WcguiVLinkViewTable(pctx, scene, pBdtbl, cBdtbl, self.__actx, parent)
        logic.setPropagationFunction(self.updateSides)
        modules.append(('vlinktab', 'Traffic Routes', logic))
        #add binding module
        modules.append(('prodbind', 'TRoutes Producers Binding', pBdtbl))
        # add binding ports
        modules.append(('consbind', 'TRoutes Consumers Binding', cBdtbl))

        return modules

    def getExcludedSystemToolboxItems(self):
        return [XPNT_ITEM_ID,CNCT_ITEM_ID]

    def getHighlightingModules(self):
        return []

    def getDirectoryDescription(self, level):
        return 'project' if level == 0 else 'sched. group'

    def getScene(self):
        return self.__scene

    def getSceneViewMouseHandler(self):
        return self.__mouseHandler

    def getClipboardHandler(self):
        return self.__clpbrdHandler

    def getSceneItemFactory(self):
        return self.__sceneItmFy

    def getContextMenuFactory(self):
        return self.__cntxtMenuF

    def getSidebarModules(self):
        return iter(self.__sidebarModules)

    def getPropertiesDialogFactory(self):
        return self.__propDiagF

    def removeElement(self, elem):
        self.__scene.removeItem(self.__scene.getSceneItem(elem))

    def updateSides(self, filtreduuid=None):
        self.__filter.setHostAsFilter(filtreduuid)
        #: update module binding table
        self.__sidebarModules[1][2].reloadFromData()
        #: update port binding table
        self.__sidebarModules[2][2].reloadFromData()


class _WcguiCockpitSystemModule(WguiCockpitSystemModuleBase):
    def __init__(self):
        super().__init__(WCMOD_SYSTEM_MAP_APP_MODULE)

    def createContext(self, pctx, pcfg, actions, actx, parent):
        path = pctx.getProjectDB().path
        hw = pctx.getHardwareFile()
        lg = pctx.getLogicFile()
        if path and hw is None:
            showErrorWindow('Project Error',
                            'Hardware project bound was not found.\n'
                            'It is possible that the path of the hardware file was changed.',
                            None)
            return None
        if path and lg is None:
            showErrorWindow('Project Error',
                            'Logic project bound was not found.\n'
                            'It is possible that the path of the logical file was changed.',
                            None)
        return _WcguiCockpitSystemContext(
            self, pctx, pcfg, actions, actx, parent)

    def getIconForCode(self, code, iconType):
        return _WCGUI_ICONS.get(code) or \
            super().getIconForCode(code, iconType)

    def _createActionObservers(self):
        odct = super()._createActionObservers()

        odct[WcguiBindingVerifierFailureHandler] = (
            WcguiBindingVerifierFailureHandler(),
            [lambda a, c, s, fs: WCACT_ACTION_BIND_VERIFY in fs])

        return odct


#: GUI Cockpit system cotnext singleton
WCGUI_COCKPIT_SYSTEM_MODULE = _WcguiCockpitSystemModule()
