# ----------------------------------------------------------------------------
# - wcgui-virtual-link-table                                                 -
# - wild cruncher:sys_map virtual link widget                                -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wcgui_virtual_link_table
   :platform:     Unix, Darwing, Windows
   :synopsis:     Wild Cruncher silkan sys_map virtuallinks widget

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
import os
import copy

from PyQt4.QtCore import QAbstractTableModel, Qt, QModelIndex
from PyQt4.QtGui import QWidget, QTableView, QHeaderView, QAbstractItemView,\
    QVBoxLayout, QHBoxLayout, QPushButton, QSplitter, QMessageBox, \
    QTableWidgetItem, QTableWidget

from wcr_core.db.whdb_utils import whdb_getNextName
from wcr_frontend.gui.wgui_utils import wgui_getOpenFile
from wcr_core.project.wprj_xmlio import WPRJ_VLINK_SILKAN_XML_MIME_TYPE
from wcr_core.common.wcmn_file_handler import WcmnFile
from wcr_frontend.modules.sys_map.gui.wcgui_binding_module_dialog import \
    WcguiAutoBindModules
from wcr_frontend.modules.vlink.gui.wvgui_vlink_gui_module import \
    WVGUI_COCKPIT_SYSTEM_MODULE
from wcr_core.project.wprj_project import wprj_load_project, \
    wprj_import_project, wprj_createProjectDB
from wcr_frontend.gui.wgui_project_editor_config import WguiProjectEditorCfg, \
    WguiProjectEditorDbCfgStore, WguiProjectEditorCfgDefaults
from wcr_frontend.gui.wgui_project_editor import WguiProjectEditor
from wcr_core.wcor_global_context import WcorGlobalContext
from wcr_core.modules.sys_map.wcmod_bus_file_handler \
    import WcmodVLinkFileHandler
from wcr_frontend import showErrorWindow


class TableWidgetDragRows(QTableView):
    def __init__(self, *args, **kwargs):
        QTableView.__init__(self, *args, **kwargs)

        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        self.viewport().setAcceptDrops(True)
        self.setDragDropOverwriteMode(False)
        # self.setDropIndicatorShown(True)

        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setDragDropMode(QAbstractItemView.DragOnly)

    def dropEvent(self, event):
        if event.source() == self and (event.dropAction() == Qt.MoveAction or
                       self.dragDropMode() == QAbstractItemView.InternalMove):
            success, row, col, topIndex = self.dropOn(event)
            if success:
                selRows = self.getSelectedRowsFast()

                top = selRows[0]
                # print 'top is %d'%top
                dropRow = row
                if dropRow == -1:
                    dropRow = self.rowCount()
                # print 'dropRow is %d'%dropRow
                offset = dropRow - top
                # print 'offset is %d'%offset

                for i, row in enumerate(selRows):
                    r = row + offset
                    if r > self.rowCount() or r < 0:
                        r = 0
                    self.insertRow(r)
                    # print 'inserting row at %d'%r


                selRows = self.getSelectedRowsFast()
                # print 'selected rows: %s'%selRows

                top = selRows[0]
                # print 'top is %d'%top
                offset = dropRow - top
                # print 'offset is %d'%offset
                for i, row in enumerate(selRows):
                    r = row + offset
                    if r > self.rowCount() or r < 0:
                        r = 0

                    for j in range(self.columnCount()):
                        # print 'source is (%d, %d)'%(row, j)
                        # print 'item text: %s'%self.item(row,j).text()
                        source = QTableWidgetItem(self.item(row, j))
                        # print 'dest is (%d, %d)'%(r,j)
                        self.setItem(r, j, source)

                # Why does this NOT need to be here?
                # for row in reversed(selRows):
                    # self.removeRow(row)

                event.accept()

        else:
            QTableView.dropEvent(event)

    def getSelectedRowsFast(self):
        selRows = []
        for item in self.selectedItems():
            if item.row() not in selRows:
                selRows.append(item.row())
        return selRows

    def droppingOnItself(self, event, index):
        dropAction = event.dropAction()

        if self.dragDropMode() == QAbstractItemView.InternalMove:
            dropAction = Qt.MoveAction

        if event.source() == self and event.possibleActions() & Qt.MoveAction and dropAction == Qt.MoveAction:
            selectedIndexes = self.selectedIndexes()
            child = index
            while child.isValid() and child != self.rootIndex():
                if child in selectedIndexes:
                    return True
                child = child.parent()

        return False

    def dropOn(self, event):
        from PyQt4.QtCore import pyqtRemoveInputHook
        import ipdb
        pyqtRemoveInputHook()
        ipdb.set_trace()

        if event.isAccepted():
            return False, None, None, None

        index = QModelIndex()
        row = -1
        col = -1
    #
    #     if self.viewport().rect().contains(event.pos()):
    #         index = self.indexAt(event.pos())
    #         if not index.isValid() or not self.visualRect(index).contains(event.pos()):
    #             index = self.rootIndex()
    #
    #     if self.model().supportedDropActions() & event.dropAction():
    #         if index != self.rootIndex():
    #             dropIndicatorPosition = self.position(event.pos(), self.visualRect(index), index)
    #
    #             if dropIndicatorPosition == QAbstractItemView.AboveItem:
    #                 row = index.row()
    #                 col = index.column()
    #                 # index = index.parent()
    #             elif dropIndicatorPosition == QAbstractItemView.BelowItem:
    #                 row = index.row() + 1
    #                 col = index.column()
    #                 # index = index.parent()
    #             else:
    #                 row = index.row()
    #                 col = index.column()
    #
    #         if not self.droppingOnItself(event, index):
    #             # print 'row is %d'%row
    #             # print 'col is %d'%col
    #             return True, row, col, index
    #
    #     return False, None, None, None
    #
    def position(self, pos, rect, index):
        r = QAbstractItemView.OnViewport
        margin = 2
        if pos.y() - rect.top() < margin:
            r = QAbstractItemView.AboveItem
        elif rect.bottom() - pos.y() < margin:
            r = QAbstractItemView.BelowItem
        elif rect.contains(pos, True):
            r = QAbstractItemView.OnItem

        if r == QAbstractItemView.OnItem and not (self.model().flags(index) & Qt.ItemIsDropEnabled):
            r = QAbstractItemView.AboveItem if pos.y() < rect.center().y() else QAbstractItemView.BelowItem

        return r


class WcguiVLinkViewTable(QWidget):
    """
    Virtual link presentation table widget.
    """

    def __init__(self, pctx, scene, pBdtbl, cBdtbl, actx, parent=None):
        """
        :param pctx: Project context
        :type pctx: WpmodSystemProjectContextP
        :param scene: Graphics scene (for current block and change signals)
        :type scene: WguiGraphicsScene
        :param pBdtbl: producer VL binding table
        :type pBdtbl: WcguiVLProdBindingTable
        :param cBdtbl: consumer VL binding table
        :type cBdtbl: WcguiVLConsBindingTable
        :param parent: Parent widget
        """
        super().__init__(parent)

        # : Project context
        self.__pctx = pctx
        self.__actx = actx
        self.__parent = parent
        self.__top = self.__pctx.getProjectDB().getTopHierarchy()
        #: Graphical scene
        self.__scene = scene
        #: Update binding tables removing invalids binds
        self.__PropagateUpdateFunc = None
        # producer binding table
        self.__pBdtbl = pBdtbl
        # consumer binding table
        self.__cBdtbl = cBdtbl

        #: Bus binding table model
        self.__modTblM = _WcguiVLinkTableModel()
        self.__modTblMAuto = _WcguiVLinkAutoTableModel()


        """:type: (WphdbPhyBusInstance) -> list[WshdbBusInstance]"""

        #: Bus binding table widget
        self.__modTblW = TableWidgetDragRows()

        self.__modTblWAuto = TableWidgetDragRows()

        #: Add entry button
        self.__linkButton = QPushButton('Link Route File')
        self.__impButton = QPushButton('Import Routes')
        self.__autoRouteButton = QPushButton('Generate routes')
        self._initElements()
        self.setLayout(self._createLayout())
        self.updateTable()

    def _initElements(self):
        self.__modTblW.setModel(self.__modTblM)
        self.__modTblW.horizontalHeader().setResizeMode(
            0, QHeaderView.ResizeToContents)
        self.__modTblW.horizontalHeader().setResizeMode(
            1, QHeaderView.Stretch)
        self.__modTblW.setDragEnabled(True)

        self.__modTblWAuto.setModel(self.__modTblMAuto)
        self.__modTblWAuto.horizontalHeader().setResizeMode(
            0, QHeaderView.ResizeToContents)
        self.__modTblWAuto.horizontalHeader().setResizeMode(
            1, QHeaderView.Stretch)
        self.__modTblWAuto.setDragEnabled(True)

        self.__modTblWAuto.doubleClicked.connect(self._openRoutesView)
        self.__modTblW.doubleClicked.connect(self._openRoutesView)
        self.__linkButton.clicked.connect(self._linkVirtualLink)
        self.__impButton.clicked.connect(self._importVirtualLink)
        self.__autoRouteButton.clicked.connect(self.autoRouteFunction)


    def _openRoutesView(self, idx):
        module = WVGUI_COCKPIT_SYSTEM_MODULE.getAppModule()
        gctx = WcorGlobalContext()
        pctx = module.createProjectContext(gctx, wprj_createProjectDB(module))

        cfgTable = pctx.getProjectDB().tops.getLocalConfiguration()
        cfgStore = WguiProjectEditorDbCfgStore(gctx.getConfigurationService())
        glblPrjtEdiCfg = WguiProjectEditorCfg(
            WguiProjectEditorCfgDefaults(), store=cfgStore)
        pcfg = WguiProjectEditorCfg(glblPrjtEdiCfg,
                                    WguiProjectEditorDbCfgStore(cfgTable))
        module.actx = self.__actx
        cctx = WVGUI_COCKPIT_SYSTEM_MODULE.createContext(
            pctx, pcfg, {}, self.__actx, self)
        if cctx:
            editor = WguiProjectEditor(pcfg, pctx, cctx)
            if self.__top.getVLinkFiles():
                filepath = self.__top.getVLinkFiles().path
            else:
                filepath = None
            mainWin = self.__actx.getParent()
            if filepath:
                top = editor.getProjectDB().getTopHierarchy()
                editor.getProjectDB().path = filepath
                top.setSourcePath(filepath)
            editor.scene.refresh()
            editor.scene.populateTable()
            mainWin.tabs.openEditor(editor.getEditorIdent(), editor)
            self.__openProtocolTab(idx, mainWin)

    def __openProtocolTab(self, idx, mainWin):
        """
        if protocol detected open the corresponding tab in traffic route view
        :param idx: Index Object
        :param mainWin: MainWindow application
        """
        model = idx.model()
        if isinstance(model, _WcguiVLinkAutoTableModel):
            idxRoute = 0
            vl = model.getEntries()[idx.row()][idxRoute]
        elif isinstance(model, _WcguiVLinkTableModel):
            idxRoute = 1
            vl = model.getEntries()[idx.row()][idxRoute][1]
        else:
            return
        protocol = vl.getVLType()
        mainWin.tabs.currentEditor.scene.openTrafficEditor(protocol)

    def removeNonExistantBoundVl(self):
        """
        Delete a Traffic routes that were bound but don't exist anymore
        """
        selIdx = self.__modTblWAuto.selectedIndexes()
        producerModel = self.__pBdtbl.getModel()
        consumerModel = self.__cBdtbl.getModel()
        messageVlsUUID = []
        for vlInst, msg in self.__modTblMAuto.getEntries():
            messageVlsUUID.append(vlInst.getUUID())
        producerRows = producerModel.getDataRows()
        for idx in sorted(range(producerModel.rowCount()), reverse=True):
            # [1] get column that contain vl data [1] get Vl instance
            vlProduced = producerRows[idx][1][1]
            if vlProduced.getUUID() not in messageVlsUUID:
                producerModel.deleteRow(idx)

        consumerRows = consumerModel.getDataRows()
        for idx in sorted(range(consumerModel.rowCount()), reverse=True):
            # [1] get column that contain vl data [1] get Vl instance
            vlConsumed = consumerRows[idx][1][1]
            if vlConsumed.getUUID() not in messageVlsUUID:
                consumerModel.deleteRow(idx)

    def _createLayout(self):
        layout = QVBoxLayout()
        layout.setMargin(0)

        layout.addWidget(self.__modTblW)

        btnLayout = QHBoxLayout()

        btnLayout.addWidget(self.__linkButton)
        btnLayout.addWidget(self.__impButton)
        layout.addLayout(btnLayout)

        layout.addWidget(QSplitter())

        layout.addWidget(self.__modTblWAuto)
        btnLayout = QHBoxLayout()
        btnLayout.addWidget(self.__autoRouteButton)
        layout.addLayout(btnLayout)

        return layout

    def ResetWithData(self):
        """
        Fill the table from the project's data
        """
        if self.__top.getVLinkBlock() and self.__top.getVLinkFiles():
            oldRoutesUUIDs = self.getAllRoutesUUIDs()
            self.__modTblM.deleteAll()
            self.__modTblMAuto.deleteAll()
            routes = [inst for inst in self.__top.getVLinkBlock().getInstances()]
            vlFileName = os.path.basename(self.__top.getVLinkFiles().path)
            if len(routes) > 0:
                for vl in routes:
                    messages = vl.getVLMessagesName()
                    if messages:
                        self.__modTblMAuto.addRow(vl, messages)
                    else:
                        self.__modTblM.addRow(vlFileName, (vl.getName(), vl))
            # compare all the routes before and after deletion
            # to see if changes occurred
            newRoutesUUIDs = self.getAllRoutesUUIDs()
            if sorted(oldRoutesUUIDs) != sorted(newRoutesUUIDs):
                self.__scene.sceneModified.emit()

    def getAllRoutesUUIDs(self):
        routes = []
        for row in self.__modTblM.getEntries():
            routes.append(row[1][1].getUUID())
        for row in self.__modTblMAuto.getEntries():
            routes.append(row[0].getUUID())
        return routes

    def _linkVirtualLink(self):
        """
        Import a new virtual link file and fill the mapper data and the table
        with is data
        """
        # Selection d'un fichier
        selected_path = wgui_getOpenFile("Link route file", self,
                                         WPRJ_VLINK_SILKAN_XML_MIME_TYPE)
        if selected_path:
            if self.__top.getVLinkFiles():
                if self.__top.getVLinkFiles().path != selected_path:
                    self.__askLinkFileErase(selected_path)
            elif self.__top.setVLinkFile(WcmnFile(selected_path)):
                self.__pctx.updateVL(selected_path)
                self.ResetWithData()
        self._unbindinvalid()
        # for some reason need to call initialize twice ... need investigation
        self.__actx.getEditor().initialize()
        self.__actx.getEditor().initialize()

    def __askLinkFileErase(self, selected_path):
        diaw = QMessageBox(self)
        diaw.setWindowTitle('Confirm Link')
        diaw.setText('A Traffic Route file is already linked, '
                     ' do you want to link a new one and erase the existing one ?')
        diaw.setInformativeText(
            'All the existing routes will be lost.')
        diaw.addButton(QMessageBox.Yes)
        diaw.addButton(QMessageBox.No)
        diaw.setIcon(QMessageBox.Warning)
        cnfm = diaw.exec()
        if cnfm == QMessageBox.Yes:
            if self.__top.setVLinkFile(WcmnFile(selected_path)):
                self.__pctx.updateVL(selected_path)
                self.ResetWithData()
                return
        elif cnfm == QMessageBox.No:
            return

    def _importVirtualLink(self):
        # Selection d'un fichier
        selected_path = wgui_getOpenFile("Import routes", self,
                                         WPRJ_VLINK_SILKAN_XML_MIME_TYPE)
        if selected_path and self.__top.getVLinkFiles():
            if self.__top.getVLinkFiles().path != selected_path:
                # Recuperation des messages
                handler = WcmodVLinkFileHandler(self.__pctx.getProjectDB().getModule())
                vlink = handler.handleFile(selected_path, self.__pctx)
                if not vlink:
                    raise ValueError("Wrong source file for Traffic Route Source.")
                vlBlock = self.__top.getVLinkBlock()
                for inst in vlink.getOperatorInstances():
                    oper = inst.getOperator()
                    allRoutes = [inst.getName() for inst in vlBlock.getInstances()]
                    routeName = whdb_getNextName(inst.getName(),
                                                 allRoutes)
                    newInst = vlBlock.createOperatorInstance(oper, routeName)
                    params = inst.getVLParametersDict(inst.getVLType())
                    newInst.setVLType(inst.getVLType())
                    newInst.setVLParameters(params)
                self.ResetWithData()
                self._unbindinvalid()

    def updateTable(self, tmpPath=None):
        """
        Update the table
        """
        self.__pctx.getMappingManager().resetVLink()
        if tmpPath:
            self.__pctx.updateVL(tmpPath)
            self.__top.getVLinkBlock().setTmpFilePath(tmpPath)
        else:
            vlf = self.__top.getVLinkFiles()
            if vlf:
                self.__pctx.updateVL(vlf.path)
                editor = self.__parent
                tabs = self.__actx.getParent().tabs
                if editor:
                    editor.createTmpFile(tabs)
        self.ResetWithData()
        self._unbindinvalid()

    def autoRouteFunction(self):
        map = self.__pctx.getMappingManager()
        if not map.logicTop:
            showErrorWindow("Route Creation",
                            'No logical view is bound', None)
            return
        bindingModule = WcguiAutoBindModules(self.__pctx, self.__scene,
                                             self.__modTblWAuto,
                                             self.__modTblMAuto,
                                             self.__pBdtbl,
                                             self.__cBdtbl,
                                            )
        oldRoutesUUIDs = self.getAllRoutesUUIDs()
        bindingModule.computeMapping()
        # compare all the routes before and after deletion
        # to see if changes occurred
        newRoutesUUIDs = self.getAllRoutesUUIDs()
        if sorted(oldRoutesUUIDs) != sorted(newRoutesUUIDs):
            self.__scene.sceneModified.emit()

    def _unbindinvalid(self):
        """
        inbind invalide physical port/virtual link and connections
        """
        self.__pctx.unbindInvalidVLinkBinding()
        if not self.__PropagateUpdateFunc is None:
            self.__PropagateUpdateFunc()

    def setPropagationFunction(self, propfunc):
        """
        give logic table a function to propagate a module update effect on
        other tables
        """
        self.__PropagateUpdateFunc = propfunc


class _WcguiVLinkTableModel(QAbstractTableModel):
    """
    Bus Binding table model.
    """

    TITLE_FILE = 'File'
    TITLE_VL = 'Traffic Routes User'
    HEADERS = [TITLE_FILE, TITLE_VL]

    def __init__(self, parent=None):
        """
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Entries as list of tuples of
        #  physical Port (pport) and logical Port (lport)
        self.__entries = []
        """:type: list[]"""

        self._update()

    def getEntries(self):
        return self.__entries

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def _doUpdate(self):
        pass

    def beginResetModel(self):
        super().beginResetModel()

    def endResetModel(self):
        self._doUpdate()
        super().endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
               0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def addRow(self, file_, vl):
        """
        Add a row at the end of the list (append)

        :param vl: traffic route name
        :type vl: | string
        :param file_: traffic route file
        :type file_: | [string]
        :return: The appended row index
        :rtype: int
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())
        self.__entries.append((file_, vl))
        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)
        self.__entries.pop(rowIndex)

        self.endRemoveRows()
        self._update()

    def deleteAll(self):
        self.deleteRows(range(0, self.rowCount()))

    def addelem(self, elem, line, column):
        """
        add the element at the line and column given : eventualy create new line
        """

        if line >= self.rowCount():
            if column == 0:
                self.addRow(elem[0], elem[1])
            return
        if column == 0:
            self.__entries[line] = elem
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if column == 0:
            info = self.__entries[row][column]
            if info is None:
                return None
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return info
            elif role == Qt.UserRole:
                return info
        elif column == 1:
            info = self.__entries[row][column]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole, Qt.UserRole]:
                return info[0]
        return None

    def flags(self, index):
        return Qt.ItemIsEditable | Qt.ItemIsDragEnabled | Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsDropEnabled


class _WcguiVLinkAutoTableModel(QAbstractTableModel):
    """
    Bus Binding table model.
    """

    TITLE_VL = 'Traffic Routes Auto'
    MESSAGES = 'Messages'
    HEADERS = [TITLE_VL, MESSAGES]

    def __init__(self, parent=None):
        """
        :param parent: Parent widget
        """
        super().__init__(parent)


        #: Entries as list of tuples of
        #  physical Port (pport) and logical Port (lport)
        self.__entries = []
        """:type: list[]"""

        self._update()

    def getEntries(self):
        return self.__entries

    def getRoutesName(self):
        return [vl.getName() for vl, _ in self.__entries]

    def getMessagesName(self):
        return [msg for _, msg in self.__entries]

    def _update(self):
        self.beginResetModel()
        self.endResetModel()

    def _doUpdate(self):
        pass

    def beginResetModel(self):
        super().beginResetModel()

    def endResetModel(self):
        self._doUpdate()
        super().endResetModel()

    def headerData(self, section, orientation, role=None):
        if role == Qt.DisplayRole or role == Qt.EditRole:
            if orientation == Qt.Horizontal and \
               0 <= section < len(self.HEADERS):
                return self.HEADERS[section]
            else:
                return str(section)
        return None

    def addRow(self, vl, messages):
        """
        Add a row at the end of the list (append)

        :param vl: traffic route name
        :type vl: | string
        :param messages: Messages to be set
        :type messages: | [string]
        :return: The appended row index
        :rtype: int
        """
        index = self.index(self.rowCount(), 0)
        self.beginInsertRows(index, self.rowCount(), self.rowCount())
        if len(messages) == 0:
            strMsg = 'No messages bound'
        elif len(messages) > 1:
            strMsg = '[{}]'.format(', '.join([msg for msg in messages]))
        else:
            strMsg = messages[0]
        self.__entries.append((vl, strMsg))

        self.endInsertRows()
        self._update()

    def deleteRows(self, rowIndices):
        """
        :param rowIndices: Iterable of indices
        :type rowIndices: Iterable[int]
        """
        # Make sure no duplicate exist
        rowList = list(set(rowIndices))
        rowList.sort(reverse=True)

        for row in rowList:
            self.deleteRow(row)

    def deleteRow(self, rowIndex):
        """
        Delete a row entry

        :param rowIndex: Row to the delete
        :type rowIndex: int
        """
        if not (0 <= rowIndex < self.rowCount()):
            raise ValueError('Index of row is out of bounds')

        index = self.index(rowIndex, 0)
        self.beginRemoveRows(index, rowIndex, rowIndex)
        self.__entries.pop(rowIndex)

        self.endRemoveRows()
        self._update()

    def deleteAll(self):
        self.deleteRows(range(0,self.rowCount()))

    def addelem(self, elem, line, column):
        """
        add the element at the line and column given : eventualy create new line
        """

        if line >= self.rowCount():
            if column == 0:
                self.addRow(elem[0], elem[1])
            return
        if column == 0:
            self.__entries[line] = elem
        self._update()

    def index(self, row, column, parentIndex=None, *args, **kwargs):
        return self.createIndex(row, column)

    def rowCount(self, parent=None, *args, **kwargs):
        return len(self.__entries)

    def columnCount(self, parent=None, *args, **kwargs):
        return len(self.HEADERS)

    def data(self, index, role=None):
        row = index.row()
        column = index.column()
        if column == 0:
            info = self.__entries[row][column]
            if info is None:
                return None
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole]:
                return info.getName()
            elif role == Qt.UserRole:
                return info.getName()
        elif column == 1:
            info = self.__entries[row][column]
            if role in [Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole, Qt.UserRole]:
                return info
        return None

    def flags(self, index):
        return Qt.ItemIsEditable | Qt.ItemIsDragEnabled | Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsDropEnabled
