# ----------------------------------------------------------------------------
# - wpgui-controller-value-provider                                          -
# - wild cruncher: controller value provider         frontend / system / gui -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpgui_controller_value_provider
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher - System GUI controller value provider

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from PyQt4.QtGui import QDialog, QComboBox, QVBoxLayout, QLabel, \
    QDialogButtonBox
from wcr_core.controller.wctl_controller_value_provider_api import \
    WctlControllerValueProviderP
from wcr_core.db_api.whdb_instance_api import WhdbOperatorInstanceP
from wcr_core.dbstore_api.wdbs_connection_point_api import WDBS_XPNT_BI
from wcr_core.modules.system.db.wphdb_host_instance import WphdbHostInstance
from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data


class _WphdbXPointTypeSelectionDialog(QDialog):
    """
    Dialog to request connection point type from user for connection point
    creation.
    """

    def __init__(self, xpntTypes, parent=None):
        """
        :param xpntTypes: Connection point types
        :type xpntTypes: Iterable[WpmodConnectionPointTypeP]
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Connection point type combo box
        self.__typeF = QComboBox()

        #: Chosen connection point type
        self.__type = None

        self.setLayout(self._createLayout())
        self._initElements(xpntTypes)

    def _createLayout(self):
        """
        Create the layout.

        :return: Layout
        :rtype: QLayout
        """
        layout = QVBoxLayout()

        layout.addWidget(
            QLabel('Please select the type of the new connection point:'))
        layout.addWidget(self.__typeF)

        buttons = QDialogButtonBox(
            QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        buttons.accepted.connect(self.accept)
        buttons.rejected.connect(self.reject)
        layout.addWidget(buttons)

        return layout

    def _initElements(self, xpntTypes):
        """
        Initialize the UI elements, as needed.

        :param xpntTypes: Connection point types
        :type xpntTypes: Iterable[WpmodConnectionPointTypeP]
        """
        self.__typeF.addItem('')

        for xptt in xpntTypes:
            self.__typeF.addItem(xptt.getName(), xptt)

    @property
    def type(self):
        """
        Get the chosen connection point type, if any.

        :return: Chosen connection point type
        :rtype: str | None
        """
        return self.__type

    def accept(self):
        self.__type = wgui_qcb_get_data(self.__typeF)

        super().accept()


class WpguiControllerValueProvider(WctlControllerValueProviderP):
    def __init__(self, pctx):
        """
        :param pctx: Project context
        :type pctx: WpmodSystemProjectContextP
        """
        self.__module = pctx.getModule()
        """:type: WpmodSystemApplicationModuleP"""

    def provideNewConnectionPointDirection(self, instance, **kwargs):
        return WDBS_XPNT_BI

    def provideNewConnectionPointType(self, instance, **kwargs):
        xpntTypes = list(self.__module.getConnectionPointTypes())
        vldTypes = None

        if isinstance(instance, WphdbHostInstance):
            vldTypes = xpntTypes
        elif isinstance(instance, WhdbOperatorInstanceP):
            xpdsc = instance.getOperator().getXPointDescriptor()
            ctps = [xpnt.ctyp for xpnt in xpdsc.getDynamicXPoints()]
            if ctps:
                vldTypes = [xtyp for xtyp in xpntTypes
                            if xtyp.getCode() in ctps]

        if not vldTypes:
            return None
        elif len(vldTypes) == 1:
            return vldTypes[0].getCode()
        else:
            diag = _WphdbXPointTypeSelectionDialog(vldTypes, None)
            diag.show()
            diag.exec_()
            ctyp = diag.type
            """:type: WpmodConnectionPointType | None"""
            return ctyp.getCode() if ctyp else None
