# ----------------------------------------------------------------------------
# - wpgui-host-instance-props-dialog                                         -
# - wild cruncher: system mapper host instance properties                    -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpgui_host_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher system mapper host instance properties

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from PyQt5.QtWidgets import QLineEdit, QSpinBox, QVBoxLayout, QLabel

from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog
from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend.modules.system.gui.wpgui_host_cards_props_widget import \
    WpguiHostCardsProperties
from wcr_frontend.modules.system.gui.wpgui_host_processes_props_widget import \
    WpguiHostProcessesProperties
from wcr_frontend.modules.system.gui.wpgui_host_connection_points_props_widget\
    import WpguiHostXPointProperties


class WpguiHostInstancePropertiesDialog(
        WguiOperatorInstancePropertiesDialog):
    _TYPE_NAME = 'Host Instance'

    _SHOW_OPERATOR_WIDGET = False

    def __init__(self, inst, pctx, updateF, selectXpnt=None, parent=None):
        """
        :param inst: Host instance
        :type inst: WphdbHostInstance
        :param pctx: Project context
        :type pctx: WpmodSystemProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param selectXpnt: Initially selected connection point, if any
        :type selectXpnt: WhdbConnectionPointP
        :param parent: Parent widget
        """
        super().__init__(inst, pctx, updateF, selectXpnt, parent)

        self.setMinimumWidth(700)
        self.setMinimumHeight(600)

    # noinspection PyMethodMayBeStatic,PyUnusedLocal
    def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
        return [_WpguiHostCapabilities(inst)]

    def _createAdditionalPropertiesWidgets(self, inst, pctx):
        return [('Processes', WpguiHostProcessesProperties(inst, pctx)),
                ('Cards', WpguiHostCardsProperties(inst, pctx))]

    # noinspection PyMethodMayBeStatic
    def _createConnectionPointsPropertiesWidget(self, inst, pctx):
        return 'Conn. Points', WpguiHostXPointProperties(inst, pctx)


class _WpguiHostCapabilities(WguiPropertiesWidgetP):
    """
    Property widget for host capabilities.
    """

    def __init__(self, host, parent=None):
        """
        :param host: Host instance
        :type host: WphdbHostInstance
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Host instance
        self.__host = host

        #: CPU field
        self.__cpuF = QLineEdit()
        #: Cores field
        self.__coresF = QSpinBox()
        #: GPU field
        self.__gpuF = QLineEdit()
        #: Memory field
        self.__memoryF = QLineEdit()
        #: Disk space field
        self.__diskF = QLineEdit()

        self.setLayout(self._createLayout())
        self._initElements()
        self.updateFields()

    def _createLayout(self):
        """
        Create the layout.

        :return: Layout
        :rtype: QLayout
        """
        layout = QVBoxLayout()
        layout.addWidget(QLabel('CPU:'))
        layout.addWidget(self.__cpuF)
        layout.addWidget(QLabel('Cores:'))
        layout.addWidget(self.__coresF)
        layout.addWidget(QLabel('GPU:'))
        layout.addWidget(self.__gpuF)
        layout.addWidget(QLabel('Memory:'))
        layout.addWidget(self.__memoryF)
        layout.addWidget(QLabel('Disk space:'))
        layout.addWidget(self.__diskF)
        return layout

    def _initElements(self):
        """
        Initialize the UI elements.
        """
        self.__coresF.setMinimum(1)

    def updateFields(self):
        host = self.__host

        self.__cpuF.setText(host.getCPU())
        self.__coresF.setValue(host.getCores())
        self.__gpuF.setText(host.getGPU())
        self.__memoryF.setText(host.getMemory())
        self.__diskF.setText(host.getDiskSpace())

    def save(self):
        host = self.__host

        host.setCPU(self.__cpuF.text())
        host.setCores(self.__coresF.value())
        host.setGPU(self.__gpuF.text())
        host.setMemory(self.__memoryF.text())
        host.setDiskSpace(self.__diskF.text())
