# ----------------------------------------------------------------------------
# - wpgui-project-props-dialog                                               -
# - wild cruncher - cockpit: project properties dialog widgets               -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpgui_project_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher project properties dialog

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
"""
from PyQt4.QtGui import QLabel, QLineEdit, QVBoxLayout, QPushButton, QIcon, \
    QHBoxLayout

from wcr_frontend.gui.wgui_project_props_dialog import \
    WguiProjectPropertiesDialog
from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend.gui.wgui_utils import wgui_getOpenFile
from wcr_core.project.wprj_xmlio import WPRJ_SILKAN_XML_MIME_TYPE
from silkan_common.scmn_file_path import scmn_expshenv, scmn_path_dir


class WpguiProjectPropertiesDialog(WguiProjectPropertiesDialog):
    """
    The WpguiProjectPropertiesDialog class is a dialog design to access
    and manipulate project metadata, project editor preferences and project
    configuration settings.
    """

    def __init__(self, prjContext, config, updateF, parent=None):
        """
        :param prjContext: Project context
        :type prjContext: WcorProjectContextP
        :param config: Project editor configuration
        :type config: WguiProjectEditorCfg
        :param updateF: Callback function to notify about saves
        :type updateF: _ -> _
        :param parent: Parent widget
        """
        super().__init__(prjContext, config, updateF, parent=parent)

    def _createProjectPropertiesWidgets(self, prjContext):
        return [('System Mapping Properties',
                 _WpguiProjectPropertiesWidget(prjContext))]


class _WpguiProjectPropertiesWidget(WguiPropertiesWidgetP):
    """
    Configuration widget for project properties.
    """

    def __init__(self, prjContext, parent=None):
        """
        :param prjContext: Project context
        :type prjContext: WpmodSystemProjectContextP
        :param parent: Parent widget
        """
        super().__init__(parent)

        #: Project info properties
        self.__prjContext = prjContext
        #: System-mapper hierarchy top-level
        self.__topBlock = prjContext.getProjectDB().getTopHierarchy()

        #: Name field
        self.__lsimPathF = QLineEdit()
        #: Name field
        self.__lsimName = QLabel()
        #: Name field
        self.__lsimInfo = QLabel()

        self.__lsimPathF.textChanged.connect(self._pathChanged)

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        """
        Create the layout for the widget.
        Should only be called once.

        :return: Layout
        :rtype: QLayout
        """
        layout = QVBoxLayout()
        layout.addWidget(QLabel('Logical Simulation File:'))
        pathLayout = QHBoxLayout()
        pathLayout.addWidget(self.__lsimPathF)
        fileButton = QPushButton()
        fileButton.setIcon(QIcon(':/img/app_was_open.png'))
        fileButton.clicked.connect(self._fileSelectClicked)
        pathLayout.addWidget(fileButton)
        layout.addLayout(pathLayout)
        layout.addWidget(self.__lsimName)
        layout.addWidget(self.__lsimInfo)

        return layout

    def _pathChanged(self, _):
        self.__lsimName.setText('')
        self.__lsimInfo.setText('')

    def _fileSelectClicked(self):
        # noinspection PyTypeChecker
        path = wgui_getOpenFile(
            'Select Logical Simulation File', self,
            WPRJ_SILKAN_XML_MIME_TYPE,
            idir=scmn_path_dir(scmn_expshenv(self.__lsimPathF.text())))
        if path:
            self.__lsimPathF.setText(path)

    def updateFields(self):
        # noinspection PyUnresolvedReferences
        file = self.__topBlock.getLogicalSimulationFile()
        if file:
            self.__lsimPathF.setText(file.path)
        else:
            self.__lsimPathF.setText('')

        lsim = self.__prjContext.getLogicalSimulation()
        if lsim is not None:
            self.__lsimName.setText(lsim.getName())
            self.__lsimInfo.setText(lsim.getInfo())
        else:
            self.__lsimName.setText('')
            self.__lsimInfo.setText('')

    def save(self):
        # noinspection PyUnresolvedReferences
        self.__topBlock.setLogicalSimulationFile(self.__lsimPathF.text())
