# ----------------------------------------------------------------------------
# - wpgui-props-dialog-factory                                               -
# - wild cruncher - cockpit: properties dialog box factory                   -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2014 clemens oertel                                   -
# - author    (c) 2014-2014 samuel longchamps                                -
# - copyright (c) 2014-2014 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wpgui_props_dialog_factory
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher properties dialog box factory

.. moduleauthor:: Clemens Oertel <clemens.oertel@silkan.com>
.. moduleauthor:: Samuel Longchamps <samuel.longchamps@silkan.com>
"""
from wcr_frontend.gui.wgui_props_dialog_factory import \
    WguiPropertiesDialogFactory
from wcr_frontend.modules.system.gui.wpgui_host_instance_props_dialog import \
    WpguiHostInstancePropertiesDialog
from wcr_frontend.modules.system.gui.wpgui_phybus_instance_props_dialog \
    import WpguiPhyBusInstancePropertiesDialog
from wcr_frontend.modules.system.gui.wpgui_project_props_dialog import \
    WpguiProjectPropertiesDialog
from wcr_core.modules.system.db.wphdb_standard_operator import WPHDB_HOST, \
    WPHDB_BUS


WPHDB_STD_OPERINST_PROPDLG = {WPHDB_HOST: WpguiHostInstancePropertiesDialog,
                              WPHDB_BUS: WpguiPhyBusInstancePropertiesDialog,
                              }


class WpguiPropertiesDialogFactory(WguiPropertiesDialogFactory):
    def __init__(self, prjContext, config, scene, actions, actx, parent=None):
        """
        :param prjContext: Project context
        :type prjContext: WpmodSystemProjectContextP
        :param config: Project editor configuration
        :type config: WguiProjectEditorCfgP
        :param scene: Graphic scene adapter
        :type scene: WguiSceneAdapterP
        :param actions: GUI actions
        :type actions: dict[str, WactActionP]
        :param actx: Action execution context
        :type actx: WactActionExecutionContextP
        :param parent: Parent widget
        """
        super().__init__(prjContext, config, scene, actions, parent)

        self.__actions = actions
        self.__actx = actx

    def _createProjectPropertiesDialog(self, prjContext, config, updateF,
                                       parent):
        def save():
            action = self.__actions.get('wcr.core.system.reloadsim')
            if action:
                action.execute(self.__actx)
            else:
                raise Exception(
                    'Could not find action in ' + str(self.__actions))
            if updateF:
                updateF()

        return WpguiProjectPropertiesDialog(
            prjContext, config, save, parent=parent)

    def _createOperatorInstancePropertiesDialog(self, inst, pctx, updateF,
                                                selectXpnt, parent):
        propDlg = WPHDB_STD_OPERINST_PROPDLG.get(inst.getOperator())
        if propDlg is not None:
            return propDlg(inst, pctx, updateF, selectXpnt, parent)
        else:
            return super()._createOperatorInstancePropertiesDialog(
                inst, pctx, updateF, selectXpnt, parent)
