# ----------------------------------------------------------------------------
# - wvgui-project-actions                                                    -
# - wild cruncher - project editor actions                                   -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvgui_project_actions
   :platform:     Unix, Darwin, Windows
   :synopsis:     Wild Cruncher project editor actions

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""

from wcr_core.action.wact_action_parameter import WactActionParameter
from wcr_core.action.wact_action_parameter_types import \
    WactActionOpenFileByPayloadParameterType
from wcr_core.common.wcmn_mime_type import WCMN_PAYLOAD_BLOCK_CSV
from wcr_core.modules.vlink.wvmod_application_module import \
    WVMOD_VLINK_APP_CODE, wvhdb_import_csv_routes
from wcr_core.modules.hardware.db.whhdb_connection_points import \
    WHHDB_XPOINT_CETRAC_CODE, WHHDB_XPOINT_ETHERNET_CODE, \
    WHHDB_XPOINT_DISCRET_CODE, WHHDB_XPOINT_FIBER_CHANNEL_CODE, \
    WHHDB_XPOINT_1553_CODE, WHHDB_XPOINT_ARINC825_CODE, \
    WHHDB_XPOINT_AFDX_CODE, WHHDB_XPOINT_ARINC429_CODE
from wcr_frontend.gui.wgui_project_actions import WguiEditorActionBase
from wcr_core.project.wprj_xmlio import WPRJ_SILKAN_CSV_MIME_TYPE
from wcr_frontend.gui.wgui_utils import wgui_getOpenFile
from wcr_frontend.modules.vlink.gui.wvgui_scene_protocol_table import *
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import *


class _WmguiEditorActionBase(WguiEditorActionBase):
    def __init__(self, name, description, *parameters):
        """
        :param name: Action name
        :type name: str
        :param description: Action description
        :type description: str
        :param parameters: Action parameter descriptions
        :type parameters: list[WactActionParameterP]
        """
        super().__init__(name, description, *parameters)

    def isEnabled(self, actx):
        if not super().isEnabled(actx):
            return False
        editor = actx.getEditor()
        """:type: WguiProjectEditor"""
        cctx = editor.cockpitSystemContext
        return cctx.code == WVMOD_VLINK_APP_CODE


class _WvguiEditorImportCSVAction(_WmguiEditorActionBase):
    """
    Project editor action to import a model.
    """

    #: Action parameters
    __PARAMETER = WactActionParameter(
        'model', 'Model file', 'Model file to import',
        WactActionOpenFileByPayloadParameterType(WCMN_PAYLOAD_BLOCK_CSV))

    def __init__(self):
        super().__init__(
            'Import Vlink from CSV...',
            'Import Traffic Route from CSV File',
            self.__PARAMETER)

    def _executeForEditor(self, editor, actx, paramValuePairs):
        file = wgui_getOpenFile("Import csv", actx.getEditor(),
                                WPRJ_SILKAN_CSV_MIME_TYPE)
        # file = self._getParamValue(self.__PARAMETER, actx, paramValuePairs)
        """:type: WcmnFile | None"""
        if not file:
            return
        pctx = actx.getProjectContext()
        tblk = pctx.getProjectDB().getTopHierarchy()
        modules = {}
        modulesInst = editor.scene.getTabModule()
        modules[WVHDB_VLINK_TYPE_AFDX] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_AFDX_CODE]]
        modules[WVHDB_VLINK_TYPE_CETRAC] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_CETRAC_CODE]]
        modules[WVHDB_VLINK_TYPE_ETNERNET] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_ETHERNET_CODE]]
        modules[WVHDB_VLINK_TYPE_DISCRETE] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_DISCRET_CODE]]
        modules[WVHDB_VLINK_TYPE_FIBER] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_FIBER_CHANNEL_CODE]]
        modules[WVHDB_VLINK_TYPE_1553] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_1553_CODE]]
        modules[WVHDB_VLINK_TYPE_ARINC825] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_ARINC825_CODE]]
        modules[WVHDB_VLINK_TYPE_ARINC429] = modulesInst[HW_2_VL_TYPE[WHHDB_XPOINT_ARINC429_CODE]]
        fl = open(file, 'r')
        wvhdb_import_csv_routes(tblk, fl, modules, editor)


#: Model csv import action
WVGUI_IMPORT_CSV_ACTION = _WvguiEditorImportCSVAction()
