# ----------------------------------------------------------------------------
# - wvgui-vlink-instance-props-dialog                                        -
# - wild cruncher - cockpit : Traffic Route properties dialog widgets         -
# ----------------------------------------------------------------------------
# - author    (c) 2016-2016 philippe lambert                                 -
# - copyright (c) 2016-2016 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wvgui_vlink_instance_props_dialog
   :platform:     Unix, Windows
   :synopsis:     Wild Cruncher instance properties dialog

.. moduleauthor:: Philippe Lambert <lambert@silkan.com>
"""
from PyQt5.QtCore import QAbstractTableModel, Qt
from PyQt5.QtWidgets import QComboBox, QLineEdit, QVBoxLayout, QLabel, \
    QTableWidgetItem, QTableWidget

from wcr_frontend.gui.wgui_utils import wgui_qcb_get_data
from wcr_frontend.gui.wgui_props_dialog_widget import WguiPropertiesWidgetP
from wcr_frontend.gui.wgui_operator_instance_props_dialog import \
    WguiOperatorInstancePropertiesDialog
from wcr_core.modules.vlink.db.wvhdb_Vlink_instance import WVHDB_VLINK_TYPES

class WvguiVLinkInstancePropertiesDialog(WguiOperatorInstancePropertiesDialog):
    _TYPE_NAME = 'Traffic Route Message Instance'

    _SHOW_OPERATOR_WIDGET = False

    def __init__(self, inst, pctx, updateF, selectXpnt=None, parent=None):
        """
        :param inst: the instance object
        :type inst: WhdbInstanceP
        :param pctx: Project context
        :type pctx: WcorProjectContextP
        :param updateF: Callback function to notify about saves
        :type updateF: () -> _
        :param parent: Parent widget
        """
        super().__init__(inst, pctx, updateF, selectXpnt, parent)


    def _createAdditionalInstancePropertiesWidgets(self, inst, pctx):
        return []#_WvguiVlinkProperties(inst)]

    # noinspection PyMethodMayBeStatic
    def _createAdditionalPropertiesWidgets(self, inst, pctx):

        return [('Configuation', WvguiConfigProperties(inst))]

    def _createConnectionPointsPropertiesWidget(self, inst, pctx):
        return None

class WvguiConfigProperties(WguiPropertiesWidgetP):
    """
    The WvguiConfigProperties is a tab widget for the Virtual link
    configuration dialog. It displays the instance parameters properties.
    """

    def __init__(self, inst, parent=None):
        """
        :param inst: instance
        :type inst: WhdbOperatorInstanceP
        :param parent: Parent widget
        """
        # base initialization
        super().__init__(parent)

        #: Instance object
        self.__vlink = inst

        #: Size field
        self.__VLT = QComboBox(parent)

        for type in WVHDB_VLINK_TYPES:
            self.__VLT.addItem(type, type)

        #: Parameter table
        self.__table = QTableWidget()

        #: Destination number size
        self.__dest = QLineEdit()

        self.setLayout(self._createLayout())
        self.updateFields()

    def _createLayout(self):
        """
        Create the layout for the widget.
        Should only be called once.

        :return: Layout
        :rtype: QLayout
        """
        self.__VLT.currentIndexChanged.connect(self.updateTable)

        table = self.__table
        table.setSortingEnabled(True)
        layout = QVBoxLayout()
        layout.addWidget(QLabel('Traffic Route Type:'))
        layout.addWidget(self.__VLT)
        layout.addWidget(QLabel('Traffic Route Destination Number: '
                                '(0 to "Undefine")'))
        layout.addWidget(self.__dest)

        layout.addWidget(QLabel('Traffic Route configuration'))
        layout.addWidget(table)
        return layout

    def updateFields(self):
        combobox = self.__VLT
        for index in range(combobox.count()):
            if combobox.itemData(index, Qt.UserRole) == \
                    self.__vlink.getVLType():
                combobox.setCurrentIndex(index)
                break
        else:
            combobox.setCurrentIndex(0)

        self.__dest.setText(str(self.__vlink.getDestNumber()))
        self.updateTable()

    def updateTable(self):
        combobox = self.__VLT
        table = self.__table
        params = list(
            self.__vlink.getVLParameters(
                vltype=wgui_qcb_get_data(combobox)))

        table.clear()
        table.setColumnCount(2)
        table.setRowCount(len(params))
        table.setHorizontalHeaderLabels(['Name', 'Value'])
        for row, param in enumerate(params):
            self._addParameter(param, row)
        table.resizeColumnsToContents()
        table.horizontalHeader().setStretchLastSection(True)

    def _addParameter(self, param, row):
        """
        Add a new parameter line in the parameter table.

        :param param: The parameter to add as tuple of label,value,key
        :type param: (str,str,str)
        :param row: Row number of the new row
        :type row: int
        """
        table = self.__table

        item = self._dis(QTableWidgetItem(param[0]))
        item.setData(Qt.UserRole, param)
        table.setItem(row, 0, item)

        value = param[1]
        item = QTableWidgetItem(str(value) if value is not None else '')
        item.setFlags(item.flags() | Qt.ItemIsEditable)
        table.setItem(row, 1, item)

    def save(self):
        combobox = self.__VLT
        self.__vlink.setVLType(wgui_qcb_get_data(combobox))
        table = self.__table
        params={}
        for row in range(table.rowCount()):
            paramk = table.item(row, 0).data(Qt.UserRole)[2]
            value = table.item(row, 1).text()
            params[paramk]=value
        self.__vlink.setVLParameters(params)
        self.__vlink.setDestNumber(int(self.__dest.text()))

