# ----------------------------------------------------------------------------
# - wmaob_matlab_init                                                        -
# - wild cruncher - model matlab initialiser                                 -
# ----------------------------------------------------------------------------
# - author    (c) 2014-2015 philippe lambert                                 -
# - copyright (c) 2014-2015 silkan / wild systems                            -
# ----------------------------------------------------------------------------

"""
.. module::       wmaob_callback_gene
   :platform:     Unix
   :synopsis:     Initialise

.. moduleauthor:: Philippe Lambert <philippe.lambert@silkan.com>
"""

import os

class WmaobCallbacksGenerator():
    """
    This class generates the cpp files for the callbacks.
    """
    def __init__(self, pctx, prof=None):
        """
        Generate the differents callback functions of the project.

        :param pctx: project context
        :type  pctx: WcorProjectContextP
        :param prof: profile name
        :type  prof: str | None
        """
        self.inst = pctx.getProjectDB().getTopHierarchy()
        self.sset = self.inst.getSchedulingGroupSet()
        if self.canUseCallback():
            funcs = self.inst.getCallbackDict().keys()
                    # loop in the set
            for name in self.sset:
            # do not process none group
                if name == 'none': continue
                for fname in funcs:
                    if self.inst.getCallbackFunction(fname) is not "" or None:
                        self.inst.CallbackFunction(fname, pctx, prof, name)

    def canUseCallback(self):
        """
        There is a restriction on using callbacks : only one scheduling group
        is allowed or all the callbacks must be empty. In other cases, global
        variables can't be shared between scheduling groups.

        :return: True if the code can be generated
        :rtype: bool
        """
        return list(self.inst.getCallbackDict().values()).count('') == \
               len(list(self.inst.getCallbackDict().values())) \
               or len(self.sset) == 1

    def getSchedulingName(self):
        """
        Can only be used if canUseCallback is True

        :return: the scheduling group name
        :rtype: str
        """
        for name in self.sset:
            # do not process none group
            if name == 'none': continue
            return name
